; $Id: adn.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    ADN
;
; ROUTINE'S PURPOSE:
;    ADN manages the simulation for the ADd Noise (ADN) module,
;    that is to call the module's initialization routine (adn_init) at the first
;    iteration of the simulation project.
;
; MODULE'S PURPOSE:
;    ADN permits to add photon noise, background noise, read-out noise and
;    dark-current noise to an image or a cube of images. ADN allows also to
;	 simulate the saturation of the CCD as well as the perturbations due to 
;    flat-field and bad-pixels.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = adn(inp_img_t, out_img_t, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t: structure of type img_t.
;    par      : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_img_t: structure of type "img_t" containing the characteristics of
;               the noisy map, the following fields are contained in
;               this structure:
;
;out_img_t =                             $
;   {                                     $
;   data_type  : out_type[0]          , $
;   data_status: !caos_data.valid     , $
;   image      : noisy_map            , $  ; IMAGE with same resolution as PSF.
;   npixel     : inp_img_t.npixel     , $  ; Number of pixels (per subaperture)
;   resolution : inp_img_t.resolution , $  ; Pixel size [arcsec/pix]
;   lambda     : inp_img_t.lambda     , $  ; Mean detection wavelength [m]
;   width      : inp_img_t.width      , $  ; Wavelength band width [m]
;   time_integ : inp_img_t.time_integ , $  ; Integration time for each image [s]
;   time_delay : 0.                   , $  ; [NOT USED WITHIN AIRY]
;   psf        : 0B                   , $  ; this is an image, not a PSF
;   background : background           , $  ; the value of the background
;   snr        : fltarr(360)          , $  ; [NOT YET EVALUATED]
;   header     : PTR_NEW('')            $  ; Header (modified by this procedure)
;   }
; *out_img_t.header = header  ;update the pointer
;
; KEYWORD PARAMETERS:
;    INIT: initialization data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;     none.
;
; CALLED NON-IDL FUNCTIONS:
;    adn_add_noise.pro : add the noise (Poisson and Gaussian) in the map
;    adn_ccd_sat.pro   : CCD saturation control
;
; MODULE MODIFICATION HISTORY:
;    module written : version beta,
;                     Serge  Correia   (OAA) [correia@arcetri.astro.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it].
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                    -call to adn_prog eliminated (useless).
;                   : january 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;
;
; MODULE MODIFICATION HISTORY:
;    module written : Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : for version 1.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                    -call to adn_prog eliminated (useless).
;                   : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                   : march 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -now having control over initial seed when generating
;		                read-out noise.
;                   : april 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added possibility of choice between three type of noise:
;                     poisson, read-out and dark-current
;                    -having control over initial seed when generating also
;                     poisson noise and dark-current noise.
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -use of variable "time_integ" now is active.
;                    -fixed bug in calculation of dark_current noise
;                   : april 2006,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -CCD saturation and full-well capacity parameters added in 
;                     order to simulate the saturation of the CCD.
;                    -function 'add_ccd_sat.pro' added.
;                   : for version 5.0,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added possibility to simulate multi-frame images.
;                     Multi-frame images cannot be saved (by WFT), but can be 
;                     only automatically reduced by using PRE.
;                    -DITHERING added: each frame can be shifted in 
;                     two different ways: 
;                     a) random shifts: the positions will be created within a 
;                        circle of given radius;
;                     b) user defined shifts: the values are stored into a file 
;                        and will be loaded in the ADN_ADD_NOISE procedure. 
;                        In this case each line of the file must contain X and 
;                        Y shifts, separated by a comma.
;                     Dithering can be applied to either each frame or every N
;                     frames (N is given by the user).
;                    -Added the MASKing of the images.
;                    -FLAT-FIELD and BAD-PIXELS perturbations added. 
;                   : march 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added another TAB in the GUI containing the possibility  
;                     of clipping negative values after RON and converting the
;                     output to an arbitrary IDL data type. 
;                    -The function "addnoise.pro" has been integrated 
;                     within the routine "adn_add_noise.pro".
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2012, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]: 
;                    -small changes in the GUI + changes in
;                     "addnoise.pro" routine. 
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -The header of the input/output image is kept
;                     up-to-date with noise information
;                    -background is now an array (from previous
;                     modules or a new array). Small changes to the GUI 
;-
;
function adn, inp_img_t, out_img_t, par, INIT=init

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialisation
error = !caos_error.ok

; module's action
if (this_iter eq 0) then error=adn_init(inp_img_t, out_img_t, par, INIT=init)
;error=adn_init(inp_img_t, out_img_t, par, INIT=init)
return, error
end
