; $Id: adn_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    adn_gen_default
;
; PURPOSE:
;    adn_gen_default generates the default parameter structure for the adn
;    module and save it in the right location.
;    (see adn.pro's header --or file airy_help.html-- for details about the
;     module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    adn_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -phot_flux parameter added in order to possibly define the
;                     flux from the observed source as well (like for the
;                     background with back_flux).
;                    -took-off useless parameters (tel. surf., int. time, etc.)
;                     and redefined background and object/source nb of photons
;                     as the nb of resulting photons per pixel in the images.
;                   : May 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : january 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS")
;                   : march 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -seed_ron parameter added in order to generate different
;                     sequence of read-out noise.
;                   : april 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -seed_pn and seed_dark parameter added in order to generate
;                     different sequence of poisson and dark-current noise.
;                    -noise parameter added to choice the type of noise
;                    -read_noise and dark_noise added (sigma for the distribution)
;                   : april 2006,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -CCD saturation and full-well capacity parameters added in order
;                     to simulate the saturation of the CCD.
;                   : march 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added "positive" and "out_type" parameters. 
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added back_file and back_variable + read_gain
;
;-
;
pro adn_gen_default

; obtain module infos
info = adn_info()

; generate module description structure
module = gen_def_module(info.mod_name, info.ver)

back_flux = make_array(360,/double)
phot_flux = make_array(360,/double)

; parameter structure
par = $
   {  $
   adn,                         $ ; structure named adn
   module       : module      , $ ; module description structure
   back_flux    : back_flux   , $ ; Background flux [photons/pixel] for up to 6 different frames
   phot_flux    : phot_flux   , $ ; Source     flux [photons/pixel] for up to 6 different frames
   back_choice  : 0B          , $ ; Background flux choice :
                                  ;    1: user-defined value
                                  ;    0: value from previous module
   back_variable: 0B,            $ ; Different background selection 
                                   ; 0 = Constant value [default]
                                   ; 1 = Variable array
   back_file    : '',            $ ; filename of the background array [must be 
   phot_choice  : 0B          , $ ; Source flux choice
                                  ;    1: user-defined value
                                  ;    0: value from previous module
   noise     : [1,1,0,0,0,0]  , $ ; 0=no, 1=yes for Photon noise, read-out,
                                $ ;      dark current noises, Flat-field, 
                                  ;      Bad Pixel and CCD saturation.
   read_noise: 10.            , $ ; Read-out noise is Gaussian distributed
                                $ ; with zero mean and $sigma$ std.dev [e-/px]
   read_gain: 1.0             , $ ; read-out noise gain [e-/adu]
   dark_noise: 0.1            , $ ; Dark current noise is Poisson distributed
                                $ ; with mean 0.1 e-/s
   full_well: 150000L		  , $ ; FULL WELL CAPACITY (e-/pixel)
   dithering: 0B              , $ ; dithering (0=no, 1=random, 2=user defined)
   dithering_every:1          , $ ; shift every # frames
   dithering_value: 1         , $ ; dithering within a circle of radius..(pixels)
   dithering_save:0B          , $ ; save the random positions (0=NO, 1=YES)
   dithering_file: ""         , $ ; dithering file
   mask_ok:0B                 , $ ; mask application (0=no, 1=yes)
   mask_file:""               , $ ; mask file (FITS file)
   flatfile:""                , $ ; FLAT FIELD map (FITS file)
   badfile:""                 , $ ; BAD PIXEL map (FITS file)
   seed_pn_choice: 1L         , $ ; random seed or fixed seed (for photon noise)?
   seed_pn   : 123444         , $ ; SEED for generation of PHOTON NOISE
   seed_ron_choice: 1L        , $ ; random seed or fixed seed (for read-out noise)?
   seed_ron  : 456777         , $ ; SEED for generation of READ-OUT NOISE (RON)
   positive  : 0B             , $ ; Clip negative values after RON adding?
   seed_dark_choice: 1L       , $ ; random seed or fixed seed (for Dark current)?
   seed_dark : 789000         , $ ; SEED for generation of DARK CURRENT NOISE
   out_type  : 4                $ ; OUTPUT can be converted to an arbitrary 
                                  ; IDL data type {1to15} (default=float=4)
   }

; save the default parameters structure in the file def_file
save, par, FILENAME=info.def_file

end
