; $Id: adn_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    adn_gui
;
; PURPOSE:
;    adn_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the ADd Noise (adn) module.
;    a parameter file called adn_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    the file is stored in the project directory proj_name located
;    in the working directory.
;    (see adn.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = adn_gui(n_module, $
;                    proj_name )
;
; INPUTS:
;    n_module : number associated to the instance of the adn module
;               [integer scalar -- n_module > 0].
;    proj_name: name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;
;    none
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -phot_flux parameter added in order to possibly define the
;                     flux from the observed source as well (like for the
;                     background with back_flux).
;                   : May 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;                    -add possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : january 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(adn_info()).help stuff added (instead of !caos_env.help).
;                   : november 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -global GUI reordering for better fitting in screens !
;                   : march 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added possibility to choice the seed for generation read-out
;                     noise.
;                   : april 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added possibility to choice between three type of noise
;                    -added the choice of seed for generation also dark-current
;                     and poisson noise.
;                   : september 2005,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added help for windows version
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -fixed bug in the calculation of dark_current noise.
;                   : april 2006,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -CCD saturation and full-well capacity parameters added 
;                     in order to simulate the saturation of the CCD
;                   : may 2007,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simplest way to call AIRY_HELP
;                   : june 2009,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -new design of the GUI: TAB-style added
;                    -DITHERING added: each frame can be shifted in 
;                     two different ways: 
;                     a) random shifts: the positions will be created within a 
;                        circle of given radius;
;                     b) user defined shifts: the values are stored into a file 
;                        and will be loaded in the ADN_ADD_NOISE procedure. 
;                        In this case each line of the file must contain X and 
;                        Y shifts, separated by a comma.
;                     Dithering can be applied to either each frame or every N
;                     frames (N is given by the user).
;                    -Added the MASKing of the images.
;                   : October 2009,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]
;                    -FLAT-FIELD and BAD-PIXEL perturbations added. 
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : march 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added another TAB containing the possibility of clipping 
;                     negative values after RON and converting the
;                     output to an arbitrary IDL data type. 
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2012, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]: 
;                    -small changes in the GUI + changes in "addnoise.pro" routine.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. The user can now
;                     load the background contained in a FITS file
;                     (that must have the correct dimensions). 
;                    -Added READ_GAIN in order to convert e- in ADU.
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro adn_gui_set, state
  
  if state.par.back_choice eq 1B then begin
     widget_control, state.id.back_arr_sel, /SENSITIVE
     if state.par.back_variable EQ 0 then begin
        fields_to_show=string(state.par.back_flux)
        pos=where(state.par.back_flux EQ 0.)
        fields_to_show[pos]=' '
        widget_control, state.id.back_flux, /SENSITIVE
        widget_control, state.id.reset_back_flux, /SENSITIVE
        widget_control, state.id.back_flux,SET_VALUE=transpose(fields_to_show)
        widget_control, state.id.back_file, SENSITIVE=0
     endif else begin
        widget_control, state.id.back_flux, SENSITIVE=0
        widget_control, state.id.reset_back_flux, SENSITIVE=0
        widget_control, state.id.back_flux,SET_VALUE=['']
        widget_control, state.id.back_file, /SENSITIVE
     endelse
  endif else begin
     widget_control, state.id.back_flux, SENSITIVE=0
     widget_control, state.id.reset_back_flux, SENSITIVE=0
     widget_control, state.id.back_flux,SET_VALUE=['']
     widget_control, state.id.back_arr_sel, SENSITIVE=0
     widget_control, state.id.back_file, SENSITIVE=0
  endelse
  
  if state.par.phot_choice eq 1B then begin
     fields_to_show=string(state.par.phot_flux)
     pos=where(state.par.phot_flux EQ 0.)
     fields_to_show[pos]=' '
     widget_control, state.id.phot_flux, /SENSITIVE
     widget_control, state.id.reset_phot_flux, /SENSITIVE
     widget_control, state.id.phot_flux,SET_VALUE=transpose(fields_to_show)
  endif else begin
     widget_control, state.id.phot_flux, SENSITIVE=0
     widget_control, state.id.reset_phot_flux, SENSITIVE=0
     widget_control, state.id.phot_flux,SET_VALUE=['']
  endelse    
  
  
  CASE state.par.noise[0]  OF
     0: BEGIN
        WIDGET_CONTROL,state.id.seed_pn, SENSITIVE=0
        WIDGET_CONTROL,state.id.seed_pn_choice, SENSITIVE=0
     END
     1: BEGIN
        WIDGET_CONTROL,state.id.seed_pn_choice,/SENSITIVE
        if state.par.seed_pn_choice eq 1 then begin
           WIDGET_CONTROL,state.id.seed_pn, SENSITIVE=0
        endif else begin
           WIDGET_CONTROL,state.id.seed_pn, /SENSITIVE 
        endelse
     END
  ENDCASE    
  
  CASE state.par.noise[1]  OF
     0: BEGIN
        WIDGET_CONTROL,state.id.read_noise, SENSITIVE=0
        WIDGET_CONTROL,state.id.read_gain, SENSITIVE=0
        WIDGET_CONTROL,state.id.seed_ron, SENSITIVE=0
        WIDGET_CONTROL,state.id.seed_ron_choice, SENSITIVE=0
     END
     1: BEGIN
        WIDGET_CONTROL,state.id.seed_ron_choice,/SENSITIVE
        WIDGET_CONTROL,state.id.read_noise, /SENSITIVE
        WIDGET_CONTROL,state.id.read_gain, /SENSITIVE
        if state.par.seed_ron_choice eq 1 then begin
           WIDGET_CONTROL,state.id.seed_ron, SENSITIVE=0
        endif else begin
           WIDGET_CONTROL,state.id.seed_ron, /SENSITIVE
        endelse
     END
  ENDCASE
  
  CASE state.par.noise[2] OF
     0: BEGIN
        WIDGET_CONTROL,state.id.dark_noise, SENSITIVE=0
        WIDGET_CONTROL,state.id.seed_dark,SENSITIVE=0
        WIDGET_CONTROL,state.id.seed_dark_choice, SENSITIVE=0
     END
     1: BEGIN
        WIDGET_CONTROL,state.id.seed_dark_choice,/SENSITIVE
        WIDGET_CONTROL,state.id.dark_noise, /SENSITIVE
        if state.par.seed_dark_choice eq 1 then begin
           WIDGET_CONTROL,state.id.seed_dark, SENSITIVE=0
        endif else begin
           WIDGET_CONTROL,state.id.seed_dark, /SENSITIVE
        endelse
     END
  ENDCASE
  
  CASE state.par.noise[3] OF
     0:  WIDGET_CONTROL,state.id.flatfile, SENSITIVE=0
     1:  WIDGET_CONTROL,state.id.flatfile, /SENSITIVE
  ENDCASE
  
  CASE state.par.noise[4] OF
     0:  WIDGET_CONTROL,state.id.badfile, SENSITIVE=0
     1:  WIDGET_CONTROL,state.id.badfile, /SENSITIVE
  ENDCASE
  
  CASE state.par.noise[5] OF
     0:  WIDGET_CONTROL,state.id.full_well, SENSITIVE=0
     1:  WIDGET_CONTROL,state.id.full_well, /SENSITIVE
  ENDCASE
  
  case state.par.dithering of
     0: begin
        WIDGET_CONTROL, state.id.dithering_value, SENSITIVE=0
        WIDGET_CONTROL, state.id.dithering_file, SENSITIVE=0
        WIDGET_CONTROL, state.id.dithering_save, SENSITIVE=0
        WIDGET_CONTROL, state.id.dithering_every, SENSITIVE=0
     end
     1: begin
        WIDGET_CONTROL, state.id.dithering_value, /SENSITIVE
        WIDGET_CONTROL, state.id.dithering_save, /SENSITIVE
        WIDGET_CONTROL, state.id.dithering_every, /SENSITIVE
        if state.par.dithering_save EQ 1 then $
           WIDGET_CONTROL, state.id.dithering_file, /SENSITIVE $
        else $
           WIDGET_CONTROL, state.id.dithering_file, SENSITIVE=0
     end
     2: begin
        WIDGET_CONTROL, state.id.dithering_value, SENSITIVE=0
        WIDGET_CONTROL, state.id.dithering_save, SENSITIVE=0
        WIDGET_CONTROL, state.id.dithering_file, /SENSITIVE
        WIDGET_CONTROL, state.id.dithering_every, /SENSITIVE
     end
  endcase
  
  if state.par.mask_ok then WIDGET_CONTROL, state.id.mask_file, /SENSITIVE $
  else WIDGET_CONTROL, state.id.mask_file, SENSITIVE=0
  
  
end

;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;
pro adn_gui_event, event

; adn_gui error management block
common error_block, error

; read the GUI state structure
widget_control, event.top, GET_UVALUE=state

; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

; handle all the other events
widget_control, event.id, GET_UVALUE=uvalue
WIDGET_CONTROL, event.TOP, GET_UVALUE=state

case uvalue of
   '' : 
   'back_choice': state.par.back_choice = event.select

   'back_flux'  : begin
                   dummyx = event.x
                   dummyy = event.y
                   widget_control, state.id.back_flux, get_Value=dummy
                   state.par.back_flux[dummyy]=dummy[dummyx,dummyy]
                  end
    
   'reset_back_flux' : begin
                          state.par.back_flux[*]=0.0
                          widget_control, state.id.back_flux, SET_Value=transpose(state.par.back_flux)
                       end

   'back_arr_sel': begin
      dummy= event.value
                                ;widget_control, state.id.back_arr_sel, GET_VALUE=dummy
      if (dummy EQ 0) then begin
         state.par.back_variable = 0B
      endif else begin
         state.par.back_variable=1B
      endelse
      adn_gui_set,state
   end

   'back_file': begin
      if event.value EQ '' then return
      state.par.back_file=event.value
   end

   'phot_choice': state.par.phot_choice = event.select

   'phot_flux'  : begin
                   dummyx = event.x
                   dummyy = event.y
                   widget_control, state.id.phot_flux, get_Value=dummy
                   state.par.phot_flux[dummyy]=dummy[dummyx,dummyy]
                  end

   'reset_phot_flux' : begin
                          state.par.phot_flux[*]=0.0
                          widget_control, state.id.phot_flux, SET_Value=transpose(state.par.phot_flux)
                       end


   'noise_photon':BEGIN
        state.par.noise[0] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    END

   'noise_ron':BEGIN
        state.par.noise[1] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    END

   'noise_dark':BEGIN
        state.par.noise[2] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    END
    'flat': begin
        state.par.noise[3] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    end
    'bad': begin
        state.par.noise[4] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    end
   'noise_sat':BEGIN
        state.par.noise[5] = event.select
        adn_gui_set, state
        WIDGET_CONTROL, event.top, SET_UVALUE=state
    END

    'read_noise':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.read_noise = dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    END
    'read_gain':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.read_gain = dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    END

    'dark_noise':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.dark_noise = dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    END
    'flatfile': BEGIN
       state.par.flatfile = event.value
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    end
    'badfile': BEGIN
       state.par.badfile = event.value
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    end
    'full_well': begin
          widget_control, event.id, GET_VALUE=dummy
          state.par.full_well=dummy
          WIDGET_CONTROL, event.top, SET_UVALUE=state
          end
    'seed_pn_choice': begin
        widget_control, state.id.seed_pn_choice, GET_VALUE=dummy
           if (dummy EQ 0) then state.par.seed_pn_choice=0
           if (dummy EQ 1) then state.par.seed_pn_choice=1
           adn_gui_set, state
    END
    'seed_pn':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.seed_pn = dummy
       ;print, dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    END

    'seed_ron_choice': begin
        widget_control, state.id.seed_ron_choice, GET_VALUE=dummy
           if (dummy EQ 0) then state.par.seed_ron_choice=0
           if (dummy EQ 1) then state.par.seed_ron_choice=1
           adn_gui_set, state
     END
     
    'seed_ron':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.seed_ron = dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
     END

    'seed_dark_choice': begin
        widget_control, state.id.seed_dark_choice, GET_VALUE=dummy
           if (dummy EQ 0) then state.par.seed_dark_choice=0
           if (dummy EQ 1) then state.par.seed_dark_choice=1
           adn_gui_set, state
     END

    'seed_dark':BEGIN
       widget_control, event.id, GET_VALUE=dummy
       state.par.seed_dark = dummy
       WIDGET_CONTROL, event.top, SET_UVALUE=state
    END
   ;dithering area
   'dithering': BEGIN
       state.par.dithering = event.value
       adn_gui_set, state
       WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'dithering_value': BEGIN
       state.par.dithering_value = event.value
       WIDGET_CONTROL, event.top, SET_UVALUE=state
   END
   'dithering_save' : BEGIN
       state.par.dithering_save = event.select
       adn_gui_set, state
       WIDGET_CONTROL, event.top, SET_UVALUE=state
   END
   'dithering_file': BEGIN
       state.par.dithering_file=event.value
       WIDGET_CONTROL, event.top, SET_UVALUE=state

   END
   'dithering_every': BEGIN
       state.par.dithering_every = event.value
       WIDGET_CONTROL, event.top, SET_UVALUE=state
   END
   ;mask area
   'mask_ok': BEGIN
      state.par.mask_ok = event.select
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END
   'mask_file': BEGIN
      if event.value EQ "" then return
      state.par.mask_file= event.value
      WIDGET_CONTROL, event.top, SET_UVALUE=state 
   END
  
   'positive' : begin
      state.par.positive =event.select
   end
   
   'out_type': begin
      state.par.out_type = event.value+2
   end
   
   'save': begin
      ; check before saving the parameter file if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='adn warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='adn information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      ; save the parameter data file
      par = state.par
      save, par, FILENAME = state.sav_file

      ; kill the GUI returning a null error
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return

   end

   ; handle event from standard help button
   'help': begin
   online_help, book=(adn_info()).help, /FULL_PATH
;     widget_control, /HOURGLASS
;     
;     CASE !VERSION.OS_FAMILY OF
;       'Windows'   :  begin
;            spawn, !caos_env.browser+" "+(adn_info()).help, /NOSHELL
;           end
;       else   :  begin
;             spawn, !caos_env.browser+" "+(adn_info()).help+" &"
;           end
;     ENDCASE
     end

   ; handle event from standard restore button
   'restore': begin

      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      ; update the current module number
      par.module.n_module = state.par.module.n_module

      ; set the default values for all the widgets
      widget_control,  state.id.back_choice,    SET_VALUE=par.back_choice
      widget_control,  state.id.back_arr_sel,    SET_VALUE=par.back_variable
      widget_control,  state.id.back_file, SET_VALUE=par.back_file
      widget_control,  state.id.phot_choice,    SET_VALUE=par.phot_choice

      widget_control,  state.id.back_flux,      SET_VALUE=transpose(par.back_flux)
      widget_control,  state.id.phot_flux,      SET_VALUE=transpose(par.phot_flux)

      widget_control,  state.id.noise_photon,   SET_VALUE= par.noise[0]
      widget_control,  state.id.noise_ron,      SET_VALUE= par.noise[1]
      widget_control,  state.id.noise_dark,     SET_VALUE= par.noise[2]
      widget_control,  state.id.flat,           SET_VALUE= par.noise[3]
      widget_control,  state.id.bad,            SET_VALUE= par.noise[4]
      widget_control,  state.id.noise_sat,      SET_VALUE= par.noise[5]

      widget_control,  state.id.read_noise,     SET_VALUE= par.read_noise
      widget_control,  state.id.read_gain,     SET_VALUE= par.read_gain
      widget_control,  state.id.dark_noise,     SET_VALUE= par.dark_noise
      widget_control,  state.id.flatfile,       SET_VALUE= par.flatfile
      widget_control,  state.id.badfile,        SET_VALUE= par.badfile
      widget_control,  state.id.full_well,      SET_VALUE= par.full_well

      WIDGET_CONTROL, state.id.seed_pn_choice,   SET_VALUE= par.seed_pn_choice
      WIDGET_CONTROL, state.id.seed_ron_choice,  SET_VALUE= par.seed_ron_choice
      WIDGET_CONTROL, state.id.seed_dark_choice, SET_VALUE= par.seed_dark_choice
      WIDGET_CONTROL, state.id.seed_pn,          SET_VALUE= par.seed_pn
      WIDGET_CONTROL, state.id.seed_ron,         SET_VALUE= par.seed_ron
      WIDGET_CONTROL, state.id.seed_dark,        SET_VALUE= par.seed_dark

      WIDGET_CONTROL, state.id.mask_ok, SET_VALUE=par.mask_ok
      WIDGET_CONTROL, state.id.mask_file, SET_VALUE=par.mask_file
      
      WIDGET_CONTROL, state.id.dithering, SET_VALUE=par.dithering
      WIDGET_CONTROL, state.id.dithering_every, SET_VALUE=par.dithering_every
      WIDGET_CONTROL, state.id.dithering_value, SET_VALUE=par.dithering_value
      WIDGET_CONTROL, state.id.dithering_save, SET_VALUE=par.dithering_save
      WIDGET_CONTROL, state.id.dithering_file, SET_VALUE=par.dithering_file
      



      ; update the state structure
      state.par = par
      adn_gui_set, state
  
   end

   ; handle event of standard cancel button (exit without saving)
   'cancel': begin
      error = !caos_error.cancel
      widget_control, event.top, /DESTROY
      return
   end

endcase

; reset the setting parameters status
adn_gui_set, state

; write the GUI state structure
widget_control, event.top, SET_UVALUE=state

return

end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function adn_gui, n_module, proj_name, GROUP_LEADER=group


; error status from the event handler procedure
common error_block, error

; retrieve the module information
info = adn_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a new one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = $
   { $
   par_base        : 0L, $
   back_choice     : 0L, $
   phot_choice     : 0L, $
   back_flux       : 0L, $
   reset_back_flux : 0L, $
   back_arr_sel    : 0L, $
   back_variable   : 0L, $
   back_file       : 0L, $
   phot_flux       : 0L, $
   reset_phot_flux : 0L, $
   noise           : 0L, $
   noise_photon    : 0L, $
   noise_ron       : 0L, $
   noise_dark      : 0L, $
   noise_sat       : 0L, $
   read_noise      : 0L, $
   read_gain       : 0L, $
   dark_noise      : 0L, $
   full_well       : 0L, $
   dithering       : 0L, $
   dithering_value : 0L, $
   dithering_every : 0L, $
   dithering_save  : 0L, $
   dithering_file  : 0L, $
   mask_ok         : 0L, $
   mask_file       : 0L, $
   flat            : 0L, $
   flatfile        : 0L, $
   bad             : 0L, $
   badfile         : 0L, $
   seed_pn_choice  : 0L, $
   seed_pn         : 0L, $
   seed_ron_choice : 0L, $
   seed_ron        : 0L, $
   positive        : 0L, $
   seed_dark_choice: 0L, $
   seed_dark       : 0L, $
   out_type        : 0L  $
   }

; build the state structure were par, id, sav_file and def_file will be stored
; (and passed to adn_gui_event).
state = $
   {    $                ; widget state structure
   sav_file: sav_file, $ ; actual name of the file where save params
   def_file: def_file, $ ; default name of the file where save params
   id      : id,       $ ; widget id structure
   par     : par       $ ; parameter structure
   }


; root base definition
modal = n_elements(group) ne 0
dummy = strupcase(info.mod_name)+' parameters setting GUI'
root_base_id = widget_base(TITLE=dummy, MODAL=modal, /COL, GROUP_LEADER=group)
; set the status structure
widget_control, root_base_id, SET_UVALUE=state
;;;;; BASE TAB DEFINITION
root_base_zero=WIDGET_BASE(root_base_id)
basetab=WIDGET_TAB(root_base_zero, location=0, uvalue='')
basetabA=WIDGET_BASE(basetab, /COL, TITLE='FLUX Information')
basetabB=WIDGET_BASE(basetab, /COL, TITLE='DITHERING')
basetabC=WIDGET_BASE(basetab, /COL, TITLE='MASK')
basetabD=WIDGET_BASE(basetab, /COL, TITLE='NOISE Perturbation')
basetabE=WIDGET_BASE(basetab, /COL, TITLE='Output')

;;;;;;
;;;;;;;;;;;
; BASE "A"
;;;;;;;;;;;
;parameters base
state.id.par_base = widget_base(basetabA, ROW=2)
;dummy = widget_label(state.id.par_base, VALUE='parameters', /FRAME)
par_base_id = widget_base(state.id.par_base, /COL)
par_base_id2= widget_base(par_base_id, /ROW)
back_flux = widget_base(par_base_id2, /FRAME, /COL)

phot_flux = widget_base(par_base_id2, /FRAME, /COL)

back_flux_value = widget_base(back_flux, /COL)

phot_flux_value = widget_base(phot_flux, /COL)

text=widget_label(back_flux_value,value='BACKGROUND FLUX')
state.id.back_choice  = cw_bgroup(back_flux_value,                       $
                    ['enter sky background flux [ph/pixel/frame]:          '], $
                                  SET_VALUE=state.par.back_choice,       $
                                  UVALUE='back_choice',                  $
                                  /NONEXCLUSIVE                          )
state.id.back_arr_sel = cw_bgroup (back_flux_value,        $
                                   ['Constant background (single value per image)',$
                                    'Variable background (Fits file shaped '+$
                                    'as input image)'],       $
                                   ROW=2,         $
                                   SET_VALUE=state.par.back_variable,   $
                                   UVALUE='back_arr_sel', $
                                   /EXCLUSIVE     )

back_table=widget_base(back_flux_value,xsize=200,ysize=160)
rlabel=make_array(360,/string,value='Angle #')
num=indgen(360)
rlabel=rlabel+strcompress(string(num),/remove_all)

button_bkg=widget_base(back_table,xoffset=58,yoffset=3,xsize=64,ysize=26)
state.id.reset_back_flux=widget_button(button_bkg,$
                      VALUE="Reset Val", $
                      UVALUE="reset_back_flux")


fields_to_show=string(state.par.back_flux)
pos=where(state.par.back_flux EQ 0.)
fields_to_show[pos]=' '
state.id.back_flux = widget_table(back_table,$
                       ALIGNMENT=2,$
                       xoffset=55,yoffset=0,$
                       value=transpose(fields_to_show),$
                       uvalue="back_flux",$
                       editable=1,$
                       COLUMN_LABELS=['Background flux'],$
                       ROW_LABELS=rlabel,$
                       COLUMN_WIDTHS=150,$
                       y_scroll_size=5   )

state.id.back_file=CW_FILENAME(back_flux_value, TITLE="Load a FITS file",$
                               VALUE=state.par.back_file,$
                               UVALUE='back_file',/ALL_EVENTS)

dummy = widget_label(back_flux,$
        VALUE=' ... otherwise the value from previous modules is considered ')

text=widget_label(phot_flux_value,value='SOURCE FLUX')
state.id.phot_choice  = cw_bgroup(phot_flux_value,                       $
                    ['enter source flux [ph/frame]:                  '], $
                                  SET_VALUE=state.par.phot_choice,       $
                                  UVALUE='phot_choice',                  $
                                  /NONEXCLUSIVE                          )


phot_table=widget_base(phot_flux_value,xsize=200,ysize=160)
button_phot=widget_base(phot_table,xoffset=58,yoffset=3,xsize=64,ysize=26)

state.id.reset_phot_flux=widget_button(button_phot,$
                      VALUE="Reset Val", $
                      UVALUE="reset_phot_flux")

fields_to_show=string(state.par.phot_flux)
pos=where(state.par.phot_flux EQ 0.)
fields_to_show[pos]=' '
state.id.phot_flux = widget_table(phot_table,$
                       ALIGNMENT=2,$
                       xoffset=55,yoffset=0,$
                       value=transpose(fields_to_show),$
                       uvalue="phot_flux",$
                       editable=1,$
                       COLUMN_LABELS=['Source flux'],$
                       ROW_LABELS=rlabel,$
                       COLUMN_WIDTHS=150,$
                       y_scroll_size=5   )


dummy = widget_label(phot_flux,$
        VALUE=' ... otherwise the value from previous modules is considered ')

;;;;;;;;;;;
; BASE "B"
;;;;;;;;;;;             
; dithering area              
dithering_base=WIDGET_BASE(basetabB, /COL, /ALIGN_CENTER)
dummy = WIDGET_LABEL(dithering_base, $ 
        value="DITHERING (multi-frames data simulation only)" )
dithering_base_c = WIDGET_BASE(dithering_base, /FRAME, /COL)
state.id.dithering=CW_BGROUP(dithering_base_c, $
                  ['no dithering    ', 'random dithering   ', $
                  'user defined dithering (load a file...)  '], $ 
                  SET_VALUE=state.par.dithering, $
                  uvalue='dithering', /EXCLUSIVE, /ROW)        
       
dithering_base_dd = WIDGET_BASE(dithering_base_c, /COL)
       
state.id.dithering_value = CW_FIELD(dithering_base_dd, $
                TITLE='within a circle of radius [pixels]', $
                /ROW,VALUE=state.par.dithering_value,UVALUE="dithering_value", $
                  /ALL_EVENTS, /INTEGER, xsize=7)


state.id.dithering_save= CW_BGROUP(dithering_base_dd, $
                         ['Save positions as ...'], $
                         SET_VALUE=state.par.dithering_save, $
                         UVALUE='dithering_save', /NONEXCLUSIVE)

state.id.dithering_file=CW_FILENAME(dithering_base_c, $
                        TITLE="Dithering filename",$
                        VALUE=state.par.dithering_file,$
                        UVALUE="dithering_file",/ALL_EVENTS)

state.id.dithering_every=CW_FIELD(dithering_base_c, $
                TITLE='Shift every # frame(s) ', $
                /ROW,VALUE=state.par.dithering_every,UVALUE="dithering_every", $
                /ALL_EVENTS, /INTEGER, xsize=7)

;;;;;;;;;;;
; BASE "C"
;;;;;;;;;;;             
; MASK area              
mask_base=WIDGET_BASE(basetabC, /COL, /ALIGN_CENTER)
dummy = WIDGET_LABEL(mask_base, $ 
        value="MASK Application" )
mask_base_c = WIDGET_BASE(mask_base, /FRAME, /COL)
state.id.mask_ok=CW_BGROUP(mask_base_c, ['Apply a mask (load a FITS file)'], $ 
                 SET_VALUE=state.par.mask_ok, $
                 uvalue='mask_ok', /NONEXCLUSIVE, /ROW)        
       

state.id.mask_file=CW_FILENAME(mask_base_c, $
                        TITLE="MASK filename",$
                        VALUE=state.par.mask_file,$
                        UVALUE="mask_file",/ALL_EVENTS, $
                        filter=["*.fits", "*.fit", "*.fts", "*.*"])


                
;;;;;;;;;;;
; BASE "D"
;;;;;;;;;;;
;NOISE base
;==========

;status = TAG_EXIST(par,'seed_pn')

noiseback = widget_base(basetabD, /COL, /FRAME)
text=widget_label(noiseback,value='    NOISE PERTURBATION',xsize=200,xoffset=200)
;IF status THEN BEGIN

noise = WIDGET_BASE(noiseback,/COL)
noise_up = WIDGET_BASE(noiseback,/ROW)
noise_down = WIDGET_BASE(noiseback,/ROW)

;-----------------------------
noise1= WIDGET_BASE(noise_up,/COL,/FRAME,xsize=250,ysize=180)
   state.id.noise_photon = $
     CW_BGROUP(noise1,['Photon Noise'],  $
               UVALUE='noise_photon',/NONEXCLUSIVE,SET_VALUE=state.par.noise[0])
   label=widget_label(noise1,value=' ')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   space=widget_label(noise1,value='')
   noise1_seed=widget_base(noise1,/row)
   state.id.seed_pn_choice=                          $
     cw_bgroup(noise1_seed,                          $
               ['Fixed Seed','Random Seed'],         $
               ROW=2,                                $
               SET_VALUE=state.par.seed_pn_choice,   $
               UVALUE='seed_pn_choice',              $
               /EXCLUSIVE,                           $
               /NO_RELEASE)
                   
   state.id.seed_pn =                                    $
     CW_FIELD(noise1_seed, TITLE='',/LONG,/ALL_EVENTS, xsize=6,   $
              VALUE=state.par.seed_pn,UVALUE='seed_pn')


;-----------------------------
noise2= WIDGET_BASE(noise_up,/COL,/FRAME,xsize=250,ysize=180)
   state.id.noise_ron =    $
     CW_BGROUP(noise2,['Read-out Noise'],  $
               UVALUE='noise_ron',/NONEXCLUSIVE,SET_VALUE=state.par.noise[1])
   state.id.read_noise= CW_FIELD(noise2, TITLE='Read-out [e- rms]', /FLOAT, xsize=6,       $
                                 VALUE=state.par.read_noise,UVALUE="read_noise",/ALL_EVENTS)

   state.id.read_gain= CW_FIELD(noise2,  TITLE='Gain     [e-/adu]', /FLOAT, xsize=6,       $
                                 VALUE=state.par.read_gain,UVALUE="read_gain",/ALL_EVENTS)

   noise2_seed=widget_base(noise2,/row)
   state.id.seed_ron_choice= cw_bgroup(noise2_seed,  $
               ['Fixed Seed','Random Seed'],         $
               ROW=2,                                $
               SET_VALUE=state.par.seed_ron_choice,  $
               UVALUE='seed_ron_choice',             $
               /EXCLUSIVE,                           $
               /NO_RELEASE)

   state.id.seed_ron =                                   $
     CW_FIELD(noise2_seed, TITLE='',/LONG,/ALL_EVENTS, xsize=6,   $
              VALUE=state.par.seed_ron,UVALUE='seed_ron')

;-----------------------------
noise3= WIDGET_BASE(noise_up,/COL,/FRAME,xsize=250,ysize=180)
   state.id.noise_dark =   $
     CW_BGROUP(noise3,['Dark-current Noise'],  $
               UVALUE='noise_dark',/NONEXCLUSIVE,SET_VALUE=state.par.noise[2])
   space=widget_label(noise3,value='')
   state.id.dark_noise=                                                   $
     CW_FIELD(noise3, TITLE='Dark-current [e-/s]', /INTEGER, xsize=6,       $
              VALUE=state.par.dark_noise,UVALUE="dark_noise",/ALL_EVENTS)
   space=widget_label(noise3,value='')
   space=widget_label(noise3,value='')
   space=widget_label(noise3,value='')
   space=widget_label(noise3,value='')
   noise3_seed=widget_base(noise3,/row)

   state.id.seed_dark_choice=                          $
     cw_bgroup(noise3_seed,                          $
               ['Fixed Seed','Random Seed'],         $
               ROW=2,                                $
               SET_VALUE=state.par.seed_dark_choice,   $
               UVALUE='seed_dark_choice',              $
               /EXCLUSIVE,                           $
               /NO_RELEASE)

   state.id.seed_dark =                                   $
     CW_FIELD(noise3_seed, TITLE='  ',/LONG,/ALL_EVENTS, xsize=6,    $
              VALUE=state.par.seed_dark,UVALUE='seed_dark')
 


;-----------------------------
noise4=WIDGET_BASE(noise_down,/COL, /FRAME, xsize=260, ysize=200)
   state.id.flat=CW_BGROUP(noise4,['FLAT-FIELD map'],  $
               UVALUE='flat',/NONEXCLUSIVE,SET_VALUE=state.par.noise[3])
   label=widget_label(noise4,value=' ')
   state.id.flatfile= cw_filename(noise4, TITLE='Load a map', $
                      VALUE=state.par.flatfile,  UVALUE='flatfile', $
                      /ALL_EVENTS )
   label=widget_label(noise4, value='The map will be normalized to unit average')
   label=widget_label(noise4, value='[See examples in ADN_DATA directory]')


;-----------------------------
noise5=WIDGET_BASE(noise_down,/COL, /FRAME, xsize=280, ysize=200)
   state.id.bad=CW_BGROUP(noise5,['BAD-PIXEL map'],  $
               UVALUE='bad',/NONEXCLUSIVE,SET_VALUE=state.par.noise[4])
   label=widget_label(noise5,value=' ')
   state.id.badfile= cw_filename(noise5, TITLE='Load a map', $
                      VALUE=state.par.badfile,  UVALUE='badfile', $
                      /ALL_EVENTS)
   label=widget_label(noise5, value='The map must have value 0 in bad positions')
   label=widget_label(noise5, value=' and 1 in good pixels')
   label=widget_label(noise5, value='[See examples in ADN_DATA directory]')


;-----------------------------
noise6= WIDGET_BASE(noise_down,/COL,/FRAME,xsize=210,ysize=200)
   state.id.noise_sat =    $
     CW_BGROUP(noise6,['CCD Saturation'],  $
               UVALUE='noise_sat',/NONEXCLUSIVE,SET_VALUE=state.par.noise[5])
   label=widget_label(noise6,value=' ')
   state.id.full_well= CW_FIELD(noise6, $
                       TITLE='Full Well'+string(10B)+'Capacity [e-/px]', $
                       /COL, /LONG, xsize=10, $
                       VALUE=state.par.full_well,UVALUE="full_well",$
                       /ALL_EVENTS)
   label=widget_label(noise6, value='The image will be thresholded')
   label=widget_label(noise6, value='above the FWC value')
   label=widget_label(noise6, value=' ')



;;;;;;;;;;;
; BASE "E"
;;;;;;;;;;;
;OUTPUT base
;==========

outbase = widget_base(basetabE, /COL, /FRAME)
text=widget_label(outbase,value='    OUTPUT   ')

   state.id.positive = cw_bgroup(outbase,  $
       ['After adding Read-Out Noise (Gaussian), clip the negative values'],$
       ROW=1,  SET_VALUE=state.par.positive, UVALUE='positive',   $
       /NONEXCLUSIVE)

text=widget_label(outbase,value='       ')

   state.id.out_type = cw_bgroup(outbase,  $
       ['INTEGER (16-bit signed integer)', $
        'LONG (32-bit signed integer)', $
        'FLOAT (32-bit, single-precision, floating point)', $
        'DOUBLE (64-bit, double-precision, floating point)'],$
       COL=1,  SET_VALUE=state.par.out_type-2, UVALUE='out_type',   $
       label_top="Convert output to type:",/EXCLUSIVE)               



; button base for control buttons (standard buttons)
btn_base_id = widget_base(root_base_id, FRAME=10, /ROW)
   dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
   cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
   if modal then widget_control, cancel_id, /CANCEL_BUTTON
   dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                         UVALUE="restore"                         )
   save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS",  $
                           UVALUE="save"                          )
   if modal then widget_control, save_id, /DEFAULT_BUTTON

; initialize all the sensitive states
adn_gui_set, state

; save the state structure of the GUI in the top base uvalue
widget_control, root_base_id, SET_UVALUE=state

; draw the GUI
widget_control, root_base_id, /REALIZE

; launch xmanager
xmanager, 'adn_gui', root_base_id, GROUP_LEADER=group

return, error
end
