; $Id: adn_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    adn_init
;
; PURPOSE:
;    adn_init executes the initialization for the ADd Noise
;    (adn) module, that is:
;
;       0- check the formal validity of the input/output structure
;       1- initialize the output structure(s) out_img_t
;
;    (see adn.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's initialization routine
;
; CALLING SEQUENCE:
;    error = adn_init(inp_img_t,  $ ; img_t input structure
;                     out_img_t,  $ ; img_t output structure
;                     par,        $ ; parameters structure
;                     INIT=init   $ ; initialization data structure
;                     )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see adn.pro's help for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -resolved ambiguity problem with add_noise function AND
;                     add_noise procedure. procedure is now adn_add_noise.
;                    -phot_flux parameter added in order to possibly re-define
;                     the flux from the observed source as well (like for the
;                     background with back_flux).
;                    -resolved confusion between nb of frames in a cube and
;                     nb of dimensions of a cube...
;                   : May 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : january 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -fixed parameter passing bug (background) in calling function
;                     adn_add_noise.pro in the case of 2 dimensions data frame.
;                   : june 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -fixed parameter passing bug (n_phot) in calling function
;                     adn_add_noise.pro in the case of 2 dimensions data frame.
;                   : march 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added parameter (seed_ron) in calling function
;                     adn_add_noise.pro.
;                   : april 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added parameters (read_noise, dark_noise, seed_pn, seed_dark,
;                     noise in calling function adn_add_noise.pro.
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -added initialization of the output parameter "time_integ"
;                   : april 2006,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -CCD saturation and full-well capacity parameters added in
;                     order to simulate the saturation of the CCD
;                   : december 2009,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added DITHERING and MASKING procedure 
;                    -added FLAT FIELD and BAD PIXEL procedure
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -The header of the input/output image is kept
;                     up-to-date with noise information
;                    -background is now an array (from previous
;                     modules or a new array). Small changes to the GUI
;-
;
function adn_init, inp_img_t,  out_img_t, par, INIT=init

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = adn_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

; STANDARD CHECKS
;
; compute and test the requested number of adn arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'adn error: par must be a structure'
if n ne 1 then message, 'adn error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module adn'

; check the input arguments

; test if any optional input exists
if n_inp gt 0 then begin
   inp_opt = info.inp_opt
endif

dummy = test_type(inp_img_t, TYPE=type)
if type eq 0 then begin         ; undefined variable
   inp_img_t = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif
if test_type(inp_img_t, /STRUC, N_EL=n, TYPE=type) then $
   message, 'adn error: wrong definition for the first input.'
if n ne 1 then message, $
   'adn error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_img_t.data_status eq !caos_data.not_valid and not inp_opt[0] then $
      message, 'undefined input is not allowed'

;------------------------------------------------------------------------------

; Is it a single 2D image, a cube of 2D images or n cubes of 3D images 
; (each cube is a set of frames taken at different parallactic angle)?
pos=where(inp_img_t.time_integ[*,1] gt 1.,num)
if num gt 0 then begin
  multi_frame=1
  print,'-----  MULTI-FRAMES DATA SIMULATION  -----'
endif else begin
  multi_frame=0
  print,'-----  SINGLE-FRAME DATA SIMULATION  -----'
endelse

header = *inp_img_t.header

nb_pix = (size(inp_img_t.image))[1]
nb_dim = (size(inp_img_t.image))[0]
max_frame=max(inp_img_t.time_integ[*,1])

if nb_dim eq 3 then begin
   nb_angle   = (size(inp_img_t.image))[3]
endif else if nb_dim eq 2 then begin
   nb_angle   = 1
endif else message, "only cubes and 2D images are supported"
noisy_map = fltarr(nb_pix, nb_pix, nb_angle,max_frame)
noisy_map = reform(noisy_map,nb_pix, nb_pix, nb_angle,max_frame)

; total number of photons from source
if par.back_choice eq 0B then begin
                                ; background defined in previous module
   background = inp_img_t.background  ; do not update the header in this case
endif else begin
   if par.back_variable EQ 0 then begin
                                ; background entered in adn_GUI
      fxaddpar, header, 'SKY_TYPE', 'VALUE', '[ADN] Sky background (user-defined)',/NOLOGICAL
      fxaddpar, header, 'SKY_NB', nb_angle, '[ADN] Sky background number of values' 
      background = fltarr(nb_pix, nb_pix, nb_angle)
      for j=0,nb_angle-1 do begin
         background[*,*,j] = par.back_flux[j]
         fxaddpar, header, 'SKY_BG'+strtrim(j,1), par.back_flux[j], '[ADN] Sky background value' 
      endfor
   endif else begin
; -- variable array: 
      fxaddpar, header, 'SKY_TYPE', 'FILE', '[ADN] Sky background (user-defined FITS file)' ,/NOLOGICAL
      background=readfits(par.back_file)
      dim_bg = size(background, /dimensions)
      if nb_angle EQ 1 then dim_bg=[dim_bg,1]
      if NOT(array_equal(dim_bg,[nb_pix,nb_pix,nb_angle])) then message, "Invalid size of the loaded BACKGROUND array."
      fxaddpar, header, 'SKY_FILE', par.back_file, '[ADN] Sky background filename'       
   endelse           
endelse

;no dithering   
dx=0
dy=0

;dithering random
seed_time=fix(systime(1,/seconds) mod 100)
if par.dithering EQ 1 then begin   
   if par.dithering_save then openw,1,par.dithering_file
   r=par.dithering_value
   dith_arr=make_array(2,nb_angle*(max_frame / par.dithering_every))
   for k=0, FIX(nb_angle*(max_frame / par.dithering_every))-1 do begin
      seed_dith=seed_time+k
      ok=0B
      while ok EQ 0 do begin
         dx=(randomn(seed_dith,/uniform)*nb_pix)-nb_pix/2.
         dy=(randomn(seed_dith,/uniform)*nb_pix)-nb_pix/2.
         if ((dx*dx+dy*dy) LE (r*r)) then ok=1B 
      endwhile
      if par.dithering_save then printf,1,strtrim(dx,1)+", "+strtrim(dy,1)
      dith_arr[0,k]=dx
      dith_arr[1,k]=dy
   endfor
   if par.dithering_save then close,1
   fxaddpar, header, 'DITHER', 'random', '[ADN] Dithering of images' 
   fxaddpar, header, 'DITH_R', par.dithering_value, '[ADN] radius of dithering' 
endif

;dithering from file
if par.dithering EQ 2 then begin
   b=""
   count=0
   openr,1,par.dithering_file
   ;count the number of lines
   while ~EOF(1) do begin
     readf,1,b 
     count=count+1
   endwhile
   if count NE nb_angle*(max_frame / par.dithering_every) then begin
      mess_err= "ADN error! Dithering file loaded must have total number of"+ $
                "lines = "+strtrim(count,1)+" AND each line must contains "+ $
                "dx, dy (shift values of the dithering separated by a comma)"   
      message, mess_err
   endif
   point_lun,1,0
   dith_arr=make_array(2,nb_angle*(max_frame / par.dithering_every))
   for k=0, nb_angle*(max_frame / par.dithering_every)-1 do begin
      ;read the values...
      readf,1,b
      dith_arr[0,k]=((strsplit(b,",",/EXTRACT))[0])
      dith_arr[1,k]=((strsplit(b,",",/EXTRACT))[1])
   endfor
   close,1
   fxaddpar, header, 'DITHER', 'user', '[ADN] Dithering of images' 
   fxaddpar, header, 'DITH_U', par.dithering_file, '[ADN] filename of dithering values' 
endif

;mask application
mask=1.
if par.mask_ok EQ 1 then begin
   fits_read, par.mask_file, mask
   NN=(size(mask))[1]
   MM=(size(mask))[2]
   P=(size(mask))[0]
   if P NE 2 then message, "The MASK must be a 2D array..."
   if NN NE MM OR NN NE nb_pix then message, "Image and MASK must have same dimensions..."
   fxaddpar, header, 'MASK', par.mask_file, '[ADN] Mask multiplication on images' 
endif

for i=0, nb_angle-1 do begin
    ; total number of source photons per frame
    if par.phot_choice eq 0B then begin
        ; source flux defined in previous module
        n_phot = total(inp_img_t.image[*,*,i],/double)
    endif else begin
        ; source flux entered in adn_GUI
        n_phot = par.phot_flux
    endelse
    inp_img_t.image[*,*,i]=inp_img_t.image[*,*,i]/total(inp_img_t.image[*,*,i],/double)*n_phot
    
    for j=0,inp_img_t.time_integ[i,1]-1 do begin
       if par.dithering GT 0 then begin
          dx=dith_arr[0,i*(max_frame/par.dithering_every)+(j/par.dithering_every)]
          dy=dith_arr[1,i*(max_frame/par.dithering_every)+(j/par.dithering_every)]
          adn_add_noise, inp_img_t.image[*,*,i], background[*,*,i], $
                         inp_img_t.time_integ[i,0], par, i, j+1, dummy, $
                         dx, dy, mask
       endif else begin
          adn_add_noise, inp_img_t.image[*,*,i], background[*,*,i], $
                         inp_img_t.time_integ[i,0], par, i, j+1, dummy, $
                         0, 0, mask
       endelse
                                ;badpixel map execution 
       if par.noise[4] then begin
          fits_read, par.badfile, badmap
          if (size(badmap))[1] NE nb_pix then $
             message, "Image and BAD PIXEL map must have the same dimensions..."
          if (size(badmap))[0] EQ 3 then badmap=badmap[*,*,i]
          dummy=dummy*badmap
       endif
       if par.noise[5] then dummy=adn_ccd_sat(dummy, par.full_well)
       noisy_map[*,*,i,j] = dummy
    endfor
endfor

;update the header with NOISE information
if par.noise[0] then fxaddpar, header, 'POISSON', par.noise[0], '[ADN] Poisson noise on the image'
if par.noise[1] then begin
   fxaddpar, header, 'RON', par.read_noise, '[ADN] Read-out noise [e-/px]'
   fxaddpar, header, 'GAIN', par.read_gain, '[ADN] Gain [e-/ADU]'
endif
if par.noise[2] then fxaddpar, header, 'DARK_C', par.dark_noise, '[ADN] Dark current [e-/s]' 
if par.noise[3] then fxaddpar, header, 'FLAT_F', par.flatfile, '[ADN] Flat-field simulation'
if par.noise[4] then fxaddpar, header, 'BAD_PIX', par.badfile, '[ADN] Bad-pixel simulation'
if par.noise[5] then fxaddpar, header, 'CCD_SAT', par.full_well, '[ADN] CCD saturation on'

; INITIALIZE THE OUTPUT STRUCTURE --------------------------------------
;
out_img_t =                             $
   {                                     $
   data_type  : out_type[0]          , $
   data_status: !caos_data.valid     , $
   image      : noisy_map            , $  ; IMAGE with same resolution as PSF.
   npixel     : inp_img_t.npixel     , $  ; Number of pixels (per subaperture)
   resolution : inp_img_t.resolution , $  ; Pixel size [arcsec/pix]
   lambda     : inp_img_t.lambda     , $  ; Mean detection wavelength [m]
   width      : inp_img_t.width      , $  ; Wavelength band width [m]
   time_integ : inp_img_t.time_integ , $  ; Integration time for each image [s]
   time_delay : 0.                   , $  ; [NOT USED WITHIN AIRY]
   psf        : 0B                   , $  ; this is an image, not a PSF
   background : background           , $  ; the value of the background
   snr        : fltarr(360)          , $  ; [NOT YET EVALUATED]
   header     : PTR_NEW('')            $  ; Header (modified by this procedure)
   }

*out_img_t.header = header  ;update the pointer
return, error
end
