; $Id: adn_add_noise.pro,v 7.0 last revision 2016/05/17 Andrea La Camera $
;+
; NAME:
;       adn_add_noise
;
; PURPOSE:
;   First of all, the routine adds the sky background to the input image. Next,
;   masking, dithering, and flat-field cases are managed. 
;   Finally, the routine calls the function "addnoise.pro" (CAOS PSE
;   library) that handles several kinds of noise (Poisson, RON, Dark, etc).  
;
;
; CATEGORY:
;       ...
;
; CALLING SEQUENCE:
;
;   adn_add_noise, im, bkgd_med, delta_t, par, angle, frame, z, dx, dy, mask
;
; INPUTS:
;
;   im        = Input map [nb_pix, nb_pix]
;   bkgd_med  = sky background flux (in photons/pixel) [FLOAT]
;   delta_t   = integration time [s] of the input image
;   par       = the structure of the parameters
;   angle     = hour angle counter [0,1,...P-1]
;   frame     = frame counter [0,1,...,P-1]
;   dx, dy    = values in pixel for dithering procedure
;   mask      = map for masking procedure 
;
; OPTIONAL INPUTS:
;   none.
;
; KEYWORD PARAMETERS:
;   none.
;
; OUTPUT:
;   z         = Noisy output [nb_pix,nb_pix]
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       common noise_seed_block, seed_pn, seed_ron, seed_dark, seed_l3ccd, $
;                                seed_xd, seed_ff
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
; EXAMPLE:
;       ...
;
; MODIFICATION HISTORY:
;       June 2000: Serge Correia (OAA),
;                  [correia@arcetri.astro.it].
;
;   modifications: march 2004,
;                  Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                 -add parameter (seed_ron) for generation read-out noise
;                  april 2004,
;                  Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                 -add parameters (seed_pn, seed_dark, sigma_DARK_NOISE, noise) 
;                  for generation poisson, read-out and dark-current noise
;                : october 2005,
;                  Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                 -fix bug in adding background/noise sequence
;                 -fix bug in calculation of DARK CURRENT noise
;                 -fix bug in calculation of "random/fixed seed" generation noise
;                : march 2010,
;                  Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                 -approx poisson ~ gauss if the signal is > 1e4 has been removed
;                : march 2011,
;                  Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                 -The function "addnoise.pro" has been integrated 
;                  within adn_add_noise (this routine).
;                : May 2016, 
;                  CAOS PSE version 7.0.
;                 -the routine "addnoise.pro" is located within the
;                  Library folder
;-
;

pro adn_add_noise, im, bkgd_med, delta_t, par, angle, frame, z, dx, dy, mask 

common noise_seed_block, seed_pn, seed_ron, seed_dark, seed_l3ccd, seed_xd, seed_ff


nb_pix = (size(im))[1]
z=im+bkgd_med    ;from version 7.0 background is an array (Fortunately, this does not change the code!)
im=0
seed_time=fix(systime(1,/seconds) mod 1e5)


;*******************************************************************************
;**********************DITHERING every # frame**********************************
;*******************************************************************************

if par.dithering GT 0 then begin
      z=fftshift(z,dx,dy)
      z=z>0.
endif


;*******************************************************************************
;*****************************MASK application**********************************
;*******************************************************************************

if par.mask_ok EQ 1 then begin
   z=z*mask
endif


;*******************************************************************************
;**************************FLAT-FIELD application*******************************
;*******************************************************************************
if par.noise[3] then begin
   fits_read, par.flatfile, flatmap
   if (size(flatmap))[1] NE nb_pix then $
          message, "Image and FLAT-FIELD map must have the same dimensions..."
   if (size(flatmap))[0] EQ 3 then flatmap=flatmap[*,*,angle]
   z=z*flatmap/total(flatmap,/double)
endif


;*******************************************************************************
;**************************NOISE perturbations*******************************
;*******************************************************************************

;noise seeds
if par.seed_pn_choice eq 1 then begin
   seed_pn=seed_time+(500.*angle+frame)
endif else begin
   seed_pn=par.seed_pn + (500.*angle + frame)
endelse

if par.seed_ron_choice eq 1 then begin
   seed_ron= seed_time+(700.*angle + frame) 
endif else begin
   seed_ron=par.seed_ron + (700.*angle + frame)
endelse

if par.seed_dark_choice eq 1 then begin 
    seed_dark=seed_time+(900.*angle + frame)                     
endif else begin
    seed_dark=par.seed_dark + (900.*angle + frame)
endelse   

seed_l3ccd=0
seed_xd=0
seed_ff=0

;call addnoise.pro

choice = par.noise[0]+(2*par.noise[1])+(4*par.noise[2])
case choice of
   0: print, "NO NOISE"  ;NO NOISE!
   1: z=addnoise(z, /PHOT_NOISE, $
                    positive=par.positive, $
                    out_type=par.out_type)  ;PHOTON NOISE
   2: z=addnoise(z, SIGMA_RON=par.read_noise/par.read_gain, $ 
                    positive=par.positive, $
                    out_type=par.out_type) ; RON
   3: z=addnoise(z, /PHOT_NOISE, $
                    SIGMA_RON=par.read_noise/par.read_gain, $
                    positive=par.positive, $
                    out_type=par.out_type) ; PHOTON + RON
   4: z=addnoise(z, SIGMA_DARK=par.dark_noise, $
                    delta_t=delta_t, $
                    positive=par.positive, $
                    out_type=par.out_type) ; DARK
   5: z=addnoise(z, /PHOT_NOISE, $
                    SIGMA_DARK=par.dark_noise, $
                    delta_t=delta_t, $
                    positive=par.positive, $
                    out_type=par.out_type) ; PHOTON + DARK
   6: z=addnoise(z, SIGMA_RON=par.read_noise/par.read_gain, $
                    SIGMA_DARK=par.dark_noise, $
                    delta_t=delta_t, $
                    positive=par.positive, $
                    out_type=par.out_type) ; RON + DARK
   7: z=addnoise(z, /PHOT_NOISE, $
                    SIGMA_RON=par.read_noise/par.read_gain, $
                    SIGMA_DARK=par.dark_noise, $
                    delta_t=delta_t, $
                    positive=par.positive, $
                    out_type=par.out_type) ; PHOTON + RON + DARK
endcase

end
