; $Id: anb.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    anb
;
; ROUTINE'S PURPOSE:
;    ANB executes the simulation for the ANalysis Binary (ANB) module.
;
; MODULE'S PURPOSE:
;    ANB module performs the astrometric and photometric analysis on a 
;    restored binary type object.   
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = anb(inp_img_t, $
;                par,       $
;                INIT=init  )
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t: structure of type img_t (image type).
;    par      : parameters structure from anb_gui.
;
; KEYWORD PARAMETERS:
;    INIT: initialisation data structure
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;     1-In some cases, namely when the object is a double star of faint
;     magnitude the MMM algorithm can return an incorrect value (0) for
;     the background evaluation.
;     For this kind of cases it is recommended to use an alternative method
;     for background evaluation.
;
;     2- The background estimation done by PRE module can be contaminated
;     by the object if the extension
;     of the object is a large fraction of the image frame. The estimated mean value
;     of the sky background and especially its rms value are then biased.
;     A simple solution is to compute directly the mean background using
;     the routine n_phot (found in .../caos/lib/, help at the IDL/CAOS prompt with
;     the command <<doc_library, "n_phot">>). For example, with a 0.01 arcsec pixel
;     size, a telescope collecting surface of 10 m^2, an integration time of 100 s
;     and an overall efficiency of 0.3, the K band mean background (per pixel) is:
;     
;        0.3*(n_phot(0.,BAND="K",DELTA_T=100.,SURF=10.))[1]*0.01^2
;     
;     Then it is necessary to enter by hand the value found within the PRE module GUI
;     (using the option "consider a user-defined background").
;
; CALLED NON-IDL FUNCTIONS:
;       ap_phot_cent.pro
;       ap_phot_cent_comp.pro
;       detect_binary_star.pro
;       legend_windt_m.pro
;       make_graph_bin.pro
;       model_binary.pro
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: october 2000,
;                     Serge Correia (OAA) [marcel@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;                   : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -help completed.
;
; MODULE MODIFICATION HISTORY:
;    module written : Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                   : for version 5.0,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -GUI resized.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -Some small changes in the GUI (Text and position of 
;                     the inputs).
;                   : march 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                     Added verbosity choice [default = no]
;-
;
function anb, inp_img_t, $ ; input structure
              par,       $ ; parameters from anb_gui
              INIT=init    ; initialization structure

common caos_block, tot_iter, this_iter

; initialization of the error code: set to "no error" as default
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = anb_init(inp_img_t, par, INIT=init)
endif else begin
   ; run section
   error = anb_prog(inp_img_t, par, INIT=init)
endelse

; back to calling program.
return, error                   ; back to calling program.
end
