; $Id: anb_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    anb_gui
;
; PURPOSE:
;    anb_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the ANB module.
;    A parameter file called anb_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    The file is stored in the project directory proj_name located
;    in the working directory.
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = anb_gui(n_module, proj_name)
;
; INPUTS:
;    n_module : integer scalar. Number associated to the instance
;               of the ANB module. n_module > 0.
;    proj_name: string. Name of the current project.
;
; OUTPUTS:
;    error: error code [long scalar]
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    none.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : march 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add convergence plots filename browser.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to version 4.0 of the whole system CAOS
;                    (variable "pack_name" added, and variable "mod_type"
;                    changed into "mod_name").
;                    -(anb_info()).help stuff added (instead of !caos_env.help).
;                   : june 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -cvparam_enable/cv_param_enable bug fixed.
;                   : september 2005,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added help for windows version
;                   : may 2007, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simplest way to call AIRY_HELP
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -GUI size reduced.
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : january 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -GUI re-organized in two TABs: one for the analysis and the 
;                     other for the results.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -Some small changes in the GUI (Text and position of 
;                     the inputs).
;                   : March 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                     Added verbosity choice [default = no]
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro anb_gui_set, state
  
;set broad band index
  widget_control, state.id.band, SET_DROPLIST_SELECT=state.par.band_index
  
  if (state.par.comparison eq 0B) then begin
     widget_control, state.id.par_bin,   SENSITIVE=0
  endif else begin
     widget_control, state.id.par_bin,   /SENSITIVE
  endelse
  
  if (state.par.cflux_dir eq 0B) then begin
     widget_control, state.id.par_flux,  SENSITIVE=0
     widget_control, state.id.par_tel,   /SENSITIVE
  endif else begin
     widget_control, state.id.par_flux,  /SENSITIVE
     widget_control, state.id.par_tel,   SENSITIVE=0
  endelse
  
  
  if (state.par.deltaM_cvplot.enable eq 0B) then begin
     widget_control, state.id.deltaM_cvplot_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.deltaM_cvplot_filename, /SENSITIVE
  endelse
  
  if (state.par.separation_cvplot.enable eq 0B) then begin
     widget_control, state.id.separation_cvplot_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.separation_cvplot_filename, /SENSITIVE
  endelse
  
  if (state.par.PA_cvplot.enable eq 0B) then begin
     widget_control, state.id.PA_cvplot_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.PA_cvplot_filename, /SENSITIVE
  endelse
  
  if (state.par.Main_mag_cvplot.enable eq 0B) then begin
     widget_control, state.id.Main_mag_cvplot_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.Main_mag_cvplot_filename, /SENSITIVE
  endelse
  
  if (state.par.Comp_mag_cvplot.enable eq 0B) then begin
     widget_control, state.id.Comp_mag_cvplot_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.Comp_mag_cvplot_filename, /SENSITIVE
  endelse
  
  if (state.par.cvparam.enable eq 0B) then begin
     widget_control, state.id.cvparam_filename, SENSITIVE=0
  endif else begin
     widget_control, state.id.cvparam_filename, /SENSITIVE
  endelse
  
  
end

;;;;;;;;;;;;;;;;;;;;;;
; anb_gui event loop ;
;;;;;;;;;;;;;;;;;;;;;;
;
pro anb_gui_event, event
  
  common error_block, error
  
; read the GUI state structure
  widget_control, event.top, GET_UVALUE=state
  
; handle a kill request (considered as a cancel event).
  if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
     error = !caos_error.cancel
     widget_control, event.top, /DESTROY
  endif
  
; handle other events.
  widget_control, event.id, GET_UVALUE=uvalue
  case uvalue of
     '' : begin
                                ; do nothing!
     end
     'comparison'    : begin
        state.par.comparison = event.value
        anb_gui_set, state
     end
     
     'd_detect'      : state.par.d_detect = event.value
     
     'tresh_detect'  : state.par.tresh_detect = event.value
     
     'centr_box_size': state.par.centr_box_size = event.value
     
     'cflux_dir'      : begin
        state.par.cflux_dir = event.value
        anb_gui_set, state
     end
     
     'cal_flux'       : state.par.cal_flux = event.value
     
     'cal_mag'        : state.par.cal_mag = event.value
     
     'band'           : state.par.band_index = event.index
     
     'tel_surf'       : state.par.tel_surf = event.value
     
     'int_time'       : state.par.int_time = event.value
     
     'efficiency'     : state.par.efficiency = event.value
     
     'table_aperphot' : if (event.type eq 0) then begin
        widget_control, event.id,  GET_VALUE =dummy
        state.par.aper_phot = dummy
     endif
     
     'bin_separation': state.par.bin_separation = event.value
     
     'bin_pa'        : state.par.bin_pa = event.value
     
     'bin_mag'       : state.par.bin_mag = event.value
     
     'bin_delt_m'    : state.par.bin_delt_m = event.value
     
     'deltaM_cvplot_enable' : begin
        state.par.deltaM_cvplot.enable = event.select
        anb_gui_set, state
     end
     'deltaM_cvplot_filename' : begin
        widget_control, state.id.deltaM_cvplot_filename , GET_VALUE=dummy
        state.par.deltaM_cvplot.filename = dummy[0]
        anb_gui_set, state
     end
     
     'separation_cvplot_enable' : begin
        state.par.separation_cvplot.enable = event.select
        anb_gui_set, state
     end
     'separation_cvplot_filename' : begin
        widget_control, state.id.separation_cvplot_filename , GET_VALUE=dummy
        state.par.separation_cvplot.filename = dummy[0]
        anb_gui_set, state
     end
     
     'PA_cvplot_enable' : begin
        state.par.PA_cvplot.enable = event.select
        anb_gui_set, state
     end
     'PA_cvplot_filename' : begin
        widget_control, state.id.PA_cvplot_filename , GET_VALUE=dummy
        state.par.PA_cvplot.filename = dummy[0]
        anb_gui_set, state
     end
     
     'Main_mag_cvplot_enable' : begin
        state.par.Main_mag_cvplot.enable = event.select
        anb_gui_set, state
     end
     'Main_mag_cvplot_filename' : begin
        widget_control, state.id.Main_mag_cvplot_filename , GET_VALUE=dummy
        state.par.Main_mag_cvplot.filename = dummy[0]
        anb_gui_set, state
     end
     
     'Comp_mag_cvplot_enable' : begin
        state.par.Comp_mag_cvplot.enable = event.select
        anb_gui_set, state
     end
     'Comp_mag_cvplot_filename' : begin
        widget_control, state.id.Comp_mag_cvplot_filename , GET_VALUE=dummy
        state.par.Comp_mag_cvplot.filename = dummy[0]
        anb_gui_set, state
     end
     
     'cvparam_enable' : begin
        state.par.cvparam.enable = event.select
        anb_gui_set, state
     end
     'cvparam_filename' : begin
        widget_control, state.id.cvparam_filename , GET_VALUE=dummy
        state.par.cvparam.filename = dummy[0]
        anb_gui_set, state
     end
    'verbosity': state.par.verbosity = event.select
    
                                ; handle event from standard save button
     'save': begin
        
        
                                ; save data in the parameter file.
                                ; check before if filename already exists
        check_file = findfile(state.sav_file)
        if check_file[0] ne "" then begin
           answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                  'do you want to overwrite it ?'],          $
                                 DIALOG_PARENT=event.top,                    $
                                 TITLE='ANB warning',                        $
                                 /QUEST)
                                ; return without saving if the user doesn't want to overwrite the
                                ; existing parameter file
           if strlowcase(answ) eq "no" then return
        endif else begin
           answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                                 DIALOG_PARENT=event.top,                    $
                                 TITLE='ANB information',                    $
                                 /INFO                                       )
                                ; inform were the parameters will be saved
        endelse
        
                                ; save the parameter data file
        par = state.par
        save, par, FILENAME = state.sav_file
        
                                ; kill the GUI returning a null error
        error = !caos_error.ok
        widget_control, event.top, /DESTROY
        return
     end
     
                                ; handle event from standard help button
     'help':begin
        online_help, book=(anb_info()).help, /FULL_PATH
     end
     
                                ; standard restore button:
     'restore': begin
        
                                ; restore the desired parameter file
        par = 0
        title = "parameter file to restore"
        restore, filename_gui(state.def_file,                          $
                              title,                                   $
                              GROUP_LEADER=event.top,                  $
                              FILTER=state.par.module.mod_name+'*sav', $
                              /NOEDIT,                                 $
                              /MUST_EXIST,                             $
                              /ALL_EVENTS                              )
        
                                ; update the current module number
        par.module.n_module = state.par.module.n_module
        
                                ; set the default values for all the widgets
        widget_control, state.id.comparison,     SET_VALUE=par.comparison
        widget_control, state.id.d_detect,       SET_VALUE=par.d_detect
        widget_control, state.id.tresh_detect,   SET_VALUE=par.tresh_detect
        widget_control, state.id.centr_box_size, SET_VALUE=par.centr_box_size
        widget_control, state.id.cflux_dir,      SET_VALUE=par.cflux_dir
        widget_control, state.id.cal_flux,       SET_VALUE=par.cal_flux
        widget_control, state.id.cal_mag,        SET_VALUE=par.cal_mag
        widget_control, state.id.band,           SET_DROPLIST_SELECT=par.band_index
        widget_control, state.id.efficiency,     SET_VALUE=par.efficiency
        widget_control, state.id.int_time,       SET_VALUE=par.int_time
        widget_control, state.id.tel_surf,       SET_VALUE=par.tel_surf
        widget_control, state.id.table_aperphot, SET_VALUE=par.aper_phot
        widget_control, state.id.bin_separation, SET_VALUE=par.bin_separation
        widget_control, state.id.bin_pa,         SET_VALUE=par.bin_pa
        widget_control, state.id.bin_mag,        SET_VALUE=par.bin_mag
        widget_control, state.id.bin_delt_m,     SET_VALUE=par.bin_delt_m
        
        widget_control, state.id.deltaM_cvplot_enable,       SET_VALUE=par.deltaM_cvplot.enable
        widget_control, state.id.deltaM_cvplot_filename,     SET_VALUE=par.deltaM_cvplot.filename
        widget_control, state.id.separation_cvplot_enable,   SET_VALUE=par.separation_cvplot.enable
        widget_control, state.id.separation_cvplot_filename, SET_VALUE=par.separation_cvplot.filename
        widget_control, state.id.PA_cvplot_enable,           SET_VALUE=par.PA_cvplot.enable
        widget_control, state.id.PA_cvplot_filename,         SET_VALUE=par.PA_cvplot.filename
        widget_control, state.id.Main_mag_cvplot_enable,     SET_VALUE=par.Main_mag_cvplot.enable
        widget_control, state.id.Main_mag_cvplot_filename,   SET_VALUE=par.Main_mag_cvplot.filename
        widget_control, state.id.Comp_mag_cvplot_enable,     SET_VALUE=par.Comp_mag_cvplot.enable
        widget_control, state.id.Comp_mag_cvplot_filename,   SET_VALUE=par.Comp_mag_cvplot.filename
        
        widget_control, state.id.cvparam_enable,             SET_VALUE=par.cvparam.enable
        widget_control, state.id.cvparam_filename,           SET_VALUE=par.cvparam.filename
        
                                ; write the reseted state structure
        state.par = par
        
                                ; write the GUI state structure
        widget_control, event.top, SET_UVALUE=state
     end
     
                                ; standard cancel button (exit without saving)
     'cancel': begin
        error = !caos_error.cancel
        widget_control, event.top, /DESTROY
        return
     end
     
  endcase
  
; write the GUI state structure
  widget_control, event.top, SET_UVALUE=state
  
  return
end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function anb_gui, n_module, proj_name, GROUP_LEADER=group
  
; error status from the event handler procedure
  common error_block, error
  
; get info structure
  info = anb_info()
  
; parameter file checks
  sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
  def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
  par=0
  check_file = findfile(sav_file)
  if check_file[0] eq '' then begin
     restore, def_file
     par.module.n_module = n_module
     if (par.module.mod_name ne info.mod_name) then     $
        message, 'the default parameter file ('+def_file+') is from another module: please take the right one'
  endif else begin
     restore, sav_file
     if (par.module.mod_name ne info.mod_name) then $
        message, 'the parameter file '+sav_file      $
                 +' is from another module: please generate a new one'
     if (par.module.ver ne info.ver) then       $
        message, 'the parameter file '+sav_file $
                 +' is not compatible: please generate it again'
  endelse
  
  
  
; build the widget id structure
  id = $
     { $                        ; widget id structure
     root_base_id    : 0L, $    ;
     verbosity       : 0L, $    ; 
     comparison      : 0L, $    ; comparison with binary model (1=yes, 0=no)
     par_base        : 0L, $    ; data widget base
     par_base_left   : 0L, $    ;
     par_base_right  : 0L, $    ;
     par_bin         : 0L, $    ; data widget binary model parameters base
     band            : 0L, $    ; observation spectral band (R,J,H,K)
     par_flux_all    : 0L, $    ;
     par_flux_quer   : 0L, $    ;
     par_flux        : 0L, $    ;
     cflux_dir       : 0L, $    ;
     cal_flux        : 0L, $    ;
     cal_mag         : 0L, $    ;
     par_tel         : 0L, $    ;
     efficiency      : 0L, $    ;
     int_time        : 0L, $    ;
     tel_surf        : 0L, $    ;
     par_ap_phot     : 0L, $    ;
     table_aperphot  : 0L, $
     bin_separation  : 0L, $             ; queried binary separation [arcsec]
     bin_pa          : 0L, $             ; queried binary position angle (PA) [degree]
     bin_mag         : 0L, $             ; queried binary main component magnitude
     bin_delt_m      : 0L, $             ; queried binary difference of magnitude
     par_an          : 0L, $             ; data widget analysis parameters base
     d_detect        : 0L, $             ; diameter of the detected star patch [pixel]
     tresh_detect    : 0L, $             ; treshold of the detection [background value]
     centr_box_size  : 0L, $             ; centroid box size [pixel]
     par_ofile       : 0L, $             ; data widget result .ps filename base
     deltaM_out_ps_file            : 0L, $ ; difference of magnitude convergence plot base
     deltaM_cvplot_enable          : 0L, $ ; enable writing difference of magnitude convergence plot on a .ps file
     deltaM_cvplot_filename        : 0L, $ ; difference of magnitude convergence plot .ps filename
     separation_out_ps_file        : 0L, $ ; separation convergence plot base
     separation_cvplot_enable      : 0L, $ ; enable writing separation convergence plot on a .ps file
     separation_cvplot_filename    : 0L, $ ; separation convergence plot .ps filename
     PA_out_ps_file                : 0L, $ ; Position angle convergence plot base
     PA_cvplot_enable              : 0L, $ ; enable writing position angle convergence plot on a .ps file
     PA_cvplot_filename            : 0L, $ ; position angle convergence plot .ps filename
     Main_mag_out_ps_file          : 0L, $ ; Main component magnitude convergence plot base
     Main_mag_cvplot_enable        : 0L, $ ; enable writing main component magnitude convergence plot on a .ps file
     Main_mag_cvplot_filename      : 0L, $ ; main component magnitude convergence plot .ps filename
     Comp_mag_out_ps_file     : 0L, $      ; Companion magnitude convergence plot base
     Comp_mag_cvplot_enable   : 0L, $      ; enable writing companion magnitude convergence plot on a .ps file
     Comp_mag_cvplot_filename : 0L, $      ; companion magnitude convergence plot .ps filename
     par_savfile                   : 0L, $ ; data widget result .sav filename base
     param_out_sav_file            : 0L, $ ; Results .sav base
     cvparam_enable                : 0L, $ ; enable saving .sav results structure
     cvparam_filename              : 0L  $ ; .sav results structure filename
     }
  
  
; build the state structure
  state = $
     {    $                     ; widget state structure
     sav_file: sav_file, $      ; name of the file where save params
     def_file: def_file, $      ; name of the file where save params
     id      : id,       $      ; widget id structure
     par     : par       $      ; parameter structure
     }
  
; root base
  modal = n_elements(group) ne 0
  title = strupcase(info.mod_name)+' parameters setting GUI'
  root_base_id = widget_base(TITLE=title, MODAL=modal, /COL, GROUP_LEADER=group)
  
; set the status structure
  widget_control, root_base_id, SET_UVALUE=state

                                ; parameters base
  state.id.par_base  = widget_base(root_base_id, FRAME=10, /ROW)
  
  
  basetab = WIDGET_TAB(state.id.par_base, location = 0, uvalue='')   
  baseA = WIDGET_BASE(basetab, /ROW, TITLE='Analysis')
  baseB = WIDGET_BASE(basetab, /COL, TITLE='Results')
  baseAL = WIDGET_BASE(baseA, /COL)
  baseAR = WIDGET_BASE(baseA, /COL)
;   dummy = widget_label(state.id.par_base_left, VALUE='parameters', /FRAME)
  
  state.id.comparison=cw_bgroup(baseAL, LABEL_LEFT='Comparison with binary model:', $
                                ['no', 'yes'], COLUMN=2, $ 
                                SET_VALUE=state.par.comparison, $
                                UVALUE='comparison', /EXCLUSIVE )
  
  state.id.par_an = widget_base(baseAL, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_an, VALUE='Analysis parameters :', /FRAME)
  
  state.id.d_detect=cw_field(state.id.par_an, $
                             TITLE='Detected star patch diameter [pixel] ',$
                             /FLOAT, VALUE=state.par.d_detect, UVALUE='d_detect', $
                             /ALL_EVENTS )
  
  state.id.tresh_detect=cw_field(state.id.par_an, $
                                 TITLE='Detection treshold [rms background]  ',$
                                 /FLOAT, VALUE=state.par.tresh_detect,  $
                                 UVALUE='tresh_detect', /ALL_EVENTS  )
  
  state.id.centr_box_size=cw_field(state.id.par_an,                              $
                                   TITLE='centroid box size [pixel]            ',$
                                   /FLOAT, VALUE=state.par.centr_box_size,  $
                                   UVALUE='centr_box_size', /ALL_EVENTS  )
  
  state.id.par_flux_all = widget_base(baseAR, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_flux_all, VALUE='Photometry parameters :', /FRAME)
  
  
  state.id.cflux_dir=cw_bgroup(state.id.par_flux_all,   $
                               LABEL_LEFT='Direct flux calibration ?', $
                               ['no', 'yes'], COLUMN=2,  $
                               SET_VALUE=state.par.cflux_dir, $
                               UVALUE='cflux_dir', /EXCLUSIVE )
  
  state.id.par_flux = widget_base(state.id.par_flux_all, /COL, FRAME=2 )
  state.id.cal_flux = cw_field(state.id.par_flux,                        $
                               TITLE='Calibrated flux [counts]             ',$
                               /FLOAT, VALUE=state.par.cal_flux, $
                               UVALUE='cal_flux', /ALL_EVENTS )
  
  state.id.cal_mag = cw_field(state.id.par_flux,                        $
                              TITLE='Corresponding magnitude in the band   ', $
                              /FLOAT, VALUE=state.par.cal_mag,  $
                              UVALUE='cal_mag', /ALL_EVENTS  )
  
  state.id.par_tel = widget_base(baseAR, /COL, FRAME=2 )
  
  band = widget_base(state.id.par_tel, /ROW)
  dummy  = widget_label(band, xsize= 100, VALUE='Band')
  state.id.band = widget_droplist(band, VALUE=state.par.band_tab, UVALUE='band')
  
  state.id.efficiency = cw_field(state.id.par_tel,                             $
                                 TITLE='overall efficiency:                  ',$
                                 /FLOAT, VALUE=state.par.efficiency,  $
                                 UVALUE='efficiency', /ALL_EVENTS  )
  
  state.id.int_time=cw_field(state.id.par_tel,                             $
                             TITLE='integration time [s]:                ',$
                             /FLOAT, VALUE=state.par.int_time,  $
                             UVALUE='int_time', /ALL_EVENTS )
  
  state.id.tel_surf = cw_field(state.id.par_tel,                             $
                               TITLE='telescope collecting surface [m^2]:  ',$
                               /FLOAT, VALUE=state.par.tel_surf,  $
                               UVALUE='tel_surf', /ALL_EVENTS  )
  
  state.id.par_ap_phot = widget_base(state.id.par_flux_all, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_ap_phot,VALUE='Aperture photometry diameters set [pixel] :')
  
  state.id.table_aperphot = widget_table(state.id.par_ap_phot,                $
                                         VALUE=state.par.aper_phot,           $
                                         /NO_HEADER, UVALUE='table_aperphot', $
                                         /EDITABLE, /ALL_EVENTS )
  
  
  
  state.id.par_bin = widget_base(baseAL, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_bin, VALUE='Binary parameters :', /FRAME)
  
  
  state.id.bin_separation=cw_field(state.id.par_bin,     $
                                   TITLE='Separation [arcsec]                  ',$
                                   /FLOATING,  VALUE=state.par.bin_separation, $
                                   UVALUE='bin_separation', /ALL_EVENTS )
  
  state.id.bin_pa=cw_field(state.id.par_bin,$
                           TITLE='Position angle [deg.]                ',$
                           /FLOATING, VALUE=state.par.bin_pa,  $
                           UVALUE='bin_pa', /ALL_EVENTS )
  
  state.id.bin_mag=cw_field(state.id.par_bin, $
                            TITLE='Principal component magnitude        ',$
                            /FLOATING, VALUE=state.par.bin_mag, $
                            UVALUE='bin_mag', /ALL_EVENTS  )
  
  state.id.bin_delt_m  = cw_field(state.id.par_bin,$
                                  TITLE='Magnitude difference                 ',$
                                  /FLOATING, VALUE=state.par.bin_delt_m, $
                                  UVALUE='bin_delt_m', /ALL_EVENTS)
                                ; .ps convergence plots of the binary parameters wrt the true parameters
  state.id.par_ofile = widget_base(baseB, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_ofile, $
                       VALUE='Results vs iterations number will be plotted in the .ps files :', $
                       /FRAME)
  state.id.deltaM_out_ps_file = widget_base(state.id.par_ofile, /ROW)
  state.id.deltaM_cvplot_enable = cw_bgroup(state.id.deltaM_out_ps_file,$
                                            ['Difference of magnitude '],$
                                            SET_VALUE=state.par.deltaM_cvplot.enable,  $
                                            UVALUE='deltaM_cvplot_enable',$
                                            /NONEXCLUSIVE)
  state.id.deltaM_cvplot_filename = cw_filename(state.id.deltaM_out_ps_file, $
                                                TITLE='',$
                                                VALUE=state.par.deltaM_cvplot.filename,$
                                                UVALUE='deltaM_cvplot_filename',$
                                                /ALL_EVENTS, xsize=60)
  state.id.separation_out_ps_file = widget_base(state.id.par_ofile, /ROW)
  state.id.separation_cvplot_enable = cw_bgroup(state.id.separation_out_ps_file,$
                                                ['Separation              '],$
                                                SET_VALUE=state.par.separation_cvplot.enable,$
                                                UVALUE='separation_cvplot_enable',$
                                                /NONEXCLUSIVE)
  state.id.separation_cvplot_filename = cw_filename(state.id.separation_out_ps_file, $
                                                    TITLE='',$
                                                    VALUE=state.par.separation_cvplot.filename,$
                                                    UVALUE='separation_cvplot_filename',$
                                                    /ALL_EVENTS, xsize=60)
  state.id.PA_out_ps_file = widget_base(state.id.par_ofile, /ROW)
  state.id.PA_cvplot_enable = cw_bgroup(state.id.PA_out_ps_file,$
                                        ['Position angle          '],$
                                        SET_VALUE=state.par.PA_cvplot.enable,$
                                        UVALUE='PA_cvplot_enable',$
                                        /NONEXCLUSIVE)
  state.id.PA_cvplot_filename = cw_filename(state.id.PA_out_ps_file,$
                                            TITLE='',$
                                            VALUE=state.par.PA_cvplot.filename,$
                                            UVALUE='PA_cvplot_filename',$
                                            /ALL_EVENTS, xsize=60)
  state.id.Main_mag_out_ps_file = widget_base(state.id.par_ofile, /ROW)
  state.id.Main_mag_cvplot_enable = cw_bgroup(state.id.Main_mag_out_ps_file,$
                                              ['Main component magnitude'],$
                                              SET_VALUE=state.par.Main_mag_cvplot.enable,$
                                              UVALUE='Main_mag_cvplot_enable',$
                                              /NONEXCLUSIVE)
  state.id.Main_mag_cvplot_filename = cw_filename(state.id.Main_mag_out_ps_file, $
                                                  TITLE='',$
                                                  VALUE=state.par.Main_mag_cvplot.filename,$
                                                  UVALUE='Main_mag_cvplot_filename',$
                                                  /ALL_EVENTS, xsize=60)
  state.id.Comp_mag_out_ps_file = widget_base(state.id.par_ofile, /ROW)
  state.id.Comp_mag_cvplot_enable = cw_bgroup(state.id.Comp_mag_out_ps_file, $
                                              ['Companion magnitude     '],$
                                              SET_VALUE=state.par.Comp_mag_cvplot.enable,$
                                              UVALUE='Comp_mag_cvplot_enable',$
                                              /NONEXCLUSIVE)
  state.id.Comp_mag_cvplot_filename = cw_filename(state.id.Comp_mag_out_ps_file, $
                                                  TITLE='',$
                                                  VALUE=state.par.Comp_mag_cvplot.filename,$
                                                  UVALUE='Comp_mag_cvplot_filename',$
                                                  /ALL_EVENTS, xsize=60)
                                ; .sav convergence parameters of the binary wrt the true parameters
  state.id.par_savfile = widget_base(baseB, /COL, FRAME=2 )
  state.id.param_out_sav_file = widget_base(state.id.par_savfile, /ROW)
  state.id.cvparam_enable = cw_bgroup(state.id.param_out_sav_file,$
                                      ['Results vs iterations number will be save in the .sav file:'],$
                                      SET_VALUE=state.par.cvparam.enable,$
                                      UVALUE='cvparam_enable',$
                                      /NONEXCLUSIVE)
  state.id.cvparam_filename = cw_filename(state.id.param_out_sav_file,$
                                          TITLE=' ',$
                                          VALUE=state.par.cvparam.filename,$
                                          UVALUE='cvparam_filename',$
                                          /ALL_EVENTS, xsize=60)

; verbosity base
  base_verbosity = widget_base(root_base_id,/ROW)  
  state.id.verbosity = cw_bgroup(base_verbosity, $
                                 ['Print analysis every iteration (verbosity)'],$
                                 SET_VALUE = state.par.verbosity, $
                                 UVALUE = 'verbosity',/NONEXCLUSIVE )
  
                                ; standard buttons
  btn_base_id = widget_base(root_base_id, FRAME=10, /ROW)
  dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
  cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
  if modal then widget_control, cancel_id, /CANCEL_BUTTON
  dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                        UVALUE="restore")
  save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS", $
                          UVALUE="save")
  if modal then widget_control, save_id, /DEFAULT_BUTTON
  
; initialize the sensitive state
  anb_gui_set, state
  
; save the state structure of the GUI in the top base uvalue
  widget_control, root_base_id, SET_UVALUE=state
  
; draw the GUI
  widget_control, root_base_id, /REALIZE
  
  xmanager, 'anb_gui', root_base_id, GROUP_LEADER=group
  
  return, error
end
