; $Id: anb_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+ 
; NAME: 
;    anb_init 
; 
; PURPOSE: 
;    anb_init executes the initialization for the ANalysis Binary  
;    (ANB) module.
; 
; CATEGORY: 
;    module's initialization routine 
; 
; CALLING SEQUENCE: 
;    error = anb_init(inp_img_t, $ ; img_t input structure
;                     par,       $ ; parameters structure
;                     INIT=init  ) ; initialization structure
; 
; INPUTS/OUTPUTS/KEYWORDS/ETC.: 
;    see anb.pro for a detailed description. 
; 
; ROUTINE MODIFICATION HISTORY: 
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : july 2008,
;                     Gabriele Desiderà (DISI) [desidera@disi.unige.it]:
;                    -multi-frame case not allowed.
;                   : March 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                     Added verbosity choice [default = no]
;
;- 
; 
function anb_init, inp_img_t, $
                   par,       $
                   INIT=init 

; CAOS global common block
common caos_block, tot_iter, this_iter

; initialization of the error code: no error as default
error = !caos_error.ok 

; retrieve the module's informations
info = anb_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

; STANDARD CHECKS
;
; compute and test the requested number of anb arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'ANB error: par must be a structure'
if n ne 1 then message, 'ANB error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module ANB'

; check the input arguments

; test if any optional input exists
if n_inp gt 0 then begin
   inp_opt = info.inp_opt
endif

dummy = test_type(inp_img_t, TYPE=type)
if type eq 0 then begin         ; undefined variable
   inp_img_t = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif
if test_type(inp_img_t, /STRUC, N_EL=n, TYPE=type) then $
   message, 'ANB error: wrong definition for the first input.'
if n ne 1 then message, $
   'ANB error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_img_t.data_status eq !caos_data.not_valid and not inp_opt[0] then $
      message, 'undefined input is not allowed'

;-------------------MODIFICA PER GESTIRE ERRORE IN CASO MULTI-FRAME
pos=where(inp_img_t.time_integ[*,1] gt 1,num)
if ((size(inp_img_t.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_img_t.image=reform(inp_img_t.image,(size(inp_img_t.image))[1],(size(inp_img_t.image))[2],(size(inp_img_t.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE (MULTI-FRAME input is not supported)',/ERROR)
      message,'WRONG INPUT TYPE (MULTI-FRAME input is not supported)'
   endelse
endif
;------------------FINE MODIFICA ERRORE PER GESTIRE CASO MULTI-FRAME

; Comparison with given binary model
if par.comparison eq 1B then begin

    ;**************************** Make model of binary **********************************
    star = replicate({info:fltarr(3)},2)
    binary = fltarr(inp_img_t.npixel,inp_img_t.npixel)


        x_cent = inp_img_t.npixel/2 + 0.5 ;centered on the pixel
        y_cent = inp_img_t.npixel/2 + 0.5

        x_main = x_cent - (par.bin_separation/inp_img_t.resolution*cos(par.bin_pa*!pi/180))/2 
        y_main = y_cent + (par.bin_separation/inp_img_t.resolution*sin(par.bin_pa*!pi/180))/2 

        x_comp = x_main + par.bin_separation/inp_img_t.resolution*cos(par.bin_pa*!pi/180)
        y_comp = y_main - par.bin_separation/inp_img_t.resolution*sin(par.bin_pa*!pi/180)

        ;subsampling rapport used to locate stars and to compute convolution with PSFs
        rap   = 1               

        sep = par.bin_separation
        pa  = par.bin_pa

        model_binary, x_main, y_main, x_comp, y_comp, sep, pa, par.bin_mag, par.bin_delt_m,          $
                      inp_img_t.npixel, par.efficiency, par.band_tab[par.band_index], par.tel_surf,  $
                      par.int_time, inp_img_t.resolution, rap, binary, star, nb_ph_main, nb_ph_comp, $
                      par.verbosity


        ;**************** photometric and astrometric evaluation vs iteration number ******************
        nb_ph_star = [nb_ph_main, nb_ph_comp]

endif 


;test if the input image is 3D (use after rft, in a single project dedicated to the analyse of already saved data) 
;or in 2D (tipically in the data flow of a deconvolution project)
if (size(inp_img_t.image))[0] eq 3 then iter = (size(inp_img_t.image))[3] else iter = tot_iter


; init structures
if par.comparison eq 1B then begin   ; Comparison with given binary model case 

   init = $
      {   $
      counter      : 0L,                                            $
      star         : star,                                          $
      nb_ph_star   : nb_ph_star,                                    $
      delta_M      : fltarr(2, iter, (size(par.aper_phot))[1]), $ ; main and companion magnitude residuals
      Mag_diff_int : fltarr(iter, (size(par.aper_phot))[1]),    $ ; residuals (original - detected) 
                                                                  ; difference of magnitude
      xc           : fltarr(2, iter),                           $ ; X centroids (main and companion)
      yc           : fltarr(2, iter),                           $ ; Y centroids (main and companion)
      sep_all      : fltarr(iter),                              $ ; absolute separation (detected)
      pa_all	   : fltarr(iter),                              $ ; absolute PA (detected)
      sep_all_rel  : fltarr(iter),                              $ ; relative separation (original - detected)
      pa_all_rel   : fltarr(iter)                               } ; relative PA (original - detected)
                                                                    
endif else begin ; absolute binary values case

   init = $
      {   $
      counter      : 0L,                                                    $
      Mag_diff_int : fltarr(iter, (size(par.aper_phot))[1]),            $ ; difference of magnitude
      xc           : fltarr(2, iter),                                   $ ; X centroids (main and companion) 
      yc           : fltarr(2, iter),                                   $ ; Y centroids (main and companion)
      sep_all      : fltarr(iter),                                      $ ; absolute separation (detected)
      pa_all	   : fltarr(iter)                                       } ; absolute PA (detected)

endelse


; back to calling program
return, error 
end
