; $Id: cbd.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    CBD
;
; ROUTINE'S PURPOSE:
;    CBD manages the simulation for the Constrained Blind (or myopic) 
;    Deconvolution (CBD) module, that is:
;       1-call the module's initialization routine cbd_init at the first
;         iteration of the simulation project,
;       2-call the module's program routine cbd_prog.
;
; MODULE'S PURPOSE:
;    CBD performs the deconvolution of a cube of images (i.e. p images
;    corresponding to different orientations of the baseline) when
;    the user don't have a PSF.
;    The module gives back an image and an estimate of the PSF(s) and has in
;    input a PSF(s) and an image(s).
;    If the user has already an initial estimate of the PSF(s) he can give
;    it directly in input; otherwise the module generates an estimate by an
;    operation based on the autocorrelation operator on the image supplied
;    in input.
;    It is possible to obtain these two reconstructions by perform
;    alternatively two partial reconstructions, the first one to the image(s)
;    and the second one to the PSF(s).
;    There are two different methods for image deconvolution and just one
;    for PSFs reconstruction. In the first case the user can choose between
;    the OS-EM method (see for details: "Application of the OS-EM method to the
;    restoration of LBT images", M. Bertero and P. Boccacci, A&AS 144,
;    May 2000, 181-186) and the LR-EM method.
;    CBD fits the PSFs dimensions to the images dimensions, if they do not
;    match, through zero-padding.
;    The background evaluation performed by PRE module is used to restore
;    the object with a correct sky-value, while the reconstructed PSFs are 
;    managed without background (this is subtracted before the deconvolution 
;    process).
;    In this version no stopping rule is given. The rates of convergence may
;    be different for different types of objects.
;    This algorithm is sensible to the number of the iterations that the user 
;    set in the reconstruction box of images and PSFs.
;    In the case of a point-like stellar object, it is easy to have a good
;    estimate of the initial PSFs.
;    In the case of diffuse objects (i.e. galaxies) is more difficult to obtain
;    a good estimate about the PSFs shape making the autocorrelation of the
;    input image and it is recommended to supply a different one.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = cbd(inp_img_t2, out_psf_t, out_img_t, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t2: structure of type img_t, corresponding to images
;    par       : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_psf_t: structure of type "img_t" (with one more feature, the filter 
;               to use to maintain in band the same PSFS) containing the 
;               characteristics of the deconvolved PSF(s).
;               The following fields are contained in this structure:
;out_psf_t = 				   $
;   {          				   $
;   data_type  : out_type[0]            , $
;   data_status: !caos_data.valid       , $
;   image      : cube2                  , $
;   psf        : 1B                     , $    ; This is a PSF
;   npixel     : N                      , $    ; Number of pixels (per subaperture)
;   resolution : inp_img_t2.resolution  , $    ; Pixel size [arcsec/pix]
;   lambda     : inp_img_t2.lambda      , $    ; Mean detection wavelength [m]
;   width      : inp_img_t2.width       , $    ; Wavelength band width [m]
;   time_integ : inp_img_t2.time_integ  , $    ; Integration time for each image [s]
;   time_delay : inp_img_t2.time_delay  , $    ; [NOT USED WITHIN AIRY]
;   background : make_array(N,N,P)      , $ ; the PSF's background is zero
;   snr        : inp_img_t2.snr         , $    ; snr
;   header     : PTR_NEW('')       $    ;header of the output PSF
;   }
;*out_psf_t.header = *inp_img_t2.header
;
;    out_img_t: structure of type "img_t" containing the characteristics of
;               the deconvolved object. The following fields are contained in this structure:
;out_img_t = 				   $
;   {          				   $
;   data_type  : out_type[0]     	 , $
;   data_status: !caos_data.valid	 , $
;   image      : cube            	 , $
;   psf        : 0B              	 , $   ; This flag indicates that this is a IMAGE
;   npixel     : N              	 , $           ; Number of pixels (per subaperture)
;   resolution : inp_img_t2.resolution   , $    ; Pixel size [arcsec/pix]
;   lambda     : inp_img_t2.lambda       , $    ; Mean detection wavelength [m]
;   width      : inp_img_t2.width        , $    ; Wavelength band width [m]
;   time_integ : inp_img_t2.time_integ   , $    ; Integration time for each image [s]
;   time_delay : inp_img_t2.time_delay   , $    ; [NOT USED WITHIN AIRY]
;   background : inp_img_t2.background   , $    ; background
;   snr        : inp_img_t2.snr          , $    ; snr
;   header     : PTR_NEW('')         $    ; header of the output object
;   }
;*out_img_t.header=*inp_img_t2.header
;
; KEYWORD PARAMETERS:
;    INIT: initialization data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : int scalar. Total number of iteration during the 
;                 simulation run.
;    this_iter  : int scalar. Number of the current iteration. It is
;                 defined only while status eq !caos_status.run.
;                 (this_iter >= 1).  
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    -other routine from the folder .../cbd/cbd_gui_lib.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: May 2005, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;
;
; MODULE MODIFICATION HISTORY:
;    module written : december 2008, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;    modifications  : for version 5.0,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -the module (previously called BLI) has been renamed in 
;                     CBD after the work done for the paper (+revisions):
;                     G. Desiderà, M. Carbillet, "Strehl-constrained iterative
;                     blind deconvolution for post-adaptive-optics data", 2009,
;                     A&A 507, pp. 1759-1762.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -output's order exchanged: (WAS) top=PSF, bottom=OBJ;
;                     (NOW IS) top=OBJ, bottom=PSF, as it is in the whole 
;                     Soft.Pack.AIRY
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : February 2014, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -GUI rewritten with TAB, several optional outputs
;                     added.  
;                   : March 2015,
;                    -Add another method of Blind Deconvolution, based on the 
;                     paper cited in the file "AIRYSoftPack6.1_From6.0to6.1.txt"
;                    -Some changes on the GUI (internal tabs added and
;                     several input fields moved here and there).
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -addded save_obj_fun and save_obj_fun_file
;                    -the header is saved in the output structure and
;                     eventually modified with the deconvolution parameters.
;                    -added optional output save: objective function
;                   : February 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    - cbd_init.pro debugged
;-
;
;;;;;;;;;;;;;;;
; module code ;
;;;;;;;;;;;;;;;
;
function cbd, inp_img_t2, out_psf_t, out_img_t, par, INIT=init

common caos_block, tot_iter, this_iter

; initialization of the error code: set to "no error" as default
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = cbd_init(inp_img_t2, out_psf_t, out_img_t, par, INIT = init)
endif else begin
   ; run section
   error = cbd_prog(inp_img_t2, out_psf_t, out_img_t, par, INIT = init)
endelse

; back to calling program.
return, error
end
