; $Id: cbd_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cbd_gen_default
;
; PURPOSE:
;    cbd_gen_default generates the default parameter structure
;    for the CBD module and save it in the right location.
;    The user doesn't need to use cbd_gen_default, it is used
;    only for developing and upgrading purposes.
;
; CATEGORY:
;    utility
;
; CALLING SEQUENCE:
;    lbd_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: december 2008,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;    modifications  : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -comments revised.
;                   : February 2014, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -GUI rewritten with TAB, several optional outputs
;                     added. 
;                   : January 2015,
;                    -Add another method of Blind Deconvolution, based on the 
;                     paper cited in the file "AIRYSoftPack6.1_From6.0to6.1.txt"
;                    -added variables: cbd_method,
;                     psf_id_filename, psf_init, sgp_verbosity
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -addded save_obj_fun and save_obj_fun_file
;-

pro cbd_gen_default

info = cbd_info()               ; obtain module infos

; Generate module description structure.

module = gen_def_module(info.mod_name, info.ver)

path = !caos_env.work+!caos_env.delim     

strehl = make_array(360,/double)


par = {                       $
      cbd,                       $ ; Structure named lbd
      module          : module,  $ 
      method          : 0B,      $ ; convolution method [0=OS-EM, 1=LR/EM, 2=ISRA]
                                   ; NOT REALLY USED. 
      cbd_method      : 0B,      $ ; blind deconvolution method:
                                   ; 0 = [Desidera et al. (2010)]'s method
                                   ; 1 = [Prato et al. (2013)]'s method
      iter_obj        : 1L,      $ 
      iter_psf        : 1L,      $
      ok_strehl       : 0L,      $ ; Strehl constraint over PSF ? [0=NO, 1=YES]
      strehl          : strehl,  $ ; Strehl constraint initializated to 0
      psf_id_filename : path,    $ ; IDEAL PSF filename 
      psf_init        : 0,       $ ; First guess of the PSF : 
                                   ; 0 = Autocorrelation of the Ideal PSF
                                   ; 1 = Ideal PSF + a suitable constant
                                   ;     (in order to satisfy strehl
                                   ;      constraints)
                                   ; 2 = Ideal PSF blurred with a gaussian 
                                   ; 3 = Autocorr of the input image
                                   ; 4 = User-Defined PSF 
      psf_userdefined : path,    $ ; user defined PSF filename
      sgp_verbosity   : 0B,      $ ; Verbosity? (0=NO, 1=YES)
      ok_image        : 0B,      $ ; initial image ? [yes=1B, no=0B]
      ok_psf          : 0B,      $ ; initial psf ? [yes=1B, no=0B]
      ok_psf_start    : 0B,      $ ; start PSF deconvolution from the provided estimation ?
      filename        : path,    $ ; initial image file address
      filename2       : path,    $ ; initial psf image file address
      ok_filter       : 0L,      $ ; filter to apply on the recontructed PSF and images [0=NO, 1=YES]
      who_filter      : 0L,      $ ; who will be filtered ?
                                   ; [0=filter only the rec'd PSFs, 1=both PSFs and images] 
      max_ide         : 0.,      $ ; max of the ideal PSF
      ok_manage_filter: 0L,      $ ; filter/ideal psf/build lbt ideal psf and then filter
      filtername      : path,    $ ; filter/idpsf 
      fil             : 0L,      $ ; filter
      npixel          : 0L,      $ ; dimension of the image/filter
      d               : 8.25,    $ ; diameter of the pupil
                                   ; (requested for the construction of the filter)            
      lambda          : 2.2e-6,  $ ; central wavelenght of the acquisition band
                                   ; (retrieved from the parameters of the image)
      width           : 4.0e-7,  $ ; bandwidth of the acquisition band
                                   ; (retrieved from the parameters of the image)
      pix             : .00512,  $ ; pixel size of the acquisition
                                ; (requested for the construction of the filter)
      baseline        : 14.4,    $ ; baseline (interferometer case)
      filtertype      : 0L,      $ ; filter type
      angles          : ' ',     $
      save_discrepancy: 0,        $ ; save the discrepancy values?
      save_obj_fun:0,      $        ; save the objective function values
      save_discrepancy_file : '', $ ; filename (ASCII file)
      save_obj_fun_file : '',     $ ; filename (ASCII file) 
      save_rho1       : 0,       $  ;save the restoration error of the REC'D PSF?
      save_rho1_file : '',       $  ; filename of restoration error (TXT or SAV file)
      load_psf_file  : '',       $  ; filename of the ground truth (PSF)
      save_rho2       : 0,       $  ;save the restoration error of the REC'D OBJ?
      save_rho2_file : '',       $  ; filename of restoration error (TXT or SAV file)
      load_obj_file  : '',       $  ; filename of the ground truth (OBJECT)
      err_type       : 0,        $  ; 0=diffuse object (error=RMSE)
                                    ; 1=point-like source (error=MARE)
      err_surf       : 104.,     $ ; telescope surface in m^2 
      err_time       : 1.,       $ ; integration time in sec
      err_lambda     : 2.2e-6,   $ ; central wavelenght in m
      err_width      : 0.4e-6,   $ ; bandwidth in m
      err_eff        : 0.3       $ ; overall efficiency
      }


; Save the default parameter structure
save, par, FILENAME=info.def_file

end
