; $Id: cbd_psf_dl.pro,v 1.3 last update: 2016/05/25, Andrea La Camera$
;+
; NAME:
;       cbd_psf_dl
;
; PURPOSE:
;       To create a generic diffraction limited PSF, based on the Airy pattern 
;       of a uniform circular aperture obscured by a circular block at the 
;       center. This situation is relevant to many common reflector telescope 
;       designs that incorporate a secondary mirror.
;
; CATEGORY:
;       utility
;
; CALLING SEQUENCE:
;       cbd_psf_dl 
;
; INPUTS:
;       none.
;
; OPTIONAL INPUTS:
;       none.
;
; KEYWORD PARAMETERS:
;       none.
;
; OUTPUT:
;      the PSF array, saved in a FITS file
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       none.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
;
; MODIFICATION HISTORY:
;- 2014/01/20, Andrea La Camera:
;             Simulation details added within the FITS header
;- 2015/01/15, Andrea La Camera:
;             Bands extended (U,L,M bands added)
; -2015/02/03, Andrea La Camera:
;            - Added Phase error, between 0 and 2*pi. The fringes may be not centered
;              on the maximum of the Airy disk....
;            - Added visualization of the result (optional)
; -2015/03/13, Andrea La Camera:
;            - Bug corrections.
; -2016/05/25, Andrea La Camera:
;           - adapted to CBD module ;)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro cbd_psf_dl_int, b, d, eps, lambda, gam, pix_scale, nb_pix, psf_i, INTERF=interf, PHASE = phase
   V=1.
   radsec_conv=180.*60.*60./!dpi   ; = 206264.8 arcsec/rad
   gam=gam/180.*!dpi
   inter_1=!dpi*pix_scale*(d/lambda)/radsec_conv
   inter_2=2*!dpi*pix_scale*(b/lambda)/radsec_conv
   print,'INTERF_1 = ', inter_1
   print,'INTERF_2 = ', inter_2
   airy=dblarr(nb_pix,nb_pix)
   x=((dindgen(nb_pix)) # (dblarr(nb_pix)+1))-nb_pix/2
   y=((dblarr(nb_pix)+1) # (dindgen(nb_pix)))-nb_pix/2
   r=sqrt(x^2+y^2)
   airy=(2.*beselj(inter_1*r,1, /double)/(inter_1*r) - $
         2.*eps*beselj(inter_1*r*eps,1, /double)/(inter_1*r))^2.
   airy=airy/(1-eps^2)^2
   airy[nb_pix/2,nb_pix/2]=1.
   phase_term=inter_2*((x*cos(gam))+(y*sin(gam)))
   
   if KEYWORD_SET(phase) then psi = phase else psi = 0.
   interf_term=1+(V*cos(phase_term+psi))
   if KEYWORD_SET(interf) then psf_i=airy*interf_term $
                          else psf_i=airy
   psf_i=psf_i/total(psf_i, /double)
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro cbd_psf_dl_set, dl_state
COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band


if (dl_state.par.interf eq 0) then begin
  widget_control, dl_state.id.baseline, SENSITIVE=0
  widget_control, dl_state.id.val_ang, SENSITIVE = 0
  widget_control, dl_state.id.reset_val_ang, SENSITIVE = 0
  widget_control, dl_state.id.phase, SENSITIVE = 0
endif else begin
  widget_control, dl_state.id.baseline, /SENSITIVE
  widget_control, dl_state.id.val_ang, /SENSITIVE
  widget_control, dl_state.id.reset_val_ang, /SENSITIVE
  widget_control, dl_state.id.phase, /SENSITIVE 
endelse

;;Indicating in GUI which bands have been selected
;;------------------------------------------------

dummy  = CLOSEST(dl_state.par.lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)

IF ((dl_state.par.lambda NE 0) AND (dummy GT -1)) THEN BEGIN
   IF (((dl_state.par.lambda - lambda_tab[dummy]) LT 1e-12)) THEN BEGIN
      flag_band     = 0B           ;Indicates a single band!!
      value[dummy]  = 1
      f_band[dummy] = 1.
   ENDIF ELSE BEGIN
      flag_band = 1B               ;Indicates wavelength range spans over more than 1 band
      lambda1   = dl_state.par.lambda ;- dl_state.par.width/2
      lambda2   = dl_state.par.lambda ;+ dl_state.par.width/2
      FOR i=0,n_band-1 DO BEGIN
         band1 = lambda_tab[i]-width_tab[i]/2.
         band2 = lambda_tab[i]+width_tab[i]/2.
         f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
         lambda1 = d1
         lambda2 = d2
      ENDFOR
      r1 = WHERE(f_band GT 0., c1)
      IF (c1 GT 0) THEN $
        value[r1] = 1.  $
      ELSE              $
        MESSAGE,'Bandwidth selected not within the standard bands. Check!!',/INFO
   ENDELSE

ENDIF

WIDGET_CONTROL,dl_state.id.band  ,SET_VALUE= value

end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cbd_psf_dl_event, event

COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band

WIDGET_CONTROL, event.ID, GET_UVALUE=choice
WIDGET_CONTROL, event.TOP, GET_UVALUE=dl_state
; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

case choice of
  'diameter': dl_state.par.diameter = event.value
  'epsilon': begin
     dl_state.par.epsilon = event.value
     if (dl_state.par.epsilon LT 0.) OR (dl_state.par.epsilon GE 1.) then begin
     a=dialog_message("The ratio must be in the range 0≤eps<1", /WARNING)
     endif
  end 
  'pix_scale': dl_state.par.pix_scale = event.value
  'n_pix' : dl_state.par.n_pix = event.value
  'lambda'       : dl_state.par.lambda  = event.value
  'menu_band': BEGIN
       WIDGET_CONTROL, event.id , GET_VALUE=dummy
       r1 = WHERE(dummy GT 0, c1)
       IF (c1 GT 0) THEN BEGIN
          min_lambda = MIN(lambda_tab[r1]);-width_tab[r1]/2.)
          max_lambda = MAX(lambda_tab[r1]);+width_tab[r1]/2.)
          lambda = (max_lambda+min_lambda)/2
          dl_state.par.lambda= lambda
       ENDIF ELSE BEGIN
          dl_state.par.lambda= 0.
       ENDELSE
       WIDGET_CONTROL, dl_state.id.lambda, SET_VALUE=dl_state.par.lambda
       cbd_psf_dl_set, dl_state
    END
  'reset':BEGIN
       dl_state.par.lambda = 0.
       WIDGET_CONTROL,dl_state.id.lambda, SET_VALUE=dl_state.par.lambda
       cbd_psf_dl_set, dl_state
   END
   'filename' : dl_state.par.filename = event.value
   'interf' :  begin
       widget_control, dl_state.id.interf, GET_VALUE=dummy
       dl_state.par.interf=dummy
       cbd_psf_dl_set,dl_state
   end
   'baseline' : dl_state.par.baseline = event.value
   'reset_val_ang' : begin
       dl_state.par.angle[*]=!VALUES.F_NAN
       widget_control, dl_state.id.val_ang, SET_Value=transpose(dl_state.par.angle)
   end
   'val_ang'  : begin
       dummyx = event.x
       dummyy = event.y
       widget_control, dl_state.id.val_ang, get_value=dummy
       dl_state.par.angle[dummyy]=dummy[dummyx,dummyy]
  end
  'phase' : begin
      dl_state.par.phase = event.value
  end
  'show': dl_state.par.show = event.select
  'save_par': begin
         a=dialog_pickfile(dialog_parent=event.top, FILTER=['*.sav'],/WRITE)
         if a EQ '' then return
         param=dl_state.par
         save, param, FILENAME=a, /VERBOSE
  end
  'load_par': begin
         a=dialog_pickfile(dialog_parent=event.top, FILTER=['*.sav'], $
                           /MUST_EXIST, /READ)
         if a EQ '' then return
         restore, a, /VERBOSE
         ;restore all parameters and set the gui::: Read structure is "PARAM"
         dl_state.par=param
         widget_control, dl_state.id.diameter, set_value=dl_state.par.diameter
         widget_control, dl_state.id.pix_scale, set_value=dl_state.par.pix_scale
         widget_control, dl_state.id.n_pix, set_value=dl_state.par.n_pix
         widget_control, dl_state.id.lambda, set_value=dl_state.par.lambda
         widget_control, dl_state.id.filename, set_value=dl_state.par.filename
         widget_control, dl_state.id.interf, set_value=dl_state.par.interf
         widget_control, dl_state.id.baseline, set_value=dl_state.par.baseline
         widget_control, dl_state.id.val_ang, set_value=transpose(dl_state.par.angle)
         widget_control, dl_state.id.phase, set_value = dl_state.par.phase
         cbd_psf_dl_set,dl_state
  end        
  'help' : begin
         a=dialog_message("NOT YET IMPLEMENTED", /INFO, dialog_parent=event.top)
         ;XDISPLAYFILE, nomefile, DONE_BUTTON="Done", Title="TECNO_INAF Help"
  end
  'web'  : begin
         a=dialog_message("NOT YET IMPLEMENTED", /INFO, dialog_parent=event.top)
  ;online_help, book="website.html"
      end
   'about' : a=dialog_message(["                   CBD_PSF_DL                 ", $
                               "Monochromatic, Diffraction-Limited PSF creator", $
                               "           written by Andrea La Camera        ", $
                               "       (DIBRIS, University of Genoa, Italy)   ", $
                               "                   May 2016                   "], $
                               /INFO, title="About this software", $
                               dialog_parent=event.top)

  'run' : begin
         print, "PLEASE, WAIT..."
         d = dl_state.par.diameter               ;Diameter of one pupil [m]
         eps = dl_state.par.epsilon       ; Obscuration / aperture ratio (0≤eps<1)
         pix_scale = dl_state.par.pix_scale       ;Detector pixel size [arcsec]   
         lambda = dl_state.par.lambda        ;Central filter wavelenght [m]
         nb_pix = dl_state.par.n_pix           ;Ingoing size array [pixels]
         if dl_state.par.interf then begin 
            b = dl_state.par.baseline               ;Baseline of the interferometer [m]
            index = WHERE(FINITE(dl_state.par.angle), nb_angles) ;Number of equidistant parallactic angles
            gam = dl_state.par.angle[index] ; Array containing the parallactic angles (degrees)
            ;print, nb_angles
            ;print, gam 
         endif else begin
            b=0.
            nb_angles=1
            gam=0.
         endelse
         psf0 = make_array(nb_pix,nb_pix, /double, value=0.)
         psf = make_array(nb_pix,nb_pix,nb_angles, /double, value=0.)
         for j=0,nb_angles-1 do begin
            cbd_psf_dl_int, b, d, eps,lambda, gam[j], pix_scale, nb_pix, psf0,$
                        interf=dl_state.par.interf, phase = dl_state.par.phase
            psf[*,*,j]=psf0
         endfor
         if (dl_state.par.filename NE '') then begin
            fits_write, dl_state.par.filename, psf
            a=readfits(dl_state.par.filename, header)
            fxaddpar, header, 'D       ',dl_state.par.diameter, "Diameter of the telescope [m]"
            fxaddpar, header, 'Obs     ',dl_state.par.epsilon,  "Obscuration / aperture ratio (0≤eps<1)"
            fxaddpar, header, 'Scale   ',dl_state.par.pix_scale,"Detector pixel size [arcsec]"
            fxaddpar, header, 'Lambda  ',dl_state.par.lambda,   "Central filter wavelenght [m]"
            fxaddpar, header, 'N_pix   ',dl_state.par.n_pix,    "Size array [pixels]"
            if dl_state.par.interf then begin 
               fxaddpar, header, 'Interf  ',dl_state.par.interf,   "Interferometric PSF"
               fxaddpar, header, 'B       ',dl_state.par.baseline,"Baseline of the interferometer [m]"
               index = WHERE(FINITE(dl_state.par.angle), nb_angles) ;Number of equidistant parallactic angles
               gam = dl_state.par.angle[index] ; Array containing the parallactic angles (degrees)
               for k=0, nb_angles-1 do fxaddpar, header, 'Angle#'+strtrim(k,1), gam[k], "Baseline angle"
            endif else fxaddpar, header, 'Single  ',dl_state.par.interf,   "Monolitic PSF"
            writefits, dl_state.par.filename, psf, header
         endif 
         if (dl_state.par.show) then begin
            XSI = 512
            YSI = 512
            for j=0,nb_angles-1 do begin
               window, j, xs = XSI, ys = YSI, title = 'ANGLE # '+strtrim(gam[j])
               tvscl, congrid(psf[*,*,j],XSI,YSI)
            endfor
         endif
         print, "...DONE!"
  end
  'exit': begin
      widget_control, event.top, /DESTROY
      return
      end
else: return
endcase ;end of main CASE 

; write the GUI dl_state structure
WIDGET_CONTROL, event.top, SET_UVALUE=dl_state


end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro cbd_psf_dl

COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band


id = { baseline:0L, diameter:0L, epsilon:0L, pix_scale:0L, $ 
       val_ang:0L, reset_val_ang:0L, $
       lambda:0L, band:0L, reset:0L, $
       n_pix:0L, filename:0L, interf:0L, $
       phase:0L, show:0L, run:0L }

angle=make_array(360,value=!VALUES.F_NAN)
par  =  {baseline:14.4, diameter:8.25, epsilon:0.11, $
         pix_scale:0.005, lambda:2.2e-6, $
         n_pix:256, angle:angle, filename:'', $ 
         interf:0, phase:0., show : 0B}

dl_state = {id:id, par:par}

;modal = n_elements(group) ne 0
base_id = widget_base(TITLE="Graphical User Interface", /COL, MBAR=bar)
menu1=WIDGET_BUTTON(bar, value="File", /MENU)
menu1b1=WIDGET_BUTTON(menu1, value="Load parameters...", uvalue="load_par")
menu1b2=WIDGET_BUTTON(menu1, value="Save parameters...", uvalue="save_par")
menu1b3=WIDGET_BUTTON(menu1, value="Exit", uvalue="exit")

menu0=WIDGET_BUTTON(bar, value="Help", /MENU)
menu0b1=WIDGET_BUTTON(menu0, value="Help", uvalue="help")
menu0b2=WIDGET_BUTTON(menu0, value="Website", uvalue="web")
menu0b3=WIDGET_BUTTON(menu0, value="About", uvalue="about")

dummy=widget_label(base_id, value="Monochromatic, diffraction-limited PSF", $
      /ALIGN_CENTER)
;;;;;;;;;;; LEFT
baseA=WIDGET_BASE(base_id, /ROW, /ALIGN_LEFT)
base_left=WIDGET_BASE(baseA, /COL, /ALIGN_LEFT,/FRAME)
dl_state.id.diameter = cw_field(base_left, TITLE= 'Diameter of telescope [m]  ', $
                         /FLOATING,  VALUE=dl_state.par.diameter,      $
                          UVALUE='diameter', /ALL_EVENTS )
dl_state.id.epsilon = cw_field(base_left, TITLE=  'Obscuration/aperture ratio ', $
                         /FLOATING,  VALUE=dl_state.par.epsilon,      $
                          UVALUE='epsilon', /ALL_EVENTS )
dl_state.id.pix_scale = cw_field(base_left, TITLE='Pixel-scale [arcsec]       ', $
                         /FLOATING,  VALUE=dl_state.par.pix_scale,      $
                          UVALUE='pix_scale', /ALL_EVENTS )
dl_state.id.n_pix = cw_field(base_left, TITLE=    'Number of pixel            ', $
                         /INTEGER,  VALUE=dl_state.par.n_pix,      $
                          UVALUE='n_pix', /ALL_EVENTS )

bandbase  = widget_base(base_left, /COL)
dummy     = WIDGET_LABEL(bandbase,VALUE='Wavelength band')
bandstuff = widget_base(bandbase, /COL)
; lambda
dl_state.id.lambda           = cw_field(bandstuff,                      $
                                    TITLE='Center wavelength [m] ', $
                                    /FLOATING,                      $
                                    VALUE=dl_state.par.lambda,         $
                                    UVALUE='lambda',                $
                                    /ALL_EVENTS                     )

; width

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Determining which bands have been chosen
;=> If a band is partially selected then it appears as having been chosen,
;   unless lambda AND width coincide exactly with values in band_tab

dummy     = N_PHOT(1.,BAND=dummy1,LAMBDA=dummy2,WIDTH=dummy3)
band_tab  = dummy1[0:9]
lambda_tab= dummy2[0:9]
width_tab = dummy3[0:9]


det_base2= WIDGET_BASE(bandbase, /ROW, /BASE_ALIGN_CENTER)
dl_state.id.reset = WIDGET_BUTTON(det_base2,UVALUE='reset',VALUE='Reset Bands')

dummy  = CLOSEST(dl_state.par.lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)

IF (((dl_state.par.lambda - lambda_tab[dummy]) LT 1e-12)) THEN BEGIN

   flag_band     = 0B                        ;Indicates a single band!!
   value[dummy]  = 1
   f_band[dummy] = 1.
   dl_state.id.band =                                                             $
     CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='menu_band', $
               BUTTON_UVALUE= INDGEN(n_band),/NONEX)

ENDIF ELSE BEGIN                             ;The selected wavelength range
                                             ;spans over two or more bands

   flag_band = 1B

   lambda1 = dl_state.par.lambda ;- dl_state.par.width/2
   lambda2 = dl_state.par.lambda ;+ dl_state.par.width/2

   FOR i=0,n_band-1 DO BEGIN
      band1 = lambda_tab[i]-width_tab[i]/2.
      band2 = lambda_tab[i]+width_tab[i]/2.
      f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
      lambda1 = d1
      lambda2 = d2
   ENDFOR

   r1 = WHERE(f_band GT 0., c1)

   IF (c1 GT 0) THEN BEGIN
      value[r1] = 1.
      dl_state.id.band =                                                            $
        CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='menu_band',$
                  BUTTON_UVALUE= INDGEN(n_band),/NONEXCL)
   ENDIF ELSE BEGIN
      MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE

ENDELSE

dl_state.id.filename = cw_filename(base_left,TITLE="Save as... ",  $
                                VALUE=dl_state.par.filename,UVALUE="filename",$
                                /ALL_EVENTS  )



;;;;;;;;;;; RIGHT
base_right=WIDGET_BASE(baseA, /COL, /ALIGN_LEFT, /FRAME)
dl_state.id.interf = cw_bgroup(base_right, $
    'Interferometric PSF',$
    SET_VALUE=dl_state.par.interf, COLUMN=2, UVALUE='interf', /NONEXCLUSIVE)

dl_state.id.baseline = cw_field(base_right, TITLE='Baseline (center-to-center distance) [m] ', $
                         /FLOATING,  VALUE=dl_state.par.baseline,      $
                          UVALUE='baseline', /ALL_EVENTS, /COL )

ang_table=widget_base(base_right,xsize=300,ysize=160)
rlabel=make_array(360,/string,value='Angle #')
num=indgen(360)
rlabel=rlabel+strcompress(string(num),/remove_all)

button_ang=widget_base(ang_table,xoffset=38,yoffset=18,xsize=64,ysize=26)
dl_state.id.reset_val_ang=widget_button(button_ang,$
                VALUE="Reset Val", $
                UVALUE="reset_val_ang")

dl_state.id.val_ang = widget_table(ang_table,$
                ALIGNMENT=2,$
                xoffset=35,yoffset=15,$
                value=transpose(dl_state.par.angle),$
                uvalue="val_ang",$
                editable=1,$
                COLUMN_LABELS=['Baseline angles (DEG)'],$
                ROW_LABELS=rlabel,$
                COLUMN_WIDTHS=150,$
                y_scroll_size=5   )	  

dl_state.id.phase = cw_field(base_right, TITLE='Phase error (0 ... 2*pi)', $
                         /FLOAT,  VALUE=dl_state.par.phase,      $
                          UVALUE='phase', /ALL_EVENTS )


;;;;;;;;;;; BOTTOM (1)
base_options = WIDGET_BASE(base_id, /ALIGN_LEFT)
dl_state.id.show = CW_BGROUP(base_options, ['Show output'], $
                          /ROW, SET_VALUE=dl_state.par.show, $
                          UVALUE='show',$
                          /NONEXCL)

base_run = WIDGET_BASE(base_id, /ALIGN_CENTER)
dl_state.id.run = WIDGET_BUTTON(base_run, value="  RUN  ", uvalue="run",xsize=200,ysize=50)


;;;;;;;;;;; BOTTOM (2)

base2  = WIDGET_BASE(base_id, FRAME=1, /ROW, /ALIGN_LEFT)
help_id = WIDGET_BUTTON(base2, VALUE='HELP',UVALUE='help')
cancel_id  = WIDGET_BUTTON(base2, VALUE='EXIT',UVALUE='exit')
;restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
;save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS',UVALUE='save')

WIDGET_CONTROL, cancel_id, /CANCEL_BUTTON
;WIDGET_CONTROL, save_id, /DEFAULT_BUTTON
WIDGET_CONTROL, base_id, /REALIZE
WIDGET_CONTROL, base_id, SET_UVALUE=dl_state
cbd_psf_dl_set,dl_state
XMANAGER, 'cbd_psf_dl', base_id, GROUP_LEADER=group

end
