; $Id: cbd_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       cbd_init
;
; PURPOSE:
;       cbd_init executes the initialization for the CBD module.
;
; CATEGORY:
;       initialization program
;
; CALLING SEQUENCE:
;    error = cbd_init(inp_img_t2,  $ ; img_t input structure
;                     out_psf_t ,  $ ; img_t output structure (PSF)
;                     out_img_t ,  $ ; img_t output structure (OBJ)
;                     par,         $ ; parameters structure
;                     INIT=init    $ ; initialization data structure
;                     )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see cbd.pro's help for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: december 2008, (based on the existing DEC module)
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]
;                   : december 2008,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -variable "time_integ" set to 0 in the output structures
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -output's order exchanged: (WAS) top=PSF, bottom=OBJ;
;                     (NOW IS) top=OBJ, bottom=PSF, as it is in the whole 
;                     Soft.Pack.AIRY
;                   : February 2014, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -several optional outputs added.  
;                   : January 2015,
;                    -Add another method of Blind Deconvolution, based on the 
;                     paper cited in the file "AIRYSoftPack6.1_From6.0to6.1.txt"
;                    -gathered all old stuff in an IF block and the
;                     new stuff in the following ELSE block.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -added optional output save: objective function
;                    -background is now an array
;                    -the header is saved in the output structure and
;                     eventually modified with the deconvolution parameters.
;                   : February 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    - cbd_init.pro debugged
;-
function cbd_init, inp_img_t2, out_psf_t, out_img_t, par, INIT=init

common caos_block, tot_iter, this_iter

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = cbd_info()

;save_init    = 0B
;restore_init = 0B

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)  
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'cbd error: par must be a structure'
if n ne 1 then message, 'cbd error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module cbd'

dummy = test_type(inp_img_t2, TYPE=type)
if type eq 0 then begin                ; undefined variable

   inp_img_t2 = $
      {        $
    data_type  : out_type[0]     , $
    data_status: !caos_data.valid $
      }
endif

if test_type(inp_img_t2, /STRUC, N_EL=n, TYPE=type) then $
   message, 'cbd error: wrong definition for the input.'

if n ne 1 then message, $
   'cbd error: input cannot be a vector of structures'

; test the data type
if inp_img_t2.data_type ne inp_type[0] then                $
   message, 'wrong input data type: '+inp_img_t.data_type $
           +' ('+inp_type[1]+' expected).'
;if inp_img_t2.data_status eq !caos_data.not_valid and not inp_opt[1] then $
;   message, 'undefined input is not allowed'


pos=where(inp_img_t2.time_integ[*,1] gt 1,num)
if ((size(inp_img_t2.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_img_t2.image=reform(inp_img_t2.image,(size(inp_img_t2.image))[1],$
                              (size(inp_img_t2.image))[2],(size(inp_img_t2.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE (MULTI-FRAME input is not supported for blind-deconvolution)',/ERROR)
      message,'WRONG INPUT TYPE (MULTI-FRAME input is not supported for blind-deconvolution)'
   endelse
endif

;read from header the RON and GAIN values                              
sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)


if (par.cbd_method EQ 0) then begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; METHOD CBD A) DESIDERA ET AL 2009  ;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
                                ;Check on the PSF (BLIND or MYOPIC ?)
   if par.ok_psf eq 1 then begin 
                                ; Upload of the PSF estimation
      fits_read,par.filename2,psf
      if (size(psf))[0] eq 2 then P2=1 else P2=(size(psf))[3]       
      if (size(inp_img_t2.image))[0] EQ 2 then P=1 else P=(size(inp_img_t2.image))[3]
      N=(size(inp_img_t2.image))[1]
      N2=(size(psf))[1]
      if P NE P2 then message, $
         'WRONG INPUT SIZE: images and PSFs do not match in number.'
      if N2 GT N then message, $
         "WRONG INPUT SIZE: provided PSF is larger than input image"
                                ; "EQUALIZATION" of the two arrays 
      if (N2 LT N) then begin
         print, 'Zero-padding of provided PSF -> new size: '+$
                strtrim(N,1)+'x'+strtrim(N,1)+'x'+strtrim(P,1)
         psf=zeropadding(psf,N,N,P) ;zeropadding is able to manage 3d arrays
      endif
   endif else begin
                                ; Calculation of the G-autocorrelation as PSF estimation 
      if (size(inp_img_t2.image))[0] eq 2 then P=1 else P=(size(inp_img_t2.image))[3]  
      N=(size(inp_img_t2.image))[1]
      psf = make_array(N,N,P,/double)
      for j=0,P-1 do begin
         tfg=fft(inp_img_t2.image[*,*,j]-inp_img_t2.background[*,*,j],-1,/double)*N*N
         tfrgg=conj(tfg)*tfg
         rgg=fft(tfrgg,1,/double)/(N*N)
         rgg= rgg-min(rgg)+((max(rgg)-min(rgg))/1000.)
         rgg=rgg/total(rgg)
         rgg=float(shift(rgg,N/2,N/2))
         psf[*,*,j]=rgg
      endfor
   endelse
                                ; "positivization" of PSF
   for j=0, P-1 do begin
      temp=positivization(psf[*,*,j])
      temp=temp/total(temp,/DOUBLE)
      psf[*,*,j]=temp
   endfor 
                                ;"positivization" of input2 (IMAGE) and flux evaluation --> c
   c=0.
   cc=0.
   for j=0,P-1 do begin
      c=c+total((inp_img_t2.image[*,*,j] - inp_img_t2.background[*,*,j]), /DOUBLE)
      cc=cc+total(inp_img_t2.image[*,*,j], /DOUBLE)
                                ; POISSON + GAUSSIAN NOISE: Add SIGMA^2 to image and background:
      inp_img_t2.image[*,*,j]=inp_img_t2.image[*,*,j]+(sigma_ron/gain_ron)^2.
      inp_img_t2.background[*,*,j]=inp_img_t2.background[*,*,j]+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image
      temp=positivization(inp_img_t2.image[*,*,j]) ;do not subtract the background here!
      inp_img_t2.image[*,*,j]=temp
   endfor
   c=c/float(P)  


   
   if (par.ok_filter ne 0) then begin
      case par.ok_manage_filter of
         0 : begin
            fits_read,par.filtername,fil
         end
         1 : begin
            fil=create_filter(par)
         end        
      endcase
   endif else begin
      fil=0
   endelse

   if (par.ok_strehl eq 1) then begin
      tf_gaussian=create_tf_gaussian(N,N)
   endif else begin
      tf_gaussian=0
   endelse
   
   upp_bound=0

endif else begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; METHOD CBD B) PRATO ET AL          ;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                                ;some initialiazations needed 
                                ;(and not fixed in the GUI, for this version!)

   SGP_init=2                   ; SGP initialization 
                                ; 0  - all zero starting point
                                ; 1  - initialization with gn
                                ; 2  - initialization with
                                ;      ones(size(gn))*sum(gn(:) - bg) / numel(gn)
                                ; 3  - user-provided starting point (double array)

   inp_img_t2.image = double(inp_img_t2.image)+(sigma_ron/gain_ron)^2. ; gaussian noise adjustment
   inp_img_t2.background = double(inp_img_t2.background)+(sigma_ron/gain_ron)^2. ; gaussian noise adjustment
   if (size(inp_img_t2.image))[0] EQ 2 then P=1 else P=(size(inp_img_t2.image))[3]
   N=(size(inp_img_t2.image))[1]
   
   c = total(total(inp_img_t2.image - inp_img_t2.background,1),1)
   
   help, c 
; sistemare questa costante (se non serve, cancellare!!!)
   dim = size(inp_img_t2.image,/dimension)
   if n_elements(dim) eq 2 then begin
      num = 1
   endif else begin
      num = dim(2)
   endelse
   
; commento ALC 2016/02/24 
; par.strehl e' un array ma il codice blind  di marco
; per ora accetta un solo valore. fino a che non verra' sistemato 
; prendiamo la media dei vari valori inseriti...
   upp_bound = par.max_ide * mean(par.strehl[0:num-1]) 
  
   ;load the IDEAL PSF (if needed)
    
   initial_psf = (["I","C","B","A","P"])[par.psf_init]
   if par.psf_init LE 2 then begin
      if file_test(par.psf_id_filename, /READ, /REGULAR) then begin
         psf_id=readfits(par.psf_id_filename)
                                ; Check for correct dimensions
         dim_psf = size(psf_id,/dimension)
         if (array_equal(dim_psf,dim) EQ 0) then message, $
            'The size of the PSF must be equal to the size of the image'
      endif else message, "ERROR while loading the file "+par.psf_id_filename
   endif       
   case initial_psf of 
      'A' : begin               ; Autocorrelation of the image
         psf = dblarr(dim)
         for ii = 0, num - 1 do begin
            psf0 = real_part(fft(fft(reform(inp_img_t2.image[*,*,ii]))*conj(fft(shift(reform(inp_img_t2.image[*,*,ii]),dim[0:1]/2))),/inverse))*dim[0]*dim[1]
            while (max(psf0) GT upp_bound) do begin
               psf0 = real_part(fft(fft(psf0)*conj(fft(shift(psf0,dim[0:1]/2))),/inverse))*dim[0]*dim[1]
            endwhile
            psf[*,*,ii] = psf0/total(psf0,/double)
         endfor
         PRINT, "MAX PSF is ", max(psf)
         PRINT, "corresponding to a SR of ", max(psf)/par.max_ide
         PRINT, "input strehl ratio = ", par.strehl[0:num-1]
         if max(psf) GT upp_bound then begin
            print, "WARNING! User-defined PSF does not satisfy SR constraint... "
            wait, 5
         endif
      end
      'B': begin                ; Ideal psf blurred with a Gaussian
         psf = dblarr(dim)
         x = (dindgen(dim[0])) # (dblarr(dim[0]) + 1)
         y = (dblarr(dim[1]) + 1) # (dindgen(dim[1]))
         r2 = (x - dim[0]/2)^2. + (y - dim[1]/2)^2.
         gamma_ini = 1.e-5
         gamma_step = 1.e-6
         for ii = 0, num - 1 do begin
            gamma = gamma_ini
            MTF = shift(fft(shift(psf_id[*,*,ii],dim[0]/2,dim[1]/2),-1,/double),$
                        dim[0]/2,dim[1]/2)*dim[0]*dim[1]
            while (max(psf0) GT upp_bound) do begin
               MTF = MTF*exp(-gamma*r2)
               psf0 = double(SHIFT(FFT(SHIFT(MTF,dim[0]/2,dim[1]/2),+1,/double),$
                                   dim[0]/2,dim[1]/2))/dim[0]/dim[1]
               gamma = gamma + gamma_step
            endwhile
            psf[*,*,ii] = psf0/total(psf0,/double)
         endfor
         PRINT, "MAX PSF is ", max(psf)
         PRINT, "corresponding to a SR of ", max(psf)/par.max_ide
         PRINT, "input strehl ratio = ", par.strehl[0:num-1]
         if max(psf) GT upp_bound then begin
            print, "WARNING! User-defined PSF does not satisfy SR constraint... "
            wait, 5
         endif
      end
      'C': begin                ; Ideal PSF plus a suitable constant
         psf = dblarr(dim)
         for ii = 0, num - 1 do begin
            b = (upp_bound[ii] -  max(psf_id[*,*,ii]))/(1. - dim[0]*dim[1]*upp_bound)
            psf0 = psf_id[*,*,ii] + b
            psf[*,*,ii] = psf0/total(psf0,/double)
         endfor
         PRINT, "MAX PSF is ", max(psf)
         PRINT, "corresponding to a SR of ", max(psf)/par.max_ide
         PRINT, "input strehl ratio = ", par.strehl[0:num-1]
         if max(psf) GT upp_bound then begin
            print, "WARNING! User-defined PSF does not satisfy SR constraint... "
            wait, 5
         endif
      end
      'I': begin                ; Autocorrelation of the Ideal PSF
         psf = dblarr(dim)
         for ii = 0, num - 1 do begin
            psf0 = real_part(fft(fft(reform(psf_id[*,*,ii]))*conj(fft(shift(reform(psf_id[*,*,ii]),dim[0:1]/2))),/inverse))*dim[0]*dim[1]
            while (max(psf0) GT upp_bound) do begin
               psf0 = real_part(fft(fft(psf0)*conj(fft(shift(psf0,dim[0:1]/2))),/inverse))*dim[0]*dim[1]
            endwhile
            psf[*,*,ii] = psf0/total(psf0,/double)
         endfor
         PRINT, "MAX PSF is ", max(psf)
         PRINT, "corresponding to a SR of ", max(psf)/par.max_ide
         PRINT, "input strehl ratio = ", par.strehl[0:num-1]
         if max(psf) GT upp_bound then begin
            print, "WARNING! User-defined PSF does not satisfy SR constraint... "
            wait, 5
         endif
         end
      'P': begin                ;user-defined Psf 
                                ;user-defined PSF is taken "as is" 
         if file_test(par.psf_userdefined, /READ, /REGULAR) then begin
            psf=readfits(par.psf_userdefined)
                                ; Check for correct dimensions
            dim_psf = size(psf,/dimension)
            if (array_equal(dim_psf,dim) EQ 0) then message, $
               'The size of the PSF must be equal to the size of the image'
         endif else message, "ERROR while loading the file "+par.psf_userdefined
                                ;check wheter the user-defined psf satisfies strehl constraint
         PRINT, "MAX PSF is ", max(psf)
         PRINT, "corresponding to a SR of ", max(psf)/par.max_ide
         PRINT, "input strehl ratio = ", par.strehl[0:num-1]
         if max(psf) GT upp_bound then begin
            print, "WARNING! User-defined PSF does not satisfy SR constraint... "
            wait, 5
         endif        
      end
      else: message, 'Please specify a correct initialization for the PSF'       
   endcase

   fil=0  ;for compatibility with "Desidera et al" part
   tf_gaussian=0 
   
endelse


;SHARED CODE OF THE 2 METHODS
;open the files for optional outputs
unit_d=0
discr=0
unit_o=0
obj_fun=0
unit_r1=0
rho1=0
true_psf=0
psf_norm2=0
unit_r2=0
rho2=0
true_obj=0
obj_norm2=0
obj_nz=0
ind_obj_nz=0
obj_magn=0
magn_par=0

if par.save_discrepancy then begin
   openw, unit_d, par.save_discrepancy_file, /GET_LUN
   if (par.cbd_method EQ 0) then discr= make_array(tot_iter, /double) $
   else discr= make_array(tot_iter+1, /double) ; in the case of method B) 
                                ; Prato et al. Discrepancy is computed once more
endif
if par.save_obj_fun then begin
   openw, unit_o, par.save_obj_fun_file, /GET_LUN
   if (par.cbd_method EQ 0) then obj_fun= make_array(tot_iter, /double) $
   else obj_fun= make_array(tot_iter+1, /double) ; in the case of method B) 
                                ; Prato et al. Discrepancy is computed once more
endif

if (par.save_rho1) then begin
   openw, unit_r1, par.save_rho1_file, /GET_LUN
   if file_test(par.load_psf_file, /READ, /REGULAR) then begin
      true_psf=double(readfits(par.load_psf_file))
                                ; Check normalization of the blurring operator A (PSF)
      Result = machar(/double)
      sumPSF = total(total(true_psf,1),1)
      checkPSF = abs(sumPSF-1.)
      size_PSF = size(checkPSF,/dimensions)
      if size_PSF eq 0 then begin
         tolCheckPSF = 1.e4*Result.eps
      endif else begin
         tolCheckPSF = make_array(size(checkPSF,/dimensions), value=1.e4*Result.eps)
      endelse
      if total(checkPSF gt tolCheckPSF) gt 0. then message, 'Not-normalized PSF: provide a normalized PSF'
   endif else message, "ERROR while loading the file "+par.load_psf_file
   if (size(true_psf))[0] eq 2 then TP=1 else TP=(size(true_psf))[3]          
   rho1 = make_array(tot_iter, TP, /double) ;one for each PSF ;)
endif

if (par.save_rho2) then begin
   openw, unit_r2, par.save_rho2_file, /GET_LUN
   if file_test(par.load_obj_file, /READ, /REGULAR) then begin
      true_obj=double(readfits(par.load_obj_file))
   endif else message, "ERROR while loading the file "+par.load_obj_file
   obj_size=(size(true_obj))[1]
   if (par.cbd_method EQ 0) then rho2 = make_array(tot_iter, /double) $ 
   else rho2 = make_array(tot_iter+1, /double) ; in the case of methot B) 
                                ;Prato et al Error is computed once more 
   if (par.err_type EQ 1B) then begin
      obj_nz = where(true_obj gt 0)
      nnz = n_elements(obj_nz)
      ind_obj_nz = dblarr(nnz,9)
      obj_magn = dblarr(nnz)
      for k=0,nnz-1 do begin
         ind_obj_nz[k,*] = [obj_nz[k]-1-obj_size, obj_nz[k]-obj_size, obj_nz[k]+1-obj_size, $
                            obj_nz[k]-1         , obj_nz[k]         , obj_nz[k]+1         , $
                            obj_nz[k]-1+obj_size, obj_nz[k]+obj_size, obj_nz[k]+1+obj_size]
         obj_magn[k] = cbd_magnitude(total(true_obj[ind_obj_nz[k,*]])/par.err_eff, $
                                     lambda=par.err_lambda, $
                                     width=par.err_width, delta_t = par.err_time, $
                                     surf = par.err_surf)
         magn_par = {lambda:par.err_lambda, width:par.err_width, delta_t:par.err_time, $
                    surf:par.err_surf, eff:par.err_eff}
      endfor
   endif
endif


; INIT structure itself

init =                         $
   {                           $
   psf          : psf,         $ ; initial psf -- from one of the two methods 
   upp_bound    : upp_bound,   $ ; up boundary for METHOD B) 
   filter       : fil,         $ ; filter to use in the PSF and/or image reconstruction
   tf_gaussian  : tf_gaussian, $ ; used by Strhel constraint
   c            : c,           $ ; normalization constant
   unit_d       : unit_d,      $ ; discr file unit
   discr        : discr,       $ ; discrepancy array
   unit_o       : unit_o,      $ ; obj_fun file unit
   obj_fun      : obj_fun,     $ ; objective function array
   unit_r1      : unit_r1,     $ ; save rho1 (PSF error) file unit
   rho1         : rho1,        $ ; array of errors (RMSE)
   true_psf     : true_psf,    $ ; PSF ground truth
   unit_r2      : unit_r2,     $ ; save rho2 (OBJ error) file unit
   rho2         : rho2,        $ ; array of errors (RMSE or MARE)
   true_obj     : true_obj,    $ ; OBJ ground truth
   obj_nz       : obj_nz,      $ ; index of star positions
   ind_obj_nz   : ind_obj_nz,  $ ; index of star positions (each one is 3x3)
   obj_magn     : obj_magn,    $ ; magnitude of each star
   magn_par     : magn_par     $ ; structure for magnitude computation
   }

; output structure initialisation

cube=make_array(N,N,1,/float,value=1.)
cube2=make_array(N,N,P,/float,value=1.)

out_img_t = 				   $
   {          				   $
   data_type  : out_type[0]     	 , $
   data_status: !caos_data.valid	 , $
   image      : cube            	 , $
   psf        : 0B              	 , $   ; This flag indicates that this is a IMAGE
   npixel     : N              	 , $           ; Number of pixels (per subaperture)
   resolution : inp_img_t2.resolution   , $    ; Pixel size [arcsec/pix]
   lambda     : inp_img_t2.lambda       , $    ; Mean detection wavelength [m]
   width      : inp_img_t2.width        , $    ; Wavelength band width [m]
   time_integ : inp_img_t2.time_integ   , $    ; Integration time for each image [s]
   time_delay : inp_img_t2.time_delay   , $    ; [NOT USED WITHIN AIRY]
   background : inp_img_t2.background   , $    ; background
   snr        : inp_img_t2.snr          , $    ; snr
   header     : PTR_NEW('')         $    ; header of the output object
   }
*out_img_t.header=*inp_img_t2.header

out_psf_t = 				   $
   {          				   $
   data_type  : out_type[0]            , $
   data_status: !caos_data.valid       , $
   image      : cube2                  , $
   psf        : 1B                     , $    ; This is a PSF
   npixel     : N                      , $    ; Number of pixels (per subaperture)
   resolution : inp_img_t2.resolution  , $    ; Pixel size [arcsec/pix]
   lambda     : inp_img_t2.lambda      , $    ; Mean detection wavelength [m]
   width      : inp_img_t2.width       , $    ; Wavelength band width [m]
   time_integ : inp_img_t2.time_integ  , $    ; Integration time for each image [s]
   time_delay : inp_img_t2.time_delay  , $    ; [NOT USED WITHIN AIRY]
   background : make_array(N,N,P)      , $ ; the PSF's background is zero
   snr        : inp_img_t2.snr         , $    ; snr
   header     : PTR_NEW('')       $    ;header of the output PSF
   }
*out_psf_t.header = *inp_img_t2.header

return, error
end
