;+
; cbd_magnitude.pro, version 2.1 
; by Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it] 
; last update: 2016/03/01
;
; PURPOSE
;    This routine computes the magnitude corresponding to the input
;    number of photons, for a given band, a given surface and a given time
;    interval. The chosen band can be either a Johnson one (from U to M), or
;    a user-defined one (specifying the central wavelength and the bandwidth).
;
; SYNOPSIS
;    out = cbd_magnitude(flux, surf=surf, delta_t=delta_t, band=band, $ 
;                        lambda=lambda, width=width)
;
; MANDATORY INPUT
;    flux       (double)  - input total flux of the source
;    either BAND or pair (LAMBDA,WIDTH) is a mandatory input
;
; OPTIONAL INPUT
;    The following options must be provided as keyword/value pairs.
;      BAND     = wavelength Johnson band [STRING].
;                 ["U","B","V","R","I","J","H","K","L","M"]
;      OR   LAMBDA   = central wavelength of the chosen band [m] [FLOAT].
;                 1e-6*[0.36,0.44,0.55,0.70,0.90,1.25,1.65,2.20,3.40]
;      +    WIDTH    = bandwidth of the chosen band [m] [FLOAT].
;                 1e-6*[0.068,0.098,0.089,0.22,0.24,0.3,0.35,0.4,0.55]
;      DELTA_T  = integrating time [s] [FLOAT], default is 1s.
;      SURF     = integrating surface [m^2] [FLOAT], default is 1m^2.
;
; OUTPUT
;    out        (double)  - calculated magnitude
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    none.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLES:
;    [1]: compute the magnitude corresponding to a flux of 10^6 photons,
;         observed in V-band, with 8m-diameter telescope and an integration
;         time of 12s:
;         IDL> print, cbd_magnitude(1e6,BAND='V',SURF=!PI*(8.^2)/4,DELTA_T=12.)
;            1.9127931
;
;    [2a]: compute the magnitude corresponding to a flux of 10^5 photons,
;         observed in a user-defined band (lambda=2.12um,width=0.02um), 
;         with 8m-diameter telescope and an integration time of 12s:
;         IDL> print, cbd_magnitude(1e5,LAMBDA=2.12e-6,WIDTH=2e-8,SURF=!PI*(8.^2)/4,DELTA_T=12.)
;            14.251272       
;
; MODIFICATIONS:
;  - from version 1.0 to 2.0:
;    extended use based on the routine n_phot.pro (CAOS PSE);
;  - version 2.1:
;    small changes and bug corrections;
;-

function cbd_magnitude, flux, surf=surf, delta_t=delta_t, band=band, lambda=lambda, $
                    width=width
on_error,2
if n_params() LT 1 then message, $
       'CALLING SEQUENCE: out = magnitude(flux, surf=surf, delta_t=delta_t, '+$ 
                             'band=band, lambda=lambda, width=width)'

if keyword_set(delta_t) then delta_t=delta_t else delta_t=1. ;default = 1 second
if keyword_set(surf) then surf=surf else surf=1. ; default = 1 m^2

;some constants
h = 6.626d-34                   ; Planck constant [Js]
c = 2.998d8                     ; light velocity [m/s]
; bands:
band_tab = ["U","B","V","R","I","J","H","K","L","M"]
; wavelength [m]:
lambda_tab = 1e-6*[0.36,0.44,0.55,0.70,0.90,1.25,1.65,2.20,3.40,5.00]
; bandwidth [m]:
width_tab  = 1e-6*[0.068,0.098,0.089,0.22,0.24,0.3,0.35,0.4,0.55,0.3]
; A0 0-magnitude star brightness [J/s/m^2/um]:
e0_tab     = 1e-10*[435.,720.,392.,176.,83.,34.,7.,3.9,0.81,0.22]
; [ref.: P.Lena, Astrophysique : methodes physiques de l'observation,
;        pp.95--96, Coll. Savoirs Actuels, InterEd./CNRS-Ed. (1996)]


j=-1
if (n_elements(band)) then begin
    repeat j=j+1 until (band_tab[j] eq band)       ; band
    if not(n_elements(lambda)) then lambda = lambda_tab[j]  ; wavelength [m]
endif else if (n_elements(lambda)) then begin
    band=''
    for i=0,n_elements(lambda_tab)-1 do begin
        if (abs(lambda_tab[i]-lambda) le width_tab[i]/2.) then begin
            j = i
            band = band_tab[i]            ; band
        endif
    endfor
    if (band eq '') then begin            ; we are in a case of a narrow band
                                          ; between two defined wide bands
        lambda_min=lambda_tab-width_tab/2 ; min lambda of the wide bands
        lambda_max=lambda_tab+width_tab/2 ; max lambda of the wide bands
        idx_min=where(abs(lambda-lambda_min) $
                      eq min(abs(lambda-lambda_min)))
                                          ; index of the closest lambda in
                                          ; lambda_min
        idx_max=where(abs(lambda-lambda_max) $
                      eq min(abs(lambda-lambda_max)))
                                          ; index of the closest lambda in
                                          ; lambda_max
                                          ; find the index of the closest band
        if (abs(lambda-lambda_min[idx_min]) le $
            abs(lambda-lambda_max[idx_max])) then begin
            j = idx_min
            band = band_tab[idx_min]      ; band
        endif else begin
            j = idx_max
            band = band_tab[idx_max]      ; band
        endelse
    endif
endif 
if not(n_elements(width)) then width = width_tab[j]  ; band-width [m]
e0 = e0_tab[j]                        ; A0 star 0-mag. brightness [J/s/m^2/um]

m=-2.5*alog10(flux*(h*c/lambda)/(float(delta_t)*float(surf)*e0*width*1d6))
return, m
end

; ==========================
; End of magnitude.pro file
; ==========================
