function cbd_project_fletcher_box, a, b, c, dia, l, u

; CBD_* VERSION FOR AIRY'S CBD MODULE contains some small
; modifications for adapting existing code to CBD's code--

; project_fletcher_box - Dai-Fletcher algorithm for separable simply constrained QPs
;   This function applies the secant-based Dai-Fletcher algorithm [1] to solve
;   the separable, singly linearly and box constrained quadratic
;   programming problem
;                           min 0.5 * x'*diag(dia)*x - c'*x
;                           subj to a'*x = b
;                                      l <= x <= u
;
;   [1] Y. H. Dai, R. Fletcher, "New algorithms for singly linearly constrained
;       quadratic programs subject to lower and upper bounds",
;       Math. Program., Ser. A 106, 403 - 421 (2006).
;
; SYNOPSIS
;   [x, biter, siter] = project_fletcher_box(a, b, c, dia, l, u)
;
; INPUT
;   a     (double)        - coefficients of x in the linear constraint
;   b     (double)        - rhs of the linear constraint
;   c     (double array)  - coefficient vector of the objective's linear term
;   dia   (double array)  - diagonal elements of the Hessian
;   l      (double array) - lower bounds in the box constraint
;   u      (double array) - upper bounds in the box constraint
;
; OUTPUT
;   x     (double array) - solution vector
;   biter (pos. integer) - total number of backtracking iterations
;   iter  (pos. integer) - total number of secant iterations
;
; ------------------------------------------------------------------------------

c = -c;
lambda = 0
dlambda = 1
tol = 1.e-9
biter = 0
siter = 0

; Bracketing Phase
x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
r = total(a*x) - b

if r lt 0. then begin
    lambdal = lambda
    rl = r
    lambda = lambda + dlambda
    x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
    r = total(a*x) - b
    while r lt 0 do begin
        biter = biter + 1
        lambdal = lambda
        s = max([rl/r - 1., 0.1])
        dlambda = dlambda + dlambda/s
        lambda = lambda + dlambda
        rl = r
        x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
        r = total(a*x) - b
    endwhile
    lambdau = lambda
    ru = r
endif else begin
    lambdau = lambda
    ru = r
    lambda = lambda - dlambda
    x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
    r = total(a*x) - b
    while r gt 0 do begin
        biter = biter + 1
        lambdau = lambda
        s = max([ru/r - 1., 0.1])
        dlambda = dlambda + dlambda/s
        lambda = lambda - dlambda
        ru = r
        x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
        r = total(a*x) - b
    endwhile
    lambdal = lambda
    rl = r
endelse

; Secant Phase
s = 1. - rl/ru
dlambda = dlambda/s
lambda = lambdau - dlambda
x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
r = total(a*x) - b

while abs(r) gt tol do begin
    siter = siter + 1
    if r gt 0 then begin
        if s le 2 then begin
            lambdau = lambda
            ru = r
            s = 1. - rl/ru
            dlambda = (lambdau - lambdal)/s
            lambda = lambdau - dlambda
        endif else begin
            s = max([ru/r - 1., 0.1])
            dlambda = (lambdau - lambda)/s
            lambda_new = max([lambda - dlambda, 0.75*lambdal + 0.25*lambda])
            lambdau = lambda
            ru = r
            lambda = lambda_new
            s = (lambdau - lambdal) / (lambdau - lambda)
        endelse
    endif else begin
        if s ge 2. then begin
            lambdal = lambda
            rl = r
            s = 1. - rl/ru
            dlambda = (lambdau - lambdal)/s
            lambda = lambdau - dlambda
        endif else begin
            s = max([rl/r - 1., 0.1])	; corrected version
            dlambda = (lambda - lambdal)/s
            lambda_new = min([lambda + dlambda, 0.75*lambdau + 0.25*lambda])
            lambdal = lambda
            rl = r
            lambda = lambda_new
            s = (lambdau - lambdal) / (lambdau - lambda)
        endelse
    endelse
    x = max([[l], [min([[u], [(c + lambda*a)/dia]], dimension=2)]], dimension=2)
    r = total(a*x) - b
endwhile

return, x

end

; =====================================
; End of project_fletcher_box.pro file
; =====================================
