function cbd_project_fletcher_nonneg, b, c, dia

; CBD_* VERSION FOR AIRY'S CBD MODULE contains some small
; modifications for adapting existing code to CBD's code--

; project_fletcher_nonneg - Dai-Fletcher algorithm for separable simply constrained QPs
;   This function applies the secant-based Dai-Fletcher algorithm [1] to solve
;   the separable, singly linearly and nonnegatively constrained quadratic
;   programming problem
;                           min 0.5 * x'*diag(dia)*x - c'*x
;                           subj to sum(x) = b
;                                      x >= 0
;
;   [1] Y. H. Dai, R. Fletcher, "New algorithms for singly linearly constrained
;       quadratic programs subject to lower and upper bounds",
;       Math. Program., Ser. A 106, 403 - 421 (2006).
;
; SYNOPSIS
;   [x, biter, siter,r] = project_fletcher_nonneg(b, c, dia)
;
; INPUT
;   b     (double)       - rhs of the linear constraint
;   c     (double array) - coefficient vector of the objective's linear term
;   dia   (double array) - diagonal elements of the Hessian
;
; OUTPUT
;   x     (double array) - solution vector
;   biter (pos. integer) - total number of backtracking iterations
;   iter  (pos. integer) - total number of secant iterations
;   r     (double)       - linear constraint residual value (sum(x)-b)
;                          at the solution
;
; ------------------------------------------------------------------------------

lambda = 0                  ; initial lambda
dlambda = 1                 ; initial step
tol_r = 1.e-11 * b          ; tolerance on the function
tol_lam = 1.e-11            ; tolerance on the step
biter = 0                   ; bracketing phase iterations
siter = 0                   ; secant phase iterations
maxprojections = 1000       ; maximum number of iterations

; Bracketing Phase
x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
r = total(x) - b;

; check abs(r) < tol
if ( abs(r) lt tol_r ) then return, x

if r lt 0. then begin
    lambdal = lambda
    rl = r
    lambda = lambda + dlambda
    x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
    r = total(x) - b
    while r lt 0 do begin
        biter = biter + 1
        lambdal = lambda
        s = max([rl/r - 1, 0.1])
        dlambda = dlambda + dlambda/s
        lambda = lambda + dlambda
        rl = r
        x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
        r = total(x) - b
    endwhile
    lambdau = lambda
    ru = r
endif else begin
    lambdau = lambda
    ru = r
    lambda = lambda-dlambda
    x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
    r = total(x) - b
    while r gt 0. do begin
        biter = biter + 1
        lambdau = lambda
        s = max([ru/r - 1., 0.1])
        dlambda = dlambda + dlambda/s
        lambda = lambda - dlambda
        ru = r
        x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
        r = total(x) - b
    endwhile
    lambdal = lambda
    rl = r
endelse

; check ru and rl
if (abs(ru) lt tol_r) then begin
   x = max([[dblarr(n_elements(c))],[(c + lambdau)/dia]], dimension=2)
   return, x
endif
if (abs(rl) lt tol_r) then begin
   x = max([[dblarr(n_elements(c))],[(c + lambdal)/dia]], dimension=2)
   return, x
endif

; Secant Phase
s = 1. - rl/ru
dlambda = dlambda/s
lambda = lambdau - dlambda
x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
r = total(x) - b
maxit_s = maxprojections - biter

; Main loop
while (abs(r) gt tol_r) and (dlambda gt tol_lam * (1 + abs(lambda))) and (siter lt maxit_s) do begin
    siter = siter + 1
    if r gt 0 then begin
        if s le 2. then begin
            lambdau = lambda
            ru = r
            s = 1. - rl/ru
            dlambda = (lambdau - lambdal)/s
            lambda = lambdau - dlambda
        endif else begin
            s = max([ru/r - 1., 0.1])
            dlambda = (lambdau - lambda) / s
            lambda_new = max([lambda - dlambda, 0.75*lambdal + 0.25*lambda])
            lambdau = lambda
            ru = r
            lambda = lambda_new
            s = (lambdau - lambdal) / (lambdau - lambda)
        endelse
    endif else begin
        if s ge 2 then begin
            lambdal = lambda
            rl = r
            s = 1. - rl/ru
            dlambda = (lambdau - lambdal)/s
            lambda = lambdau - dlambda
        endif else begin
            s = max([rl/r - 1., 0.1])
            dlambda = (lambda - lambdal) / s
            lambda_new = min([lambda + dlambda, 0.75*lambdau + 0.25*lambda])
            lambdal = lambda
            rl = r
            lambda = lambda_new
            s = (lambdau - lambdal) / (lambdau-lambda)
        endelse
    endelse
    x = max([[dblarr(n_elements(c))],[(c + lambda)/dia]], dimension=2)
    r = total(x) - b
endwhile

return, x

END

; ========================================
; End of project_fletcher_nonneg.pro file
; ========================================
