pro cbd_sgp_deblurring_bound_multi, A, gn, x, iter, err, Discr, times, $
                    bg = bg, initialization = initialization, maxit = maxit, obj = obj, data_scaling = data_scaling, $
                    stopcrit = stopcrit, tol = tol, verb = verb, omega = omega, upp_bound = upp_bound, center = center, magn_par = magn_par
; CBD_* VERSION FOR AIRY'S CBD MODULE contains some small
; modifications for adapting existing code to CBD's code--

; cbd_sgp_deblurring_bound_multi - SGP algorithm for non-regularized deblurring
;   This function solves a multiple image deblurring problem by applying the SGP
;   algorithm to the minimization of the generalized Kullback-Leibler
;   divergence with no regularization [1,2]:
;
;      min sum_k KL(A^(k)*x + bg, gn^(k))
;       x in OMEGA
;
;   where KL(u,v) is the generalized Kullback-Leibler divergence between
;   vectors u and v, bg is the background, gn^(k) are the observed data and
;   the feasible set OMEGA is x(i) >= 0;
;
;   [1] S. Bonettini, R. Zanella, L. Zanni,
;       "A scaled gradient projection method for constrained image deblurring",
;       Inverse Problems 25(1), 2009, January, 015002.
;   [2] M. Prato, R. Cavicchioli, L. Zanni, P. Boccacci, M. Bertero
;     "Efficient deconvolution methods for astronomical imaging: algorithms
;     and IDL-GPU codes", Astronomy & Astrophysics 539, 2012, A133.
;
; SYNOPSIS
;    sgp_deblurring_blind_multi, A, gn, x, iter, err, Discr, times [, opts]
;
; MANDATORY INPUT
;   A   (double array) - measuring NxNxP array used to apply the blurring
;                        operator, that is to compute A^(k)*x
;                        REMARK: all columns of A must sum-up to 1
;   gn  (double array) - measured images
;
; OPTIONAL INPUT
;   The following options must be provided as keyword/value pairs.
;   'OBJ'              - Exact solution, for error calculation (double array)
;   'BG'               - Background value (array)
;   'INITIALIZATION'   - Choice for starting point:
;                        0  - all zero starting point
;                        1  - initialization with gn
;                        2  - initialization with
;                               ones(size(gn))*sum(gn(:) - bg) / numel(gn)
;                        x0 - user-provided starting point (double array)
;                        DEFAULT = 2
;   'MAXIT'            - Maximum number of iterations (integer)
;                        DEFAULT = 1000
;   'VERB'             - Verbosity level (integer)
;                        0 - silent
;                        1 - print configuration parameters at startup and
;                            some information at each iteration
;                        DEFAULT = 0
;   'STOPCRIT'         - Choice for stopping rule (integer)
;                        1 -> iter > MAXIT
;                        2 -> ||x_k - x_(k-1)|| <= tol*||x_k|| OR iter > MAXIT
;                        3 -> |KL_k - KL_(k-1)| <= tol*|KL_k| OR iter > MAXIT
;                        4 -> (2/N)*KL_k <= tol OR iter > MAXIT
;                        DEFAULT = 1
;   'TOL'              - Tolerance used in the stopping criterion
;                        DEFAULT = 1e-4 if STOPCRITERION = 2 or 3
;                        DEFAULT = 1+1/mean(gn) if STOPCRITERION = 4
;   'OMEGA'            - Constraints type (string):
;                        'nonneg'   : non negativity
;                        'nonneg_eq': non negativity and flux conservation
;                                    ( estimated flux is sum(gn(:) - bg) )
;                        'low_up_eq': lower end upper bound plus equality
;                        'special' : 'low_up_eq' + center value set equal to upp_bound
;                        DEFAULT: 'nonneg'
;   'UPP_BOUND'        - Upper bound for the PSF
;                        DEFAULT = 1e10
;   'CENTER'           - 2 components vector with the coordinates of the psf center;
;   'MAGN_PAR'          - Parameters for MARE computation
;                magn_par.lambda: electromagnetic band of the observation
;                magn_par.width : electromagnetic band of the observation
;                magn_par.surf: integrating surface
;                magn_par.delta_t: observation time amplitude (s)
;                magn_par.eff : overall efficiency
;   'DATA_SCALING'     - Data scaling:
;                        false : nonscaled data
;                        true  : scaled data
;                        DEFAULT = 1
;
; OUTPUT
;   x                  - Reconstructed data
;   iter               - Number of iterations
;   err                - Error value at each iteration. If OBJ was not given,
;                        then err is the empty matrix.
;   discr              - Discrepancy value after each iteration:
;                            D = 2/numel(x_k) * KL( Ax_k + bg, gn)
;   times              - Time elapsed after each iteration
;
; ------------------------------------------------------------------------------
; modification of May2016, ALC
; - bg is now an array!!!
; ------------------------------------------------------------------------------


; start the clock
t0 = systime(2)

;;;;;;;;;;;;;;;;;;;;;;;;;;
; SGP default parameters ;
;;;;;;;;;;;;;;;;;;;;;;;;;;
alpha_min = 1.e-5        ; alpha lower bound
alpha_max = 1.e5        ; alpha upper bound
theta = 0.4              ; backtracking parameter
beta = 1.e-4             ; for sufficient decrease
initalpha = 1.3          ; initial alpha
M = 1                    ; memory in obj. function value (if M = 1 monotone)
Malpha = 3               ; alfaBB1 memory
tau = 0.5                ; alternating parameter
initflag = 2             ; 2 -> constant image
errflag = 0B             ; 0 -> no error computation
magnflag = 0B       ; 0 -> no MARE computation
err = -1

if (keyword_set(maxit) - 1)         then maxit = 1000             ; maximum number of iterations
if (keyword_set(stopcrit) - 1)      then stopcrit = 1             ; 1 -> number of iterations
if (keyword_set(verb) - 1)          then verb = 0                 ; 0 -> silent
if (keyword_set(omega) - 1)         then omega = 'nonneg'         ; non negativity constraints
if (keyword_set(upp_bound) - 1)     then upp_bound = 1e10         ; upper_bound for the PSF (unused with default omega)
if (keyword_set(data_scaling) - 1)  then begin              ; data scaling
   data_scaling = 1B
endif else begin
   if data_scaling eq 'N' then begin
      data_scaling = 0B
   endif else begin
      data_scaling = 1B
   endelse
endelse

; Problem size
dim = size(gn,/dimensions)
if (size(gn,/n_dimensions) eq 2) then begin
   num = 1
endif else begin
   num = dim[2]
endelse
im_size = dim[0:1]

TF = dcomplexarr(dim)
CTF = TF
for i = 0, num - 1 do begin
    TF[*,*,i] = fft(shift(reform(A[*,*,i]),im_size/2))*dim[0]*dim[1]
    CTF[*,*,i] = conj(TF[*,*,i])
endfor

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read the optional parameters ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if keyword_set(obj) then errflag = 1B
if keyword_set(magn_par) then magnflag = 1B
if keyword_set(initialization) then begin
    if product(size(initialization,/dimensions)) gt 1 then begin
       initflag = 999
       x = initialization       ; initial x provided by user
    endif else begin
       initflag = initialization
    endelse
endif

;;;;;;;;;;;;;;;;;;
; starting point ;
;;;;;;;;;;;;;;;;;;
case initflag of
    0: x = dblarr(im_size)                                                ; all zeros
    1: x = gn[*,*,0]                                             ; gn
    2: x = total(gn-bg)/n_elements(gn)*make_array(im_size,value=1.,/double)     ; same flux as gn - bg
    999: if not(array_equal(size(x,/dimensions),im_size)) then  begin           ; x is explicitly given, check dimensions
            message, 'Invalid size of the initial point.'
         endif
    else: message, 'Unknown initialization option.'
endcase


;;;;;;;;;;;;;;;;;;
; stop criterion ;
;;;;;;;;;;;;;;;;;;
if (stopcrit ne 1 and stopcrit ne 2 and stopcrit ne 3 and stopcrit ne 4) then begin
    message, 'Unknown stopping criterion:', stopcrit
end

if (keyword_set(tol) - 1) then begin
    if (stopcrit eq 2 or stopcrit eq 3) then tol = 1.e-4
    if (stopcrit eq 4) then tol = 1. + 1./mean(gn)
end

;;;;;;;;;;;;;;;;
; data scaling ;
;;;;;;;;;;;;;;;;
if data_scaling then begin
    scaling = max(gn)
    gn = gn/scaling
    bg = bg/scaling
    x = x/scaling
endif else begin
    scaling = 1.
endelse

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; change the nonpositive pixels of the observed image ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
par = machar(/double)
vmin = min(gn(where(gn gt 0)))
gn_nonpos = where(gn le 0, n_nonpos)
if (n_nonpos gt 0) then gn(gn_nonpos) = vmin*par.eps^2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; some computations needed only once ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
N = product(im_size)                             ; pixels in the image
flux = total(total(gn-bg,1),1);-total(bg,/double)    ; exact flux
iter = 1                                         ; iteration counter
alpha = initalpha                                ; initial alpha
Valpha = alpha_max * (dblarr(Malpha)+1.)         ; memory buffer for alpha
Fold = -1.e30 * (dblarr(M)+1.)                    ; memory buffer for obj. func.
Discr_coeff = 2./(N*num)*scaling                  ; discrepancy coefficient
ONE = make_array(im_size,value = num, /double)

;;;;;;;;;;;;;;;;;
; projection type
;;;;;;;;;;;;;;;;;
case omega of
    'nonneg'    : pflag = 0
    'nonneg_eq' : pflag = 1
    'low_up_eq' : pflag = 2
    'special'   : pflag = 3
else            : message, 'projection ' + omega +' is not implemented'
endcase

;;;;;;;;;;;;;;;;;;;;;
; vector allocation ;
;;;;;;;;;;;;;;;;;;;;;
if errflag then begin
   err = dblarr(maxit+1)
   obj = obj[*]
   obj = obj/scaling
   if magnflag then begin
      obj_nz = where(obj gt 0)
      nnz = n_elements(obj_nz)
      ind_obj_err = dblarr(nnz,9)
      obj_magn = dblarr(nnz)
      for k=0,nnz-1 do begin
          ind_obj_err[k,*] = [obj_nz[k] - 1 - im_size[0], obj_nz[k] - im_size[0], obj_nz[k] + 1 - im_size[0], $
                              obj_nz[k] - 1             , obj_nz[k]             , obj_nz[k] + 1             , $
                              obj_nz[k] - 1 + im_size[0], obj_nz[k] + im_size[0], obj_nz[k] + 1 + im_size[0]]
          obj_magn[k] = cbd_magnitude(total(obj[ind_obj_err[k,*]])/magn_par.eff*scaling, $
                                      lambda=magn_par.lambda, width=magn_par.width, $
                                      delta_t = magn_par.delta_t, surf = magn_par.surf)
      endfor
   endif else begin
      obj_sum = total(obj*obj)
   endelse
endif

Discr = dblarr(maxit+1)
times = dblarr(maxit+1)

;;;;;;;;;;;;;;;;
; start of SGP ;
;;;;;;;;;;;;;;;;

; projection of the initial point
t0=systime(1)

x = x[*]
Nx = n_elements(x)
case pflag of
       0 : begin
              x_nonpos =  where(x lt 0, n_nonpos)    ; non negativity
              if (n_nonpos gt 0) then x(x_nonpos) = 0.
           end
       1 : x = cbd_project_fletcher_nonneg(mean(flux), x, dblarr(Nx) + 1.)  ; non negativity and flux conservation
       2 : begin
              x = cbd_project_fletcher_box(dblarr(Nx) + 1., 1./scaling, -x, dblarr(Nx) + 1., $
                                dblarr(Nx), dblarr(Nx) + upp_bound/scaling)   ; lower and upper constraints and normalization
           end
       3 : begin
              center = center[1]*true_size[0] + center[0]
              x = cbd_project_fletcher_box(dblarr(Nx-1) + 1., (1.-upp_bound)/scaling, -[x[0:center-1],x[center+1:Nx-1]],$
                                       dblarr(Nx-1) + 1., dblarr(Nx-1), dblarr(Nx-1) + upp_bound/scaling)
              x = [x[0:center-1], upp_bound/scaling, x[center:Nx-2]]
           end
endcase

; error
if errflag then begin
   if magnflag then begin
      err[0] = 0.
      for k=0,n_elements(obj_nz)-1 do begin
         m_star = cbd_magnitude(total(x[ind_obj_err[k,*]],/double)/magn_par.eff*scaling, $
                                lambda=magn_par.lambda, width=magn_par.width, $
                                delta_t = magn_par.delta_t, surf = magn_par.surf)
         err[0] = err[0] + (abs(m_star - obj_magn[k])/obj_magn[k])
      endfor
      err[0] = err[0]/n_elements(obj_nz)
   endif else begin
      e = x - obj
      err[0] = sqrt(total(e*e)/obj_sum)
   endelse
endif

x = reform(x,im_size)

; objective function value
x_tf = cbd_afunction_multi(x,TF)
den = x_tf + bg
temp = gn/den
g = cbd_afunction_multi(temp,CTF)
if num gt 1 then begin
    g = total(g,3)
endif
g = ONE - g
fv = total(gn*alog(temp)) + total(x_tf) - total(flux)

;;;;;  bounds for the scaling matrices ;;;;;;;;
temp = cbd_afunction_multi(gn,CTF)
tmp = temp[*,*,0]
y = (flux[0]/(flux[0] + total(bg[*,*,0])))*tmp
y_min = min(y[where(y gt 0)])
y_max = max(y)
for i = 1, num-1 do begin
    tmp = temp[*,*,i]
    y = (flux[i]/(flux[i] + total(bg[*,*,i])))*tmp
    y_min = min([min(y[where(y gt 0)]),y_min])
    y_max = max([max(y),y_max])
endfor

X_low_bound = y_min              ; Lower bound for the scaling matrix
X_upp_bound = y_max             ; Upper bound for the scaling matrix

if X_upp_bound/X_low_bound lt 50. then begin
   X_low_bound = X_low_bound/10.
   X_upp_bound = X_upp_bound*10.
endif
; discrepancy
Discr[0] = Discr_coeff * fv

; scaling matrix
if initflag eq 0 then begin
    XX = make_array(size(x,/dimensions), value = 1., /double)
endif else begin
    XX = x
    ; bounds
    XX_toolow = where(XX lt X_low_bound, nlow)
    if (nlow gt 0) then XX(XX_toolow) = X_low_bound
    XX_toohigh = where(XX gt X_upp_bound, nhigh)
    if (nhigh gt 0) then XX(XX_toohigh) = X_upp_bound
endelse

if (pflag eq 1) or (pflag eq 2) or (pflag eq 3) then D = 1./XX

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; tolerance for stop criterion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (stopcrit eq 2) then tol = tol*tol

;;;;;;;;;;;
; main loop
;;;;;;;;;;;
loop = 1B
while loop do begin
    ; store alpha and objective function values
    Valpha[0:Malpha-2] = Valpha[1:Malpha-1]
    if M gt 1 then Fold[0:M-2] = Fold[1:M-1]
    Fold[M-1] = fv

    ; compute descent direction
    y = x - alpha*XX*g

    ; projection onto the feasible set
    case pflag of
       0 : begin
             y_nonpos =  where(y lt 0, n_nonpos)    ; non negativity
             if (n_nonpos gt 0) then y(y_nonpos) = 0.
           end
       1 : begin
             y = y[*]
             D = D[*]
             y = cbd_project_fletcher_nonneg(flux, y*D, D)  ; non negativity and flux conservation
             y = reform(y,im_size)
           end
       2 : begin
             y = y[*]
             D = D[*]
             y = cbd_project_fletcher_box(dblarr(n_elements(y)) + 1., 1./scaling, -y*D, D, $
                               dblarr(n_elements(y)), dblarr(n_elements(y)) + upp_bound/scaling)   ; lower and upper constraints and normalization
             y = reform(y,im_size)
           end
       3 : begin
             y = y[*]
             D = D[*]
             y = cbd_project_fletcher_box(dblarr(n_elements(y) - 1) + 1., (1.-upp_bound)/scaling, -[y[0:center-1],y[center+1:n_elements(y)-1]]*$
                                      [D[0:center-1],D[center+1:n_elements(y)-1]],$
                                      [D[0:center-1],D[center+1:n_elements(y)-1]], dblarr(n_elements(y)-1), dblarr(n_elements(y)-1) + upp_bound/scaling)
             y = [y[0:center-1], upp_bound/scaling, y[center:n_elements(y)-2]]
             y = reform(y,im_size)
           end
    endcase

    d = y - x

    ; backtracking loop for linesearch
    gd = total(d*g)
    lam = 1.

    fcontinue = 1B
    d_tf = cbd_afunction_multi(d,TF)   ; exploiting linearity
    fr = max(Fold)

    while fcontinue do begin
        xplus = x + lam*d
        x_tf_try = x_tf + lam*d_tf
        den = x_tf_try + bg
        temp = gn/den
        fv = total(gn*alog(temp)) + total(x_tf_try) - total(flux)

        if ( fv le fr + beta * lam * gd OR lam lt 1.e-12) then begin
            x = xplus
            sk = lam*d
            x_tf = x_tf_try
            gtemp = cbd_afunction_multi(temp,CTF)
            if num gt 1 then begin
                gtemp = total(gtemp,3)
            endif
            gtemp = ONE - gtemp

            yk = gtemp - g
            if pflag EQ 3 then yk[center] = 0.
            g = gtemp
            fcontinue = 0B
        endif else begin
            lam = lam * theta
        endelse
    endwhile

    if (fv ge fr AND verb gt 0) then print, 'WARNING: fv >= fr'

    ; update the scaling matrix and the steplength
    XX = x
    XX_toolow = where(XX lt X_low_bound, nlow)
    if (nlow gt 0) then XX(XX_toolow) = X_low_bound
    XX_toohigh = where(XX gt X_upp_bound, nhigh)
    if (nhigh gt 0) then XX(XX_toohigh) = X_upp_bound
    D = 1./XX

    sk2 = sk*D
    yk2 = yk*XX
    bk = total(sk2*yk)
    ck = total(yk2*sk)
    if (bk le 0) then begin
        alpha1 = min([10.*alpha,alpha_max])
    endif else begin
        alpha1BB = total(sk2*sk2)/bk
        alpha1 = min([alpha_max, max([alpha_min, alpha1BB])])
    end
    if (ck le 0) then begin
        alpha2 = min([10.*alpha,alpha_max])
    endif else begin
        alpha2BB = ck/total(yk2*yk2)
        alpha2 = min([alpha_max, max([alpha_min, alpha2BB])])
    end

    Valpha[Malpha-1] = alpha2

    if (iter le 20) then begin
       alpha = min(Valpha)
    endif else begin
        if (alpha2/alpha1 lt tau) then begin
           alpha = min(Valpha)
           tau = tau*0.9
        endif else begin
            alpha = alpha1
            tau = tau*1.1
        endelse
    endelse
    times[iter] = systime(2) - t0
    iter = iter + 1

    alpha = double(float(alpha))

    ; error
    if errflag then begin
       if magnflag then begin
          err[iter-1]=0.
          for k=0,n_elements(obj_nz)-1 do begin
             m_star = cbd_magnitude(total(x[ind_obj_err[k,*]],/double)/magn_par.eff*scaling, $
                                    lambda= magn_par.lambda, width=magn_par.width, $
                                    delta_t = magn_par.delta_t, surf = magn_par.surf)
             err[iter-1]=err[iter-1]+(abs(m_star-obj_magn[k])/obj_magn[k])
           endfor
          err[iter-1]=err[iter-1]/n_elements(obj_nz)
       endif else begin
          e = x[*] - obj
          err[iter-1] = sqrt(total(e*e)/obj_sum)
       endelse
    endif

    Discr[iter-1] = Discr_coeff * fv

    ;;;;;;;;;;;;;;;;;
    ; stop criteria ;
    ;;;;;;;;;;;;;;;;;
    case stopcrit of
        1: begin
              if verb gt 0 then begin
                 print, 'it ', iter-1, ' of ', maxit
              endif
           end
        2: begin
              normstep = total(sk*sk)
              loop = (normstep gt tol*total(x*x))
              if verb gt 0 then begin
                 print, 'it ', iter-1, ', || x_k - x_k-1 || ^2 / || x_k || ^2 ', normstep, ', tol ', tol
              endif
           end
        3: begin
              reldecrease = abs(fv - Fold[M-1])/abs(fv)
              loop = (reldecrease gt tol)
              if verb gt 0 then begin
                 print, 'it ', iter-1, ', | f_k - f_k-1 | / | f_k | ', reldecrease, ', tol ', tol
              endif
           end
        4: begin
              if iter ne 1 then begin
                 loop = (Discr[iter-1] gt tol)
              endif else begin
                 loop = 1
              endelse
              if verb gt 0 then begin
                 print, 'it ', iter-1, ', D_k ', Discr[iter-1], ', tol ', tol
              endif
           end
    endcase

    if iter gt maxit then loop = 0B

    if verb gt 0 then begin
      print, 'Iteration:', iter,  '  Fobj:', fv, '  Alpha:', alpha, '  Lambda:', lam
    endif

endwhile

x = x * scaling
gn = gn * scaling
bg = bg * scaling

if errflag then begin
   obj = obj * scaling
   err = err[0:iter-1]
endif

Discr = Discr[0:iter-1]
times = times[0:iter-1]
iter = iter - 1

end

; ==========================================
; End of sgp_deblurring_bound_multi.pro file
; ==========================================
