; $Id: psf_interf.pro,v 1.0 1999/05/05$
;+
; NAME:
;       psf_interf
;
; PURPOSE:
;
;       Model interferometric PSF for an interferometer composed of
;       two pupils of diameter D separeted by a center-to-center baseline b, 
;       at the desired monochromatic wavelength and parallactic orientation.
;       Assumptions are perfects optics, AO correction and cophasing.
;
; CATEGORY:
;       ...
;
; CALLING SEQUENCE:
;
;       psf_interf, b, d, lambda, gam, pix_size, nb_pix, psf
;
; INPUTS:
;
;       b               = interferometer baseline in meter [FLOAT]
;       d               = one pupil diameter in meter [FLOAT]
;       lambda          = central filter wavelenght in meter [FLOAT]
;       gam             = parallactic angle in degree CCW [FLOAT]
;       pix_size        = detector pixel size [FLOAT]
;       nb_pix          = array number of pixels [INT]
;
; OPTIONAL INPUTS:
;       none.
;
; KEYWORD PARAMETERS:
;
; OUTPUT:
;
;       psf = energy normalised to 1 interferometric PSF [FLOAT] = Array[nb_pix,nb_pix]
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       none.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
; EXAMPLE:
;       ...
;
; MODIFICATION HISTORY:
;     
;-

pro psf_interf, b, d, lambda, gam, pix_size, nb_pix, psf_i


V=1.
i0=nb_pix/2.
j0=nb_pix/2.


radsec_conv=360.*60*60/(2*!pi)
gam=gam/180*!pi

inter_1=!pi*pix_size*(d/lambda)/radsec_conv
inter_2=2*!pi*pix_size*(b/lambda)/radsec_conv


airy=fltarr(nb_pix,nb_pix)


for i=0, nb_pix-1 do begin
for j=0, nb_pix-1 do begin

        r=sqrt(float((i-i0)^2+(j-j0)^2))
        airy[i,j]=(2*beselj(inter_1*r,1)/(inter_1*r))^2
        airy[i0,j0]=1.
        phase_term=inter_2*(float((i-i0)*cos(gam)+(j-j0)*sin(gam)))
        interf_term=1+(V*cos(phase_term))

        psf_i[i,j]=airy[i,j]*interf_term

endfor
endfor

psf_i=psf_i/total(psf_i)

return
end
