; $Id: cbd_prog.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       cbd_prog
;
; PURPOSE:
;       cbd_prog represents the scientific algorithm for the CBD module.
;
; CATEGORY:
;       scientific program
;
; CALLING SEQUENCE:
;       error = cbd_prog(inp_img_t2,  $ ; img_t input structure
;                        out_psf_t ,  $ ; img_t output structure (PSF)
;                        out_img_t ,  $ ; img_t output structure (OBJ)
;                        par,         $ ; parameters structure
;                        INIT=init    $ ; initialization data structure
;                       )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see module help for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: december 2008,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -output's order exchanged: (WAS) top=PSF, bottom=OBJ;
;                     (NOW IS) top=OBJ, bottom=PSF, as it is in the whole 
;                     Soft.Pack.AIRY
;                   : February 2014, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -several optional outputs added.  
;                   : January 2015,
;                    -Add another method of Blind Deconvolution, based on the 
;                     paper cited in the file "AIRYSoftPack6.1_From6.0to6.1.txt"
;                    -Some changes on the GUI (internal tabs added and
;                     several input fields moved here and there).
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added optional output save: objective function
;
;-
function cbd_prog, inp_img_t2, out_psf_t,  out_img_t, par, INIT = init

; CAOS global common block
common caos_block, tot_iter, this_iter

; initialization of the error code: no error as default
info  = cbd_info()
error = !caos_error.ok


if (par.cbd_method EQ 0) then begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; METHOD CBD A) DESIDERA ET AL 2009  ;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
   fn=out_img_t.image[*,*,0]
   N=(size(out_img_t.image))[1]
   M=(size(out_img_t.image))[2]
   if (size(inp_img_t2.image))[0] EQ 3 then P=(size(inp_img_t2.image))[3] else P=1
   
   if this_iter eq 1 then begin
      psf=init.psf 
   endif else begin
      psf=out_psf_t.image
   endelse
   
                                ;read of the initial image
   if (par.ok_image NE 0B) and (this_iter EQ 1) then begin
      fits_read, par.filename, fn
   endif
   
                                ;image's normalization to c 
   if this_iter EQ 1 then begin
      fntot=total(double(fn))
      fn=init.c*(fn/fntot)
   endif
   
   psfn=make_array(N,M,P,/complex,value=0.)
   if P NE 1 then begin
      for j=0,P-1 do begin
         psfn[*,*,j]=fft(shift(psf[*,*,j],N/2,M/2),-1,/double)*N*M
      endfor
   endif else begin
      psfn=fft(shift(psf,N/2,M/2),-1,/double)*N*M
   endelse
   
   temp=make_array(N,M,/complex)
   
                                ;Choice of deconvolution method
   case par.method of
      
      0: begin                  ; OS-EM method
         for i=0,par.iter_obj-1 do begin
            fn=osemcbd(P, fn, N, M, psfn, inp_img_t2.image, inp_img_t2.background[*,*,0:P-1], init.c)
         endfor
      end
      
      1: begin                  ; LR/EM method
         for i=0,par.iter_obj-1 do begin
            fn=lrmcbd(P, fn, N, M, psfn, inp_img_t2.image, inp_img_t2.background[*,*,0:P-1], init.c)
         endfor
      end
      
   endcase
   
   if (par.ok_filter eq 1) and (par.who_filter eq 1) then begin
      fil=make_array(N,M,/double)
      for i=0,P-1 do begin
         fil=fil+init.filter[*,*,i]
      endfor
      fil=fil < 1. 
      fn=apply_filter(fn,fil)
                                ;fn=(fn/total(double(fn))) 
   endif
   fn=double(fn)
   out_img_t.image[*,*,0]=fn
   
                                ;error :: NEW ::
   if (par.save_rho2) then begin
      if (par.err_type) then begin
                                ;point-like object --> magnitude
         init.rho2[this_iter-1]=0.
         for i=0,n_elements(init.obj_nz)-1 do begin
            m_star = cbd_magnitude(total(fn[init.ind_obj_nz[i,*]],/double)/par.err_eff, lambda=par.err_lambda, $
                                   width=par.err_width, delta_t = par.err_time, surf = par.err_surf)
            init.rho2[this_iter-1]=init.rho2[this_iter-1]+(abs(m_star-init.obj_magn[i])/init.obj_magn[i])
         endfor
         init.rho2[this_iter-1]=init.rho2[this_iter-1]/n_elements(init.obj_nz)
      endif else begin
                                ;diffuse object --> RMSE
         init.rho2[this_iter-1] = sqrt(total(abs(fn-init.true_obj)^2,/DOUBLE))/init.obj_norm2
      endelse
   endif
   
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;; Inner step on the PSF
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   psf=double(fn)
   psft=fft(shift(psf,N/2,M/2),-1,/double)*N*M
   
   fn=out_psf_t.image
   
   if (par.ok_psf_start NE 0B) and (this_iter EQ 1) then begin
      for j=0,P-1 do begin
         fn[*,*,j]=init.psf[*,*,j]
      endfor
   endif
   
   for j=0,P-1 do begin
      for i=0,par.iter_psf-1 do begin
         fnt=FFT(fn[*,*,j],-1,/double)*N*M
         temp=(psft*fnt)
         temp=FFT(temp,1,/double)/(N*M)
         temp=double(temp+inp_img_t2.background[*,*,j])
         temp=inp_img_t2.image[*,*,j]/temp
         temp=FFT(temp,-1,/double)*N*M
         temp=(CONJ(psft)*temp)
         temp=double(FFT(temp,1,/double)/(N*M))
         fn[*,*,j]=temp*fn[*,*,j]
         fn[*,*,j]=fn[*,*,j]>0. ;mettere positivizzazione furba!
         
         if (par.ok_filter eq 1) then begin
            fn[*,*,j]=apply_filter(fn[*,*,j],init.filter[*,*,j])
            fn[*,*,j]=(fn[*,*,j]/(total(double(fn[*,*,j]))))
         endif else begin
            fn[*,*,j]=(fn[*,*,j]/(total(double(fn[*,*,j]))))
         endelse      
      endfor
      if (par.ok_strehl eq 1) then begin ;filtering of the results
         temp=adjust_strehl(fn[*,*,j],init.tf_gaussian,par.strehl[j], par.max_ide, N, M)
         out_psf_t.image[*,*,j]=double(temp)
      endif else begin
         out_psf_t.image[*,*,j]=double(fn[*,*,j])
      endelse
                                ;error  :: NEW ::
      if (par.save_rho1) then begin
                                ;diffuse object --> RMSE
         init.rho1[this_iter-1,j]=sqrt(total(abs(out_psf_t.image[*,*,j]-init.true_psf[*,*,j])^2,/DOUBLE))/sqrt(total(init.true_psf[*,*,j]*init.true_psf[*,*,j]))
      endif
   endfor
   
   den = make_array(N,M,P)
   for j=0,P-1 do begin
      x=out_img_t.image[*,*,0]    ; red'd object
      h=out_psf_t.image[*,*,j]    ; red'd psf
      den[*,*,j]=float(FFT(FFT(x,-1,/DOUBLE)*FFT(SHIFT(h,N/2,M/2),-1,/DOUBLE),1,/DOUBLE)*N*M)
      den[*,*,j]+=inp_img_t2.background[*,*,j]
   endfor
   temp = inp_img_t2.image/den
   temp = total(inp_img_t2.image*alog(temp),/DOUBLE) + total(den,/DOUBLE) - total(inp_img_t2.image,/DOUBLE)
                                ;discrepancy :: NEW ::
   if (par.save_discrepancy) then init.discr[this_iter-1]=temp/(0.5*N*M*P)
   if (par.save_obj_fun) then init.obj_fun[this_iter-1]=temp
   
                                ;write optional outputs
   if (par.save_rho1) then begin
      printf, init.unit_r1, this_iter-1, init.rho1[this_iter-1,*]
      if this_iter GE LONG(tot_iter) then close,init.unit_r1
   endif
   if (par.save_rho2) then begin
      printf, init.unit_r2, this_iter-1, init.rho2[this_iter-1]
      if this_iter GE LONG(tot_iter) then close,init.unit_r2
   endif
   if (par.save_discrepancy) then begin
      printf, init.unit_d, this_iter-1, init.discr[this_iter-1]
      if this_iter GE LONG(tot_iter) then close,init.unit_d
   endif
   if (par.save_obj_fun) then begin
      printf, init.unit_o, this_iter-1, init.obj_fun[this_iter-1]
      if this_iter GE LONG(tot_iter) then close,init.unit_o
   endif
   
   
endif else begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; METHOD CBD B) PRATO ET AL          ;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   stopcrit=1
   SGP_init=2
   tol0=1e-7

   dim = size(init.psf,/dimension)
   if n_elements(dim) eq 2 then num = 1 else num = dim(2)

   if this_iter EQ 1 then begin
   ;;; First step on the image
      if (par.save_rho2) then begin
         if (par.err_type EQ 1) then begin
            cbd_sgp_deblurring_bound_multi, init.psf, inp_img_t2.image, x, iter, err, discr, times, $
                                            maxit = par.iter_obj, bg = inp_img_t2.background, $
                                            initialization = SGP_init, obj = init.true_obj, $
                                            verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit, $
                                            magn_par = init.magn_par
         endif else begin
            cbd_sgp_deblurring_bound_multi, init.psf, inp_img_t2.image, x, iter, err, discr, times, $
                                            maxit = par.iter_obj, bg = inp_img_t2.background, $
                                            initialization = SGP_init, obj = init.true_obj, $
                                            verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit
         endelse
         init.rho2[0] = err[n_elements(err)-1]
      endif else begin
         cbd_sgp_deblurring_bound_multi, init.psf, inp_img_t2.image, x, iter, err, discr, times, $
                                         maxit = par.iter_obj, bg = inp_img_t2.background, $
                                         initialization = SGP_init, $
                                         verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit
      endelse
      if (par.save_discrepancy) then init.discr[0]=discr[n_elements(discr)-1]/2.*product(dim)
      if (par.save_obj_fun) then init.obj_fun[0]=discr[n_elements(discr)-1]
      out_img_t.image=x
                                ;write optional outputs
      if (par.save_rho2) then begin
         printf, init.unit_r2, -1, init.rho2[0]
      endif
      if (par.save_discrepancy) then begin
         printf, init.unit_d, -1, init.discr[0]
      endif
      if (par.save_obj_fun) then begin
         printf, init.unit_o, -1, init.obj_fun[0]
      endif
   endif

   ;restoring previous results
   x = out_img_t.image
   psf = out_psf_t.image

   ;OUTER CYCLE :::: 
   c = total(x)                 ; to normalize the imaging matrix in the psf cycle
   
   ;;; Inner step on the psf
   for it_num_psf = 0, num - 1 do begin
      if (par.save_rho1) then begin
         cbd_sgp_deblurring_bound_multi, x/c, reform(inp_img_t2.image[*,*,it_num_psf])/c, psf_temp, iter, err, discr, times, $
                                         maxit = par.iter_psf, bg = inp_img_t2.background/c, $
                                         initialization = reform(psf[*,*,it_num_psf]), obj = reform(init.true_psf[*,*,it_num_psf]), $
                                         verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit, $
                                         omega = 'low_up_eq', upp_bound = init.upp_bound, $
                                         data_scaling = 'N'
         init.rho1[this_iter-1,it_num_psf] = err[n_elements(err)-1]
      endif else begin
         cbd_sgp_deblurring_bound_multi, x/c, reform(inp_img_t2.image[*,*,it_num_psf])/c, psf_temp, iter, err, discr, times, $
                                         maxit = par.iter_psf, bg = inp_img_t2.background/c, $
                                         initialization = reform(psf[*,*,it_num_psf]), $
                                         verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit, $
                                         omega = 'low_up_eq', upp_bound = init.upp_bound, $
                                         data_scaling = 'N'
      endelse
      psf[*,*,it_num_psf] = psf_temp
   endfor
   
   
    ;;; Inner step on the image
   if (par.save_rho2) then begin
      if (par.err_type EQ 1) then begin
         cbd_sgp_deblurring_bound_multi, psf, inp_img_t2.image, x, iter, err, discr, times, $
                                         maxit = par.iter_obj, bg = inp_img_t2.background, initialization = x, $
                                         obj = init.true_obj, verb = par.sgp_verbosity, tol = tol0, $
                                         stopcrit = stopcrit, magn_par = init.magn_par
      endif else begin
         cbd_sgp_deblurring_bound_multi, psf, inp_img_t2.image, x, iter, err, discr, times, $
                                         maxit = par.iter_obj, bg = inp_img_t2.background, initialization = x, $
                                         obj = init.true_obj, verb = par.sgp_verbosity, tol = tol0, $
                                         stopcrit = stopcrit
      endelse
      init.rho2[this_iter] = err[n_elements(err)-1]
   endif else begin
      cbd_sgp_deblurring_bound_multi, psf, inp_img_t2.image, x, iter, err, discr, times, $
                                      maxit = par.iter_obj, bg = inp_img_t2.background, initialization = x, $
                                      verb = par.sgp_verbosity, tol = tol0, stopcrit = stopcrit
   endelse
   if (par.save_discrepancy) then init.discr[this_iter] = discr[n_elements(discr)-1]/2.*product(dim)
   if (par.save_obj_fun) then init.obj_fun[this_iter] = discr[n_elements(discr)-1]
   
   out_img_t.image[*,*,0]=x
   out_psf_t.image=psf
   
   
                                ;write optional outputs
   if (par.save_rho1) then begin
      printf, init.unit_r1, this_iter-1, init.rho1[this_iter-1,*]
      if this_iter GE LONG(tot_iter) then close,init.unit_r1
   endif
   if (par.save_rho2) then begin
      printf, init.unit_r2, this_iter-1, init.rho2[this_iter]
      if this_iter GE LONG(tot_iter) then close,init.unit_r2
   endif
   if (par.save_discrepancy) then begin
      printf, init.unit_d, this_iter-1, init.discr[this_iter]
      if this_iter GE LONG(tot_iter) then close,init.unit_d
   endif
   if (par.save_obj_fun) then begin
      printf, init.unit_o, this_iter-1, init.obj_fun[this_iter]
      if this_iter GE LONG(tot_iter) then close,init.unit_o
   endif

endelse   

;update the headers (every keyword is written on both the outputs)
if this_iter GE LONG(tot_iter) then begin
   header =  *out_img_t.header
   if par.cbd_method EQ 0 then begin
;DESIDERA ET AL keywords 
      fxaddpar, header, 'CBD_ALG', par.cbd_method, '[CBD] Desidera et al (2009) algorithm'
      if par.ok_filter then begin
         fxaddpar, header, 'BAND_FIL', par.ok_filter, '[CBD] Apply a band filter'
         if par.ok_manage_filter EQ 0 then begin
            fxaddpar, out_psf_t.header, 'BAND_USER', par.filtername, '[CBD] Band filter file name'
         endif else begin
            fxaddpar, header, 'BAND_DIA', par.d, '[CBD] Band filter - diameter [m]'
            fxaddpar, header, 'BAND_RES', par.pix, '[CBD] Band filter - resolut [arcsec]'
            fxaddpar, header, 'BAND_DIM', par.npixel, '[CBD] Band filter - pixel'
            fxaddpar, header, 'BAND_ANG', par.angles, '[CBD] Band filter - angles'
            fxaddpar, header, 'BAND_BL', par.baseline, '[CBD] Band filter - baseline [m]'
            fxaddpar, header, 'BAND_LEN', par.lambda, '[CBD] Band filter - lambda [m]'
            fxaddpar, header, 'BAND_WID', par.width, '[CBD] Band filter - width [m]'
            fxaddpar, header, 'BAND_TYP', par.filtertype, '[CBD] Band filter type [st-ga-bu]'
         endelse
      endif
      if par.ok_strehl then begin
         fxaddpar, header, 'STREHL_C', par.ok_strehl, '[CBD] Apply the Strehl Constraint'
         a=where(par.strehl GT 0, N_SR)
         for k=0,N_SR-1 do fxaddpar, header, 'SR'+strtrim(k,1), par.strehl[k], '[CBD] Strehl value (mean)'
         fxaddpar, header, 'MAX_IDE', par.max_ide, '[CBD] Maximum value of the ID PSF'
      endif
      fxaddpar, header, 'PSF_INIT', par.ok_psf, '[CBD] PSF initialization (Autocorr/User)'
      if par.ok_psf then fxaddpar, header, 'PSF_FILE', par.filename2, '[CBD] User-defined PSF init'
      fxaddpar, header, 'OBJ_INIT', par.ok_image, '[CBD] OBJ initialization (Constant/User)'
      if par.ok_image then fxaddpar, header, 'OBJ_FILE', par.filename, '[CBD] User-defined OBJ init'
      fxaddpar, header, 'PSFSTART', par.ok_psf_start, '[CBD] PSF BOX start (constant/previous)'
   endif else begin
;PRATO ET AL keywords
      fxaddpar, header, 'CBD_ALG', par.cbd_method, '[CBD] Prato et al (2015) algorithm'
      fxaddpar, header, 'STREHL_C', par.ok_strehl, '[CBD] Apply the Strehl Constraint'
      a=where(par.strehl GT 0, N_SR)
      for k=0,N_SR-1 do fxaddpar, header, 'SR'+strtrim(k,1), par.strehl[k], '[CBD] Strehl value (mean)'
      fxaddpar, header, 'MAX_IDE', par.max_ide, '[CBD] Maximum value of the ID PSF'
      a = ["AutoCorr of ID","ID + Constant","Gaussian Blurring","Autocorr. of img","User-defined"]
      fxaddpar, header, 'PSF_INIT', a[par.psf_init], '[CBD] PSF initialization'
      if par.psf_init LE 2 then fxaddpar, header, 'PSF_IDE', par.psf_id_filename, '[CBD] Ideal PSF filename'   
      if par.psf_init EQ 4 then fxaddpar, header, 'PSF_USER', par.psf_userdefined, '[CBD] User-defined PSF filename'   
   endelse 
;Other common keywords 
   fxaddpar, header, 'ITER_OBJ', par.iter_obj, '[CBD] number of iterations on the OBJ'
   fxaddpar, header, 'ITER_PSF', par.iter_psf, '[CBD] number of iterations on the PSF'
   fxaddpar, header, 'ITER_TOT', par.iter_psf, '[CBD] global number of iterations'

;save the header into the two outputs
   *out_img_t.header=header
   *out_psf_t.header=header
endif

;end of cbd_prog.pro
end
