; $Id: cnv.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cnv
;
; ROUTINE'S PURPOSE:
;    cnv manages the simulation for the CoNVolution (CNV) module,
;    that is to call the module's initialization routine cnv_init at the first
;         iteration of the simulation project.
;
; MODULE'S PURPOSE:
;    CNV permits to make a convolution between a 2D (3D) PSFs and an object
;    2D (3D) map. The parameters related to the observing conditions (efficiency,
;    integration time, telescope surface collecting surface) can be chosen.
;    The pixel size of the resulting cube of images is driven by the PSFs pixel
;    size, whatever is the object map one. Both the object map and the PSFs
;    arrays are by the way put in 2^N (N integer) pixels arrays, if they are
;    not originally 2^N arrays. The maximum size for this last operation is
;    set to 2048 pixels (that corresponds to N=11).
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = cnv(inp_img_t, inp_src_t, out_img_t, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t: structure of type img_t.
;    inp_src_t: structure of type src_t.
;    par      : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_img_t: structure of type "img_t" containing the characteristics of
;               the convolved map, the following fields are contained in
;               this structure:
;
;out_img_t =                         $
;{                                   $
;   data_type  : out_type[0]         , $
;   data_status: !caos_data.valid    , $
;   image      : image               , $ ; IMAGE with same resolution as PSF.
;   npixel     : nb_pix_opt          , $ ; Number of pixels (per subaperture)
;   resolution : inp_img_t.resolution, $ ; Pixel size [arcsec/pix]
;   lambda     : lambda[0]           , $ ; Mean detection wavelength [m]
;   width      : width[0]            , $ ; Wavelength band width [m]
;   time_integ : par.int_time        , $ ; Integration time for each image [s]
;   time_delay : 0.                  , $ ; [NOT USED WITHIN AIRY]
;   psf        : 0B                  , $ ; IMAGE=1, PSF=0
;   background : back_frame          , $ ; sky background array [ph/pixel] 
;   snr        : 0.                  , $ ; [NOT YET EVALUATED]
;   header     : PTR_NEW('')           $ ; header of the two structures merged
;}
;*out_img_t.header = header ; update the pointer
;
; KEYWORD PARAMETERS:
;    INIT: initialisation data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    The object map and the PSFs arrays are put to 2^N (N integer) pixels
;    arrays if they are not originally 2^N arrays.
;    Doing it the maximum size is set to 2048 pixels (that corresponds to N=11).
;
; CALLED NON-IDL FUNCTIONS:
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                    -call to cnv_prog eliminated (useless).
;                    -parameters needed for image complete definition now
;                     considered (total efficiency, telescope collecting
;                     surface, integration time).
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;                   : december 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -use of variable "time_integ" activated for the output structure.
;
; MODULE MODIFICATION HISTORY:
;    module written : version beta,
;                     Serge  Correia   (OAA) [correia@arcetri.astro.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it].
;    modifications  : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                   : for version 5.0;
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added the possibility to convolve objects and PSF
;                     "many to many" (it was "one to many").
;                    -added possibility to simulate multi-frame images.
;                     Multi-frame images cannot be saved (by WFT), but
;                     only auto-reduced by PRE module.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -The header of the input/output image is kept
;                     up-to-date with convolution information
;                    -background is now an array, which dimensions are
;                     the same of the input image.
;
;-
;
function cnv, inp_img_t, inp_src_t, out_img_t, par, INIT=init

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialisation
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then $
   error = cnv_init(inp_img_t, inp_src_t, out_img_t, par, INIT=init)

return, error
end
