; $Id: cnv_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cnv_gui
;
; PURPOSE:
;    cnv_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the CoNVolution (CNV) module.
;    a parameter file called cnv_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    the file is stored in the project directory proj_name located
;    in the working directory.
;    (see cnv.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = cnv_gui(n_module, $
;                    proj_name )
;
; INPUTS:
;    n_module : number associated to the instance of the CNV module
;               [integer scalar -- n_module > 0].
;    proj_name: name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    none
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -parameters needed for image complete definition added
;                     (total efficiency, telescope collecting surface,
;                     integration time).
;                   : may 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -added possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : february 2003,
;                     Barbara Anconelli (DISI) [1997s019@educ.disi.unige.it]:
;                    -real acquisition buttons added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(cnv_info()).help stuff added (instead of !caos_env.help).
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition button commented...
;                   : september 2005
;                     Barbara Anconelli (DISI) [anconelli@disiunige.it]
;                    - added help for windows version
;                   : december 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -output tag "int_time" activate to export Integration
;                     Times in the output structure
;                   : may 2007, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simpler way to call AIRY_HELP.
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;
;-
;
;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;
pro cnv_gui_event, event

; cnv_gui error management block
common error_block, error

; read the GUI state structure
widget_control, event.top, GET_UVALUE=state

; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

; handle all the other events
widget_control, event.id, GET_UVALUE=uvalue
case uvalue of

   'tel_surf'   : state.par.tel_surf = event.value

   'int_time'  : begin
                   dummyx = event.x
                   dummyy = event.y
                   widget_control, state.id.int_time, get_Value=dummy
                   if dummyx eq 0 then begin
                    state.par.int_time[dummyy,dummyx]=double(dummy[dummyx,dummyy])
                   endif else begin
                    state.par.int_time[dummyy,dummyx]=double(fix(dummy[dummyx,dummyy]))
                    widget_control, state.id.int_time,  SET_Value=transpose(state.par.int_time)             
                   endelse
                  end
    
   'reset_int_time' : begin
                          state.par.int_time[*,0]=0
                          state.par.int_time[*,1]=1
                          widget_control, state.id.int_time, SET_Value=transpose(state.par.int_time)
                       end




   'efficiency' : state.par.efficiency = event.value

;;   'enable_real': begin
;;        widget_control, state.id.real, GET_VALUE=dummy
;;        if (dummy EQ 1) then begin
;;          widget_control, state.id.tel_surf, SENSITIVE=0
;;          widget_control, state.id.efficiency, SENSITIVE=0
;;          widget_control, state.id.int_time_1, SENSITIVE=0
;;          widget_control, state.id.int_time_2, SENSITIVE=0
;;          widget_control, state.id.int_time_3, SENSITIVE=0
;;          widget_control, state.id.int_time_4, SENSITIVE=0
;;          widget_control, state.id.int_time_5, SENSITIVE=0
;;          widget_control, state.id.int_time_6, SENSITIVE=0
;;          ;set the default values for all the widgets and for the parameters
;;          state.par.tel_surf    = 94.
;;          state.par.int_time[0] = 1.
;;          state.par.int_time[1] = 1.
;;          state.par.int_time[2] = 1.
;;          state.par.int_time[3] = 1.
;;          state.par.int_time[4] = 1.
;;          state.par.int_time[5] = 1.
;;          state.par.efficiency  = .4
;;          state.par.real_acq    = 1B
;;          widget_control,  state.id.tel_surf,   SET_VALUE=state.par.tel_surf
;;          widget_control,  state.id.int_time_1, SET_VALUE=state.par.int_time[0]
;;          widget_control,  state.id.int_time_2, SET_VALUE=state.par.int_time[1]
;;          widget_control,  state.id.int_time_3, SET_VALUE=state.par.int_time[2]
;;          widget_control,  state.id.int_time_4, SET_VALUE=state.par.int_time[3]
;;          widget_control,  state.id.int_time_5, SET_VALUE=state.par.int_time[4]
;;          widget_control,  state.id.int_time_6, SET_VALUE=state.par.int_time[5]
;;          widget_control,  state.id.efficiency, SET_VALUE=state.par.efficiency
;;          widget_control,  state.id.real,       SET_VALUE=state.par.real_acq
;;        endif else begin
;;          widget_control, state.id.tel_surf,   /SENSITIVE
;;          widget_control, state.id.efficiency, /SENSITIVE
;;          widget_control, state.id.int_time_1, /SENSITIVE
;;          widget_control, state.id.int_time_2, /SENSITIVE
;;          widget_control, state.id.int_time_3, /SENSITIVE
;;          widget_control, state.id.int_time_4, /SENSITIVE
;;          widget_control, state.id.int_time_5, /SENSITIVE
;;          widget_control, state.id.int_time_6, /SENSITIVE
;;          widget_control, state.id.real,       SET_VALUE=0
;;          state.par.real_acq=0B
;;        endelse
;;        par=state.par
;;      end

   'save': begin

      ; check before saving the parameter file if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='CNV warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='CNV information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      ; save the parameter data file
      par = state.par
      save, par, FILENAME = state.sav_file

      ; kill the GUI returning a null error
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return

   end

   ; handle event from standard help button
   'help': begin
   online_help, book=(cnv_info()).help, /FULL_PATH
;         widget_control, /HOURGLASS
;        
;       CASE !VERSION.OS_FAMILY OF
;         'Windows'   :     begin
;                    spawn, !caos_env.browser+" "+(cnv_info()).help,/NOSHELL
;                   end
;         else    :  begin
;                     spawn, !caos_env.browser+" "+(cnv_info()).help+" &"
;                   end
;       ENDCASE
   end
   ; handle event from standard restore button
   'restore': begin

      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      ; update the current module number
      par.module.n_module = state.par.module.n_module

      ; set the default values for all the widgets
      widget_control,  state.id.tel_surf,     SET_VALUE=par.tel_surf
      widget_control,  state.id.int_time,     SET_VALUE=transpose(par.int_time)
      widget_control,  state.id.efficiency,   SET_VALUE=par.efficiency
;;      widget_control,  state.id.real,         SET_VALUE=0
      widget_control,  state.id.tel_surf,     /SENSITIVE
      widget_control,  state.id.efficiency,   /SENSITIVE
      ; update the state structure
      state.par = par

   end

   'cancel': begin
      error = !caos_error.cancel
      widget_control, event.top, /DESTROY
      return
   end

endcase

; write the GUI state structure
widget_control, event.top, SET_UVALUE=state

return

end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function cnv_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
common error_block, error

; retrieve the module information
info = cnv_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a new one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = $
   { $
   root_base_id            : 0L, $
   efficiency              : 0L, $
    int_time               : 0., $
    reset_int_time         : 0L, $
;;   real                  : 0L, $
   tel_surf                : 0L  $
   }

; build the state structure were par, id, sav_file and def_file will be stored
; (and passed to cnv_gui_event).
state = $
   {    $
   sav_file: sav_file, $
   def_file: def_file, $
   id      : id,       $
   par     : par       $
   }

; root base definition
modal = n_elements(group) ne 0
dummy = strupcase(info.mod_name)+' parameters setting GUI'
root_base_id = widget_base(TITLE=dummy, MODAL=modal, /COL, GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

param = widget_base(root_base_id, FRAME=10, ROW=4)
dummy = widget_label(param, VALUE='parameters', /FRAME)
param = widget_base(param, xsize=300,ysize=270,/COL)

state.id.efficiency = cw_field(param,                                        $
                               TITLE='Overall efficiency:          ',        $
                               /FLOAT,                                       $
                               VALUE=state.par.efficiency,                   $
                               UVALUE='efficiency',                          $
                               XSIZE=11,                                     $
                               /ALL_EVENTS                                   )


int_time_table=widget_base(param,xsize=230,ysize=160)
rlabel=make_array(360,/string,value='Angle #')
num=indgen(360)
rlabel=rlabel+strcompress(string(num),/remove_all)

button_int_time=widget_base(int_time_table,xoffset=8,yoffset=18,xsize=64,ysize=26)
state.id.reset_int_time=widget_button(button_int_time,$
                      VALUE="Reset Val", $
                      UVALUE="reset_int_time")

state.id.int_time = widget_table(int_time_table,$
                       ALIGNMENT=2,$
                       xoffset=5,yoffset=15,$
                       value=transpose(state.par.int_time),$
                       uvalue="int_time",$
                       editable=1,$
                       COLUMN_LABELS=['Integration x frame [s]','Frames #'],$
                       ROW_LABELS=rlabel,$
                       COLUMN_WIDTHS=[145,60],$
                       y_scroll_size=5   )


label=widget_label(param,value=' ')
state.id.tel_surf = cw_field(param,                                          $
                               TITLE='Telescope'+string(10B)+'collecting surface [m^2]:    ', $
                             /FLOAT,                                         $
                             VALUE=state.par.tel_surf,                       $
                             UVALUE='tel_surf',                              $
                             XSIZE=11,                                       $
                             /ALL_EVENTS                                     )

;;state.id.real = cw_bgroup(param,                         $
;;                          ['Real Time Acquisition  '],   $
;;                          SET_VALUE=0.,                  $
;;                          UVALUE="enable_real",          $
;;                          /NONEXCLUSIVE                  )
;;
;;;controls for the button of real acquisition
;;if (state.par.real_acq EQ 1B) then begin
;;        widget_control, state.id.real,       SET_VALUE=1
;;        widget_control, state.id.tel_surf,   SENSITIVE=0
;;        widget_control, state.id.efficiency, SENSITIVE=0
;;        widget_control, state.id.int_time_1, SENSITIVE=0
;;        widget_control, state.id.int_time_2, SENSITIVE=0
;;        widget_control, state.id.int_time_3, SENSITIVE=0
;;        widget_control, state.id.int_time_4, SENSITIVE=0
;;        widget_control, state.id.int_time_5, SENSITIVE=0
;;        widget_control, state.id.int_time_6, SENSITIVE=0
;;endif else begin
;;        widget_control, state.id.tel_surf,   /SENSITIVE
;;        widget_control, state.id.efficiency, /SENSITIVE
;;        widget_control, state.id.int_time_1, /SENSITIVE
;;        widget_control, state.id.int_time_2, /SENSITIVE
;;        widget_control, state.id.int_time_3, /SENSITIVE
;;        widget_control, state.id.int_time_4, /SENSITIVE
;;        widget_control, state.id.int_time_5, /SENSITIVE
;;        widget_control, state.id.int_time_6, /SENSITIVE
;;endelse

; button base for control buttons (standard buttons)
btn_base_id = widget_base(root_base_id, FRAME=10, /ROW)
   dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
   cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
   if modal then widget_control, cancel_id, /CANCEL_BUTTON
   dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                         UVALUE="restore"                         )
   save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS",  $
                           UVALUE="save"                          )
   if modal then widget_control, save_id, /DEFAULT_BUTTON

; save the state structure of the GUI in the top base uvalue
widget_control, root_base_id, SET_UVALUE=state

; draw the GUI
widget_control, root_base_id, /REALIZE

; launch xmanager
xmanager, 'cnv_gui', root_base_id, GROUP_LEADER=group

return, error
end
