; $Id: cnv_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cnv_init
;
; PURPOSE:
;    cnv_init executes the initialization for the CoNVolution
;    (CNV) module, that is:
;
;       0- check the formal validity of the input/output structure
;       1- initialize the output structure(s) out_cnv_t
;
;    (see cnv.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's initialization routine
;
; CALLING SEQUENCE:
;    error = cnv_init(inp_img_t,  $ ; img_t input structure
;                     inp_src_t,  $ ; src_t input structure
;                     out_cnv_t,  $ ; img_t output structure
;                     par,        $ ; parameters structure
;                     INIT=init   $ ; initialization data structure
;                     )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see cnv.pro's help for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -parameters needed for image complete definition now
;                     considered (total efficiency, telescope collecting
;                     surface, integration time).
;                   : may 2001, Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : october 2002,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -odd AND even cases of bn of pixels for PSF now handled.
;                   : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition buttons added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition buttons commented...
;                   : february 2004,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it]:
;                    -odd and even cases of nb of pixels for PSF now handled
;                     ALSO for multiple frames (this case was forgotten during
;                     the "october 2002" modifications).
;                   : december 2004
; 	                  Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -one-value-vector IDL6+ bug corrected (ex. n_phot->n_phot[0]).
;                   : december 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -initialized tag "int_time" containing the Integration Times for
;                     each frame of the data cube (max 6 frames) for the output structure.
;                   : march 2007, 
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -modified the routine (after MAKE THE CONVOLUTION 
;                     OBJECT-PSF) in order to make convolution of 
;                     single/multiple objects with single/multiple psfs  
;                    -added tag "angle" to save the baseline angles
;                   : march 2010,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -The normalization to the correct flux is performed just 
;                     before the convolution. Please note that I also changed
;                     convolution.pro
;-
;
function cnv_init, inp_img_t, inp_src_t, out_img_t, par, INIT=init
  
; error code initialization
  error = !caos_error.ok
  
; retrieve the module's informations
  info = cnv_info()
  
; get the individual output structure types
  if info.out_type ne '' then out_type = str_sep(info.out_type,",")
  
; STANDARD CHECKS
;
; compute and test the requested number of cnv arguments
  n_par = 1                     ; the parameter structure is always present within the arguments
  if info.inp_type ne '' then begin
     inp_type = str_sep(info.inp_type,",")
     n_inp    = n_elements(inp_type)
  endif else n_inp = 0
  if info.out_type ne '' then begin
     out_type = str_sep(info.out_type,",")
     n_out    = n_elements(out_type)
  endif else n_out = 0
  n_par = n_par + n_inp + n_out
  if n_params() ne n_par then message, 'wrong number of arguments'
  
; test the parameter structure
  if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
     message, 'CNV error: par must be a structure'
  if n ne 1 then message, 'CNV error: par cannot be a vector of structures'
  if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
     message, 'par must be a parameter structure for the module CNV'
  
; check the input arguments
  
; test if any optional input exists
  if n_inp gt 0 then begin
     inp_opt = info.inp_opt
  endif
  
  dummy = test_type(inp_img_t, TYPE=type)
  if type eq 0 then begin       ; undefined variable
     inp_img_t = $
        {        $
        data_type  : inp_type[0],         $
        data_status: !caos_data.not_valid $
        }
  endif
  if test_type(inp_img_t, /STRUC, N_EL=n, TYPE=type) then $
     message, 'CNV error: wrong definition for the first input.'
  if n ne 1 then message, $
     'CNV error: first input cannot be a vector of structures'
  
; test the data type
  if inp_type[0] ne 'gen_t' then begin
     if inp_img_t.data_type ne inp_type[0] then                $
        message, 'wrong input data type: '+inp_img_t.data_type $
                 +' ('+inp_type[0]+' expected).'
  endif
  
  if inp_img_t.data_status eq !caos_data.not_valid and not inp_opt[0] then $
     message, 'undefined input is not allowed'
  
  
  dummy = test_type(inp_src_t, TYPE=type)
  if type eq 0 then begin       ; undefined variable
     inp_src_t = $
        {        $
        data_type  : inp_type[1],         $
        data_status: !caos_data.not_valid $
        }
  endif
  
  if test_type(inp_src_t, /STRUC, N_EL=n, TYPE=type) then $
     message, 'CNV error: wrong definition for the second input.'
  
  if n ne 1 then message, $
     'CNV error: second input cannot be a vector of structures'
  
; test the data type
  if inp_src_t.data_type ne inp_type[1] then                $
     message, 'wrong input data type: '+inp_src_t.data_type $
              +' ('+inp_type[1]+' expected).'
  if inp_src_t.data_status eq !caos_data.not_valid and not inp_opt[1] then $
     message, 'undefined input is not allowed'
  

; MAKE THE CONVOLUTION OBJECT-PSF
;
;header 
header = *inp_img_t.header

; rebin the source map with the PSF pixel size
  nb_pix_map = (size(inp_src_t.map))[1]
  nb_pix     = round(nb_pix_map*inp_src_t.scale_xy*206265./inp_img_t.resolution) 
  
; Test that the array size of the source is larger or equal of that of the PSF
  if (size(inp_img_t.image))[1] gt nb_pix then message, $
     'CNV ERROR: the source array size is smaller than that of the PSF !!'
  
;number of frames in OBJECT(src) and PSF(img) (cube or single image)
  if (size(inp_src_t.map))[0] eq 3 then      $
     nb_frame_src=(size(inp_src_t.map))[3]   $
  else nb_frame_src = 1
  if (size(inp_img_t.image))[0] eq 3 then    $
     nb_frame_img=(size(inp_img_t.image))[3] $
  else nb_frame_img = 1
  nb_frame=nb_frame_src > nb_frame_img
  
;check if nb_frame_src (if GT 1) is equal to nb_frame_img (if GT 1)
; in order to convolve each obj-frame with the correspondent psf-frame
  if (nb_frame_src GT 1) AND (nb_frame_img GT 1) AND $
     (nb_frame_src NE nb_frame_img) then $
        message, "CNV ERROR: the number of frames of the object must be equal"+ $
                 " to the PSF's one."
  
; search the closest 2^n array size
  
  n_max      = 11
  nb_pix_opt = 0
  i = 1
  while (2^i lt nb_pix) do i = i+1
  if (i le n_max) then nb_pix_opt = 2^i else nb_pix_opt = 0.
  
  print, '-------------------------------------------------------------------'
  print, 'CNV:'
  print, "number of frames - OBJ                     = ", nb_frame_src
  print, "number of frames - PSF                     = ", nb_frame_img
  print, "resulting array dimension [px]             = ", nb_pix
  if i gt n_max then begin
     print, "resulting closest 2^n array dimension greater than 2048 pixels"
  endif else begin
     print, "resulting closest 2^n array dim. [px]      = ", long(nb_pix_opt)
  endelse
  
; find the n_phot value in the vector from OBJ and compute the nb of ph/frame
  n_phot = inp_src_t.n_phot[where(inp_src_t.n_phot ne 0.)]
  n_phot=n_phot[0]
  n_phot_frame = fltarr(nb_frame)
  
  
;convolution cycle
  
  psf_map = fltarr(nb_pix_opt, nb_pix_opt, nb_frame_img)
  src_map = fltarr(nb_pix_opt, nb_pix_opt, nb_frame_src)
  image   = fltarr(nb_pix_opt, nb_pix_opt, nb_frame)
  
  dummy   = (size(inp_img_t.image))[1]
  
  for j=0,nb_frame-1 do begin
     if nb_frame_src EQ 1 then begin
        tmp_src=inp_src_t.map
        k_src=0
     endif else begin
        tmp_src=inp_src_t.map[*,*,j]
        k_src=1
     endelse
     if nb_frame_img EQ 1 then begin
        tmp_img=inp_img_t.image 
        k_img=0
     endif else begin
        tmp_img=inp_img_t.image[*,*,j]
        k_img=1
     endelse
     n_phot_frame[j]=n_phot[0]*par.efficiency*par.tel_surf*par.int_time[j]
                                ; put the rebinned source into the closest 2^n array dimension
     if nb_pix mod 2 eq 0 then begin
                                ; even case
        src_map[nb_pix_opt/2-nb_pix/2:nb_pix_opt/2+nb_pix/2-1, $
                nb_pix_opt/2-nb_pix/2:nb_pix_opt/2+nb_pix/2-1, $
                j*k_src] = congrid(tmp_src, nb_pix, nb_pix)
     endif else begin
                                ; odd case
        src_map[nb_pix_opt/2-nb_pix/2:nb_pix_opt/2+nb_pix/2, $
                nb_pix_opt/2-nb_pix/2:nb_pix_opt/2+nb_pix/2, $
                j*k_src] = congrid(tmp_src, nb_pix, nb_pix)
     endelse
                                ; renormalise the map after the rebin and add n_phot of the j-th frame
     src_map[0,0,j*k_src]=src_map[*,*,j*k_src]/$
                          total(src_map[*,*,j*k_src],/double)*n_phot_frame[j]
     
                                ; put the PSF (normalized) into the source array size
     if dummy/2 eq dummy/2. then begin
        psf_map[nb_pix_opt/2-dummy/2:nb_pix_opt/2+dummy/2-1, $
                nb_pix_opt/2-dummy/2:nb_pix_opt/2+dummy/2-1, $
                j*k_img] = tmp_img / total(tmp_img,/double)
     endif else begin
        psf_map[nb_pix_opt/2-dummy/2:nb_pix_opt/2+dummy/2, $
                nb_pix_opt/2-dummy/2:nb_pix_opt/2+dummy/2, $
                j*k_img] = tmp_img / total(tmp_img,/double)
     endelse
     
                                ;convolution !
     convolution, src_map[*,*,j*k_src], psf_map[*,*,j*k_img], temp, nb_pix_opt
     image[*,*,j] = temp
  endfor
  
; find the background value in the src_t data structure (it's a number!)
  background = inp_src_t.background[where(inp_src_t.n_phot ne 0.)]
; the background now is an array!
  back_frame = fltarr(nb_pix_opt, nb_pix_opt, nb_frame)
;convert the background from [ph/s/m^2/arcsec^2] into [ph/pixel] for each frame
;and update the keyword in the header with the value
  fxaddpar, header, 'SKY_TYPE', 'VALUE', '[CNV] Sky background (from OBJ)' 
  fxaddpar, header, 'SKY_NB', nb_frame, '[CNV] Sky background number of values'
  for j=0,nb_frame-1 do begin
     back_val = background[0]*par.efficiency*par.tel_surf*par.int_time[j]*(inp_img_t.resolution)^2
     back_frame[*,*,j]= back_val
     fxaddpar, header, 'SKY_BG'+strtrim(j,1), back_val, '[CNV] Sky background value' 
  endfor

; find the wavelength value in the vector
lambda = inp_src_t.lambda[where(inp_src_t.n_phot ne 0.)]

; find the bandwidth value in the vector
width = inp_src_t.width[where(inp_src_t.n_phot ne 0.)]

;add specific keywords (convolution parameters) to the header
for k=0,nb_frame-1 do fxaddpar, header, 'EXPTIME'+strtrim(k,1),  par.int_time[j], '[CNV] Exposure Time [s]'
fxaddpar, header, 'COL_AREA',  par.tel_surf, '[CNV] Collecting area [m^2]'
fxaddpar, header, 'EFFICIEN',  par.efficiency, '[CNV] Overall efficiency'

; UPDATE THE HEADERS AND MERGE THEM (src --> img)
;retrieve header from src_t and clean
h = *inp_src_t.header
fxhclean, h
;add keywords to final header (img_t)
for k=0,n_elements(h)-1 do begin
   keyw = strmid(h[k],0,8)
   valu= fxpar(h, keyw, comment=comm) 
   ;search for an existing keyword and add only new keywords
   ;existing keywords are not updated 
   a=fxpar(header, keyw, count=b)
   if ((b EQ 0) AND (keyw NE 'COMMENT ')) then fxaddpar, header, keyw, valu, comm
   if keyw EQ 'COMMENT ' then begin
      for jj=0,n_elements(valu)-1 do  fxaddpar, header, keyw, valu[jj], comm[jj]
   endif
endfor

; INITIALIZE THE OUTPUT STRUCTURE
;
out_img_t =                         $
{                                   $
   data_type  : out_type[0]         , $
   data_status: !caos_data.valid    , $
   image      : image               , $ ; IMAGE with same resolution as PSF.
   npixel     : nb_pix_opt          , $ ; Number of pixels (per subaperture)
   resolution : inp_img_t.resolution, $ ; Pixel size [arcsec/pix]
   lambda     : lambda[0]           , $ ; Mean detection wavelength [m]
   width      : width[0]            , $ ; Wavelength band width [m]
   time_integ : par.int_time        , $ ; Integration time for each image [s]
   time_delay : 0.                  , $ ; [NOT USED WITHIN AIRY]
   psf        : 0B                  , $ ; IMAGE=1, PSF=0
   background : back_frame          , $ ; sky background array [ph/pixel] 
   snr        : 0.                  , $ ; [NOT YET EVALUATED]
   header     : PTR_NEW('')           $ ; header of the two structures merged
}

*out_img_t.header = header ; update the pointer
return, error
end
