; $Id: dec.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    dec
;
; ROUTINE'S PURPOSE:
;    DEC manages the simulation for the image deconvolution (DEC) module,
;    that is:
;       1-call the module's initialization routine dec_init at the first
;         iteration of the simulation project,
;       2-call the module's program routine dec_prog.
;
; MODULE'S PURPOSE:
;
;       DEC performs the deconvolution of a cube of images (i.e. P images
;    corresponding to different orientations of the baseline). There are
;    five different methods for image deconvolution, according to the noise
;    model. For Poisson noise, the Richardson-Lucy (RL) and the Ordered-Subset 
;    Expectation Maximization (OSEM) methods are available. Recently
;    (from v.6.1) we added the Scaled Gradient Projection (SGP) method
;    [7]. It is an optimized method for solving the minimization of the KL
;    divergence. 
;       As concerns Gaussian noise both the Image Space Reconstruction
;    Algorithm (ISRA) and the OS-ISRA are implemented. (see [1] for details).
;       
;       DEC also allows to reduce the boundary effects in the reconstructed
;    image (see [2,8] for details).
;       When the object is complex, as a high dynamic range image, a regularized 
;    version of the algorithm can give better results. In this cases, eight 
;    different regularizations are available in DEC: Tikhonov (0th order), 
;    Tikhonov (2nd order, aka Laplacian), Cross-Entropy, Edge Preserving MRF, 
;    High Dynamic Range (HDR), Mistral, Hypersurface, and Tikhonov 1st
;    order (see [8] for details). Depending on the regularization chosen, 
;    one or a couple of parameters control the efficacy of the regularization. 
;
;       For RL and OSEM methods, two accelerated versions of the
;    algorithms are also available, following the Biggs and Andrews
;    method (see [3,4] for details).

;       The background (now an array) evaluation performed by PRE
;    module is used to restore the object with a correct sky-value.
;
;       In this new version of DEC up to five different stopping rules are given. 
;    Concerning SGP, RL, and OSEM algorithms, five stopping rules are implemented:
;    1) Set a total number of iterations and stop the algorithm when this number
;       is reached. This is valid for all available methods;
;    2) Stop the iterations when the relative L2-norm of the difference between
;       the current iteration and the previous one is less than a given tolerance.
;       i.e.   || f(k) - f(k-1) ||^2 / || f(k) ||^2 < tol
;       where f(k) is the current iteration, f(k-1) the previous one,
;       and ||.||^2 the L2-norm (squared) 
;    3) Stop the iterations when the total functional is approximately constant, 
;       according to a user-defined tolerance. 
;       i.e. | J(k) - J(k-1) | / | J(k) | < tol
;       where J(k) is the objective function defined as J_0 + beta * J_R, 
;       J_0 the kullback-leibler divergence, J_R the regularization
;       functional, and beta the regularization parameter (see [8] for
;       more information). In the case of no-regularization (beta=0), 
;       then J becomes simply J_0.
;    4) Stop the iteration when the discrepancy function crosses a
;       given tolerance (that should be around 1). This 
;       criterion is called discrepancy principle for Poisson data (see [5]).   
;    5) Stop the iterations when the relative r.m.s error reaches a minimum 
;       value. This stopping rule can be used only in the case of numerical 
;       simulations, when the true object is known.
;    The first and the last stopping rules are also available for ISRA and 
;    OS-ISRA. 
;
;       In DEC we also implemented an algorithm for super-resolving compact objects
;    such as a binary system with an angular separation smaller than the angular
;    resolution of the telescope. (see [1,6] for details). The method is based on 
;    a simple modification of the RL/OSEM (and SGP)  method and in
;    general consist of 3 steps: the first one requires a large number
;    of RL/OSEM (or SGP) iterations (typically 10^4, smaller numbers
;    in the case of SGP), which are used to estimate the domain of the unresolved 
;    object; the second one is a RL/OSEM restoration (typically 5000 iterations) 
;    initialized with the mask of the domain, as estimated in step 1. These two 
;    steps are used to estimate the positions of the two stars while their 
;    magnitudes can be obtained in a possible 3rd step by solving a simple 
;    least-squares problem. The first two steps are included in DEC. 
;    In the second step it is possible to choose, in the GUI, the image and the 
;    mask used to initialized the method. The mask is an image with values 0/1. 
;    There are 3 kind of masks: the 1st is a mask based on percentage of the 
;    image maximum, the 2nd one is a circular mask, and the 3rd is a 
;    user-defined mask.
;
;       Finally the discrepancy, the objective function, and - in the 
;    simulation case - the restoration error can be also saved as
;    ASCII files. 
;
;    References:
;    [1] La Camera et al., 2012, "AIRY: a complete tool for the simulation 
;        and the reconstruction of astronomical images",
;        Proc. SPIE 8445, 84453D 
;    [2] Anconelli et al., 2006, "Reduction of boundary effects in multiple 
;        image deconvolution with an application to LBT LINC-NIRVANA", 
;        A&A 448, 1217–1224.
;    [3] Biggs and Andrews, 1997, "Acceleration of iterative image 
;        restoration algorithms", Applied Optics 36, 1766. 
;    [4] Biggs and Andrews, 1998, "Asymmetric iterative blind deconvolution 
;        of multiframe images", Proc. SPIE 3461, 328–338.
;    [5] Bertero et al., 2010, "A discrepancy principle for Poisson data",
;        Inverse Problems 26, 5004.
;    [6] Anconelli et al., 2005, "Restoration of interferometric images. 
;        IV. An algorithm for super-resolution of stellar systems", Astronomy 
;        and Astrophysics 431, 747–755.
;    [7] Bonettini et al, 2009, "A scaled gradient projection
;        method for constrained image deblurring", Inverse
;        Problems 25, 015002.
;    [8] Prato et al, 2016, "Reconstruction of high-dynamic
;        range Poisson data with an application to Io imaging",
;        Inverse Problems, submitted.
;    [9] La Camera et al, 2016, "The Software Package AIRY 7.0:
;        new efficient deconvolution methods for post-adaptive
;        optics data", Proc. SPIE, in Adaptive Optics Systems V (AS106)
;
;    Please pay attention to the fact that the TOP BOX-INPUT of the module dec
;    is supposed to receive images, while the BOTTOM BOX-INPUT of the module
;    dec is supposed to receive PSFs.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = dec(inp_img_t1, inp_img_t2, out_img_t, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t1: structure of type img_t, corresponding to PSF's
;    inp_img_t2: structure of type img_t, corresponding to images
;    par       : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_img_t: structure of type "img_t" containing the characteristics of
;               the deconvolved object, the following fields are contained in
;               this structure:
;
;out_img_t = 				      $
;   {          				      $
;   data_type  : out_type[0]     	    , $
;   data_status: !caos_data.valid	    , $
;   image      : img  	                    , $
;   psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
;   npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
;   resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
;   lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
;   width      : inp_img_t2.width            , $ ; Wavelength band width [m]
;   time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
;   time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
;   background : inp_img_t2.background       , $ ; background
;   snr        : inp_img_t2.snr              , $ ; snr
;   header     : PTR_NEW('')                   $ ; header of the output file
;   }
;*out_img_t.header = *inp_img_t2.header ; update the pointer
;
; KEYWORD PARAMETERS:
;    INIT: initialization data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : int scalar. Total number of iteration during the 
;                 simulation run.
;    this_iter  : int scalar. Number of the current iteration. It is
;                 defined only while status eq !caos_status.run.
;                 (this_iter >= 1).  
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    -other routine from the folders .../dec/dec_gui_lib 
;     and .../dec/dec_lib
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: november 1999, 
;                     Massimo Barbati (USG) [massimo@dirac.disi.unige.it].
;    modifications  : march 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to dec version of the CAOS structure (v 2.0).
;                   : may 2000,
;                     Massimo Barbati (USG) [massimo@dirac.disi.unige.it]:
;                    -Help form, Restore form.
;                    -Variables Names, Comments.
;                   : july 2000,
;                     Massimo Barbati (USG) [massimo@dirac.disi.unige.it]:
;                    -beta version for package AIRY.
;                   : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed + a few minor changes.
;                   : Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -help completed.
;
; MODULE MODIFICATION HISTORY:
;    module written : Massimo Barbati (USG) [massimo@dirac.disi.unige.it].
;    modifications  : for version 2.0,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -second output of type "src_t" added.
;                    -super-resolution stuff added.
;                   : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                   : for version 2.0, 
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -added the possibility to choose between the OS-EM method
;                     and the LR/EM method (par.method added). 
;                   : for version 2.0,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"real acquisition" button commented and 2nd output of type
;                    "src_t" momentaneously eliminated.
;                    -normalization constant "c" from PAR to INIT structure.
;                    -background substracted from PSF too now.
;                   : for version 3.0,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                    -added a dec accelerate version of OS-EM method (par.acc 
;                     addded)
;                    -ISRA method inserted
;                    -added regularization for OS-EM method.
;                    -visualization of a mask added.
;                    -restructuration of interface
;                    -choice of differents regularizations type added
;                   : for version 4.0,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -use of variable "time_integ" activated for the output structure
;                    -added possibility to reduce boundary effect
;                   : for version 5.0,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -eliminated variables concerning regularizations with 
;                     alpha=0.5 and alpha=1.
;                    -image type (img_t) completed (tag "angle" added).                  
;                   : may 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -routine debugged.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : April 2012 (for version 6.0),
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -module rewritten: new method (OS-ISRA), new regularizations,
;                     new accelerations added. Debugging of existing algorithms.
;                     Old regularizations and old accelerations have been removed. 
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added gain_ron parameter (gain of the detector, in order to 
;                     convert RON from electrons/pixel to ADU/pixel);
;                    -added SGP algorithm. Also added the
;                     "deconvolve_sgp" sub-routine.
;                   : November 2013,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr],
;                    -1D case debugged (NN=N before right definition of N).
;                   : December 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -1D case added (debugged) for SGP method.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array (from previous modules)
;                    -two new regularizations added:
;                     HYPERSURFACE and TIKHONOV 1st ORDER
;                    -all the 8 regularizations are available for SGP now
;                     (i.e. they are available for ALL methods).
;                    -added optional output save: objective function
;                    -GUI debugged (small changes on text and
;                     displacement of the widgets)
;                    -stopping rules modified (same rules for RL,OSEM,SGP)
;                    -The header of the input/output image is kept
;                     up-to-date with deconvolution information
;                   : July 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -bug correction on the GUI (for regularization HS)
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -if a stopping rule is satisfied, the variable "last_iter" is 
;                     used for storing current iteration, next "this_iter" is set to the 
;                     total iteration number, and then "tot_iter" is updated with 
;                     the current iteration number (i.e the very last iteration).  
;-
;
;;;;;;;;;;;;;;;
; module code ;
;;;;;;;;;;;;;;;
;
function dec, inp_img_t1, inp_img_t2, out_img_t, par, INIT=init

common caos_block, tot_iter, this_iter

; initialization of the error code: set to "no error" as default
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = dec_init(inp_img_t1, inp_img_t2, out_img_t, par, INIT = init)
endif else begin
   ; run section
   error = dec_prog(inp_img_t1, inp_img_t2, out_img_t, par, INIT = init)
endelse

; back to calling program.
return, error
end
