; $Id: dec_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    dec_gen_default
;
; PURPOSE:
;    dec_gen_default generates the default parameter structure
;    for the dec module and save it in the right location.
;    the user doesn't need to use dec_gen_default, it is used
;    only for developing and upgrading purposes.
;
; CATEGORY:
;    utility
;
; CALLING SEQUENCE:
;    dec_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: may 2000,
;                     Massimo Barbati (OAA) [].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                    -useless INIT file stuff eliminated.
;                   : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition button added.
;                    -super-resolution stuff added.
;                   : march 2003, 
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -add the possibility to choose between the OS-EM method and 
;                     the LR/EM method (par.method added). 
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition button commented and second output of type
;                    "src_t" momentaneously eliminated.
;                    -normalization constant "c" from PAR to INIT structure.
;                    -background substracted from PSF too now.
;                    : july 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -comment about mask saving added
;                    -second type of mask and choice of mask added
;                    -add procedure dec_gui_set, state
;                   : may 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added dec_mask_gui.pro for the choice of a mask
;                   : august 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added dec_reg_gui.pro for regularization with OS-EM method.
;                    -visualization of a mask added.
;                    -menu for the choice of acceleration step added.
;                    -new design of interface
;                   : november 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -choice of different regularizations type added
;                   : january 2006,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added possibility to reduce boundary effect
;                   : for version 4.0
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -use of variable "time_integ" activated for the output 
;                     structure
;                   : may 2007,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -eliminated variables concerning regularizations with 
;                     alpha=0.5 and alpha=1.
;                   : April 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it],
;                    -eliminated some parameters and added new parameters for 
;                     version 6.0.
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added gain_ron parameter (gain of the detector, in order to 
;                     convert RON from electrons/pixel to ADU/pixel);
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -changes: par.mu --> par.beta and 
;                              par.threshold --> par.delta
;                              par.init_image --> par.init_alg
;                    -added: par.flux_con (fllux constrained)
;                    -regularizations from 6 to 8 added
;                    -addded save_obj_fun and save_obj_fun_file
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -Added optional input for the case of Multi-Step
;                     Method (MSM)
;-
;
pro dec_gen_default

info = dec_info()               ; obtain module infos

; Generate module description structure.

module = gen_def_module(info.mod_name, info.ver)

path = !caos_env.work+!caos_env.delim   
     
par =                  $
   {                   $
   dec,                $        ; Structure named DEC
   module    : module, $
   method    : 4,      $        ; deconvolution method (0=RL, 1=OSEM, 2=ISRA, 3=OSISRA, 4=SGP)
                                ; default is SGP
   reg	     : 0,      $        ; regularizations [0=none, 1=TI0, 2=TI2, 3=ENT, 4=MRF, 
                                ;                  5=HDR,  6=MIS, 7=HYP, 8=TI1]
   beta        : 0.0d,   $        ; regularization parameter 
   delta       : 0.0d,   $        ; threshold for the regularization
   flux_con    : 0B,   $        ;     
   acc       : 0,      $        ; accelerations [0=none, 1=acc.lin, 2=acc.exp]
   ee_threshold: 0.95, $        ; threshold of the PSF's Enclosed Energy 
   bound     : 0B,     $        ; reduce boundary effects [0=no, 1=yes]
   init_alg  : 0B,     $        ; initialization of algorithm [0=constant array, 
                                ;                              1=user-defined image]
   filename_image  : path, $    ; initial image file address
   init_mask   : 0B,   $        ; super-resolution mask  [0=no, 1=yes]
   msm : 0B, $                  ; multi-step method case (load a diffuse image)
   msm_file : '', $             ; the diffuse image
   mask_choice: 0.,    $        ; consider perc-mask, diam-mask or user-defined mask
   mask_npix  : 128,   $        ; mask dimension (must be the same dim of initial img)
   mask_perc : 0.5,    $        ; Percentage of image maximum (for domain selection=mask)
   mask_diam : 0,      $        ; nb of pixel for the diameter of the mask
   save_mask : 0B,     $        ; save the mask? [0=no, 1=yes]
   mask_filename : path,   $    ; mask file name
   user_mask : path, $          ; user-defined mask filename 
   stopping_rule : 0, $         ;stopping criteria: 0=iter>TOT_ITER; 1=Discrepancy crosses 1; 
                                ; 2=total functional is approx. constant; 3=min of restoration error.
   stop_tol : 1.d0, $           ;tolerance for the given stopping rule 
   load_obj_file : '' , $       ; object map for the stopping rule nb 3 
   save_discrepancy: 0, $       ; save the discrepancy values
   save_obj_fun:0,      $       ; save the objective function values
   save_discrepancy_file : '', $ ; filename (ASCII file)
   save_obj_fun_file : '',     $ ; filename (ASCII file) 
   save_rho : 0, $               ;save the restoration error values
   save_rho_file : '', $         ;filename of restoration error (ASCII file)
   verb : 0B $ ; verbosity of the algorithm(s)
   }   

; Save the default parameter structure
save, par, FILENAME=info.def_file

end
