; $Id: dec_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    dec_gui
;
; PURPOSE:
;    dec_gui generates the Graphical User Interface (GUI) for setting the
;    parameters of the dec module. A parameter file
;    called dec_yyyyy.sav is created, where yyyyy is the number n_module
;    associated to the the module instance.  The file is stored in the
;    project directory proj_name located in the working directory.
;    (see dec.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = dec_gui(n_module, proj_name)
;
; INPUTS:
;    n_module  : number associated to the intance of the dec module.
;                [integer scalar -- n_module > 0].
;    proj_name : name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    dec_mask_gui.pro
;    dec_reg_gui.pro
;
; ROUTINE MODIFICATION HISTORY:
;   routine written: july 2000,
;                    Massimo Barbati (USG) [massimo@dirac.disi.unige.it].
;   modifications  : july 2000,
;                    Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                   -a few minor modifications (help, default buttons, warning
;                    messages).
;                  : february 2003,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -real time acquisition button and second output of type
;                    "src_t" added.
;                   -super-resolution stuff added.
;                  : february 2003,
;                    Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                   -adapted to version 4.0 of the whole system CAOS
;                    (variable "pack_name" added, and variable "mod_type"
;                    changed into "mod_name").
;                   -(dec_info()).help stuff added (instead of !caos_env.help).
;                  : march 2003,
;                    Serge Correia (AIP) [scorreia@aip.de]:
;                   -added the possibility to choose between the OS-EM
;                    method and the LR/EM method (par.method added).
;                  : june 2003,
;                    Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -real acquisition button commented and second output of type
;                    "src_t" momentaneously eliminated.
;                   -normalization constant "c" from PAR to INIT structure.
;                   -background substracted from PSF too now.
;                  : july 2003,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -comment about mask saving added
;                   -second type of mask and choice of mask added
;                   -added procedure dec_gui_set.
;                  : february 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added possibility of choice of a accelerate version for the
;                    OS-EM method
;                  : mai 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added dec_mask_gui.pro for the choice of a mask
;                  : august 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added dec_reg_gui.pro for regularization with OS-EM method.
;                   -visualization of a mask added.
;                   -menu for the choice of acceleration step added.
;                   -restructuration of interface
;                  : november 2004,
;                   Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -choice of differents regularizations type added
;                  : september 2005,
;                   -added help for Windows version
;                  : january 2006,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added possibility to reduce boundary effect
;                   : for version 4.0,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -use of variable "time_integ" activated for the output structure
;                   : for version 5.0, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simpler way to call AIRY_HELP.
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : may 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -routine debugged.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : April 2012, 
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it],
;                    -GUI rewritten with TAB, (Super-Resolution) MASK sub-GUI  
;                     changed, Regularizations sub-GUI eliminated.
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added gain_ron parameter (gain of the detector, in order to 
;                     convert RON from electrons/pixel to ADU/pixel);
;                    -added SGP method and relative stopping
;                     rules. Modifications to the dec_gui_set procedure.
;                   : July 2015,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added MISTRAL regularization, similar to the one
;                     described in Mugnier et al, 2004, JOSAA, vol. 21(10)
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -changes: par.mu --> par.beta and 
;                              par.threshold --> par.delta
;                    -two new regularizations added:
;                     HYPERSURFACE and TIKHONOV 1st ORDER
;                    -all the 8 regularizations are available for SGP now
;                     (i.e. they are available for ALL methods).
;                    -SGP has (now) a flux constrained
;                    -added optional output save: objective function
;                    -GUI debugged (small changes on text and
;                     displacement of the widgets; added verbosity)
;                    -stopping rules modified (same rules for RL,OSEM,SGP)
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -Added optional input for the case of Multi-Step
;                     Method (MSM)
;                   : July 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -bug correction on the GUI (for regularization HS)
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro dec_gui_set, state

; CONTROLS ON SELECTED METHOD AND OTHER STUFF
  widget_control, state.id.method, SET_COMBOBOX_SELECT=state.par.method
;RL && OSEM stopping rules ++ SGP 
  if ((state.par.method LE 1) OR (state.par.method EQ 4)) then begin
     stopping= ['#1 [ALL] Stop when this_iter > tot_iter (set up in the worksheet)       ', $
                '#2 [ALL] Stop when || f(k) - f(k-1) ||^2 < tol * || f(k) ||^2      OR #1', $
                '#3 [ALL] Stop when | J(k) - J(k-1) | < tol * | J(k) |              OR #1', $
                '#4 [ALL] Stop when the discrepancy crosses the tolerance value     OR #1', $
                '#5 [SIM] Stop when the restoration error reaches a minimum value   OR #1']
     widget_control, state.id.stopping_rule, SET_VALUE=stopping
     widget_control, state.id.stopping_rule, SET_COMBOBOX_SELECT=state.par.stopping_rule
     widget_control, state.id.stop_tol, /SENSITIVE
     widget_control, state.id.save_discrepancy, /SENSITIVE
     widget_control, state.id.save_discrepancy_file, /SENSITIVE
     widget_control, state.id.save_obj_fun, /SENSITIVE
     widget_control, state.id.save_obj_fun_file, /SENSITIVE
     if (state.par.stopping_rule GE 1) AND (state.par.stopping_rule LE 3)then begin
        widget_control, state.id.stop_tol, /SENSITIVE 
     endif else begin
        widget_control, state.id.stop_tol, SENSITIVE=0 
     endelse
     widget_control, state.id.save_discrepancy, /SENSITIVE
     widget_control, state.id.save_discrepancy_file, /SENSITIVE
     widget_control, state.id.save_obj_fun, /SENSITIVE
     widget_control, state.id.save_obj_fun_file, /SENSITIVE
  endif
  
;ISRA && OS-ISRA stopping rules
  if ((state.par.method EQ 2) OR (state.par.method EQ 3)) then begin   
     state.par.msm = 0B
     state.par.msm_file = ''
     widget_control, state.id.msm, SET_VALUE=state.par.msm
     widget_control, state.id.msm_file, SET_VALUE=state.par.msm_file     
     widget_control, state.id.msm, SENSITIVE=0
     widget_control, state.id.msm_file, SENSITIVE=0
     stopping= ['#1 [ALL] Stop when this_iter > tot_iter (set up in the worksheet)       ', $
                '#2 [SIM] Stop when the restoration error reaches a minimum value   OR #1']
     widget_control, state.id.stopping_rule, SET_VALUE=stopping
     widget_control, state.id.stopping_rule, SET_COMBOBOX_SELECT=state.par.stopping_rule
     if state.par.stopping_rule GT 0 then begin
        widget_control, state.id.stopping_rule, SET_COMBOBOX_SELECT=state.par.stopping_rule-3
     endif
     widget_control, state.id.stop_tol, SENSITIVE=0
     state.par.save_discrepancy=0
     state.par.save_obj_fun=0
     widget_control, state.id.save_discrepancy, SET_VALUE=state.par.save_discrepancy                 
     widget_control, state.id.save_discrepancy, SENSITIVE=0
     widget_control, state.id.save_discrepancy_file, SENSITIVE=0
     widget_control, state.id.save_obj_fun, SET_VALUE=state.par.save_obj_fun                 
     widget_control, state.id.save_obj_fun, SENSITIVE=0
     widget_control, state.id.save_obj_fun_file, SENSITIVE=0
  endif

;controls for the button of initial image
  if (state.par.init_alg EQ 1B ) then begin
     widget_control, state.id.filename_image, /SENSITIVE
     widget_control, state.id.init_alg, SET_VALUE=1
  endif else begin
     widget_control, state.id.filename_image, SENSITIVE=0
     widget_control, state.id.init_alg, SET_VALUE=0
  endelse

;-----------------------------------------------------
 
; FLUX CONSTRAINED
  if (state.par.method EQ 4) then widget_control, state.id.flux_con, /SENSITIVE else $
     widget_control, state.id.flux_con, SENSITIVE=0
  
;  BOUNDARY
  if state.par.bound then widget_control, state.id.ee_threshold, /SENSITIVE else $
     widget_control, state.id.ee_threshold, SENSITIVE=0
    
; REGULARIZATIONS
  widget_control, state.id.reg, /SENSITIVE
  widget_control, state.id.reg, SET_COMBOBOX_SELECT=state.par.reg
  case state.par.reg of
     0 : begin                  ;NONE
        widget_control, state.id.beta, SENSITIVE=0
        widget_control, state.id.delta, SENSITIVE=0
     end
     1: begin                   ;TIKHONOV (0th order)
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     2: begin                   ;LAPLACIAN (second order tikhonov)
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     3: begin                   ;ENTROPY
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     4: begin                   ;EDGE-PRESERVING MRF
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     5: begin                   ;HDR
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     6: begin                   ;MISTRAL
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     7: begin                   ;hypersurface
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     8: begin                   ;tikhonov first order
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     9: begin
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     else:                      ;NOTHING TO DO!
  endcase

; ACCELERATIONS
  if (state.par.method LE 3) then begin
     widget_control, state.id.acc, /SENSITIVE
     widget_control, state.id.acc, SET_COMBOBOX_SELECT=state.par.acc
  endif else begin
     widget_control, state.id.acc, SENSITIVE=0
     widget_control, state.id.acc, SET_COMBOBOX_SELECT=0
  endelse

;------------------------------  
;controls for the button of mask (Super-resolution)
  if (state.par.init_mask EQ 1B ) then begin
     widget_control, state.id.button_mask, /SENSITIVE
     widget_control, state.id.msm, SENSITIVE=0
  endif else begin
     widget_control, state.id.button_mask, SENSITIVE=0
     widget_control, state.id.msm, /SENSITIVE
  endelse

;controls for the Multi-Step Method (MSM)
  if (state.par.msm EQ 1B) then begin
     widget_control, state.id.msm_file, /SENSITIVE
     widget_control, state.id.init_mask, SENSITIVE=0
  endif else begin
     widget_control, state.id.msm_file, SENSITIVE=0
     widget_control, state.id.init_mask, /SENSITIVE
  endelse
  
  
;OPTIONAL OUTPUTS ---------------------------------------------
  if (state.par.save_discrepancy) then begin
     widget_control, state.id.save_discrepancy_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_discrepancy_file, SENSITIVE=0
  endelse   
  
  if (state.par.save_obj_fun) then begin
     widget_control, state.id.save_obj_fun_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_obj_fun_file, SENSITIVE=0
  endelse   
  
  if (state.par.save_rho) then begin
     widget_control, state.id.save_rho_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_rho_file, SENSITIVE=0
  endelse   
;--------------------------------------------------------------
  
 
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dec_gui_event, event

   common error_block, error

   widget_control, event.top, get_uvalue=state

   if tag_names(event, /structure_name) eq 'widget_kill_request' then begin
       error = !caos_error.cancel
       widget_control, event.top, /destroy
   endif

   ; events
   widget_control, event.id, get_uvalue = uvalue
   case uvalue of

   ; standard help button
   'help': online_help, book=(dec_info()).help, /FULL_PATH
   
   ; standard restore button:
   'restore' : begin
      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      widget_control, state.id.method,         SET_COMBOBOX_SELECT=par.method
      widget_control, state.id.flux_con,       SET_VALUE=par.flux_con
      widget_control, state.id.verb,           SET_VALUE=par.verb
      widget_control, state.id.acc,            SET_COMBOBOX_SELECT=par.acc
      widget_control, state.id.reg,            SET_COMBOBOX_SELECT=par.reg
      widget_control, state.id.beta,           SET_VALUE=par.beta
      widget_control, state.id.delta,          SET_VALUE=par.delta
      widget_control, state.id.bound,          SET_VALUE=par.bound
      widget_control, state.id.ee_threshold,   SET_VALUE=fix(par.ee_threshold*100)
      widget_control, state.id.init_alg,     SET_VALUE=par.init_alg
      widget_control, state.id.filename_image, SET_VALUE=par.filename_image
      widget_control, state.id.init_mask,      SET_VALUE=par.init_mask
      widget_control, state.id.msm,           SET_VALUE=par.msm
      widget_control, state.id.msm_file,      SET_VALUE=par.msm_file
      widget_control, state.id.stopping_rule,  SET_COMBOBOX_SELECT=par.stopping_rule
      widget_control, state.id.stop_tol, SET_VALUE=par.stop_tol
      widget_control, state.id.load_obj_file,  SET_VALUE=par.load_obj_file
      widget_control, state.id.save_discrepancy, SET_VALUE=par.save_discrepancy
      widget_control, state.id.save_discrepancy_file, SET_VALUE=par.save_discrepancy_file
      widget_control, state.id.save_obj_fun, SET_VALUE=par.save_obj_fun
      widget_control, state.id.save_obj_fun_file, SET_VALUE=par.save_obj_fun_file
      widget_control, state.id.save_rho, SET_VALUE=par.save_rho
      widget_control, state.id.save_rho_file, SET_VALUE=par.save_rho_file
      ; update the current module number
      par.module.n_module = state.par.module.n_module
      ; update the state structure
      state.par = par
      ;update all GUI
      dec_gui_set, state
   end

   ; standard save button:
    'save' : begin

      ; check before saving the parameter file if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='dec warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='dec information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      par = state.par
      save, par, FILENAME = state.sav_file
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return

   end

    ; standard cancel button (exit without saving)
    'cancel': begin
       error = !caos_error.cancel
       WIDGET_CONTROL, event.top, /DESTROY
       return
    end
    
    'method': begin
       state.par.method=event.index
       dec_gui_set, state
    end
    
    'flux_con' : begin
       widget_control, state.id.flux_con, GET_VALUE=dummy
       state.par.flux_con=dummy
       dec_gui_set, state
    end
    
    'bound': begin
       widget_control, state.id.bound, GET_VALUE=dummy
       state.par.bound=dummy
       dec_gui_set, state
    end
    
    'ee_threshold' : state.par.ee_threshold = float(event.value)/100.
        
    'verb' : begin
       widget_control, state.id.verb, GET_VALUE=dummy
       state.par.verb=dummy
    end
    
    'acc': begin
       state.par.acc=event.index
       dec_gui_set, state
    end
    
    'reg': begin
       state.par.reg=event.index
       dec_gui_set, state
    end

    'beta' : begin
       state.par.beta=event.value
    end
    
    'delta' : begin
       state.par.delta = event.value
    end
    
    
    'init_alg': begin
       widget_control, state.id.init_alg, GET_VALUE=dummy
       state.par.init_alg=dummy
       dec_gui_set, state
    end
    
    'filename_image': begin
       widget_control, state.id.filename_image , GET_VALUE=dummy
       state.par.filename_image=dummy
    end
    
    'init_mask': begin
       widget_control, state.id.init_mask, GET_VALUE=dummy
       state.par.init_mask=dummy
       dec_gui_set, state
    end
    'open_mask_gui': begin	    
       if (state.par.init_mask EQ 1B) then begin 
          title = "DEC mask parameter GUI"
          dummy = dec_mask_gui(title, state, GROUP_LEADER=event.top)
          state.par.mask_choice=dummy.mask_choice
          state.par.mask_npix=dummy.mask_npix
          state.par.mask_perc=dummy.mask_perc
          state.par.mask_diam=dummy.mask_diam
          state.par.user_mask=dummy.user_mask
          state.par.save_mask=dummy.save_mask
          state.par.mask_filename=dummy.mask_filename
       endif
    end
    
    'msm': begin
       state.par.msm = event.select
       dec_gui_set, state
    end
    'msm_file': begin
       state.par.msm_file = event.value
    end
    
    'stopping_rule': begin
       val = event.index
       if ((state.par.method EQ 2) OR (state.par.method EQ 3)) then begin
          if val GT 0 then val = val + 3
       endif
       state.par.stopping_rule=val
       dec_gui_set, state
    end
    'stop_tol': begin
       state.par.stop_tol = event.value
    end
    'load_obj_file': begin
       state.par.load_obj_file=event.value
    end
    'save_discrepancy':begin
       state.par.save_discrepancy = event.select
       dec_gui_set, state
    end
    'save_discrepancy_file':begin
       state.par.save_discrepancy_file=event.value
    end
    'save_obj_fun':begin
       state.par.save_obj_fun = event.select
       dec_gui_set, state
    end
    'save_obj_fun_file':begin
       state.par.save_obj_fun_file=event.value
    end
    'save_rho':begin
       state.par.save_rho = event.select
       dec_gui_set, state
    end
    'save_rho_file':begin
       state.par.save_rho_file=event.value
    end       
    else:                       ;nothing to do!
    endcase
    widget_control, event.top, set_uvalue=state
;    return
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function dec_gui, n_module, proj_name, GROUP_LEADER=group

common error_block, error

; retrieve the module information
info = dec_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a dec one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = $
   { $
   root_base_id    : 0L, $
   method          : 0L, $
   flux_con        : 0L, $
   bound           : 0L, $
   ee_threshold    : 0L, $
   acc             : 0L, $
   reg             : 0L, $
   beta            : 0L, $
   delta           : 0L, $
   verb            : 0L, $
   init_alg        : 0L, $
   filename_image  : 0L, $
   init_mask       : 0L, $
   button_mask     : 0L, $
   msm             : 0L, $
   msm_file        : 0L, $
   stopping_rule   : 0L, $
   stop_tol        : 0L, $
   load_obj_text   : 0L, $
   load_obj_file   : 0L, $
   save_discrepancy : 0L, $
   save_discrepancy_file : 0L, $
   save_obj_fun : 0L, $
   save_obj_fun_file : 0L, $
   save_rho : 0L, $
   save_rho_file : 0L $
   }

state = $
  {     $
  sav_file : sav_file, $
  def_file : def_file, $
  id       : id      , $
  par      : par       $
  }

; root base definition
modal= N_ELEMENTS(group) NE 0
title= STRUPCASE(info.mod_name)+' parameter setting GUI'
root_base_id = WIDGET_BASE(TITLE=title,MODAL=modal,/COL,GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

; parameters base
par_base = widget_base(root_base_id, /FRAME, /ROW)

par_base_id = widget_base(par_base, /COL)

base  = WIDGET_BASE(par_base_id,  /COLUMN)
text  = WIDGET_LABEL(base, VALUE=' DEConvolution module ', /ALIGN_LEFT  )
text  = WIDGET_LABEL(base, VALUE='  [ Top input is for the image(s), '  $
                    +'while bottom input is for the PSF(s). ]'     )

basetab=WIDGET_TAB(par_base_id, location=0, uvalue='')
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabA=WIDGET_BASE(basetab, /COL, TITLE='       DECONVOLUTION METHODS       ')

base_top = WIDGET_BASE(basetabA, /FRAME, /ROW)
base_method = WIDGET_BASE(base_top, /FRAME, /COLUMN)
text  = WIDGET_LABEL(base_method, VALUE='DECONVOLUTION METHOD')
   methods=['Richardson-Lucy or Expectation Maximization    (RL) ', $
            'Ordered-Subset Expectation Maximization      (OSEM) ', $
            'Image Space Restoration Algorithm            (ISRA) ',$
            'Ordered-Subset version of ISRA            (OS-ISRA) ', $
            'Scaled Gradient Projection                    (SGP) ']
state.id.method = widget_combobox(base_method, value=methods, UVALUE='method')
state.id.init_alg = cw_bgroup(base_method, $
                              LABEL_TOP= 'Initialization of the algorithm: ',$
                             ['Constant array','User-defined image'], /COL,   $
                             SET_VALUE=0,  UVALUE='init_alg',   /EXCLUSIVE   )
state.id.filename_image = cw_filename(base_method, TITLE='  ', $
                          VALUE=state.par.filename_image, $
                          UVALUE='filename_image', /ALL_EVENTS )
state.id.verb = cw_bgroup(base_method,  'Verbosity of the selected method',$
                          COLUMN=1,  UVALUE='verb',  /NONEXCLUSIVE, $
                          SET_VALUE=state.par.verb )

base_add = WIDGET_BASE(base_top, /FRAME, /COLUMN)
text  = WIDGET_LABEL(base_add, VALUE='ADD-ONs of selected method')
state.id.flux_con = cw_bgroup(base_add,  'Flux constraint on each reconstruction',$
                              COLUMN=1,  UVALUE='flux_con',  /NONEXCLUSIVE, $
                              SET_VALUE=state.par.flux_con )
state.id.bound = cw_bgroup(base_add,  'Boundary effects reduction',$
                           COLUMN=1,  UVALUE='bound',  /NONEXCLUSIVE, $
                           SET_VALUE=state.par.bound )
state.id.ee_threshold = WIDGET_SLIDER(base_add, $
                                      value=fix(state.par.ee_threshold*100), $
                                      xsize=50, uvalue='ee_threshold', $
                                      title="Percentage of the PSF's Enclosed Energy")
text=widget_label(base_add, value='Regularization:', /ALIGN_LEFT)          
reg=['none', 'Tikhonov - 0th order ', 'Tikhonov - 2nd order (Laplacian)', 'Cross-Entropy', 'Edge-preserving MRF', $
     'High-dynamic range','Mistral', 'Hypersurface', 'Tikhonov - 1st order','L1 Norm'] 
state.id.reg=widget_combobox(base_add, value=reg, uvalue='reg')
state.id.beta = cw_field(base_add, $
                         TITLE='Regularization parameter (Beta):   ', /ROW, xsize=10, $
                         UVALUE='beta', value=state.par.beta, /FLOAT, /ALL_EVENTS)  
state.id.delta = cw_field(base_add, $
                          TITLE='Threshold (Delta):                 ', /ROW, /FLOAT, xsize=10, $
                          UVALUE='delta', value=state.par.delta, /ALL_EVENTS)
text=widget_label(base_add, value='Acceleration:', /ALIGN_LEFT)                           
acc=['none', 'Biggs and Andrews LINEAR acc.', $
     'Biggs and Andrews EXPONENTIAL acc.']
state.id.acc = widget_combobox(base_add,  value=acc, uvalue='acc') 

base_bottom=WIDGET_BASE(basetabA, /FRAME, /COLUMN)
base_mask = WIDGET_BASE(base_bottom, /FRAME, /COL)
base_mask2 = WIDGET_BASE(base_mask, /ROW)
state.id.init_mask = cw_bgroup(base_mask2,  'Super-resolution mask    ',   $
                             COLUMN=2,  SET_VALUE=state.par.init_mask,     $
                             UVALUE='init_mask',  /NONEXCLUSIVE            )
state.id.button_mask = widget_button(base_mask2, VALUE="Edit...", $
                                     UVALUE='open_mask_gui',xsize=150)
text  = WIDGET_LABEL(base_mask,VALUE='[NB: for super-resolution '  $
             +'it is necessary to provide first the initial image, then the mask.]')

base_msm=WIDGET_BASE(base_bottom, /FRAME, /ROW)
state.id.msm = CW_BGROUP(base_msm, $
                         ['[MSM] Multi-Step Method: load diffuse image'], $
                         /COL, set_value=state.par.msm, UVALUE='msm',  /NONEXCLUSIVE)
state.id.msm_file = cw_filename(base_msm, TITLE='Enter the path of the file', $
                                VALUE=state.par.msm_file, UVALUE='msm_file', /ALL_EVENTS )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabB=WIDGET_BASE(basetab, /COL, TITLE='STOPPING RULES AND OPTIONAL OUTPUTS')
base_stopping_rule = WIDGET_BASE(basetabB, /FRAME, /COL)

dummy = widget_label(base_stopping_rule, value='Choose a stopping criterium among:')
   stopping= ['#1 [ALL] Stop when this_iter > tot_iter (set up in the worksheet)       ', $
              '#2 [ALL] Stop when || f(k) - f(k-1) ||^2 < tol * || f(k) ||^2      OR #1', $
              '#3 [ALL] Stop when | J(k) - J(k-1) | < tol * | J(k) |              OR #1', $
              '#4 [ALL] Stop when the discrepancy crosses the tolerance value     OR #1', $
              '#5 [SIM] Stop when the restoration error reaches a minimum value   OR #1']
state.id.stopping_rule = widget_combobox(base_stopping_rule, value=stopping, $
                         uvalue='stopping_rule')


state.id.stop_tol = CW_FIELD(base_stopping_rule, $
            value=state.par.stop_tol, xsize=15,$
            UVALUE='stop_tol', /ROW,/return_events, /FLOAT, $
            TITLE="tolerance of the selected rule:       [ENTER]")
        

base_optional=widget_base(basetabB, /col, /FRAME) 
dummy=widget_label(base_optional, value='Optional outputs: ', /ALIGN_LEFT)
base_optional2=widget_base(base_optional, /row) 

base_optional_sx=widget_base(base_optional2, /col) 
 
state.id.save_discrepancy=CW_BGROUP(base_optional_sx, $
                 ['[ALL] Save the discrepancy value at each iteration'], $
                          /COL, set_value=state.par.save_discrepancy, $
                          UVALUE='save_discrepancy',  /NONEXCLUSIVE)
state.id.save_discrepancy_file = cw_filename(base_optional_sx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_discrepancy_file,      $
                        UVALUE='save_discrepancy_file',             $
                        /ALL_EVENTS                         )

state.id.save_obj_fun=CW_BGROUP(base_optional_sx, $
                 ['[ALL] Save the objective function value at each iteration'], $
                          /COL, set_value=state.par.save_obj_fun, $
                          UVALUE='save_obj_fun',  /NONEXCLUSIVE)
state.id.save_obj_fun_file = cw_filename(base_optional_sx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_obj_fun_file,      $
                        UVALUE='save_obj_fun_file',             $
                        /ALL_EVENTS                         )

base_optional_dx=widget_base(base_optional2, /col) 

state.id.save_rho=CW_BGROUP(base_optional_dx, $
                 ["[SIM] Save the restoration error at each iteration"], $
                          /COL, set_value=state.par.save_rho, $
                          UVALUE='save_rho',  /NONEXCLUSIVE)

;state.id.load_obj_text = widget_label(base_stopping_rule_rho, $
;                        value="[#4 cont'd]         ")    
state.id.load_obj_file = cw_filename(base_optional_dx,       $
                        TITLE='Enter the path of the OBJECT map ', $
                        VALUE=state.par.load_obj_file,      $
                        UVALUE='load_obj_file',             $
                        /ALL_EVENTS   )

state.id.save_rho_file = cw_filename(base_optional_dx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_rho_file,      $
                        UVALUE='save_rho_file',             $
                        /ALL_EVENTS                         )

;Standard buttons
;----------------
base2      = WIDGET_BASE(root_base_id, FRAME=10, /ROW)
help_id    = WIDGET_BUTTON(base2, VALUE='HELP'              ,UVALUE='help')
cancel_id  = WIDGET_BUTTON(base2, VALUE='CANCEL'            ,UVALUE='cancel')
restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS'   ,UVALUE='save')

if modal then widget_control, cancel_id, /cancel_button
if modal then widget_control, save_id, /default_button

widget_control, root_base_id, set_uvalue=state
widget_control, root_base_id, /realize

dec_gui_set, state

xmanager, 'dec_gui', root_base_id, group_leader=group

return, error
end
