; $Id: dec_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       dec_init
;
; PURPOSE:
;       dec_init executes the initialization for the dec module.
;
; CATEGORY:
;       initialization program
;
; CALLING SEQUENCE:
;    error = dec_init(inp_img_t1,  $ ; img_t input structure
;                     inp_img_t2,  $
;                     out_img_t ,  $ ; img_t output structure
;                     par,         $ ; parameters structure
;                     INIT=init    $ ; initialization data structure
;                     )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see dec.pro's header for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: may 2000,
;                     Massimo Barbati (USG) [].
;    modifications  : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -second output of type "src_t" added.
;                    -super-resolution stuff added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : march 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -added the possibility to choose between OS-EM method and
;                     LR/EM method (par.method added).
;                   : june 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition button commented and second output of
;                     type "src_t" momentaneously eliminated.
;                    -normalization constant "c" from PAR to INIT structure.
;                    -background substracted from PSF too now.
;                   : may 2007,
;                     Marcel Carbillet (LUAN) [marcel.carbillet@unice.fr]:
;                    -kc and gc were not defined for the 1D case -> fixed.
;                   : April 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -For version 6.0 - init procedure changed; introduced two
;                     new functions: positivization.pro and zeropadding.pro
;                     (see CAOS_Library and/or pack_lib)
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added gain_ron parameter (gain of the detector, in order to 
;                     convert RON from electrons/pixel to ADU/pixel);
;                    -added SGP algorithm. dec_init now distinguishes between
;                     SGP and other (old) methods.
;                   : November 2013,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr],
;                    -1D case debugged (NN=N before right definition of N).
;                   : December 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -1D case added (debugged) for SGP method.
;                   : January 2015,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -bug corrections (in equalization of 3d images)
;                   : June 2015,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -small corrections in the boundary effects correction part
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array (depending on the case 1D, 2D, 3D)
;                    -the header is saved in the output structure and
;                     eventually modified with the deconvolution parameters.
;                    -added optional output for objective function
;                    -debugged SGP part of the software
;                    -the initializations of the methods have been moved  
;                     inside to two specific procedures (in the dec_lib)
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -Added optional input for the case of Multi-Step
;                     Method (MSM)
;                   : Januray 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added common block (called parameters) to this routine
;                     and to dec_prog in order to correctly save the parameter 
;                     "par.delta" (delta_ori) in header. (for future reference: 
;                     this can be used for any other parameter)
;-
;
function dec_init, inp_img_t1, inp_img_t2, out_img_t, par, INIT=init

common caos_block, tot_iter, this_iter
common parameters, delta_ori

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = dec_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")


n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'dec error: par must be a structure'
if n ne 1 then message, 'dec error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module dec'


dummy = test_type(inp_img_t1, TYPE=type)
if type eq 0 then begin         ; undefined variable

   inp_img_t1 = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif

if test_type(inp_img_t1, /STRUC, N_EL=n, TYPE=type) then $
   message, 'dec error: wrong definition for the first input.'
if n ne 1 then message, $
   'dec error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t1.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t1.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_img_t1.data_status eq !caos_data.not_valid  then $
      message, 'undefined input is not allowed'

dummy = test_type(inp_img_t2, TYPE=type)
if type eq 0 then begin                ; undefined variable

   inp_img_t2 = $
      {        $
    data_type  : out_type[0]     , $
    data_status: !caos_data.valid $
      }
endif

if test_type(inp_img_t2, /STRUC, N_EL=n, TYPE=type) then $
   message, 'dec error: wrong definition for the second input.'

if n ne 1 then message, $
   'dec error: second input cannot be a vector of structures'

; test the data type
if inp_img_t2.data_type ne inp_type[1] then                $
   message, 'wrong input data type: '+inp_img_t.data_type $
           +' ('+inp_type[1]+' expected).'
;if inp_img_t2.data_status eq !caos_data.not_valid and not inp_opt[1] then $
;   message, 'undefined input is not allowed'


dummy = where(tag_names(par) eq 'INIT_FILE',count)
if (count GT 0) then begin
    ;; the module allows the saving and restoring of the initialization
    ;; structure
   if par.init_file ne '' then begin
      save_init    = par.init_save
      restore_init = not save_init
      file_exists  = (findfile(par.init_file))[0] ne ''
   endif
endif


; initialization of the error code: no error as default
info  = dec_info()
error = !caos_error.ok

;Multi-frame case --> error 
pos=where(inp_img_t2.time_integ[*,1] gt 1,num)
if ((size(inp_img_t2.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_img_t2.image=reform(inp_img_t2.image,(size(inp_img_t2.image))[1],$
                                               (size(inp_img_t2.image))[2],$
                                               (size(inp_img_t2.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE: MULTI-FRAME input is not '+$
                           'supported for deconvolution.',/ERROR)
      message,'WRONG INPUT TYPE: MULTI-FRAME input is not '+$
                           'supported for deconvolution.'
   endelse
endif

;saving the original parameter DELTA in its common block variable
delta_ori = par.delta

if par.method LE 3 then begin   ;;;;;;;;;;;;;;   RL, OSEM, ISRA, and OSISRA METHODS 
   dec_init_RLetal_methods, inp_img_t1, inp_img_t2, out_img_t, par, init
endif else begin   ;;;;;;;;;;;;;;    SCALED GRADIENT PROJECTION  (SGP) METHOD 
   dec_init_SGP_method, inp_img_t1, inp_img_t2, out_img_t, par, init
endelse

img=make_array(init.NN,init.NN)           ; the reconstructed image (defined in S)

out_img_t = 				      $
   {          				      $
   data_type  : out_type[0]     	    , $
   data_status: !caos_data.valid	    , $
   image      : img  	                    , $
   psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
   npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
   resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
   lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
   width      : inp_img_t2.width            , $ ; Wavelength band width [m]
   time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
   time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
   background : inp_img_t2.background       , $ ; background
   snr        : inp_img_t2.snr              , $ ; snr
   header     : PTR_NEW('')                   $ ; header of the output file
   }

*out_img_t.header = *inp_img_t2.header ; update the pointer

return, error
end
