; $Id: dec_prog.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       dec_prog
;
; PURPOSE:
;       dec_prog represents the scientific algorithm for DEC module.
;
; CATEGORY:
;       scientific program
;
; CALLING SEQUENCE:
;       error = dec_prog(inp_img_t1,  $ ; img_t input structure
;                        inp_img_t2,  $
;                        out_img_t ,  $ ; img_t output structure
;                        par,         $ ; parameters structure
;                        INIT=init    $ ; initialization data structure
;                       )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see dec.pro's header for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: may 2000,
;                     Massimo Barbati (USG) [].
;    modifications  : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -second output of type "src_t" added.
;                    -super-resolution stuff added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                   : june 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -second output of type "src_t" eliminated.
;                    -normalization constant "c" from PAR to INIT structure.
;                   : february 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                    -dec accelerate version (Lanteri et al.) of OS-EM method 
;                     added
;                   : august 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -possibility of regularization in OS-EM method added.
;                    -different values for the acceleration step in OS-EM method 
;                     considered.
;                   : november 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -bug in acceleration corrected.
;                    -choice of differents regularizations type added.
;                   : january 2006,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it];
;                    -added possibility to reduce boundary effect
;                   : october 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -use of variable "time_integ" activated for the output structure
;                   : may 2007,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                    -eliminated variables concerning regularizations with 
;                     alpha=0.5 and alpha=1.
;                   : April 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -For version 6.0: 
;                    initial stuff has been moved in dec_init (here it is useless)
;                    The main algorithms have been re-written and moved within the 
;                    "deconvolve.pro" procedure. The accelerations follow 
;                    the Biggs&Andrews scheme. Added stopping rules for both
;                    RL/OSEM and ISRA/OS-ISRA.
;                   : April 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -added SGP method. The whole steps are computed
;                     within the "deconvolve_sgp" sub-routine. 
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added optional output for objective function
;                    -stopping rules modified (same rules for RL,OSEM,SGP)
;                    -at the end of the iterations, the header is updated
;                    -added common variable "last_iter" used for
;                     saving iterations number after stopping rule
;                   : Januray 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added common block (called parameters) to this routine
;                     and to dec_init in order to correctly save the parameter 
;                     "par.delta" (delta_ori) in header. (for future reference: 
;                     this can be used for any other parameter)
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -if a stopping rule is satisfied, the variable "last_iter" is 
;                     used for storing current iteration, next "this_iter" is set to the 
;                     total iteration number, and then "tot_iter" is updated with 
;                     the current iteration number (i.e the very last iteration).  
;
function dec_prog, inp_img_t1, inp_img_t2, out_img_t, par, INIT = init

; CAOS global common block
common caos_block, tot_iter, this_iter

common dec_iter, last_iter
common parameters, delta_ori

if this_iter EQ 1 then last_iter=this_iter

if par.method LE 3 then begin   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;RL, OSEM, ISRA, and OS-ISRA methods   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   case par.acc of
      0: deconvolve, init.fn, init, par, out, this_iter ; NO ACCELERATION      
      1: begin                                          ; LINEAR ACCELERATION (or ADDITIVE ACC)
         if (this_iter EQ 1) then begin 
            deconvolve, init.fn, init, par, out, this_iter
            init.acc_arrays[*,*,0]=out
            init.acc_arrays[*,*,1]=out      
         endif else begin
            deconvolve, init.fn, init, par, out, this_iter        
            delta2=out-init.acc_arrays[*,*,1]     
            delta1=init.acc_arrays[*,*,0]-init.acc_arrays[*,*,2] 
            lambda=total(delta2*delta1,/DOUBLE) / total(abs(delta1)^2.,/double)
            init.lambda=(0.>lambda<1.)                       ;to limit lambda values      
            ek=out + (lambda * (out-init.acc_arrays[*,*,0])) ; e(k)
            ek = ek > 0.                                     ;positivization on e(k)     
            init.acc_arrays[*,*,2]=init.acc_arrays[*,*,1]    ; store e(k-1)
            init.acc_arrays[*,*,1]=ek                        ; store e(k)
            init.acc_arrays[*,*,0]=out                       ;store f(k-1) for next it
            out = ek                                         ;next it is computed by using ek
         endelse
      end
      2: begin                  ;EXPONENTIAL ACCELERATION (or MULTIPLICATIVE ACC)
         if (this_iter EQ 1) then begin
            deconvolve, init.fn, init, par, out,this_iter
            init.acc_arrays[*,*,0]=out
            init.acc_arrays[*,*,1]=out      
         endif else begin 
            eps=1d-15
            deconvolve, init.fn, init, par, out,this_iter
            logdelta2=init.R+eps ;define logdelta2 as R+eps
            dummy=init.acc_arrays[*,*,1]
            logdelta2[init.posR]=alog(out[init.posR]/dummy[init.posR])
            pos=where(finite(logdelta2) EQ 0, count)
            if count GT 0 then logdelta2[pos]=eps     
            logdelta1=init.R+eps ;idem for logdelta1
            dummy=init.acc_arrays[*,*,0]
            dummy1=init.acc_arrays[*,*,2]
            logdelta1[init.posR]=alog(dummy[init.posR]/dummy1[init.posR])
            pos=where(finite(logdelta1) EQ 0, count)
            if count GT 0 then logdelta1[pos]=eps
            lambda=total(out*logdelta2*init.acc_arrays[*,*,0]*logdelta1,/DOUBLE)/$
                   total(abs(init.acc_arrays[*,*,0]*logdelta1)^2.,/double)
            init.lambda=(0.>lambda<1.) ;to limit lambda values      
            ek=init.R+eps
            dummy=init.acc_arrays[*,*,0]
            ek[init.posR]=out[init.posR]*((out[init.posR]/dummy[init.posR])^init.lambda) ; e(k) is out
            pos=where(finite(ek) EQ 0, count)                                            ; NOT NECESSARY????
            if count GT 0 then ek[pos]=eps 
            init.acc_arrays[*,*,2]=init.acc_arrays[*,*,1] ; store e(k-1)
            init.acc_arrays[*,*,1]=ek                     ; store e(k)
            init.acc_arrays[*,*,0]=out                    ;store f(k-1) for next it
            out = ek
         endelse
      end   
   endcase  
                                ;fn is defined in the R region
   init.fn=out
                                ;output image is defined in the S region
   out_img_t.image=out[(init.N-init.NN)/2:(init.N+init.NN)/2-1, $
                       (init.N-init.NN)/2:(init.N+init.NN)/2-1] 
   
   
                                ; restoration error evaluation
   if (par.save_rho) OR (par.stopping_rule EQ 4) then begin
      init.rho[this_iter]=sqrt(total(abs(out_img_t.image-init.object)^2.,/double))/sqrt(total(abs(init.object)^2.,/double))
   endif


                                ;Is the stopping rule satisfied?
   case par.stopping_rule of
                                ; STOPPING RULE #1 (TOT_ITER REACHED!)
      0: begin                  ;really un-useful ;)
         if this_iter GE tot_iter then begin
            if (par.verb) then print, '[#1]  --> STOP AT ITER = ' , this_iter
            last_iter = this_iter
         endif
      end
                                ; STOPPING RULE #2 (NORM OF THE OBJECT)
      1: begin
         if (this_iter EQ 1) then init.prev = out_img_t.image ; save the reconstruction
         if (this_iter GE 2) then begin
            diff = out_img_t.image-init.prev
            normstep = total(diff*diff,/double) / total(out_img_t.image*out_img_t.image,/double)
            if (normstep LT par.stop_tol) then begin
               if (par.verb) then print, '[#2]  --> STOP AT ITER = ' , this_iter
               if (par.verb) then print, ' || f(k) - f(k-1) ||^2 / || f(k) ||^2 = '+strtrim(normstep,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
            endif
            init.prev=out_img_t.image
         endif
      end
                                ;  STOPPING RULE #3  (TOTAL FUNCTIONAL IS APPROX CONSTANT)
      2: begin
         if (this_iter GE 2) then begin
            reldecrease = abs(init.obj_fun[this_iter]-init.obj_fun[this_iter-1])/abs(init.obj_fun[this_iter])
            if (reldecrease LT par.stop_tol) then begin
               if (par.verb) then print, '[#3]  --> STOP AT ITER = ' , this_iter
               if (par.verb) then print, ' | J(k) - J(k-1) | / | J(k) | = '+strtrim(reldecrease,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
            endif
         endif
      end
                                ;  STOPPING RULE #4  (DISCREPANCY CROSSES THE GIVEN TOLERANCE ~~
                                ;  should be around 1)
      3: begin
         if (this_iter GT 2) AND (init.discrepancy[this_iter] LE par.stop_tol) then begin
            if (par.verb) then print, '[#4]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' D(k) = '+strtrim(init.discrepancy[this_iter],1), ' ;   tol = '+strtrim(par.functional_tol,1)
                last_iter = LONG(this_iter)
                this_iter=LONG(tot_iter)
                tot_iter=LONG(last_iter)
         endif
      end
                                   ;  STOPPING RULE #5  (MINIMUM OF THE RESTORATION ERROR)
      4: begin
         if (this_iter GT 2) AND (init.rho[this_iter] GT init.rho[this_iter-1]) then begin
            if (par.verb) then print, '[#5]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' RHO (k) = '+strtrim(init.rho[this_iter],1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
         endif
      end
      else: message, 'Unknown stopping criterion:', stopcrit
   endcase

endif else begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SGP METHOD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                                ; call sub-routine  
   deconvolve_sgp, init.x, init, par, out
                                ; the whole algorithm is managed
                                ; within this routine. Even the
                                ; stopping rules. 
                                ;output image is defined in the S
                                ;region
   init.x = out
   out = out[init.obj_index]
   out = reform(out,init.obj_size)
   out = out * init.scaling
   out_img_t.image=out;[(init.work_size[0]-init.obj_size[0])/2:(init.work_size[0]+init.obj_size[0])/2-1, (init.work_size[1]-init.obj_size[1])/2:(init.work_size[1]+init.obj_size[1])/2-1] 
   
endelse   

;write optional outputs
if (par.save_rho) then begin
   printf, init.unit_r, this_iter, init.rho[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_r
endif

if (par.save_discrepancy) then begin
   printf, init.unit_d, this_iter, init.discrepancy[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_d
endif

if (par.save_obj_fun) then begin
   printf, init.unit_o, this_iter, init.obj_fun[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_o
endif


;update the header
;if this_iter GE LONG(tot_iter) then begin
   header =  *out_img_t.header
   method = ['RL','OSEM','IRSA','OS-ISRA','SGP']
   fxaddpar, header, 'METHOD  ', method[par.method], '[DEC] deconvolution method'
   if par.bound then begin
      fxaddpar, header, 'BOUND', fix(par.bound), '[DEC] Boundary Effect Correction '
      fxaddpar, header, 'EE_THRES', par.ee_threshold, '[DEC] Enclosed Energy threshold'
   endif
   if par.method EQ 4 then begin
      fxaddpar, header, 'FLUX_CON',fix(par.flux_con), '[DEC] Flux constraint'
   endif
   if (par.reg GT 0) then begin
      which_reg = ['none', 'T0', 'T2', 'CE', 'MRF', 'HDR', 'Mistral', 'HS', 'T1','L1-norm']
      fxaddpar, header, 'REG_TYPE',which_reg[par.reg], '[DEC] Regularization'
      fxaddpar, header, 'REG_BETA',par.beta, '[DEC] Regularization parameter (beta)'
      if (par.reg EQ 4) OR (par.reg EQ 5) OR (par.reg EQ 6)  OR (par.reg EQ 7) then begin
         fxaddpar, header, 'REG_DELT',delta_ori, '[DEC] Regularization parameter (delta)'
      endif
   endif
   if (par.acc EQ 1) then begin
      fxaddpar, header, 'ACC_LIN ', par.acc, '[DEC] Acceleration B&A LINEAR'
   endif
   if (par.acc EQ 2) then begin
      fxaddpar, header, 'ACC_EXP ', par.acc, '[DEC] Acceleration B&A EXP'
   endif
   if (par.init_alg EQ 0) then begin
      fxaddpar, header, 'INIT_ALG', par.init_alg, '[DEC] Algorithm initialized with constant array'
   endif else begin
      fxaddpar, header, 'INIT_ALG', par.init_alg, '[DEC] Algorithm initialized with user-defined image'
      fxaddpar, header, 'FILENAME', par.filename_image, '[DEC] Initialization filename'
   endelse
   if (par.init_mask) then begin
      fxaddpar, header, 'SUPERRES', par.init_mask, '[DEC] Super-Resolution'
      fxaddpar, header, 'MASK_TYP', par.mask_choice, '[DEC] Super-Resolution mask choice'
      fxaddpar, header, 'MASK_PIX', par.mask_npix, '[DEC] Super-Resolution mask size'
      fxaddpar, header, 'MASK_PER', par.mask_perc, '[DEC] Super-Resolution percentage'
      fxaddpar, header, 'MASK_DIA', par.mask_diam, '[DEC] Super-Resolution mask diameter'
   endif
   fxaddpar, header, 'STOP_CRI', par.stopping_rule+1, '[DEC] stopping rule #'
   if (par.stopping_rule GT 1) AND (par.stopping_rule NE 4) then begin
      fxaddpar, header, 'STOP_TOL', par.stop_tol, '[DEC] stopping tolerance (if set)'
   endif
   fxaddpar, header, 'ITER    ', last_iter, '[DEC] Number of iterations'

   *out_img_t.header = header
;endif

end
