; $Id: fsm_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    fsm_gui
;
; PURPOSE:
;    fsm_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the FSM module.
;    A parameter file called fsm_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    The file is stored in the project directory proj_name located
;    in the working directory.
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = fsm_gui(n_module, proj_name)
;
; INPUTS:
;    n_module : integer scalar. Number associated to the instance
;               of the FSM module. n_module > 0.
;    proj_name: string. Name of the current project.
;
; OUTPUTS:
;    error: error code [long scalar]
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    none.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(fsm_info()).help stuff added (instead of !caos_env.help).
;                   : september 2005
;                     Barbara Anconelli
;                    - added  help for windows version
;                   : may 2007, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simplest way to call AIRY_HELP 
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro fsm_gui_set, state

;set broad band index
widget_control, state.id.band, SET_DROPLIST_SELECT=state.par.band_index

if (state.par.cflux_dir eq 0B) then begin
  widget_control, state.id.par_flux,  SENSITIVE=0
  widget_control, state.id.par_tel,   /SENSITIVE
endif else begin
  widget_control, state.id.par_flux,  /SENSITIVE
  widget_control, state.id.par_tel,   SENSITIVE=0
endelse

end

;;;;;;;;;;;;;;;;;;;;;;
; fsm_gui event loop ;
;;;;;;;;;;;;;;;;;;;;;;
;
pro fsm_gui_event, event

common error_block, error

; read the GUI state structure
widget_control, event.top, GET_UVALUE=state

; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

; handle other events.
widget_control, event.id, GET_UVALUE=uvalue
case uvalue of

   'tresh_detect'  : state.par.tresh_detect = event.value

   'cflux_dir'     : begin
                      state.par.cflux_dir = event.value
                      fsm_gui_set, state
                     end

   'cal_flux'      : state.par.cal_flux = event.value

   'cal_mag'       : state.par.cal_mag = event.value

   'band'          : state.par.band_index = event.index

   'tel_surf'      : state.par.tel_surf = event.value

   'int_time'      : state.par.int_time = event.value

   'efficiency'    : state.par.efficiency = event.value

   'table_aperphot': if (event.type eq 0) then begin
                     widget_control, event.id,  GET_VALUE =dummy
                     state.par.aper_phot = dummy
                     endif


   ; handle event from standard save button
   'save': begin


      ; save data in the parameter file.
      ; check before if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='FSM warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='FSM information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      ; save the parameter data file
      par = state.par
      save, par, FILENAME = state.sav_file

      ; kill the GUI returning a null error
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return
   end

  ; handle event from standard help button
     'help': begin
   online_help, book=(fsm_info()).help, /FULL_PATH
;         widget_control, /HOURGLASS
;         
;       CASE !VERSION.OS_FAMILY OF
;         'Windows'   :     begin
;                    spawn, !caos_env.browser+" "+(fsm_info()).help,/NOSHELL
;                   end
;         else    :  begin
;                     spawn, !caos_env.browser+" "+(fsm_info()).help+" &"
;                   end
;       ENDCASE
   end
   ; standard restore button:
   'restore': begin

      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      ; update the current module number
      par.module.n_module = state.par.module.n_module

      ; set the default values for all the widgets
      widget_control, state.id.tresh_detect,   SET_VALUE=par.tresh_detect
      widget_control, state.id.cflux_dir,      SET_VALUE=par.cflux_dir
      widget_control, state.id.cal_flux,       SET_VALUE=par.cal_flux
      widget_control, state.id.cal_mag,        SET_VALUE=par.cal_mag
      widget_control, state.id.band,           SET_DROPLIST_SELECT=par.band_index
      widget_control, state.id.efficiency,     SET_VALUE=par.efficiency
      widget_control, state.id.int_time,       SET_VALUE=par.int_time
      widget_control, state.id.tel_surf,       SET_VALUE=par.tel_surf
      widget_control, state.id.table_aperphot, SET_VALUE=par.aper_phot

      ; write the reseted state structure
      state.par = par

      ; write the GUI state structure
      widget_control, event.top, SET_UVALUE=state
   end

   ; standard cancel button (exit without saving)
   'cancel': begin
      error = !caos_error.cancel
      widget_control, event.top, /DESTROY
      return
   end

endcase

; write the GUI state structure
widget_control, event.top, SET_UVALUE=state

return
end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function fsm_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
common error_block, error

; get info structure
info = fsm_info()

; parameter file checks
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
    restore, def_file
    par.module.n_module = n_module
    if (par.module.mod_name ne info.mod_name) then     $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
    restore, sav_file
    if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file      $
              +' is from another module: please generate a new one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse



; build the widget id structure
id = $
   { $                     ; widget id structure
   root_base_id    : 0L,  $ ;
   par_base        : 0L,  $ ;
   par_an          : 0L,  $ ;
   tresh_detect    : 0L,  $ ;
   par_flux_all    : 0L,  $ ;
   par_flux_quer   : 0L,  $ ;
   par_flux        : 0L,  $ ;
   cflux_dir       : 0L,  $ ;
   cal_flux        : 0L,  $ ;
   cal_mag         : 0L,  $ ;
   par_tel         : 0L,  $ ;
   band            : 0L,  $ ;
   efficiency      : 0L,  $ ;
   int_time        : 0L,  $ ;
   tel_surf        : 0L,  $ ;
   par_ap_phot     : 0L,  $ ;
   table_aperphot  : 0L   $
   }

; build the state structure
state = $
   {    $                   ; widget state structure
   sav_file: sav_file, $    ; name of the file where save params
   def_file: def_file, $    ; name of the file where save params
   id      : id,       $    ; widget id structure
   par     : par       $    ; parameter structure
   }

; root base
modal = n_elements(group) ne 0
title = strupcase(info.mod_name)+' parameters setting GUI'
root_base_id = widget_base(TITLE=title, MODAL=modal, /COL, GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

   ; parameters base
   state.id.par_base = widget_base(root_base_id, FRAME=10, ROW=6)
   dummy = widget_label(state.id.par_base, VALUE='parameters', /FRAME)
   par_base_id = widget_base(state.id.par_base, /COL)

  state.id.par_an = widget_base(state.id.par_base, /COL, FRAME=5 )
  dummy = widget_label(state.id.par_an, VALUE='Detection parameters :', /FRAME)


  state.id.tresh_detect         = cw_field(state.id.par_an,                              $
                                           TITLE='Detection treshold [rms background]  ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.tresh_detect,                 $
                                           UVALUE='tresh_detect',                        $
                                           /ALL_EVENTS                                   )

  state.id.par_flux_all = widget_base(state.id.par_base, /COL, FRAME=5 )
  dummy = widget_label(state.id.par_flux_all, VALUE='Photometry parameters :', /FRAME)


  state.id.cflux_dir = cw_bgroup(state.id.par_flux_all,                                  $
                                 LABEL_LEFT='Direct flux calibration ?',                 $
                                 ['no', 'yes'],                                          $
                                 COLUMN=2,                                               $
                                 SET_VALUE=state.par.cflux_dir,                          $
                                 UVALUE='cflux_dir',                                     $
                                 /EXCLUSIVE                                              )

  state.id.par_flux = widget_base(state.id.par_flux_all, /COL, FRAME=2 )
  state.id.cal_flux                 = cw_field(state.id.par_flux,                        $
                                           TITLE='Calibrated flux [counts]             ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.cal_flux,                     $
                                           UVALUE='cal_flux',                            $
                                           /ALL_EVENTS                                   )

  state.id.cal_mag                  = cw_field(state.id.par_flux,                        $
                                           TITLE='Correspondente magnitude in the band ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.cal_mag,                      $
                                           UVALUE='cal_mag',                             $
                                           /ALL_EVENTS                                   )

  state.id.par_tel = widget_base(state.id.par_flux_all, /COL, FRAME=2 )

  band                    = widget_base(state.id.par_tel, /ROW)
  dummy                   = widget_label(band, xsize= 100, VALUE='Band')
  state.id.band           = widget_droplist(band, VALUE=state.par.band_tab, UVALUE='band')

  state.id.efficiency           = cw_field(state.id.par_tel,                             $
                                           TITLE='overall efficiency:                  ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.efficiency,                   $
                                           UVALUE='efficiency',                          $
                                           /ALL_EVENTS                                   )

  state.id.int_time             = cw_field(state.id.par_tel,                             $
                                           TITLE='integration time [s]:                ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.int_time,                     $
                                           UVALUE='int_time',                            $
                                           /ALL_EVENTS                                   )

  state.id.tel_surf             = cw_field(state.id.par_tel,                             $
                                           TITLE='telescope collecting surface [m^2]:  ',$
                                           /FLOAT,                                       $
                                           VALUE=state.par.tel_surf,                     $
                                           UVALUE='tel_surf',                            $
                                           /ALL_EVENTS                                   )

  state.id.par_ap_phot = widget_base(state.id.par_flux_all, /COL, FRAME=2 )
  dummy = widget_label(state.id.par_ap_phot, VALUE='Aperture photometry diameters set [pixel] :')

  state.id.table_aperphot            = widget_table(state.id.par_ap_phot,                $
                                                    VALUE=state.par.aper_phot,           $
                                                    /NO_HEADER,                          $
                                                    UVALUE='table_aperphot',             $
                                                    /EDITABLE,                           $
                                                    /ALL_EVENTS                          )
   ; standard buttons
   btn_base_id = widget_base(root_base_id, FRAME=10, /ROW)
      dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
      cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
      if modal then widget_control, cancel_id, /CANCEL_BUTTON
      dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                            UVALUE="restore")
      save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS", $
                              UVALUE="save")
      if modal then widget_control, save_id, /DEFAULT_BUTTON

; initialize the sensitive state
fsm_gui_set, state

; save the state structure of the GUI in the top base uvalue
widget_control, root_base_id, SET_UVALUE=state

; draw the GUI
widget_control, root_base_id, /REALIZE

xmanager, 'fsm_gui', root_base_id, GROUP_LEADER=group

return, error
end
