; $Id: fsm_prog.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    fsm_prog
;
; PURPOSE:
;    fsm_prog represents the scientific algorithm for the
;    Find Star Module (FSM) module.
;
; CATEGORY:
;    module's program routine
;
; CALLING SEQUENCE:
;    error = fsm_prog(inp_img_t, $ ; img_t input structure
;                     par,       $ ; parameters structure
;                     INIT=init  ) ; initialization structure
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see module help for a detailed description. 
;
; ROUTINE MODIFICATION HISTORY: 
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;                   : december 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -final display changed from log- to square-root-scale.
;
;-
;
function fsm_prog, inp_img_t, $
                   par,       $
                   INIT=init

; CAOS global common block
common caos_block, tot_iter, this_iter

; initialization of the error code: no error as default
error = !caos_error.ok

; program itself
init.counter = init.counter+1

;test if the input image is 3D (use after rft, in a single project dedicated to the analyse of already saved data) 
;or in 2D (tipically in the data flow of a deconvolution project)
if (size(inp_img_t.image))[0] eq 3 then begin 
    init.data = inp_img_t.image[*,*,(size(inp_img_t.image))[3]-1]  ; last frame of the cube
endif else begin
    init.data = inp_img_t.image
endelse

if (init.counter mod tot_iter EQ 0) then begin 

    ; find parameters 
    find_par     = {                                          $
                 fwhm_find       : 2.,                    $
                 roundlim_find   : [-90.,90.],            $
                 sharplim_find   : [-10., 10.],           $
                 xest            : 2,                     $
                 yest            : (size(init.data))[1]-1,$
                 sigma           : par.tresh_detect       $
                                                          }

    print, '-------------------------------------------------------------------'
    print, 'FSM:'

    ; aperture photometry parameters
    print, 'aperture photometry set = ',  par.aper_phot

    ; Flux calibration
    if par.cflux_dir eq 0B then begin 
        mag_0           = par.cal_mag 
        zero_pt         = par.efficiency*n_phot(mag_0, BAND=par.band_tab[par.band_index], $
                                                SURF=par.tel_surf, DELTA_T=par.int_time)
        pt_0            = zero_pt[0]
    endif else begin 
        mag_0           = par.cal_mag 
        pt_0            = par.cal_flux
    endelse

    amas_phot_m, init.data, find_par, pt_0, mag_0, par.aper_phot, res, BACK_RMS=back_rms

    window, /FREE, xsize=450, ysize=410, xpos=0, ypos=0, $
    title = 'Detect stars position (Astrolib find)'
    dim = size(init.data)
    nx = dim[1] & ny = dim[2]
    zoom_fac = 1
    ;arr2disp = alog10(1.+ init.data)
    arr2disp = init.data^0.5

                cont_image, arr2disp[nx/2-nx/(2*zoom_fac):nx/2+nx/(2*zoom_fac)-1,  $
                                     ny/2-ny/(2*zoom_fac):ny/2+ny/(2*zoom_fac)-1], $
                  /nocontour, /colorbar

                dum = ((size(size(res.flux)))[1])-1
                xyouts, 0.3, 0.97,string((size(res.flux))[dum],'(I4)')+' !17detected stars!X', $
                  charsize=1, /normal
                ; fix oplot one value bug
                if (size(res.xcenter))[0] eq 0 then begin 
                    tmpx = fltarr(2) & tmpy = fltarr(2)
                    tmpx = [res.xcenter-(nx/2-nx/(2*zoom_fac)), res.xcenter-(nx/2-nx/(2*zoom_fac))] 
                    tmpy = [res.ycenter-(ny/2-ny/(2*zoom_fac)), res.ycenter-(ny/2-ny/(2*zoom_fac))]
                    oplot, tmpx, tmpy , PSym=sym(6), color=255B, symsize=zoom_fac
                endif else begin 
                    oplot, res.xcenter-(nx/2-nx/(2*zoom_fac)), res.ycenter-(ny/2-ny/(2*zoom_fac)), $
                           PSym=sym(6), color=255B, symsize=zoom_fac 
                    oplot, res.xcenter-(nx/2-nx/(2*zoom_fac)), res.ycenter-(ny/2-ny/(2*zoom_fac)), $
                           PSym=1, color=255B
                endelse    

endif

; back to calling program
return, error
end
