; $Id: mcd.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    mcd
;
; ROUTINE'S PURPOSE:
;    MCD manages the simulation for the image mcdonvolution (MCD) module,
;    that is:
;       1-call the module's initialization routine mcd_init at the first
;         iteration of the simulation project,
;       2-call the module's program routine mcd_prog.
;
; MODULE'S PURPOSE:
;
;       MCD performs the deconvolution of high dynamic range images
;    [1,2,3,4]. The module assumes that the unknown object $\mathbf{f}$ 
;    is the sum of two components: the point-like part f_P containing
;    the bright sources of the object superimposed on the extended and
;    smooth part f_E. Moreover MCD assumes to know the positions of
;    the bright sources, i.e. the user must provide a mask M_P which
;    is 1 in the pixels of the sources and 0 elsewhere. Alternatively,
;    the user can set the mask to 1 in small regions around the known 
;    positions. 
;
;    References:
;    [1] A. La Camera et al., SPIE Proc. 8455, 84553D (2012).
;    [2] A. La Camera et al., PASP 126,(936), 180 (2014).
;    [3] M. Prato et al., Inverse Problems (submitted), (2016)
;    [4] A. La Camera et al., SPIE Proc. 9909-297 (2016)
;
;    Please pay attention to the fact that the TOP BOX-INPUT of the module mcd
;    is supposed to receive images, while the BOTTOM BOX-INPUT of the module
;    mcd is supposed to receive PSFs. 
;    Concerning the output, the TOP BOX-OUTPUT is for the point-like
;    component, while the BOTTOM BOX-OUTPUT is for the diffuse one.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = mcd(inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t1: structure of type img_t, corresponding to PSF's
;    inp_img_t2: structure of type img_t, corresponding to images
;    par       : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_img_tE: structure of type "img_t" containing the characteristics of
;                the deconvolved object (DIFFUSE COMPONENT), the following 
;                fields are contained in this structure:
;; out_img_tE = 				      $
;;    {          				      $
;;    data_type  : out_type[0]     	    , $
;;    data_status: !caos_data.valid	    , $
;;    image      : img  	                    , $
;;    psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
;;    npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
;;    resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
;;    lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
;;    width      : inp_img_t2.width            , $ ; Wavelength band width [m]
;;    time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
;;    time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
;;    background : inp_img_t2.background       , $ ; background
;;    snr        : inp_img_t2.snr              , $ ; snr
;;    header     : PTR_NEW('')                 $   ; header of the output file
;;    }
;; *out_img_tE.header = *inp_img_t2.header
;
;    out_img_tP: structure of type "img_t" containing the characteristics of
;                the deconvolved object (POINT-LIKE COMPONENT), the following 
;                fields are contained in this structure:
;; out_img_tP = 				      $
;;    {          				      $
;;    data_type  : out_type[0]     	    , $
;;    data_status: !caos_data.valid	    , $
;;    image      : img  	                    , $
;;    psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
;;    npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
;;    resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
;;    lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
;;    width      : inp_img_t2.width            , $ ; Wavelength band width [m]
;;    time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
;;    time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
;;    background : inp_img_t2.background       , $ ; background
;;    snr        : inp_img_t2.snr              , $ ; snr
;;    header     : PTR_NEW('')                 $   ; header of the output file
;;    }
;; *out_img_tP.header = *inp_img_t2.header
;
; KEYWORD PARAMETERS:
;    INIT: initialization data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : int scalar. Total number of iteration during the 
;                 simulation run.
;    this_iter  : int scalar. Number of the current iteration. It is
;                 defined only while status eq !caos_status.run.
;                 (this_iter >= 1).  
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    -other routine from the folder ./mcd_lib
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: June 2016, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;    modifications  : 
;
; MODULE MODIFICATION HISTORY:
;    module written : Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;    modifications  : 
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -if a stopping rule is satisfied, the variable "last_iter" is 
;                     used for storing current iteration, next "this_iter" is set to the 
;                     total iteration number, and then "tot_iter" is updated with 
;                     the current iteration number (i.e the very last iteration).  
;                    -added common block (called parameters_mcd) to routines
;                     mcd_init and mcd_prog in order to correctly save the parameter 
;                     "par.delta" (delta_ori) in header. (for future reference: 
;                     this can be used for any other parameter)
;-
;
;;;;;;;;;;;;;;;
; module code ;
;;;;;;;;;;;;;;;
;
function mcd, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, INIT=init

common caos_block, tot_iter, this_iter
common mcd_iter, last_iter

; initialization of the error code: set to "no error" as default
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = mcd_init(inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, INIT = init)
endif else begin
   ; run section
   error = mcd_prog(inp_img_t1, inp_img_t2,out_img_tE, out_img_tP, par, INIT = init)
endelse

; back to calling program.
return, error
end
