; $Id: mcd_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    mcd_gui
;
; PURPOSE:
;    mcd_gui generates the Graphical User Interface (GUI) for setting the
;    parameters of the mcd module. A parameter file
;    called mcd_yyyyy.sav is created, where yyyyy is the number n_module
;    associated to the the module instance.  The file is stored in the
;    project directory proj_name located in the working directory.
;    (see mcd.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = mcd_gui(n_module, proj_name)
;
; INPUTS:
;    n_module  : number associated to the intance of the mcd module.
;                [integer scalar -- n_module > 0].
;    proj_name : name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;
; ROUTINE MODIFICATION HISTORY:
;   routine written: May 2016,
;                    Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;   modifications  : 
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -bug correction on the GUI (for regularization HS)
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro mcd_gui_set, state

;controls for the button of initial image (P)
  if (state.par.init_P EQ 1B ) then begin
     widget_control, state.id.init_P_filename, /SENSITIVE
     widget_control, state.id.init_P, SET_VALUE=1
  endif else begin
     widget_control, state.id.init_P_filename, SENSITIVE=0
     widget_control, state.id.init_P, SET_VALUE=0
  endelse

;controls for the button of initial image (E)
  if (state.par.init_E EQ 1B ) then begin
     widget_control, state.id.init_E_filename, /SENSITIVE
     widget_control, state.id.init_E, SET_VALUE=1
  endif else begin
     widget_control, state.id.init_E_filename, SENSITIVE=0
     widget_control, state.id.init_E, SET_VALUE=0
  endelse
 
; FLUX CONSTRAINED
  if (state.par.method EQ 1) then widget_control, state.id.flux_con, /SENSITIVE else $
     widget_control, state.id.flux_con, SENSITIVE=0
  
;  BOUNDARY
  if state.par.bound then widget_control, state.id.ee_threshold, /SENSITIVE else $
     widget_control, state.id.ee_threshold, SENSITIVE=0
    
; REGULARIZATIONS
  widget_control, state.id.reg, /SENSITIVE
  widget_control, state.id.reg, SET_COMBOBOX_SELECT=state.par.reg
  case state.par.reg of
     0 : begin                  ;NONE
        widget_control, state.id.beta, SENSITIVE=0
        widget_control, state.id.delta, SENSITIVE=0
     end
     1: begin                   ;TIKHONOV (0th order)
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     2: begin                   ;LAPLACIAN (second order tikhonov)
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     3: begin                   ;ENTROPY
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     4: begin                   ;EDGE-PRESERVING MRF
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     5: begin                   ;HDR
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     6: begin                   ;MISTRAL
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     7: begin                   ;hypersurface
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, /SENSITIVE
     end
     8: begin                   ;tikhonov first order
        widget_control, state.id.beta, /SENSITIVE
        widget_control, state.id.delta, SENSITIVE=0
     end
     else:                      ;NOTHING TO DO!
  endcase

  
;OPTIONAL OUTPUTS ---------------------------------------------
  if (state.par.save_discrepancy) then begin
     widget_control, state.id.save_discrepancy_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_discrepancy_file, SENSITIVE=0
  endelse   
  
  if (state.par.save_obj_fun) then begin
     widget_control, state.id.save_obj_fun_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_obj_fun_file, SENSITIVE=0
  endelse   
  
  if (state.par.save_rho) then begin
     widget_control, state.id.save_rho_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_rho_file, SENSITIVE=0
  endelse   
;--------------------------------------------------------------
  
 
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro mcd_gui_event, event

   common error_block, error

   widget_control, event.top, get_uvalue=state

   if tag_names(event, /structure_name) eq 'widget_kill_request' then begin
       error = !caos_error.cancel
       widget_control, event.top, /destroy
   endif

   ; events
   widget_control, event.id, get_uvalue = uvalue
   case uvalue of

   ; standard help button
   'help': online_help, book=(mcd_info()).help, /FULL_PATH
   
   ; standard restore button:
   'restore' : begin
      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      widget_control, state.id.method,         SET_COMBOBOX_SELECT=par.method
      widget_control, state.id.flux_con,       SET_VALUE=par.flux_con
      widget_control, state.id.verb,           SET_VALUE=par.verb
      widget_control, state.id.reg,            SET_COMBOBOX_SELECT=par.reg
      widget_control, state.id.beta,           SET_VALUE=par.beta
      widget_control, state.id.delta,          SET_VALUE=par.delta
      widget_control, state.id.bound,          SET_VALUE=par.bound
      widget_control, state.id.ee_threshold,   SET_VALUE=fix(par.ee_threshold*100)
      widget_control, state.id.init_P,         SET_VALUE=par.init_P
      widget_control, state.id.init_E,         SET_VALUE=par.init_E
      widget_control, state.id.init_P_filename,SET_VALUE=par.init_P_filename
      widget_control, state.id.init_E_filename,SET_VALUE=par.init_E_filename
      widget_control, state.id.stopping_rule,  SET_COMBOBOX_SELECT=par.stopping_rule
      widget_control, state.id.stop_tol, SET_VALUE=par.stop_tol
      widget_control, state.id.load_obj_file,  SET_VALUE=par.load_obj_file
      widget_control, state.id.save_discrepancy, SET_VALUE=par.save_discrepancy
      widget_control, state.id.save_discrepancy_file, SET_VALUE=par.save_discrepancy_file
      widget_control, state.id.save_obj_fun, SET_VALUE=par.save_obj_fun
      widget_control, state.id.save_obj_fun_file, SET_VALUE=par.save_obj_fun_file
      widget_control, state.id.save_rho, SET_VALUE=par.save_rho
      widget_control, state.id.save_rho_file, SET_VALUE=par.save_rho_file
      ; update the current module number
      par.module.n_module = state.par.module.n_module
      ; update the state structure
      state.par = par
      ;update all GUI
      mcd_gui_set, state
   end

   ; standard save button:
    'save' : begin

      ; check before saving the parameter file if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='mcd warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='mcd information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      par = state.par
      save, par, FILENAME = state.sav_file
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return

   end

    ; standard cancel button (exit without saving)
    'cancel': begin
       error = !caos_error.cancel
       WIDGET_CONTROL, event.top, /DESTROY
       return
    end
    
    'method': begin
       state.par.method=event.value
       mcd_gui_set, state
    end
    
    'mask': begin
       widget_control, state.id.mask, GET_VALUE=dummy
       state.par.mask=dummy
    end

    'flux_con' : begin
       widget_control, state.id.flux_con, GET_VALUE=dummy
       state.par.flux_con=dummy
       mcd_gui_set, state
    end
    
    'bound': begin
       widget_control, state.id.bound, GET_VALUE=dummy
       state.par.bound=dummy
       mcd_gui_set, state
    end
    
    'ee_threshold' : state.par.ee_threshold = float(event.value)/100.
        
    'verb' : begin
       widget_control, state.id.verb, GET_VALUE=dummy
       state.par.verb=dummy
    end
    
    'reg': begin
       state.par.reg=event.index
       mcd_gui_set, state
    end

    'beta' : begin
       state.par.beta=event.value
    end
    
    'delta' : begin
       state.par.delta = event.value
    end
    
    'init_P': begin
       widget_control, state.id.init_P, GET_VALUE=dummy
       state.par.init_P=dummy
       mcd_gui_set, state
    end
    
    'init_E': begin
       widget_control, state.id.init_E, GET_VALUE=dummy
       state.par.init_E=dummy
       mcd_gui_set, state
    end

    'init_P_filename': begin
       widget_control, state.id.init_P_filename, GET_VALUE=dummy
       state.par.init_P_filename=dummy
    end

    'init_E_filename': begin
       widget_control, state.id.init_E_filename, GET_VALUE=dummy
       state.par.init_E_filename=dummy
    end
    
    'stopping_rule': begin
       val = event.index
       if ((state.par.method EQ 2) OR (state.par.method EQ 3)) then begin
          if val GT 0 then val = val + 3
       endif
       state.par.stopping_rule=val
       mcd_gui_set, state
    end
    'stop_tol': begin
       state.par.stop_tol = event.value
    end
    'load_obj_file': begin
       state.par.load_obj_file=event.value
    end
    'save_discrepancy':begin
       state.par.save_discrepancy = event.select
       mcd_gui_set, state
    end
    'save_discrepancy_file':begin
       state.par.save_discrepancy_file=event.value
    end
    'save_obj_fun':begin
       state.par.save_obj_fun = event.select
       mcd_gui_set, state
    end
    'save_obj_fun_file':begin
       state.par.save_obj_fun_file=event.value
    end
    'save_rho':begin
       state.par.save_rho = event.select
       mcd_gui_set, state
    end
    'save_rho_file':begin
       state.par.save_rho_file=event.value
    end       
    else:                       ;nothing to do!
    endcase
    widget_control, event.top, set_uvalue=state
;    return
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function mcd_gui, n_module, proj_name, GROUP_LEADER=group

common error_block, error

; retrieve the module information
info = mcd_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a mcd one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = $
   { $
   root_base_id    : 0L, $
   method          : 0L, $
   flux_con        : 0L, $
   bound           : 0L, $
   ee_threshold    : 0L, $
   reg             : 0L, $
   beta            : 0L, $
   delta           : 0L, $
   verb            : 0L, $
   init_P          : 0L, $
   init_E          : 0L, $
   init_P_filename : 0L, $
   init_E_filename : 0L, $
   mask            : 0L, $
   stopping_rule   : 0L, $
   stop_tol        : 0L, $
   load_obj_text   : 0L, $
   load_obj_file   : 0L, $
   save_discrepancy : 0L, $
   save_discrepancy_file : 0L, $
   save_obj_fun : 0L, $
   save_obj_fun_file : 0L, $
   save_rho : 0L, $
   save_rho_file : 0L $
   }

state = $
  {     $
  sav_file : sav_file, $
  def_file : def_file, $
  id       : id      , $
  par      : par       $
  }

; root base definition
modal= N_ELEMENTS(group) NE 0
title= STRUPCASE(info.mod_name)+' parameter setting GUI'
root_base_id = WIDGET_BASE(TITLE=title,MODAL=modal,/COL,GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

; parameters base
par_base = widget_base(root_base_id, /FRAME, /ROW)

par_base_id = widget_base(par_base, /COL)

base  = WIDGET_BASE(par_base_id,  /COLUMN)
text  = WIDGET_LABEL(base, VALUE=' MCD module ', /ALIGN_LEFT  )
text  = WIDGET_LABEL(base, VALUE='  [ Top input is for the image(s), while bottom input is for the PSF(s). ]                  '     )
text  = WIDGET_LABEL(base, VALUE='  [ Top output is for the point-like component, while bottom output is for the diffuse one.]'     )

basetab=WIDGET_TAB(par_base_id, location=0, uvalue='')
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabA=WIDGET_BASE(basetab, /COL, TITLE='       MULTI-COMPONENTS METHODS       ')

base_top = WIDGET_BASE(basetabA, /FRAME, /ROW)
base_method = WIDGET_BASE(base_top, /FRAME, /COLUMN)

state.id.method = cw_bgroup(base_method, LABEL_TOP='DECONVOLUTION METHOD: ', $
                            ['MC-RL', 'MC-SGP'], /ROW, SET_VALUE=par.method, $
                            UVALUE='method', /EXCLUSIVE)

state.id.mask = cw_filename(base_method,       $
                            TITLE='Enter the path of the MASK file [Mandatory]', $
                            VALUE=state.par.mask,      $
                            UVALUE='mask',             $
                            /ALL_EVENTS                         )

state.id.flux_con = cw_bgroup(base_method,  'Flux constraint on each reconstruction',$
                              COLUMN=1,  UVALUE='flux_con',  /NONEXCLUSIVE, $
                              SET_VALUE=state.par.flux_con )
state.id.bound = cw_bgroup(base_method,  'Boundary effects reduction',$
                           COLUMN=1,  UVALUE='bound',  /NONEXCLUSIVE, $
                           SET_VALUE=state.par.bound )
state.id.ee_threshold = WIDGET_SLIDER(base_method, $
                                      value=fix(state.par.ee_threshold*100), $
                                      xsize=50, uvalue='ee_threshold', $
                                      title="Percentage of the PSF's Enclosed Energy")
text=widget_label(base_method, value='Regularization (on diffuse component):', /ALIGN_LEFT)          
reg=['none', 'Tikhonov - 0th order ', 'Tikhonov - 2nd order (Laplacian)', 'Cross-Entropy', 'Edge-preserving MRF', $
     'High-dynamic range','Mistral', 'Hypersurface', 'Tikhonov - 1st order'] 
state.id.reg=widget_combobox(base_method, value=reg, uvalue='reg')
state.id.beta = cw_field(base_method, $
                         TITLE='Regularization parameter (Beta):   ', /ROW, xsize=10, $
                         UVALUE='beta', value=state.par.beta, /FLOAT, /ALL_EVENTS)  
state.id.delta = cw_field(base_method, $
                          TITLE='Threshold (Delta):                 ', /ROW, /FLOAT, xsize=10, $
                          UVALUE='delta', value=state.par.delta, /ALL_EVENTS)

state.id.verb = cw_bgroup(base_method,  'Verbosity of the algorithm',$
                          COLUMN=1,  UVALUE='verb',  /NONEXCLUSIVE, $
                          SET_VALUE=state.par.verb )

base_right = WIDGET_BASE(base_top, /FRAME, /COLUMN)
text  = WIDGET_LABEL(base_right, VALUE='Initializations of the algorithm:')
state.id.init_P = cw_bgroup(base_right, $
                            LABEL_LEFT= 'Point-like component: ',$
                            ['Default array','User-defined image'], /ROW,   $
                            SET_VALUE=par.init_P,  UVALUE='init_P',   /EXCLUSIVE   )
state.id.init_P_filename = cw_filename(base_right, TITLE='                      ', $
                                       VALUE=state.par.init_P_filename, $
                                       UVALUE='init_P_filename', /ALL_EVENTS )
state.id.init_E = cw_bgroup(base_right, $
                            LABEL_LEFT= 'Diffuse component:    ',$
                            ['Default array','User-defined image'], /ROW,   $
                            SET_VALUE=par.init_E,  UVALUE='init_E',   /EXCLUSIVE   )
state.id.init_E_filename = cw_filename(base_right, TITLE='                      ', $
                                       VALUE=state.par.init_E_filename, $
                                       UVALUE='init_E_filename', /ALL_EVENTS )




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabB=WIDGET_BASE(basetab, /COL, TITLE='STOPPING RULES AND OPTIONAL OUTPUTS')
base_stopping_rule = WIDGET_BASE(basetabB, /FRAME, /COL)

dummy = widget_label(base_stopping_rule, value='Choose a stopping criterium among:')
   stopping= ['#1 [ALL] Stop when this_iter > tot_iter (set up in the worksheet)       ', $
              '#2 [ALL] Stop when || f(k) - f(k-1) ||^2 < tol * || f(k) ||^2      OR #1', $
              '#3 [ALL] Stop when | J(k) - J(k-1) | < tol * | J(k) |              OR #1', $
              '#4 [ALL] Stop when the discrepancy crosses the tolerance value     OR #1', $
              '#5 [SIM] Stop when the restoration error reaches a minimum value   OR #1']
state.id.stopping_rule = widget_combobox(base_stopping_rule, value=stopping, $
                         uvalue='stopping_rule')


state.id.stop_tol = CW_FIELD(base_stopping_rule, $
            value=state.par.stop_tol, xsize=15,$
            UVALUE='stop_tol', /ROW,/return_events, /FLOATING, $
            TITLE="tolerance of the selected rule:       [ENTER]")
        

base_optional=widget_base(basetabB, /col, /FRAME) 
dummy=widget_label(base_optional, value='Optional outputs: ', /ALIGN_LEFT)
base_optional2=widget_base(base_optional, /row) 

base_optional_sx=widget_base(base_optional2, /col) 
 
state.id.save_discrepancy=CW_BGROUP(base_optional_sx, $
                 ['[ALL] Save the discrepancy value at each iteration'], $
                          /COL, set_value=state.par.save_discrepancy, $
                          UVALUE='save_discrepancy',  /NONEXCLUSIVE)
state.id.save_discrepancy_file = cw_filename(base_optional_sx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_discrepancy_file,      $
                        UVALUE='save_discrepancy_file',             $
                        /ALL_EVENTS                         )

state.id.save_obj_fun=CW_BGROUP(base_optional_sx, $
                 ['[ALL] Save the objective function value at each iteration'], $
                          /COL, set_value=state.par.save_obj_fun, $
                          UVALUE='save_obj_fun',  /NONEXCLUSIVE)
state.id.save_obj_fun_file = cw_filename(base_optional_sx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_obj_fun_file,      $
                        UVALUE='save_obj_fun_file',             $
                        /ALL_EVENTS                         )

base_optional_dx=widget_base(base_optional2, /col) 

state.id.save_rho=CW_BGROUP(base_optional_dx, $
                 ["[SIM] Save the restoration error at each iteration"], $
                          /COL, set_value=state.par.save_rho, $
                          UVALUE='save_rho',  /NONEXCLUSIVE)

;state.id.load_obj_text = widget_label(base_stopping_rule_rho, $
;                        value="[#4 cont'd]         ")    
state.id.load_obj_file = cw_filename(base_optional_dx,       $
                        TITLE='Enter the path of the DIFFUSE COMPONENT map ', $
                        VALUE=state.par.load_obj_file,      $
                        UVALUE='load_obj_file',             $
                        /ALL_EVENTS   )

state.id.save_rho_file = cw_filename(base_optional_dx,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_rho_file,      $
                        UVALUE='save_rho_file',             $
                        /ALL_EVENTS                         )

;Standard buttons
;----------------
base2      = WIDGET_BASE(root_base_id, FRAME=10, /ROW)
help_id    = WIDGET_BUTTON(base2, VALUE='HELP'              ,UVALUE='help')
cancel_id  = WIDGET_BUTTON(base2, VALUE='CANCEL'            ,UVALUE='cancel')
restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS'   ,UVALUE='save')

if modal then widget_control, cancel_id, /cancel_button
if modal then widget_control, save_id, /default_button

widget_control, root_base_id, set_uvalue=state
widget_control, root_base_id, /realize

mcd_gui_set, state

xmanager, 'mcd_gui', root_base_id, group_leader=group

return, error
end
