; $Id: mcd_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       mcd_init
;
; PURPOSE:
;       mcd_init executes the initialization for the mcd module.
;
; CATEGORY:
;       initialization program
;
; CALLING SEQUENCE:
;    error = mcd_init(inp_img_t1,  $ ; img_t input structure (PSF)
;                     inp_img_t2,  $ ; (IMAGE)
;                     out_img_tE,  $ ; img_t output structure (Diffuse part)
;                     out_img_tP,  $ ; (POINT-LIKE part)
;                     par,         $ ; parameters structure
;                     INIT=init    $ ; initialization data structure
;                     )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see mcd.pro's header for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]
;    modifications  : 
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added common block (called parameters_mcd) to this routine
;                     and to mcd_prog in order to correctly save the parameter 
;                     "par.delta" (delta_ori) in header. (for future reference: 
;                     this can be used for any other parameter)
;-
;
function mcd_init, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, INIT=init

common caos_block, tot_iter, this_iter
common parameters_mcd, delta_ori

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = mcd_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'mcd error: par must be a structure'
if n ne 1 then message, 'mcd error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module mcd'

dummy = test_type(inp_img_t1, TYPE=type)
if type eq 0 then begin         ; undefined variable
   inp_img_t1 = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif

if test_type(inp_img_t1, /STRUC, N_EL=n, TYPE=type) then $
   message, 'mcd error: wrong definition for the first input.'
if n ne 1 then message, $
   'mcd error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t1.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t1.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_img_t1.data_status eq !caos_data.not_valid  then $
      message, 'undefined input is not allowed'

dummy = test_type(inp_img_t2, TYPE=type)
if type eq 0 then begin                ; undefined variable

   inp_img_t2 = $
      {        $
    data_type  : out_type[0]     , $
    data_status: !caos_data.valid $
      }
endif

if test_type(inp_img_t2, /STRUC, N_EL=n, TYPE=type) then $
   message, 'mcd error: wrong definition for the second input.'

if n ne 1 then message, $
   'mcd error: second input cannot be a vector of structures'

; test the data type
if inp_img_t2.data_type ne inp_type[1] then                $
   message, 'wrong input data type: '+inp_img_t.data_type $
           +' ('+inp_type[1]+' expected).'
;if inp_img_t2.data_status eq !caos_data.not_valid and not inp_opt[1] then $
;   message, 'undefined input is not allowed'


dummy = where(tag_names(par) eq 'INIT_FILE',count)
if (count GT 0) then begin
    ;; the module allows the saving and restoring of the initialization
    ;; structure
   if par.init_file ne '' then begin
      save_init    = par.init_save
      restore_init = not save_init
      file_exists  = (findfile(par.init_file))[0] ne ''
   endif
endif


; initialization of the error code: no error as default
info  = mcd_info()
error = !caos_error.ok

;Multi-frame case --> error 
pos=where(inp_img_t2.time_integ[*,1] gt 1,num)
if ((size(inp_img_t2.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_img_t2.image=reform(inp_img_t2.image,(size(inp_img_t2.image))[1],$
                                               (size(inp_img_t2.image))[2],$
                                               (size(inp_img_t2.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE: MULTI-FRAME input is not '+$
                           'supported for deconvolution.',/ERROR)
      message,'WRONG INPUT TYPE: MULTI-FRAME input is not '+$
                           'supported for deconvolution.'
   endelse
endif

;saving the original parameter DELTA in its common block variable
delta_ori = par.delta

; multi-component methods are developed for 2D-3D case,
; in case of 1D array an error message is printed...
IF size(inp_img_t1.image,/N_DIM) EQ 1 THEN message, "Multi-Component methods are developed for 2D-3D case only."

if par.method EQ 0 then begin   ;;;;;;;;;;;;;;  MC-RL METHOD 
   mcd_init_MCRL, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, init
endif else begin   ;;;;;;;;;;;;;;    MC-SGP METHOD 
   mcd_init_MCSGP, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, init
endelse

img=make_array(init.NN,init.NN) ; the reconstructed image (defined in S)

out_img_tE = 				      $
   {          				      $
   data_type  : out_type[0]     	    , $
   data_status: !caos_data.valid	    , $
   image      : img  	                    , $
   psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
   npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
   resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
   lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
   width      : inp_img_t2.width            , $ ; Wavelength band width [m]
   time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
   time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
   background : inp_img_t2.background       , $ ; background
   snr        : inp_img_t2.snr              , $ ; snr
   header     : PTR_NEW('')                 $   ; header of the output file
   }

*out_img_tE.header = *inp_img_t2.header

out_img_tP = 				      $
   {          				      $
   data_type  : out_type[0]     	    , $
   data_status: !caos_data.valid	    , $
   image      : img  	                    , $
   psf        : 0B              	    , $ ; This flag indicates that this is a IMAGE
   npixel     : init.NN              	    , $ ; Number of pixels (per subaperture)
   resolution : inp_img_t2.resolution       , $ ; Pixel size [arcsec/pix]
   lambda     : inp_img_t2.lambda           , $ ; Mean detection wavelength [m]
   width      : inp_img_t2.width            , $ ; Wavelength band width [m]
   time_integ : inp_img_t2.time_integ       , $ ; Integration time for each image [s]
   time_delay : inp_img_t2.time_delay       , $ ; [NOT USED WITHIN AIRY]
   background : inp_img_t2.background       , $ ; background
   snr        : inp_img_t2.snr              , $ ; snr
   header     : PTR_NEW('')                   $ ; header of the output file
   }

*out_img_tP.header = *inp_img_t2.header

return, error
end
