; $Id: mcd_prog.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;
;+
; NAME:
;       mcd_prog
;
; PURPOSE:
;       mcd_prog represents the scientific algorithm for MCD module.
;
; CATEGORY:
;       scientific program
;
; CALLING SEQUENCE:
;       error = mcd_prog(inp_img_t1,  $ ; img_t input structure
;                        inp_img_t2,  $
;                        out_img_tE,  $ ; img_t output structure
;                        out_img_tP,  $ ; img_t output structure
;                        par,         $ ; parameters structure
;                        INIT=init    $ ; initialization data structure
;                       )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see mcd.pro's header for a detailed description.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;    modifications  : 
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -if a stopping rule is satisfied, the variable "last_iter" is 
;                     used for storing current iteration, next "this_iter" is set to the 
;                     total iteration number, and then "tot_iter" is updated with 
;                     the current iteration number (i.e the very last iteration).  
;                    -added common block (called parameters_mcd) to this routine
;                     and to mcd_init in order to correctly save the parameter 
;                     "par.delta" (delta_ori) in header. (for future reference: 
;                     this can be used for any other parameter)
; 
;-


;;;;;;;;;;;;;;;;;;;;;;;;
function jzero, g, den
;;;;;;;;;;;;;;;;;;;;;;;;
  g=double(g)
  den=double(den)
  index=where(g GT 0.)
  t = total(g[index]*alog(g[index]/den[index]),/double) + total(den - g,/double)
  return, t
end


function mcd_prog, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, INIT = init

; CAOS global common block
common caos_block, tot_iter, this_iter

common mcd_iter, last_iter
common parameters_mcd, delta_ori

if this_iter EQ 1 then last_iter=this_iter

if par.method EQ 0 then begin 
   f1=init.f1
   f2=init.f2
   
                                ;update the regularization functions U and V
   grad_dec = reg_grad_dec(f2, par)
   
;reg_functions, f2, init_reg, par_reg, U, V
                                ;iteration
   fnt=FFT((f2+f1),-1,/DOU)
   sum = 0d
   j0 = 0d
   for j=0,init.P-1 do begin
      tmp=init.image2[*,*,j]
      den = double(FFT(init.image1[*,*,j]*fnt, +1, /DOU)) + init.bg[*,*,j]
      dummy = tmp/den
      sum = sum + double(FFT((conj(init.image1[*,*,j])*FFT(dummy,-1,/DOU)),+1,/DOU))
      j0 = j0 + jzero(tmp, den)
   endfor
                                ;next iterate(s)
   f2=init.w*f2/(1.+(par.beta*grad_dec.V_R*init.w))*(sum+par.beta*grad_dec.U_R) ;boundary effects applied only here!
   f1=init.w*f1*sum             ; no boundary on the stellar component
   
                                ; update arrays 
   init.discrepancy[this_iter] = j0*2./(init.P*n_elements(f2))
   init.obj_fun[this_iter] = j0 + par.beta*reg_f(f2, par)
   
                                ; save the output
   init.f1 = f1                 ; f1 and f2 are defined in the R region
   init.f2 = f2
   out_img_tP.image = f1[(init.N-init.NN)/2:(init.N+init.NN)/2-1, (init.N-init.NN)/2:(init.N+init.NN)/2-1]
   out_img_tE.image = f2[(init.N-init.NN)/2:(init.N+init.NN)/2-1, (init.N-init.NN)/2:(init.N+init.NN)/2-1] ;output image is defined in the S region
                                ; restoration error evaluation 
                                ; ONLY ON THE DIFFUSE PART
   if (par.save_rho) OR (par.stopping_rule EQ 4) then begin
      init.rho[this_iter]=sqrt(total(abs(out_img_tE.image-init.object)^2.,/double))/sqrt(total(abs(init.object)^2.,/double))
   endif
   
                                ; stopping rule
   case par.stopping_rule of
                                ; STOPPING RULE #1 (TOT_ITER REACHED!)
      0: begin                  ;really un-useful ;)
         if this_iter GE tot_iter then begin
            if (par.verb) then print, '[#1]  --> STOP AT ITER = ' , this_iter
            last_iter = this_iter
         endif
      end
                                ; STOPPING RULE #2 (NORM OF THE OBJECT)
      1: begin
         if (this_iter EQ 1) then init.prev = out_img_tE.image+out_img_tP.image ; save the reconstruction
         if (this_iter GE 2) then begin
            sum = out_img_tE.image + out_img_tP.image
            diff = sum - init.prev
            normstep = total(diff*diff,/double) / total(sum*sum,/double)
            if (normstep LT par.stop_tol) then begin
               if (par.verb) then print, '[#2]  --> STOP AT ITER = ' , this_iter
               if (par.verb) then print, ' || f(k) - f(k-1) ||^2 / || f(k) ||^2 = '+strtrim(normstep,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
            endif
            init.prev=out_img_tE.image+out_img_tP.image
         endif
      end
                                ;  STOPPING RULE #3  (TOTAL FUNCTIONAL IS APPROX CONSTANT)
      2: begin
         if (this_iter GE 2) then begin
            reldecrease = abs(init.obj_fun[this_iter]-init.obj_fun[this_iter-1])/abs(init.obj_fun[this_iter])
            if (reldecrease LT par.stop_tol) then begin
               if (par.verb) then print, '[#3]  --> STOP AT ITER = ' , this_iter
               if (par.verb) then print, ' | J(k) - J(k-1) | / | J(k) | = '+strtrim(reldecrease,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
            endif
         endif
      end
                                ;  STOPPING RULE #4  (DISCREPANCY CROSSES THE GIVEN TOLERANCE ~~
                                ;  should be around 1)
      3: begin
         if (this_iter GT 2) AND (init.discrepancy[this_iter] LE par.stop_tol) then begin
            if (par.verb) then print, '[#4]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' D(k) = '+strtrim(init.discrepancy[this_iter],1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
         endif
      end
                                   ;  STOPPING RULE #5  (MINIMUM OF THE RESTORATION ERROR)
      4: begin
         if (this_iter GT 2) AND (init.rho[this_iter] GT init.rho[this_iter-1]) then begin
            if (par.verb) then print, '[#5]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' RHO (k) = '+strtrim(init.rho[this_iter],1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
         endif
      end
      else: message, 'Unknown stopping criterion:', stopcrit
   endcase
   
   
endif else begin
                                ; MCSGP main loop
   init.Valpha[0:init.Malpha-2] = init.Valpha[1:init.Malpha-1]
   if init.M gt 1 then init.Fold[0:init.M-2] = init.Fold[1:init.M-1]
   init.Fold[init.M-1] = init.fv
                                ; compute step along the negative scaled gradient
   y_d = init.x_d - init.alpha*init.XX_d*init.g_d
   y_p = init.x_p - init.alpha*init.XX_p*init.g_p
   
                                ; projection
   if par.flux_con then begin
      temp = [y_d[init.regionR_index],y_p]
      D = reform([init.D_d[init.regionR_index],init.D_p])
      temp = project_fletcher_nonneg(reform([init.ATM[init.regionR_index],init.ATM[init.ind_po]]), init.c*init.dim[2], reform(temp)*D, D)
      y_d[init.regionR_index] = temp[0:init.n_R-1]
      y_p = temp[init.n_R:init.n_R+init.n_po-1]
   endif else begin
      temp =  where(y_d lt 0, n_nonpos)
      if (n_nonpos gt 0) then y_d[temp] = 0.
      temp =  where(y_p lt 0, n_nonpos)
      if (n_nonpos gt 0) then y_p[temp] = 0.
   endelse
                                ; compute descent direction
   d_d = y_d - init.x_d
   d_p = y_p - init.x_p
   gd = total(d_d*init.g_d,/double) + total(d_p*init.g_p,/double)
   d = d_d
   d[init.ind_po] = d[init.ind_po] + d_p
   
   
                                ; Armijo linesearch
   lam = 1.
   fcontinue = 1B
   d_tf = Afunction(d,init.TF)  ; exploiting linearity
   fr = max(init.Fold)
   while fcontinue do begin
      xplus_d = init.x_d + lam*d_d
      xplus_p = init.x_p + lam*d_p
      x_tf_try = init.x_tf + lam*d_tf
      temp = init.gn/(x_tf_try + init.bg)
      if (par.reg gt 0) then begin
         f_R = reg_f(xplus_d, par) 
      endif else begin
         f_R = 0.
      endelse
      f_0 = total(init.gn[init.obj_index_multi]* alog(temp[init.obj_index_multi]),/double) + total(x_tf_try[init.obj_index_multi],/double) - total(init.flux,/double)
      init.fv = f_0 + par.beta*f_R
      if (init.fv le fr + init.beta*lam*gd) OR (lam lt 1.e-12) then begin
         init.x_d = xplus_d
         init.x_p = xplus_p
         sk = lam*[d_d[*],d_p]
         init.x_tf = x_tf_try
         gtemp = Afunction(temp,init.CTF)
         if init.dim[2] gt 1 then begin
            gtemp = total(gtemp,3,/double)
         endif
         if (par.reg gt 0) then begin
            grad_dec = reg_grad_dec(init.x_d, par)
         endif else begin
            grad_dec = {U_R:0., V_R:0.}
         endelse
         gtemp_d = init.ATM - gtemp + par.beta*(grad_dec.V_R - grad_dec.U_R)
         gtemp_p = init.ATM[init.ind_po] - gtemp[init.ind_po]
         yk = [gtemp_d[*] - init.g_d[*], gtemp_p - init.g_p]
         init.g_d = gtemp_d
         init.g_p = gtemp_p
         fcontinue = 0B
      endif else begin
         lam = lam * init.theta
      endelse
   endwhile
   if (init.fv ge fr) and (par.verb gt 0) then print, 'WARNING: fv >= fr'
   
                                ; compute new steplength and scaling matrices
                                ; scaling matrix - diffuse part
   temp = init.ATM + par.beta*grad_dec.V_R
   Weight_R = dblarr(init.work_size)
   Weight_R[init.regionR_index] = 1./temp[init.regionR_index]
   init.XX_d = Weight_R*init.x_d
   XX_toolow = where(init.XX_d lt init.X_low_bound_d, nlow)
   if (nlow gt 0) then init.XX_d[XX_toolow] = init.X_low_bound_d
   XX_toohigh = where(init.XX_d gt init.X_upp_bound_d, nhigh)
   if (nhigh gt 0) then init.XX_d[XX_toohigh] = init.X_upp_bound_d
   if (par.bound) then begin
      temp = dblarr(work_size)
      temp[init.regionR_index] = init.XX_d[init.regionR_index]
      init.XX_d = temp
   endif
   init.D_d = dblarr(init.work_size)
   init.D_d[init.regionR_index] = 1./init.XX_d[init.regionR_index]
                                ; scaling matrix - pointwise part
   init.XX_p = init.Weight[init.ind_po]*init.x_p
   XX_toolow = where(init.XX_p lt init.X_low_bound_p, nlow)
   if (nlow gt 0) then init.XX_p[XX_toolow] = init.X_low_bound_p
   XX_toohigh = where(init.XX_p gt init.X_upp_bound_p, nhigh)
   if (nhigh gt 0) then init.XX_p[XX_toohigh] = init.X_upp_bound_p
   init.D_p = 1./init.XX_p
   
   
                                ; steplength
   sk2 = sk*[init.D_d[*],init.D_p]
   yk2 = yk*[init.XX_d[*],init.XX_p]
   bk = total(sk2*yk,/double)
   ck = total(yk2*sk,/double)
   if (bk le 0) then begin
      alpha1 = min([10.*init.alpha,init.alpha_max])
   endif else begin
      alpha1BB = total(sk2*sk2,/double)/bk
      alpha1 = min([init.alpha_max, max([init.alpha_min, alpha1BB])])
   end
   if (ck le 0) then begin
      alpha2 = min([10.*init.alpha,init.alpha_max])
   endif else begin
      alpha2BB = ck/total(yk2*yk2,/double)
      alpha2 = min([init.alpha_max, max([init.alpha_min, alpha2BB])])
   end
   init.Valpha[init.Malpha-1] = alpha2
   if (this_iter le 20) then begin
      init.alpha = min(init.Valpha)
   endif else begin
      if (alpha2/alpha1 lt init.tau) then begin
         init.alpha = min(init.Valpha)
         init.tau = init.tau*0.9
      endif else begin
         init.alpha = alpha1
         init.tau = init.tau*1.1
      endelse
   endelse
   init.alpha = double(float(init.alpha))
   init.discrepancy[this_iter] = init.Discr_coeff * f_0
   init.obj_fun[this_iter] = init.fv * init.scaling
                                ; compute image x
   x = init.x_d
   x[init.ind_po] = x[init.ind_po] + init.x_p
                                ; error
   if init.errflag then begin
      e = init.x_d[init.obj_index] - init.obj[*]
      init.rho[this_iter] = sqrt(total(e*e,/double)/init.obj_sum)
   endif
                                ; stopping criteria
   case par.stopping_rule of
                                ; STOPPING RULE #1 (TOT_ITER REACHED!)
      0: begin                  ;really un-useful ;)
         if this_iter GE tot_iter then begin
            if (par.verb) then print, '[#1]  --> STOP AT ITER = ' , this_iter
            last_iter = this_iter
         endif
      end
                                ; STOPPING RULE #2 (NORM OF THE OBJECT)
      1: begin  
         normstep = total(sk*sk,/double) / total(x*x,/double)
         if (normstep lt par.stop_tol) then begin
            if (par.verb) then print, '[#2]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' || f(k) - f(k-1) ||^2 / || f(k) ||^2 = '+strtrim(normstep,1), ' ;   tol = '+strtrim(par.stop_tol,1)
            last_iter = LONG(this_iter)
            this_iter=LONG(tot_iter)
            tot_iter=LONG(last_iter)
         endif
      end
                                ;  STOPPING RULE #3  (TOTAL FUNCTIONAL IS APPROX CONSTANT)
      2: begin
         reldecrease = abs(init.fv - init.Fold[init.M-1])/abs(init.fv)
         if (reldecrease lt par.stop_tol) then begin
            if (par.verb) then print, '[#3]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' | J(k) - J(k-1) | / | J(k) | = '+strtrim(reldecrease,1), ' ;   tol = '+strtrim(par.stop_tol,1)
            last_iter = LONG(this_iter)
            this_iter=LONG(tot_iter)
            tot_iter=LONG(last_iter)
         endif
      end
                                ;  STOPPING RULE #4  (DISCREPANCY CROSSES THE GIVEN TOLERANCE ~~
                                ;  should be around 1)
      3: begin
         if (this_iter GT 2) AND (init.discrepancy[this_iter] lt par.stop_tol) then begin
            if (par.verb) then print, '[#4]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' D(k) = '+strtrim(init.discrepancy[this_iter],1), ' ;   tol = '+strtrim(par.stop_tol,1)
            last_iter = LONG(this_iter)
            this_iter=LONG(tot_iter)
            tot_iter=LONG(last_iter)
         endif
      end
                                ;  STOPPING RULE #5  (MINIMUM OF THE RESTORATION ERROR)
      4: begin
         if (this_iter GT 2) AND (init.rho[this_iter] GT init.rho[this_iter-1]) then begin
            if (par.verb) then print, '[#5]  --> STOP AT ITER = ' , this_iter
            if (par.verb) then print, ' RHO (k) = '+strtrim(init.rho[this_iter],1)
            last_iter = LONG(this_iter)
            this_iter=LONG(tot_iter)
            tot_iter=LONG(last_iter)
         endif
      end
      else: message, 'Unknown stopping criterion:', stopcrit
   endcase

   if par.verb gt 0 then begin
      print, 'Iteration:', this_iter,  '  Fobj:', init.fv, '  Alpha:', init.alpha, '  Lambda:', lam
   endif
   
                                ; save the output
   out = init.x_d[init.obj_index]
   out = reform(out,init.obj_size)
   out = out * init.scaling
   out_img_tE.image = out
   
   x = init.x_p
   out = dblarr(init.work_size)
   out[init.ind_po] = x
   out = out[init.obj_index]
   out = reform(out,init.obj_size)
   out = out * init.scaling
   out_img_tP.image = out
   
endelse 

                                ; write optional outputs
if (par.save_rho) then begin
   printf, init.unit_r, this_iter, init.rho[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_r
endif

if (par.save_discrepancy) then begin
   printf, init.unit_d, this_iter, init.discrepancy[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_d
endif

if (par.save_obj_fun) then begin
   printf, init.unit_o, this_iter, init.obj_fun[this_iter]
   if this_iter GE LONG(tot_iter) then close,init.unit_o
endif



                                ;update the header
header = *out_img_tE.header
method = ['MCRL', 'MCSGP']
fxaddpar, header, 'METHOD  ', method[par.method], '[MCD] deconvolution method'
if par.method EQ 1 then begin
   fxaddpar, header, 'FLUX_CON',par.flux_con, '[MCD] Flux constraint'
endif
fxaddpar, header, 'INIT_P', par.init_P, '[MCD] Initialization of Point-like part'
if (par.init_P EQ 1) then begin
   fxaddpar, header, 'FILE_P', par.init_P_filename, '[MCD] Initialization filename'
endif
fxaddpar, header, 'INIT_E', par.init_E, '[MCD] Initialization of Diffuse part'
if (par.init_E EQ 1) then begin
   fxaddpar, header, 'FILE_E', par.init_E_filename, '[MCD] Initialization filename'
endif
if par.bound then begin
   fxaddpar, header, 'BOUND', par.bound, '[MCD] Boundary Effect Correction '
   fxaddpar, header, 'EE_THRES', par.ee_threshold, '[MCD] Enclosed Energy threshold'
endif
if (par.reg GT 0) then begin
   fxaddpar, header, 'REG_TYPE',par.reg, '[MCD] Regularization #'
   fxaddpar, header, 'REG_BETA',par.beta, '[MCD] Regularization parameter (beta)'
   if (par.reg EQ 4) OR (par.reg EQ 5) OR (par.reg EQ 6)  OR (par.reg EQ 7) then begin
      fxaddpar, header, 'REG_DELT',delta_ori, '[MCD] Regularization parameter (delta)'
   endif
endif
fxaddpar, header, 'STOP_CRI', par.stopping_rule+1, '[MCD] stopping rule #'
if (par.stopping_rule GT 1) AND (par.stopping_rule NE 4) then begin
   fxaddpar, header, 'STOP_TOL', par.stop_tol, '[MCD] stopping tolerance (if set)'
endif
fxaddpar, header, 'ITER    ', last_iter, '[MCD] Number of iterations'

*out_img_tE.header = header
*out_img_tP.header = header


end 
