; $Id: obj.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    obj
;
; ROUTINE'S PURPOSE:
;    OBJ manages the simulation for the OBJect (OBJ) module,
;    that is:
;      1-call the module's initialization routine obj_init.pro at the first
;        iteration of the project
;      2-call the module's program routine obj_prog.pro otherwise.
;
; MODULE'S PURPOSE:
;    OBJ permits to define map of different types of astronomical objects as 
;    a target for observation and/or subsequent image reconstruction/deconvolution.
;    
;    Current object types are: single star, binary object, open cluster, 
;    planetary nebulae, supernovae remnant, spiral galaxy, Young Stellar Objects (YSOs), 
;    stellar surface and user-defined.
;
;    For each object type the map is defined by its dimension [npixel] and pixel size. 
;
;    In particular, some remarks concerning the definition of some objects are the 
;    following : 
;    
;    Single star : pixel located at [npixel/2,npixel/2] containing all the flux 
;                  (i.e. unresolved star or point source). 
;                  One may pay attention to the fact that such a definition will 
;                  creates artifacts if one use PSFs (for both the convolution/
;                  modelization and deconvolution) which are centered on 
;                  [npixel-1/2,npixel-1/2]. 
;                  A work around is to use a dirac map (pixel at [0,0] set to 1, 
;                  0 elsewhere) within the user-defined object type. 
;    
;    Binary object : Formed by two point-source stars for which the barycenter 
;                    (not photocenter) is located in [npixel/2,npixel/2]. 
;                    In order to center each component on a pixel, the separation
;                    and position angle may vary slightly from the one set by 
;                    user in the GUI. A warning appears if such a variation is 
;                    more than 1%. 
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = obj(out_obj_t, par, INIT=init)
;
; OUTPUT:
;    error: long scalar (error code). see !caos_error var in caos_init.pro.
;
; INPUTS:
;    none.
;
; INCLUDED OUTPUTS:
;    out_obj_t: structure of type "src_t" containing the characteristics of
;               the selected source, the following fields are contained in
;               this structure:
;out_obj_t =                       $
;   {                              $
;   data_type  : info.out_type[0], $
;   data_status: !caos_data.valid, $
;   off_axis   : init.off_axis,    $
;   pos_ang    : init.pos_ang,     $
;   dist_z     : init.dist_z,      $
;   map        : init.map,         $
;   scale_xy   : init.scale_xy,    $
;   coord      : init.coord,       $
;   scale_z    : init.scale_z,     $
;   n_phot     : init.n_phot,      $
;   background : init.background,  $
;   lambda     : init.lambda,      $
;   width      : init.width,       $
;   constant   : init.constant,    $
;   header     : PTR_NEW('')       $
;   }
;*out_obj_t.header = header  ; update the pointer
;
; KEYWORD PARAMETERS:
;    INIT: named variable undefined or containing a scalar when obj is
;          called for the first time. As output the named variable will
;          contain a structure of the initialization data. For the following
;          calls of obj, the keyword INIT has to be set to the structure
;          returned by the first call.
;
; COMMON BLOCKS:
;    common caos_ao_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    The "Object total magnitude" field within the sub-GUIs "YSOs"
;    and "Spiral galaxy" remains to the original value although it has been changed
;    and the whole set of parameters was saved before.
;    This is just a display problem, since the change of this value is effectively
;    taken into account (as it can be seen at the IDL/CAOS prompt line).
;
; PROCEDURE:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    -n_phot  : for number of photons calculus (see in .../lib).
;    -spec2mag: to transform a V-magnitude into any other band
;               magnitude (see in .../lib).
;    -other routines from the astrolib/astron IDL library
;    -other routines from the folder .../obj/obj_lib
;                     and the folder .../obj/obj_gui_lib
;
; ROUTINE MODIFICATION HISTORY:
;    program written: may 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;                    -call to obj_prog eliminated (useless).
;                   : november 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -help updated.
;
; MODULE MODIFICATION HISTORY:
;    program written: version beta,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;                   : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;                    -obj_prog eliminated since it was irrelevant.
;                   : for version 2.0 too,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -object type single star added.
;                    -help updated.
;                   : for version 5.0,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -flux constants are now "float" instead of "int",
;                    -object type (obj_t) completed (tag angle added),
;                    -modeling of elliptical disk, gaussian and stellar surface debugged.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New sub-GUIs for Planetary Nebula, SNR, Spiral galaxy, YSO, 
;                     and user-defined objects. 
;                    -Bug fixed in parameter restoring procedure (esp. Planetary 
;                     nebula and SNR).
;                    -Small changes in the text of the GUI.
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -keywords added to the header (that is now created
;                     whether the map is saved or not), most commons are:
;                     OBJECT, BAND, and PIX_SIZE. All the keywords are
;                     described in the FitsHeader_Keywords_definition.txt file. 
;-
;
function obj, out_obj_t, $
              par,       $
              INIT=init

COMMON caos_block, tot_iter, this_iter

error = !caos_error.ok          ; Init error code: no error as default

IF (this_iter EQ 0) THEN error= obj_init(out_obj_t, par, INIT=init)

return, error
END
