; $Id: obj_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    obj_gui
;
; PURPOSE:
;    obj_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the OBJect (OBJ) module.
;    a parameter file called obj_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the the module instance.
;    the file is stored in the project directory proj_name located
;    in the working directory.
;
; CATEGORY:
;    Graphical User Interface program
;
; CALLING SEQUENCE:
;    error = obj_gui(n_module, proj_name)
;
; INPUTS:
;    n_module : integer scalar. Number associated to the instance
;               of the OBJ module. n_module > 0.
;    proj_name: string. Name of the current project.
;
; OUTPUTS:
;    error: long scalar, error code (see !caos_error var in caos_init.pro).
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON_IDL FUNCTIONS:
;    n_phot
;    spec2mag
;
; MODIFICATION HISTORY:
;    program written: may 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -widget indicating the type of object selected added.
;                    -pixel size is no more changed to the spectral band
;                     default value when the spectral band itself is changed.
;                    -a few other minor changes.
;                   : september 2000,
;                     Serge Correia (OAA)
;                    -a few modifications made.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(obj_info()).help stuff added (instead of !caos_env.help).
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -sky background control added.
;                   : november 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -added the object type "single star".
;                   : september 2005
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;                    - added help for windows version
;                   :January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Flux constants are now "float" instead of "int".
;                   : may 2007, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simplest way to call AIRY_HELP.
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New sub-GUIs for Planetary Nebula, SNR, Spiral galaxy, YSO, 
;                     and user-defined objects. 
;                    -Bug fixed in parameter restoring procedure (esp. Planetary 
;                     nebula and SNR).
;                    -Small changes in the text of the GUI.
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting provedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro obj_gui_set, state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

;write the gui state structure in current state.id.obj_base[] (if different from default)
;if state.type_obj ne state.type_obj_nb then                           $
;widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state

if (state.par.save_obj_fits.enable eq 0B) then begin
  widget_control, state.id.save_obj_filename, SENSITIVE=0
endif else begin
  widget_control, state.id.save_obj_filename, /SENSITIVE
endelse

;print, ">>>>>>>>>", state.par.obj_nb
end

;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;



;---------------------------- Single star widget ------------------------------

pro single_star_gui_set, state

;set broad band index
widget_control, state.id.single_star_band, SET_COMBOBOX_SELECT = state.par.band_index

end

pro single_star_gui_event, event


widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state

widget_control, event.id,  GET_UVALUE=uval

;print, uval

case uval of

'band': begin
    state.par.band_index = event.index
    ;;;set default pixel size value
    ;;state.par.pix_size = state.par.pix_size_tab[event.index]
    ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
    ;change the field extension value , field magnitude value and re-display object
    ;in the current type object gui
    state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
    update_obj_par, state
    obj_gui_set, state
    end


'single_star_mag' : begin
    state.par.single_star_mag = event.value
    end


endcase

;default setting
single_star_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end


pro single_star_gui, state

  state.id.obj_base[state.type_obj] = widget_base(title ='Single star',$
                          /COLUMN, GROUP_LEADER=state.id.root_base_id)
  obj_up=widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
  obj_down=widget_base(state.id.obj_base[state.type_obj],/FRAME, /COLUMN)
  single_star_band=widget_base(obj_up, /ROW)
  dummy= widget_label(single_star_band, VALUE='Band: ')
  state.id.single_star_band = widget_combobox(single_star_band, $
                         VALUE=state.par.band_tab, UVALUE='band')
  state.id.single_star_mag = cw_field(obj_up, $
               TITLE='Star magnitude:            ',  $
               /FLOATING, VALUE=state.par.single_star_mag, $
               UVALUE='single_star_mag', /ALL_EVENTS       )
  state.id.single_star_field_extension  = cw_field(obj_down, $ 
               TITLE='(Field extension in arcsec)', /FLOATING,  $
               VALUE=state.par.field_extension,              $
               UVALUE='single_star_field_extension', /NOEDIT )
;default setting
single_star_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
xmanager, 'single_star_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id

return
end


;------------- Binary stars widget ---------------------------------------

pro binary_star_gui_set, state

;set broad band index
widget_control, state.id.bin_band, SET_COMBOBOX_SELECT = state.par.band_index

end

pro binary_star_gui_event, event
widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval

;print, uval

case uval of

'band': begin
   state.par.band_index = event.index
   ;;;set default pixel size value
   ;;state.par.pix_size = state.par.pix_size_tab[event.index]
   ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state
   obj_gui_set, state
   end

'bin_separation': begin
   state.par.bin_separation = event.value
   end

'bin_pa': begin
   state.par.bin_pa = event.value
   end

'bin_mag': begin
   state.par.bin_mag = event.value
   end

'bin_delt_m': begin
   state.par.bin_delt_m = event.value
   end

endcase

;default setting
binary_star_gui_set, state
;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state
;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end


pro binary_star_gui, state

   state.id.obj_base[state.type_obj]=widget_base( $ 
                                     title ='Binary type object characteristics', $
                                     /COLUMN, GROUP_LEADER=state.id.root_base_id )
   obj_up = widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
   obj_down = widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
   bin_band = widget_base(obj_up, /ROW)
   dummy = widget_label(bin_band,  VALUE='Band: ')
   state.id.bin_band = widget_combobox(bin_band, VALUE=state.par.band_tab, UVALUE='band')
   state.id.bin_separation = cw_field(obj_up, $
                    TITLE='Separation (arcsec):           ', $
                    /FLOATING, VALUE=state.par.bin_separation,               $
                    UVALUE='bin_separation', /ALL_EVENTS  )
   state.id.bin_pa = cw_field(obj_up, $
                    TITLE='Position angle (deg):          ',         $
                    /FLOATING, VALUE=state.par.bin_pa, UVALUE='bin_pa', /ALL_EVENTS )
   state.id.bin_mag = cw_field(obj_up,$
                    TITLE='Principal component magnitude: ',         $
                    /FLOATING, VALUE=state.par.bin_mag, UVALUE='bin_mag',           $
                    /ALL_EVENTS)
   state.id.bin_delt_m = cw_field(obj_up, $
                    TITLE='Magnitude difference:          ',     $
                    /FLOATING, VALUE=state.par.bin_delt_m, UVALUE='bin_delt_m', $
                    /ALL_EVENTS)
   state.id.bin_field_extension = cw_field(obj_down, $
                    TITLE='(Field extension in arcsec)    ', $
                    /FLOATING, VALUE=state.par.field_extension,          $
                    UVALUE='bin_field_extension', /NOEDIT )
;default setting
binary_star_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
xmanager, 'binary_star_gui', state.id.obj_base[state.type_obj], $
          GROUP_LEADER=state.id.root_base_id
return
end



;-------------------- Random generated open cluster widget --------------------

pro open_cluster_gui_set, state

;set broad band index
widget_control, state.id.clust_band, SET_COMBOBOX_SELECT = state.par.band_index

end

pro open_cluster_gui_event, event


widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval
;print, uval

case uval of

'band': begin
    state.par.band_index = event.index
    ;;;set default pixel size value
    ;;state.par.pix_size = state.par.pix_size_tab[event.index]
    ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
    ;change the field extension value , field magnitude value and re-display object
    ;in the current type object gui
    state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
    update_obj_par, state
    obj_gui_set, state
    end

'clust_nb_star': begin
    state.par.clust_nb_star = event.value
    end

'clust_mag_min': begin
    state.par.clust_mag_min = event.value
    end

'clust_mag_max': begin
    state.par.clust_mag_max = event.value
    end

'clust_seed': begin
    state.par.clust_seed = event.value
    end
endcase

;default setting
open_cluster_gui_set, state
;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state
;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state
return
end


pro open_cluster_gui, state
   state.id.obj_base[state.type_obj]= widget_base( $
                  title ='Randomn generated cluster characteristics', /COLUMN, $
                                        GROUP_LEADER=root_base_id              )
   obj_up = widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
   obj_down = widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
   clust_band = widget_base(obj_up, /ROW)
   dummy = widget_label(clust_band,  VALUE='Band: ')
   state.id.clust_band = widget_combobox(clust_band, VALUE=state.par.band_tab, $
                         UVALUE='band')
   state.id.clust_nb_star  = cw_field(obj_up, TITLE='Star number:                    ', $
                             /INTEGER, VALUE=state.par.clust_nb_star,  $
                             UVALUE='clust_nb_star',/ALL_EVENTS  )
   state.id.clust_mag_min  = cw_field(obj_up, TITLE='Minimum star magnitude:         ', $
                             /FLOATING, VALUE=state.par.clust_mag_min,    $
                             UVALUE='clust_mag_min',  /ALL_EVENTS  )
   state.id.clust_mag_max  = cw_field(obj_up, TITLE='Maximum star magnitude:         ', $
                             /FLOATING, VALUE=state.par.clust_mag_max, $
                             UVALUE='clust_mag_max', /ALL_EVENTS   )
   state.id.clust_seed     = cw_field(obj_up, TITLE='Randomn cluster generation seed:', $
                             /LONG, VALUE=state.par.clust_seed, UVALUE='clust_seed',   $
                             /ALL_EVENTS  )
   state.id.clust_field_extension=cw_field(obj_down,$
                             TITLE='(Field extension in arcsec)     ', /FLOATING, $
                             VALUE=state.par.field_extension,             $
                             UVALUE='clust_field_extension', /NOEDIT )
;default setting
open_cluster_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
xmanager, 'open_cluster_gui', state.id.obj_base[state.type_obj], $
          GROUP_LEADER=state.id.root_base_id
return
end



;---------------------------- Planetary Nebulae widget -------------------------

; Calculation of the flux (photons/s/m2) of an emission lines object ***********

function spectrum_flux, state
h = 6.626075d-34   ; Planck constant in international unit system
c = 3.0d8          ; Light velocity in international unit system
nb_lines = (size(state.par.neb.spectrum.spectrum_flux))[2]
;define the band wavelength bound limits
lambda_inf = state.par.lambda - state.par.width/2.
lambda_sup = state.par.lambda + state.par.width/2.
;compute ph and mag
dummy=state.par.neb_spectra_nb
ph = 0.d                ; photon number per second per m^2 (DOUBLE VALUE!)
for i=0, nb_lines-1 do begin
    lam=state.par.neb.spectrum.lambda[i]
    if  (lam ge lambda_inf) and (lam le lambda_sup) then begin
       val=state.par.neb.user_def_spectrum.flux[dummy,i]
       ph = ph + (val*1d-3/(h*c/(lam*1d-10)))
    endif
endfor
return, ph
end

; ******************************************************************************

pro planetary_nebulae_gui_set, state

;;;;; NOTHING TO DO????

end

pro planetary_nebulae_gui_event, event
widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval
;print, uval
case uval of
'neb_extension' : begin
   state.par.neb.extension[state.par.obj_nb] = event.value
   ;re-display object
   display_object, state
   end

'spectra_choice' : begin
   case event.select of
   1 : begin
     ;plot the current spectrum
      if state.par.flag eq -1 then window, /FREE
      state.par.flag = !D.window
      plot_neb_spectrum, state
      end
   0 : begin
      if state.par.flag ne -1 then wdelete, state.par.flag
      state.par.flag = -1
      end
   endcase
end

'spectra_number' : begin
   ;update the spectra index
   state.par.neb_spectra_nb = event.index
   widget_control, state.id.neb_spectra_choice, GET_VALUE = dummy
   if dummy eq 1 then begin
      ;update the values of the emission lines fluxes
      nb_lines = (size(state.par.neb.spectrum.spectrum_flux))[2]
      for i=0, nb_lines-1 do begin
          widget_control, state.id.neb_em_lines[i], $
                          SET_VALUE=state.par.neb.spectrum.spectrum_flux[event.index,i]
          state.par.neb.user_def_spectrum.flux[event.index, i] = $
                          state.par.neb.spectrum.spectrum_flux[event.index,i]
      endfor
      ;plot the current spectrum
      if state.par.flag eq -1 then window, /FREE
      state.par.flag = !D.window
      state.par.neb.user_def_spectrum.status = ''
      plot_neb_spectrum, state

      ;compute nb_photons / second / m^2 from spectrum
       state.par.neb.flux = spectrum_flux(state)
       widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   endif
end

'neb_lambda': begin
   state.par.lambda = event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   end

'neb_width': begin
   state.par.width = event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   end
 
string(state.par.neb.user_def_spectrum.lambda[0]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 0]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[1]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 1]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[2]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 2]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[3]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 3]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[4]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 4] = event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[5]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 5]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[6]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 6]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[7]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 7]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
end

string(state.par.neb.user_def_spectrum.lambda[8]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 8]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[9]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 9]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[10]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 10]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[11]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 11]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[12]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 12]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[13]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 13]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[14]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 14]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[15]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 15]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[16]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 16]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[17]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 17]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[18]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 18]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[19]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 19]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[20]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 20]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[21]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 21]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[22]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 22]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[23]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 23]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[24]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 24]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[25]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 25]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[26]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 26]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[27]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 27]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[28]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 28]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[29]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 29]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[30]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 30]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end
string(state.par.neb.user_def_spectrum.lambda[31]) : begin
   state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb, 31]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.neb.flux = spectrum_flux(state)
   widget_control, state.id.neb_flux, SET_VALUE=state.par.neb.flux
   ;plot the updated spectrum
   state.par.neb.user_def_spectrum.status = ' (modified)'
   plot_neb_spectrum, state
   end

'neb_sky_flux' : state.par.sky_flux = event.value
else: ;do nothing!
endcase

;default setting
planetary_nebulae_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end

pro planetary_nebulae_gui, state


  state.id.obj_base[state.type_obj] = widget_base($
                               title='Nebulae type object characteristics',    $
                              /COLUMN, GROUP_LEADER=state.id.root_base_id )
  tab1=WIDGET_TAB(state.id.obj_base[state.type_obj], location=0, uvalue='')
  ;first tab
  obj_info= widget_base(tab1, /ROW, TITLE="Object information")
  obj_info_left=widget_base(obj_info, /COL, /FRAME)
  obj_info_right=widget_base(obj_info, /COL, /FRAME)
  neb_name= widget_base(obj_info_left, /COL, /FRAME)
  dummy=widget_label(neb_name, value="Select an object:", /ALIGN_LEFT)
  state.id.neb_name_obj= widget_combobox(neb_name, VALUE=state.par.neb.name_obj,$
                         UVALUE='neb_name_obj')
  state.id.neb_image= widget_draw(neb_name, xsize=256, ysize=256, /FRAME)
  extension= widget_base(obj_info_left, /COL, /FRAME)
  state.id.neb_extension= cw_field(extension, $
                           TITLE='Object extension in arcsec [ENTER]',$
                           /FLOATING, xsize=8, $
                           /RETURN_EVENTS, UVALUE='neb_extension',   $
                           VALUE=state.par.neb.extension[state.par.obj_nb]     )
  state.id.neb_field_extension = cw_field(extension, $
                           TITLE='(Field extension in arcsec)       ',$
                           /FLOATING,VALUE=state.par.field_extension,$
                           /NOEDIT, xsize=8)
  wavelength_range= widget_base(obj_info_right, /FRAME, /COL)
  dummy= widget_label(wavelength_range, $
                   VALUE='WAVELENGTH RANGE CONSIDERED   ')
  dummy= widget_base(wavelength_range, /COL)
  state.id.neb_lambda= cw_field(dummy,$
                      TITLE='Central wavelength [Angstroem]                  ',$
                      /FLOATING,VALUE=state.par.lambda, UVALUE='neb_lambda',  $
                      /ALL_EVENTS)
  state.id.neb_width= cw_field(dummy, $
                      TITLE='Width [Angstroem]                               ',$
                      /FLOATING, VALUE=state.par.width, UVALUE='neb_width',              $
                      /ALL_EVENTS)
  neb_flux= widget_base(obj_info_right, /FRAME, /COL)
  dummy= widget_label(neb_flux, $
                   VALUE='TOTAL FLUX IN WAVELENGTH RANGE')
  state.id.neb_flux  = cw_field(neb_flux, $
                      TITLE='(Resulted emission lines flux [photons/s/m^2])  ',$
                      /FLOATING, VALUE=state.par.neb.flux, /NOEDIT )
  state.id.neb_sky_flux  = cw_field(neb_flux, $
                      TITLE='Sky background flux [photons/s/m^2]             ',$
                      /FLOATING, VALUE=state.par.sky_flux, $
                      UVALUE='neb_sky_flux', /ALL_EVENTS)
  ;second tab
  emission= widget_base(tab1, /COL, TITLE="Emission Lines")
  spectra_neb= widget_base(emission, /COL, /FRAME)
  state.id.neb_spectra_choice  = cw_bgroup(spectra_neb, $
               ['Emmission Lines library [4102-7325 A]                      '],$
               SET_VALUE=state.par.neb_spectra_choice, UVALUE='spectra_choice',$
               /NONEXCLUSIVE )
  state.id.neb_spectra_number = widget_list(spectra_neb, ysize=5, xsize=10, $
                               VALUE = state.par.neb.spectrum.name,  $
                               UVALUE='spectra_number' )
  neb_emission_lines=widget_base(emission, /FRAME, /COL)
  dummy= widget_label(neb_emission_lines, VALUE='EMISSION LINES FLUX (mW/m2)')
  neb_emission_base= widget_base(neb_emission_lines, /ROW)
  em_lines_text = ['H delta '  , 'H gamma '  , 'OIII    '  , 'HeI     '  , 'HeI     '  , $
                   'HeII    '  , 'HeII    '  , 'ArIV    '  , 'H beta  '  , 'HeI     '  , $
                   '[OIII]  '  , '[OIII]  '  , 'NI      '  , 'HeII    '  , 'ClIII   '  , $
                   'ClIII   '  , 'NII     '  , 'HeI     '  , 'OI      '  , 'SIII    '  , $
                   'OI      '  , 'ArV     '  , '[NII]   '  , 'H alpha '  , '[NII]   '  , $
                   'HeI     '  , 'SII     '  , 'SII     '  , 'ArV     '  , 'HeI     '  , $
                   'ArIII   '  , '[OII]   '                                              ]

  nb_lines = (size(em_lines_text))[1]
  num_by_col = 8
  neb_emission_lines_col= [widget_base(neb_emission_base, /COL),   $
                           widget_base(neb_emission_base, /COL),   $
                           widget_base(neb_emission_base, /COL),   $
                           widget_base(neb_emission_base, /COL)    ]
  for i=0, nb_lines-1 do begin

     state.id.neb_em_lines[i]= cw_field(neb_emission_lines_col[i/num_by_col], $
         TITLE=em_lines_text[i] + '(' + $
         strtrim(state.par.neb.user_def_spectrum.lambda[i],1)+' A):',/FLOAT, $
         VALUE=state.par.neb.user_def_spectrum.flux[state.par.neb_spectra_nb,i],$
         /ALL_EVENTS, UVALUE=string(state.par.neb.user_def_spectrum.lambda[i]), $
         xsize=13)
  endfor

;default setting
planetary_nebulae_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
display_object, state
xmanager, 'planetary_nebulae_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id
return
end

pro plot_neb_spectrum, state

;current spectra index
dummy = state.par.neb_spectra_nb
;emission lines total number in the spectrum
dim = size(state.par.neb.spectrum.spectrum_flux)
nb_lines = dim[2]
;plot the emission lines (width=10 angstroem)
plotcolorfill_modif, state.par.neb.spectrum.lambda,                         $
               state.par.neb.user_def_spectrum.flux[dummy,*],               $
               PSYM=1,                                                      $
               TITLE = state.par.neb.spectrum.name[dummy] +                 $
                        state.par.neb.user_def_spectrum.status,             $
               WIDTH =10.,                                                  $
               XTITLE='lambda [Angstroem]',                                 $
               YTITLE='Emission lines flux [mW/m2]',                        $
               COLOR=200,                                                   $
               POSITION=ASPECT(0.7, Margin=0.15)

maxV = max(state.par.neb.user_def_spectrum.flux[dummy,*])

;display the emission lines names
for i=0, nb_lines-1 do begin
    if state.par.neb.user_def_spectrum.flux[dummy,i] ge 0.05*maxV then      $
    xyouts, state.par.neb.spectrum.lambda[i],                               $
            state.par.neb.user_def_spectrum.flux[dummy,i]+0.05*maxV,        $
            state.par.neb.em_lines_text[i],                                 $
            ALIGNMENT=0.5,                                                  $
            CHARSIZE=1.
endfor
return
end

;--------------- Supernovae Remnant (SNR) widget -------------------------------

function SNR_spectrum_flux, state
h = 6.626075d-34   ; Planck constant in international unit system
c = 3.0d8          ; Light velocity in international unit system
nb_lines = (size(state.par.SNR.spectrum.spectrum_flux))[2]
;define the band wavelength bound limits
lambda_inf = state.par.lambda - state.par.width/2.
lambda_sup = state.par.lambda + state.par.width/2.
;compute ph and mag
dummy=state.par.SNR_spectra_nb
ph = 0.d                ; photon number per second per m^2
for i=0, nb_lines-1 do begin
    lam=state.par.SNR.spectrum.lambda[i]
    if  (lam ge lambda_inf) and (lam le lambda_sup) then begin
       val=state.par.SNR.user_def_spectrum.flux[dummy,i]
       ph = ph + (val*1d-3/(h*c/(lam*1d-10)))
    endif
endfor
return, ph
end


pro SNR_gui_set, state
;NOTHING TO DO??
end

pro SNR_gui_event, event


widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state

widget_control, event.id,  GET_UVALUE=uval

;print, uval

case uval of

'SNR_extension' : begin
   state.par.SNR.extension[state.par.obj_nb] = event.value
   ;re-display object
   display_object, state
   end

'spectra_choice' : begin
   case event.select of
   
      1 : begin
               ;plot the current spectrum
               if state.par.flag eq -1 then window, /FREE
               state.par.flag = !D.window
               plot_SNR_spectrum, state
   
            end
      0 : begin
               if state.par.flag ne -1 then wdelete, state.par.flag
               state.par.flag = -1
            end
   endcase
   end

'spectra_number' : begin
   ;update the spectra index
   state.par.SNR_spectra_nb = event.index
   
   widget_control, state.id.SNR_spectra_choice, GET_VALUE = dummy
   if dummy eq 1 then begin
         ;update the values of the emission lines fluxes
         nb_lines = (size(state.par.SNR.spectrum.spectrum_flux))[2]
   
         for i=0, nb_lines-1 do begin
            widget_control, state.id.SNR_em_lines[i],                                     $
                           SET_VALUE=state.par.SNR.spectrum.spectrum_flux[event.index,i]
   
            state.par.SNR.user_def_spectrum.flux[event.index, i] =                        $
               state.par.SNR.spectrum.spectrum_flux[event.index,i]
         endfor
   
         ;plot the current spectrum
         if state.par.flag eq -1 then window, /FREE
         state.par.flag = !D.window
         state.par.SNR.user_def_spectrum.status = ''
         plot_SNR_spectrum, state
   
         ;compute nb_photons / second / m^2 from spectrum
         state.par.SNR.flux = SNR_spectrum_flux(state)
         widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   
   endif
   end

'SNR_lambda': begin
   state.par.lambda = event.value
   
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   end

'SNR_width': begin
   state.par.width = event.value
   
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   end

string(state.par.SNR.user_def_spectrum.lambda[0]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 0]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[1]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 1]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[2]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 2]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[3]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 3]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[4]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 4]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[5]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 5]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[6]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 6]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[7]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 7]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[8]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 8]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[9]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 9]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[10]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 10]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[11]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 11]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[12]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 12]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[13]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 13]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[14]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 14]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[15]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 15]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[16]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 16]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[17]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 17]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[18]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 18]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[19]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 19]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[20]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 20]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[21]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 21]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[22]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 22]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[23]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 23]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[24]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 24]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[25]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 25]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[26]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 26]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[27]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 27]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[28]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 28]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[29]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 29]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[30]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 30]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end
string(state.par.SNR.user_def_spectrum.lambda[31]) : begin
   state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb, 31]=event.value
   ;compute nb_photons / second / m^2 from spectrum
   state.par.SNR.flux = SNR_spectrum_flux(state)
   widget_control, state.id.SNR_flux, SET_VALUE=state.par.SNR.flux
   ;plot the updated spectrum
   state.par.SNR.user_def_spectrum.status = ' (modified)'
   plot_SNR_spectrum, state
   end

'SNR_sky_flux': state.par.sky_flux = event.value
else: ;nothing to do!

endcase

;default setting
SNR_gui_set, state
;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state
;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state
return
end

pro SNR_gui, state


  state.id.obj_base[state.type_obj]= widget_base($
                   title ='SuperNovae Remnant type object characteristics',    $
                   /COLUMN,GROUP_LEADER=state.id.root_base_id)
  tab1=widget_tab(state.id.obj_base[state.type_obj], location=0, uvalue='')
  ;first tab
  obj_info= widget_base(tab1, /ROW, TITLE="Object information")  
  obj_info_left= widget_base(obj_info, /COL, /FRAME)
  obj_info_right= widget_base(obj_info, /COL, /FRAME)
  SNR_name= widget_base(obj_info_left, /COL, /FRAME)
  dummy= widget_label(SNR_name, VALUE='Select an object:', /ALIGN_LEFT)
  state.id.SNR_name_obj= widget_combobox(SNR_name, VALUE=state.par.SNR.name_obj,$
                                         UVALUE='SNR_name_obj')
  state.id.SNR_image= widget_draw(SNR_name, xsize=256, ysize=256, /FRAME)
  extension= widget_base(obj_info_left, /COL, /FRAME)
  state.id.SNR_extension= cw_field(extension, $
                          TITLE='Object extension in arcsec [ENTER]',$
                          /FLOAT, $
                          /RETURN_EVENTS, UVALUE='SNR_extension', $
                          VALUE=state.par.SNR.extension[state.par.obj_nb],$
                          xsize=8)
  state.id.SNR_field_extension = cw_field(extension, $ 
                          TITLE='(Field extension in arcsec)       ', /FLOATING, $
                          VALUE=state.par.field_extension, /NOEDIT, xsize=8 )
  wavelength_range= widget_base(obj_info_right, /FRAME, /COL)
  dummy= widget_label(wavelength_range,  $
           VALUE='WAVELENGTH RANGE CONSIDERED   ')
  dummy= widget_base(wavelength_range, /COL)
  state.id.SNR_lambda= cw_field(dummy, $
                           TITLE='Central wavelength [Angstroem]                  ', $
                           /FLOATING, VALUE=state.par.lambda,  UVALUE='SNR_lambda', $
                           /ALL_EVENTS)
  state.id.SNR_width= cw_field(dummy, $
                           TITLE='Width [Angstroem]                               ',$
                           /FLOATING,  VALUE=state.par.width,  UVALUE='SNR_width',  $
                           /ALL_EVENTS)
  SNR_flux= widget_base(obj_info_right, /FRAME, /COL)
  dummy= widget_label(SNR_flux, $
           VALUE='TOTAL FLUX IN WAVELENGTH RANGE')
  state.id.SNR_flux  = cw_field(SNR_flux, $
                           TITLE='(Resulted emission lines flux [photons/s/m^2])  ',$
                           /FLOATING,VALUE=state.par.SNR.flux, /NOEDIT )
  state.id.SNR_sky_flux  = cw_field(SNR_flux, $
                           TITLE='Sky background flux [photons/s/m^2]             ',$
                           /FLOATING,VALUE=state.par.sky_flux, UVALUE='SNR_sky_flux',/ALL_EVENTS)
  ;second tab
  emission=WIDGET_BASE(tab1, /COL, TITLE="Emission Lines")
  spectra_SNR= widget_base(emission, /COL, /FRAME)
  state.id.SNR_spectra_choice  = cw_bgroup(spectra_SNR,  $
              ['Emmission Lines library [4102-7325 A]                      '],$
              SET_VALUE=state.par.SNR_spectra_choice,$
              UVALUE='spectra_choice', /NONEXCLUSIVE)
  state.id.SNR_spectra_number  = widget_list(spectra_SNR, ysize=5, xsize=10, $
                               VALUE = state.par.SNR.spectrum.name,$
                               UVALUE='spectra_number' )
  SNR_emission_lines=widget_base(emission, /FRAME, /COL)
  dummy= widget_label(SNR_emission_lines, xsize= 350, $
                  VALUE='EMISSION LINES FLUX (mW/m2)')
  SNR_emission_base= widget_base(SNR_emission_lines, /ROW)
  em_lines_text = ['H delta '  , 'H gamma '  , 'OIII    '  , 'HeI     '  ,  $
      'HeI     '  , $
      'HeII    '  , 'HeII    '  , 'ArIV    '  , 'H beta  '  , 'HeI     '  , $
      '[OIII]  '  , '[OIII]  '  , 'NI      '  , 'HeII    '  , 'ClIII   '  , $
      'ClIII   '  , 'NII     '  , 'HeI     '  , 'OI      '  , 'SIII    '  , $
      'OI      '  , 'ArV     '  , '[NII]   '  , 'H alpha '  , '[NII]   '  , $
      'HeI     '  , 'SII     '  , 'SII     '  , 'ArV     '  , 'HeI     '  , $
      'ArIII   '  , '[OII]   '                                              ]

  nb_lines = (size(em_lines_text))[1]
  num_by_col = 8
  SNR_emission_lines_col= [widget_base(SNR_emission_base, /COL),   $
                           widget_base(SNR_emission_base, /COL),   $
                           widget_base(SNR_emission_base, /COL),   $
                           widget_base(SNR_emission_base, /COL)    ]
  for i=0, nb_lines-1 do begin

  state.id.SNR_em_lines[i]= cw_field(SNR_emission_lines_col[i/num_by_col], $
         TITLE=em_lines_text[i] + '(' +$
         strtrim(state.par.SNR.user_def_spectrum.lambda[i],1)+' A):', /FLOAT, $
         VALUE=state.par.SNR.user_def_spectrum.flux[state.par.SNR_spectra_nb,i],$
         /ALL_EVENTS, UVALUE=string(state.par.SNR.user_def_spectrum.lambda[i]), $
         xsize=13 )
  endfor

;default setting
SNR_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
display_object, state
xmanager, 'SNR_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id
return
end

pro plot_SNR_spectrum, state

;current spectra index
dummy = state.par.SNR_spectra_nb

;emission lines total number in the spectrum
dim = size(state.par.SNR.spectrum.spectrum_flux)
nb_lines = dim[2]

;plot the emission lines (width=10 angstroem)
plotcolorfill_modif, state.par.SNR.spectrum.lambda,                   $
         state.par.SNR.user_def_spectrum.flux[dummy,*],               $
         PSYM=1,                                                      $
         TITLE = state.par.SNR.spectrum.name[dummy] +                 $
                 state.par.SNR.user_def_spectrum.status,              $
         WIDTH =10.,                                                  $
         XTITLE='lambda [Angstroem]',                                 $
         YTITLE='Emission lines flux [mW/m2]',                        $
         COLOR=200,                                                   $
         POSITION=ASPECT(0.7, Margin=0.15)

maxV = max(state.par.SNR.user_def_spectrum.flux[dummy,*])

;display the emission lines names
for i=0, nb_lines-1 do begin
    if state.par.SNR.user_def_spectrum.flux[dummy,i] ge 0.05*maxV then   $
    xyouts, state.par.SNR.spectrum.lambda[i],                            $
            state.par.SNR.user_def_spectrum.flux[dummy,i]+0.05*maxV,     $
            state.par.SNR.em_lines_text[i],                              $
            ALIGNMENT=0.5,                                               $
            CHARSIZE=1.
endfor

return
end


;---------------- Spiral galaxy  widget ----------------------------------------

pro spiral_galaxy_gui_set, state
;set object nb
widget_control, state.id.gal_name_obj, SET_COMBOBOX_SELECT=state.par.obj_nb
;set default extension
widget_control, state.id.gal_extension, $
      SET_VALUE=state.par.gal.extension[state.par.obj_nb]

if (state.par.gal.wave_range_choice[state.par.obj_nb] eq 0) then begin    
; broad band case

  widget_control, state.id.gal_broad_band_choice, SET_VALUE=0
  if (state.par.gal.bright_mag_choice[state.par.obj_nb] eq 1B) then begin
      widget_control, state.id.gal_tot_mag_choice, SET_VALUE=0B
      widget_control, state.id.gal_bright_mag_choice, SET_VALUE=1B
      widget_control, state.id.gal_bright, /SENSITIVE
      widget_control, state.id.gal_field_mag, /SENSITIVE
      widget_control, state.id.gal_tot_mag, SENSITIVE=0
  endif else begin
      widget_control, state.id.gal_bright_mag_choice, SET_VALUE=0B
      widget_control, state.id.gal_tot_mag_choice, SET_VALUE=1B
      widget_control, state.id.gal_bright, SENSITIVE=0
      widget_control, state.id.gal_field_mag, SENSITIVE=0
      widget_control, state.id.gal_tot_mag, /SENSITIVE
  endelse
  widget_control, state.id.gal_bright_mag, /SENSITIVE
  widget_control, state.id.gal_narrow_band, SENSITIVE=0
  widget_control, state.id.gal_broad_band_choice, SET_VALUE=0
  widget_control, state.id.gal_bright, $
       SET_VALUE=state.par.gal.bright[state.par.obj_nb]
  ;update object total field magnitude
  if state.par.field_extension lt state.par.gal.extension[state.par.obj_nb] then begin
    state.par.gal_field_mag = state.par.gal.bright[state.par.obj_nb]          $
                     -2.5*alog10(state.par.field_extension^2.)
  endif else begin
    state.par.gal_field_mag = state.par.gal.bright[state.par.obj_nb]          $
                     -2.5*alog10(state.par.gal.extension[state.par.obj_nb]^2.)
  endelse
  widget_control, state.id.gal_field_mag, SET_VALUE=state.par.gal_field_mag
endif else begin              ; custom wavelength range
  widget_control, state.id.gal_broad_band_choice, SET_VALUE=1
  widget_control, state.id.gal_bright_mag, SENSITIVE=0
  widget_control, state.id.gal_narrow_band, /SENSITIVE
endelse

;set broad band index
widget_control, state.id.gal_band, SET_COMBOBOX_SELECT = state.par.band_index

end


pro spiral_galaxy_gui_event, event


widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state

widget_control, event.id,  GET_UVALUE=uval

;print, uval

case uval of


'gal_name_obj'  : begin
   state.par.obj_nb=event.index
   ;re-display object
   display_object, state
   end

'gal_extension' : begin
   state.par.gal.extension[state.par.obj_nb] = event.value
   
   ;re-display object
   display_object, state
   end

'band'      : begin
   state.par.band_index = event.index
   
   ;;;set default pixel size value
   ;;state.par.pix_size = state.par.pix_size_tab[event.index]
   ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
   
   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state
   
   obj_gui_set, state
   end

'gal_bright'    : state.par.gal.bright[state.par.obj_nb] = event.value

'gal_tot_mag'   : state.par.gal.tot_mag[state.par.obj_nb] = event.value

'gal_broad_band_choice' : state.par.gal.wave_range_choice[state.par.obj_nb] = event.value

'gal_bright_mag_choice' : state.par.gal.bright_mag_choice[state.par.obj_nb] = event.select

'gal_tot_mag_choice' : state.par.gal.bright_mag_choice[state.par.obj_nb] = non(event.select)

'gal_lambda'      : state.par.lambda = event.value

'gal_width'       : state.par.width = event.value

'gal_flux'        : state.par.gal.flux[state.par.obj_nb] = event.value

'gal_sky_flux'    : state.par.sky_flux = event.value


endcase

;default setting
spiral_galaxy_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end

pro spiral_galaxy_gui, state


  state.id.obj_base[state.type_obj] = widget_base( $
                          title ='Spiral galaxy type object characteristics', $
                          /COLUMN, GROUP_LEADER=state.id.root_base_id )
  ;no tabs here
  obj_info=widget_base(state.id.obj_base[state.type_obj], /ROW)
  obj_info_left= widget_base(obj_info, /COL, /FRAME)
  obj_info_right= widget_base(obj_info, /COL, /FRAME)
  gal_name= widget_base(obj_info_left, /COL, /FRAME)
  dummy= widget_label(gal_name, VALUE='Select an object:', /ALIGN_LEFT)
  state.id.gal_name_obj= widget_combobox(gal_name, VALUE=state.par.gal.name_obj,$
                          UVALUE='gal_name_obj')
  state.id.gal_image= widget_draw(gal_name, xsize=256, ysize=256, /FRAME) 
  extension= widget_base(obj_info_left, /COL, /FRAME)
  state.id.gal_extension= cw_field(extension, $
                        TITLE='Object extension in arcsec [ENTER]', /FLOAT, $
                        /RETURN_EVENTS, UVALUE='gal_extension', xsize=8, $
                        VALUE=state.par.gal.extension[state.par.obj_nb]  )
  state.id.gal_field_extension = cw_field(extension, $
                        TITLE='(Field extension in arcsec)       ', /FLOATING, $
                           VALUE=state.par.field_extension, /NOEDIT, xsize=8 )
  state.id.gal_broad_band_choice = cw_bgroup(obj_info_right, /COL,  $
                  ['Standard broad band filter range   ',  $
                   'Custom wavelength range            '], $
                  SET_VALUE=state.par.gal.wave_range_choice[state.par.obj_nb],$
                  UVALUE='gal_broad_band_choice',/EXCLUSIVE)
  gal_phot= widget_base(obj_info_right, /COL)
  state.id.gal_bright_mag= widget_base(gal_phot, /COL, /FRAME)
  dummy= widget_label(state.id.gal_bright_mag,$
                        VALUE='STANDARD BROAD BAND FILTER RANGE', /ALIGN_LEFT)
  gal_band= widget_base(state.id.gal_bright_mag, /ROW)
  dummy= widget_label(gal_band,VALUE='Band:   ')
  state.id.gal_band= widget_combobox(gal_band, VALUE=state.par.band_tab, $
                     UVALUE='band')
  gal_bright= widget_base(state.id.gal_bright_mag, /ROW)
  gal_bright_left= widget_base(gal_bright, /COL, /FRAME)
  gal_bright_right= widget_base(gal_bright, /COL, /FRAME)
  state.id.gal_bright_mag_choice = cw_bgroup(gal_bright_left,     $
               ['Object average brightness (mag/arcsec^2)  [ENTER]'],  $
               SET_VALUE=state.par.gal.bright_mag_choice[state.par.obj_nb], $
               UVALUE='gal_bright_mag_choice',/NONEXCLUSIVE  )
  state.id.gal_bright= cw_field(gal_bright_left, $
                       TITLE='                  ', xsize=12, /FLOAT,  $
                       VALUE=state.par.gal.bright[state.par.obj_nb],$
                       UVALUE='gal_bright', /RETURN_EVENTS )
  state.id.gal_field_mag= cw_field(gal_bright_left, /ROW, $
                          TITLE='(Resulted field total magnitude)',$
                          /FLOAT, VALUE=state.par.gal_field_mag,   $
                          /NOEDIT, xsize=8 )
  state.id.gal_tot_mag_choice = cw_bgroup(gal_bright_right,$
               ['Object total magnitude [ENTER]'],$
               SET_VALUE=non(state.par.gal.bright_mag_choice[state.par.obj_nb]), $
               UVALUE='gal_tot_mag_choice',/NONEXCLUSIVE)

  state.id.gal_tot_mag= cw_field(gal_bright_right, $
                        TITLE='         ', /FLOAT, xsize=12, $
                        VALUE=state.par.gal.tot_mag[state.par.obj_nb],$
                        UVALUE='gal_tot_mag', /ALL_EVENTS )

  state.id.gal_narrow_band= widget_base(gal_phot, /COL, /FRAME)
  dummy= widget_label(state.id.gal_narrow_band, $
                     VALUE='CUSTOM WAVELENGTH RANGE', /ALIGN_LEFT)
  wavelength_range= widget_base(state.id.gal_narrow_band, /COL)
  dummy= widget_base(wavelength_range, /COL)
  state.id.gal_lambda= cw_field(dummy, TITLE='Central wavelength [Angstroem]        ', $
                      /FLOATING, VALUE=state.par.lambda,  UVALUE='gal_lambda', $
                      /ALL_EVENTS)
  state.id.gal_width= cw_field(dummy,  TITLE='Width [Angstroem]                     ', $
                      /FLOATING, VALUE=state.par.width,  UVALUE='gal_width', $
                      /ALL_EVENTS)
  flux= widget_base(wavelength_range, /COL)
  state.id.gal_flux= cw_field(flux,    TITLE='Flux [photons/s/m^2]                  ', $
                     /FLOATING,VALUE=state.par.gal.flux[state.par.obj_nb], $
                     UVALUE='gal_flux', /ALL_EVENTS)
  state.id.gal_sky_flux= cw_field(flux,TITLE='Sky background flux [photons/s/m^2]   ', $
                      /FLOATING, VALUE=state.par.sky_flux, UVALUE='gal_sky_flux', $
                      /ALL_EVENTS)

;default setting
spiral_galaxy_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
display_object, state
xmanager, 'spiral_galaxy_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id
return
end

;---------------------------- YSOs widget ------------------------------------------------------------------

pro YSO_gui_set, state
;set object nb
widget_control, state.id.YSO_name_obj, SET_COMBOBOX_SELECT=state.par.obj_nb
;set default extension
widget_control, state.id.YSO_extension, $
      SET_VALUE=state.par.YSO.extension[state.par.obj_nb]
if (state.par.YSO.wave_range_choice[state.par.obj_nb] eq 0) then begin   
 ; broad band case
  widget_control, state.id.YSO_broad_band_choice, SET_VALUE=0
  if (state.par.YSO.bright_mag_choice[state.par.obj_nb] eq 1B) then begin
      widget_control, state.id.YSO_tot_mag_choice, SET_VALUE=0B
      widget_control, state.id.YSO_bright_mag_choice, SET_VALUE=1B
      widget_control, state.id.YSO_bright, /SENSITIVE
      widget_control, state.id.YSO_field_mag, /SENSITIVE
      widget_control, state.id.YSO_tot_mag, SENSITIVE=0
  endif else begin
      widget_control, state.id.YSO_bright_mag_choice, SET_VALUE=0B
      widget_control, state.id.YSO_tot_mag_choice, SET_VALUE=1B
      widget_control, state.id.YSO_bright, SENSITIVE=0
      widget_control, state.id.YSO_field_mag, SENSITIVE=0
      widget_control, state.id.YSO_tot_mag, /SENSITIVE
  endelse
  widget_control, state.id.YSO_bright_mag, /SENSITIVE
  widget_control, state.id.YSO_narrow_band, SENSITIVE=0
  widget_control, state.id.YSO_broad_band_choice, SET_VALUE=0
  widget_control, state.id.YSO_bright, $ 
         SET_VALUE=state.par.YSO.bright[state.par.obj_nb]
  ;update YSO total field magnitude
  if state.par.field_extension lt state.par.YSO.extension[state.par.obj_nb] then begin
    state.par.YSO_field_mag = state.par.YSO.bright[state.par.obj_nb]          $
                   -2.5*alog10(state.par.field_extension^2.)
  endif else begin
    state.par.YSO_field_mag = state.par.YSO.bright[state.par.obj_nb]          $
                   -2.5*alog10(state.par.YSO.extension[state.par.obj_nb]^2.)
  endelse
  widget_control, state.id.YSO_field_mag, SET_VALUE=state.par.YSO_field_mag
endif else begin              ; custom wavelength range
  widget_control, state.id.YSO_broad_band_choice, SET_VALUE=1
  widget_control, state.id.YSO_bright_mag, SENSITIVE=0
  widget_control, state.id.YSO_narrow_band, /SENSITIVE
endelse

;set broad band index
widget_control, state.id.YSO_band, SET_COMBOBOX_SELECT = state.par.band_index


end


pro YSO_gui_event, event
widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval
;print, uval
case uval of

'YSO_name_obj'  : begin
   state.par.obj_nb=event.index

   ;re-display object
   display_object, state

   ; update image caption and credit
   widget_control, state.id.YSO_caption1, SET_VALUE = 'image caption : ' + $
                        state.par.YSO.caption1[state.par.obj_nb],/DYNAMIC_RESIZE
   widget_control, state.id.YSO_caption2, SET_VALUE = '                ' + $
                        state.par.YSO.caption2[state.par.obj_nb],/DYNAMIC_RESIZE
   widget_control, state.id.YSO_caption3, SET_VALUE = '                ' + $
                        state.par.YSO.caption3[state.par.obj_nb], /DYNAMIC_RESIZE
   widget_control, state.id.YSO_credit,   SET_VALUE = 'image credit  : ' + $
                        state.par.YSO.credit[state.par.obj_nb],/DYNAMIC_RESIZE
   end

'YSO_extension' : begin
   state.par.YSO.extension[state.par.obj_nb] = event.value

   ;re-display object
   display_object, state
   end

'band'      : begin
   state.par.band_index = event.index
   
   ;;;set default pixel size value
   ;;state.par.pix_size = state.par.pix_size_tab[event.index]
   ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
   
   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state
   
   obj_gui_set, state
   end

'YSO_bright'    : state.par.YSO.bright[state.par.obj_nb] = event.value

'YSO_tot_mag'   : state.par.YSO.tot_mag[state.par.obj_nb] = event.value

'YSO_broad_band_choice' : state.par.YSO.wave_range_choice[state.par.obj_nb] = event.value

'YSO_bright_mag_choice' : state.par.YSO.bright_mag_choice[state.par.obj_nb] = event.select

'YSO_tot_mag_choice' : state.par.YSO.bright_mag_choice[state.par.obj_nb] = non(event.select)

'YSO_lambda'         : state.par.lambda = event.value

'YSO_width'          : state.par.width = event.value

'YSO_flux'           : state.par.YSO.flux[state.par.obj_nb] = event.value

'YSO_sky_flux'       : state.par.sky_flux = event.value
else: ;do nothing!
endcase

;default setting
YSO_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end

pro YSO_gui, state


  state.id.obj_base[state.type_obj]= widget_base($ 
                   title ='YSO type object characteristics',    $
                   /COLUMN,  GROUP_LEADER=state.id.root_base_id  )
  tab1=WIDGET_TAB(state.id.obj_base[state.type_obj], location=0, uvalue='')
  ;FIRST TAB ---
  obj_info= widget_base(tab1, /ROW, title="Object information")
  obj_info_left= widget_base(obj_info, /COL, /FRAME)
  obj_info_right= widget_base(obj_info, /COL, /FRAME)
  YSO_name= widget_base(obj_info_left, /COL, /FRAME)
  dummy = widget_label(YSO_name, VALUE='Select an object:', /ALIGN_LEFT)
  state.id.YSO_name_obj= widget_combobox(YSO_name, VALUE=state.par.YSO.name_obj, $
                                         UVALUE='YSO_name_obj')
  state.id.YSO_image= widget_draw(YSO_name, xsize=256, ysize=256, /FRAME)
  extension= widget_base(obj_info_left, /COL, /FRAME)
  state.id.YSO_extension= cw_field(extension,$
         TITLE='Object extension in arcsec [ENTER]',xsize=8, $
         /FLOAT,  /RETURN_EVENTS,  UVALUE='YSO_extension', $
         VALUE=state.par.YSO.extension[state.par.obj_nb] )
  state.id.YSO_field_extension = cw_field(extension, xsize=8, $
         TITLE='(Field extension in arcsec)       ', /FLOATING, $
         VALUE=state.par.field_extension, /NOEDIT)
         
  state.id.YSO_broad_band_choice = cw_bgroup(obj_info_right, $
                  ['Standard broad band filter range ','Custom wavelength range '], $
                  SET_VALUE=state.par.YSO.wave_range_choice[state.par.obj_nb], $
                  UVALUE='YSO_broad_band_choice', /EXCLUSIVE)

  YSO_phot= widget_base(obj_info_right, /COL)
  state.id.YSO_bright_mag= widget_base(YSO_phot, /FRAME, /COL)
  dummy= widget_label(state.id.YSO_bright_mag,$
               VALUE='STANDARD BROAD BAND FILTER RANGE', /ALIGN_LEFT)
  YSO_band= widget_base(state.id.YSO_bright_mag, /ROW)
  dummy= widget_label(YSO_band, VALUE='Band:   ')
  state.id.YSO_band= widget_combobox(YSO_band, VALUE=state.par.band_tab, UVALUE='band')
  YSO_bright= widget_base(state.id.YSO_bright_mag, /ROW)
  YSO_bright_left=widget_base(YSO_bright, /COL, /FRAME)
  YSO_bright_right=widget_base(YSO_bright, /COL, /FRAME)
  state.id.YSO_bright_mag_choice = cw_bgroup(YSO_bright_left,$
               ['Object average brightness (mag/arcsec^2)  [ENTER]'], $
               SET_VALUE=state.par.YSO.bright_mag_choice[state.par.obj_nb],   $
               UVALUE='YSO_bright_mag_choice', /NONEXCLUSIVE)
  state.id.YSO_bright= cw_field(YSO_bright_left, $
                       TITLE='                  ', xsize=12, /FLOAT, $
                    VALUE=state.par.YSO.bright[state.par.obj_nb],$
                    UVALUE='YSO_bright', /ALL_EVENTS )
  state.id.YSO_field_mag= cw_field(YSO_bright_left,$
                    TITLE='(Resulted field total magnitude)', $
                    /FLOAT, VALUE=state.par.YSO_field_mag,xsize=8,/NOEDIT )

  state.id.YSO_tot_mag_choice = cw_bgroup(YSO_bright_right,$
            ['Object total magnitude [ENTER]'], $
            SET_VALUE=non(state.par.YSO.bright_mag_choice[state.par.obj_nb]),  $
            UVALUE='YSO_tot_mag_choice',/NONEXCLUSIVE )
  state.id.YSO_tot_mag= cw_field(YSO_bright_right, $ 
                        TITLE='         ', /FLOAT, xsize=12, $
                        VALUE=state.par.YSO.tot_mag[state.par.obj_nb],  $
                        UVALUE='YSO_tot_mag', /ALL_EVENTS )

  state.id.YSO_narrow_band= widget_base(YSO_phot, /FRAME, /COL)
  dummy= widget_label(state.id.YSO_narrow_band,        $
                         VALUE='CUSTOM WAVELENGTH RANGE', /ALIGN_LEFT)
  wavelength_range= widget_base(state.id.YSO_narrow_band, /COL)
  dummy= widget_base(wavelength_range, /COL)
  state.id.YSO_lambda= cw_field(dummy, $
                   TITLE='Central wavelength [Angstroem]        ', $
                   /FLOATING, VALUE=state.par.lambda,  UVALUE='YSO_lambda', $
                   /ALL_EVENTS)
  state.id.YSO_width= cw_field(dummy, $
                   TITLE='Width [Angstroem]                     ',$
                   /FLOATING, VALUE=state.par.width, $
                   UVALUE='YSO_width', /ALL_EVENTS )
  flux= widget_base(wavelength_range, /COL)
  state.id.YSO_flux= cw_field(flux, $
                   TITLE='Flux [photons/s/m^2]                  ',$
                   /FLOATING, VALUE=state.par.YSO.flux[state.par.obj_nb],$
                   UVALUE='YSO_flux', /ALL_EVENTS )
  state.id.YSO_sky_flux= cw_field(flux, $
                   TITLE='Sky background flux [photons/s/m^2]   ', $
                   /FLOATING, VALUE=state.par.sky_flux,  UVALUE='YSO_sky_flux', $
                   /ALL_EVENTS)

  ;SECOND TAB
  explain= widget_base(tab1, /COL, title="Credits")

  state.id.YSO_caption1 = widget_label(explain, VALUE='image caption : ' $
                    +string(state.par.YSO.caption1[state.par.obj_nb]),   $
                    /ALIGN_LEFT)

  state.id.YSO_caption2 = widget_label(explain, VALUE='                '                    $
                    +string(state.par.YSO.caption2[state.par.obj_nb]),   $
                    /ALIGN_LEFT)

  state.id.YSO_caption3 = widget_label(explain, VALUE='                '                    $
                    +string(state.par.YSO.caption3[state.par.obj_nb]),   $
                    /ALIGN_LEFT)

  state.id.YSO_credit   = widget_label(explain, VALUE='image credit  : '                    $
                    + string(state.par.YSO.credit[state.par.obj_nb]),    $
                    /ALIGN_LEFT)
  
 
;default setting
YSO_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
display_object, state
xmanager, 'YSO_gui', state.id.obj_base[state.type_obj], $
       GROUP_LEADER=state.id.root_base_id
return
end

;--------------------- Stellar surface widget ----------------------------------

pro star_surface_gui_set, state

;set broad band index
widget_control, state.id.star_band, SET_COMBOBOX_SELECT = state.par.band_index

end

pro star_surface_gui_event, event
widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval
case uval of
   
'band': begin
   state.par.band_index = event.index
   
   ;;;set default pixel size value
   ;;state.par.pix_size = state.par.pix_size_tab[event.index]
   ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size
   
   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state
   
   obj_gui_set, state
   end

'star_mag':begin
   state.par.star_mag = event.value
   end

'star_diam':begin
   state.par.star_diam = event.value
   end

'star_ax_ratio': begin
   state.par.star_ax_ratio = event.value
   end

'spot_diam': begin
   state.par.spot_diam = event.value
   end

'spot_flux': begin
   state.par.spot_flux = event.value
   end

'spot_dist': begin
   state.par.spot_dist = event.value
   end

'spot_pa': begin
   state.par.spot_pa = event.value
   end

endcase

;default setting
star_surface_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end


pro star_surface_gui, state


  state.id.obj_base[state.type_obj]= widget_base( $
               title ='Stellar surface structure', /COLUMN,$
               GROUP_LEADER=state.id.root_base_id)
  obj_up= widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
  obj_down= widget_base(state.id.obj_base[state.type_obj], /FRAME, /COLUMN)
  star_band= widget_base(obj_up, /ROW)
  dummy= widget_label(star_band, VALUE='Band: ')
  state.id.star_band= widget_combobox(star_band, VALUE=state.par.band_tab, $
                      UVALUE='band')
  state.id.star_mag= cw_field(obj_up, $
                     TITLE='Star band-magnitude:            ', /FLOATING, $
                     VALUE=state.par.star_mag, UVALUE='star_mag', /ALL_EVENTS )

  state.id.star_diam= cw_field(obj_up, $
                     TITLE='Star apparent diameter (mas):   ', /FLOATING, $
                     VALUE=state.par.star_diam, UVALUE='star_diam',      $
                     /ALL_EVENTS)
  state.id.star_ax_ratio= cw_field(obj_up, $
                     TITLE='Star axial ratio:               ', $
                     /FLOATING, VALUE=state.par.star_ax_ratio,   $
                     UVALUE='star_ax_ratio',/ALL_EVENTS )
  state.id.spot_diam= cw_field(obj_up, $
                     TITLE='Spot apparent diameter (mas):   ',$
                     /FLOATING,VALUE=state.par.spot_diam,$
                     UVALUE='spot_diam', /ALL_EVENTS )
  state.id.spot_flux= cw_field(obj_up, $
                     TITLE='Spot flux (% of total):         ',$
                     /FLOATING, VALUE=state.par.spot_flux,  UVALUE='spot_flux',$
                     /ALL_EVENTS)
  state.id.spot_dist= cw_field(obj_up, $
                     TITLE='Spot distance from center (mas):',$
                     /FLOATING, VALUE=state.par.spot_dist, UVALUE='spot_dist', $
                     /ALL_EVENTS)
  state.id.spot_pa= cw_field(obj_up,$
                     TITLE='Spot position angle (deg.):     ', $
                     /FLOATING,VALUE=state.par.spot_pa, UVALUE='spot_pa',    $
                     /ALL_EVENTS)
  state.id.star_field_extension = cw_field(obj_down, $
                     TITLE='(Field extension in arcsec)     ', /FLOATING, $
                     VALUE=state.par.field_extension,/NOEDIT )
;default setting
star_surface_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
xmanager, 'star_surface_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id
return
end


;------------------- User-defined object widget --------------------------------

pro user_def_gui_set, state

;set default extension
widget_control, state.id.user_def_extension, $
     SET_VALUE=state.par.user_def.extension[state.par.obj_nb]

if (state.par.user_def.wave_range_choice[state.par.obj_nb] eq 0) then begin    
; broad band case

  widget_control, state.id.user_def_broad_band_choice, SET_VALUE=0

  if (state.par.user_def.bright_mag_choice[state.par.obj_nb] eq 1B) then begin
      widget_control, state.id.user_def_tot_mag_choice, SET_VALUE=0B
      widget_control, state.id.user_def_bright_mag_choice, SET_VALUE=1B
      widget_control, state.id.user_def_bright, /SENSITIVE
      widget_control, state.id.user_def_field_mag, /SENSITIVE
      widget_control, state.id.user_def_tot_mag, SENSITIVE=0

  endif else begin
      widget_control, state.id.user_def_bright_mag_choice, SET_VALUE=0B
      widget_control, state.id.user_def_tot_mag_choice, SET_VALUE=1B
      widget_control, state.id.user_def_bright, SENSITIVE=0
      widget_control, state.id.user_def_field_mag, SENSITIVE=0
      widget_control, state.id.user_def_tot_mag, /SENSITIVE

  endelse


  widget_control, state.id.user_def_bright_mag, /SENSITIVE
  widget_control, state.id.user_def_narrow_band, SENSITIVE=0

  widget_control, state.id.user_def_broad_band_choice, SET_VALUE=0
  widget_control, state.id.user_def_bright, $
      SET_VALUE=state.par.user_def.bright[state.par.obj_nb]


  ;update user def object total field magnitude
  if state.par.field_extension lt state.par.user_def.extension[state.par.obj_nb] then begin
    state.par.user_def_field_mag = state.par.user_def.bright[state.par.obj_nb]    $      
               -2.5*alog10(state.par.field_extension^2.)
  endif else begin

    state.par.user_def_field_mag = state.par.user_def.bright[state.par.obj_nb]          $
               -2.5*alog10(state.par.user_def.extension[state.par.obj_nb]^2.)
  endelse
  widget_control, state.id.user_def_field_mag, SET_VALUE=state.par.user_def_field_mag

endif else begin              ; custom wavelength range

  widget_control, state.id.user_def_broad_band_choice, SET_VALUE=1
  widget_control, state.id.user_def_bright_mag, SENSITIVE=0
  widget_control, state.id.user_def_narrow_band, /SENSITIVE
endelse

;set broad band index
widget_control, state.id.user_def_band, SET_COMBOBOX_SELECT = state.par.band_index

end


pro user_def_gui_event, event
widget_control, event.top, GET_UVALUE=state
widget_control, state.id.root_base_id, GET_UVALUE=state
widget_control, event.id,  GET_UVALUE=uval
case uval of

'user_def_obj_filename'  : begin
   widget_control, state.id.user_def_obj_filename , GET_VALUE=dummy
   state.par.user_def.obj_file = dummy[0]

   ;re-display object
   ;display_object, state
   end

'user_def_display'   : display_object, state

'user_def_extension' : begin
   state.par.user_def.extension[state.par.obj_nb] = event.value

   ;re-display object
   display_object, state
   end

'band': begin
   state.par.band_index = event.index

   ;;;set default pixel size value
   ;;state.par.pix_size = state.par.pix_size_tab[event.index]
   ;;widget_control, state.id.pix_size, SET_VALUE=state.par.pix_size

   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state

   obj_gui_set, state
   end

'user_def_bright'    : state.par.user_def.bright[state.par.obj_nb] = event.value

'user_def_tot_mag'   : state.par.user_def.tot_mag[state.par.obj_nb] = event.value

'user_def_broad_band_choice' : state.par.user_def.wave_range_choice[state.par.obj_nb] = event.value

'user_def_bright_mag_choice' : state.par.user_def.bright_mag_choice[state.par.obj_nb] = event.select

'user_def_tot_mag_choice' : state.par.user_def.bright_mag_choice[state.par.obj_nb] = non(event.select)

'user_def_lambda'         : state.par.lambda = event.value

'user_def_width'          : state.par.width = event.value

'user_def_flux'           : state.par.user_def.flux[state.par.obj_nb] = event.value

'user_def_sky_flux'       : state.par.sky_flux = event.value


endcase

;default setting
user_def_gui_set, state

;write the gui state structure in event.top
widget_control, event.top, SET_UVALUE=state

;write the gui state structure in root_base_id
widget_control, state.id.root_base_id, SET_UVALUE=state

return
end

pro user_def_gui, state


  state.id.obj_base[state.type_obj]= widget_base( $
                          title ='User-defined object characteristics',    $
                          /COLUMN, GROUP_LEADER=state.id.root_base_id  )

  obj_info = widget_base(state.id.obj_base[state.type_obj], /ROW)
  obj_info_left = widget_base(obj_info, /COL, /FRAME)
  obj_info_right = widget_base(obj_info, /COL, /FRAME)
  
  user_def_name= widget_base(obj_info_left, /COL, /FRAME)
  user_def_name_dis= widget_base(user_def_name, /COL)
  state.id.user_def_obj_filename = cw_filename(user_def_name_dis,            $
                     TITLE='map filename (.fits)',   xsize=40,       $
                     VALUE=state.par.user_def.obj_file,     $
                     UVALUE='user_def_obj_filename', /ALL_EVENTS)
  user_def_display= widget_button(user_def_name_dis, VALUE='DISPLAY MAP',  $
                     UVALUE='user_def_display')
  state.id.user_def_image= widget_draw(user_def_name, xsize=256, ysize=256, /FRAME)



  extension= widget_base(obj_info_left, /COL, /FRAME)
  state.id.user_def_extension= cw_field(extension,$
                    TITLE='Object extension in arcsec [ENTER]', /FLOAT, $
                    /RETURN_EVENTS, UVALUE='user_def_extension',xsize=8, $
                    VALUE=state.par.user_def.extension[state.par.obj_nb])
  state.id.user_def_field_extension = cw_field(extension,$
                    TITLE='(Field extension in arcsec)       ', /FLOATING, $
                    VALUE=state.par.field_extension, /NOEDIT, xsize=8)

  state.id.user_def_broad_band_choice = cw_bgroup(obj_info_right,$
                ['Standard broad band filter range ',$
                 'Custom wavelength range          '],$
                 SET_VALUE=state.par.user_def.wave_range_choice[state.par.obj_nb], $
                 UVALUE='user_def_broad_band_choice', /EXCLUSIVE)
  user_def_phot= widget_base(obj_info_right, /COL)
  state.id.user_def_bright_mag= widget_base(user_def_phot, /FRAME, /COL)
  dummy= widget_label(state.id.user_def_bright_mag,    $
                  VALUE='STANDARD BROAD BAND FILTER RANGE', /ALIGN_LEFT)
  user_def_band= widget_base(state.id.user_def_bright_mag, /ROW)
  dummy= widget_label(user_def_band, VALUE='Band:   ')
  state.id.user_def_band= widget_combobox(user_def_band, $
                         VALUE=state.par.band_tab, UVALUE='band')
  user_def_bright= widget_base(state.id.user_def_bright_mag, /ROW)
  user_def_bright_left = widget_base(user_def_bright, /COL, /FRAME)
  user_def_bright_right = widget_base(user_def_bright, /COL, /FRAME)
  state.id.user_def_bright_mag_choice = cw_bgroup(user_def_bright_left, $
                 ['Object average brightness (mag/arcsec^2)  [ENTER]'], $
                SET_VALUE=state.par.user_def.bright_mag_choice[state.par.obj_nb], $
                UVALUE='user_def_bright_mag_choice', /NONEXCLUSIVE)
  state.id.user_def_bright= cw_field(user_def_bright_left, $
                            TITLE='                   ', /FLOAT,xsize=12, $
                            VALUE=state.par.user_def.bright[state.par.obj_nb], $
                            UVALUE='user_def_bright', /ALL_EVENTS)
  state.id.user_def_field_mag= cw_field(user_def_bright_left,$
                TITLE='(Resulted field total magnitude)', /FLOAT,$
                VALUE=state.par.user_def_field_mag, /NOEDIT, xsize=8)
  
  state.id.user_def_tot_mag_choice = cw_bgroup(user_def_bright_right, $
                     ['Object total magnitude [ENTER]'], $
                     SET_VALUE=non(state.par.user_def.bright_mag_choice[state.par.obj_nb]), $
                     UVALUE='user_def_tot_mag_choice', /NONEXCLUSIVE)
  state.id.user_def_tot_mag= cw_field(user_def_bright_right, $
                      TITLE='         ', /FLOAT, $
                      VALUE=state.par.user_def.tot_mag[state.par.obj_nb],  $
                      UVALUE='user_def_tot_mag',/ALL_EVENTS, xsize=12)
  state.id.user_def_narrow_band= widget_base(user_def_phot, /FRAME, /COL)
  dummy= widget_label(state.id.user_def_narrow_band,$
                      VALUE='CUSTOM WAVELENGTH RANGE', /ALIGN_LEFT)
  wavelength_range= widget_base(state.id.user_def_narrow_band, /COL)
  dummy= widget_base(wavelength_range, /COL)
  state.id.user_def_lambda= cw_field(dummy,$
                     TITLE='Central wavelength [Angstroem]        ', $
                     /FLOATING,VALUE=state.par.lambda, UVALUE='user_def_lambda', $
                     /ALL_EVENTS)
  state.id.user_def_width= cw_field(dummy, $
                     TITLE='Width [Angstroem]                     ',$
                     /FLOATING, VALUE=state.par.width,  UVALUE='user_def_width', $
                     /ALL_EVENTS)
  flux = widget_base(wavelength_range, /COL)
  state.id.user_def_flux= cw_field(flux, $
                     TITLE='Flux [photons/s/m^2]                  ', $
                     /FLOATING, VALUE=state.par.user_def.flux[state.par.obj_nb],$
                     UVALUE='user_def_flux',/ALL_EVENTS)
  state.id.user_def_sky_flux=cw_field(flux,$
                     TITLE='Sky background flux [photons/s/m^2]   ',$
                     /FLOATING, VALUE=state.par.sky_flux,UVALUE='user_def_sky_flux', $
                     /ALL_EVENTS)

;default setting
user_def_gui_set, state
widget_control, state.id.obj_base[state.type_obj], SET_UVALUE=state
widget_control, state.id.obj_base[state.type_obj], /REALIZE
display_object, state
xmanager, 'user_def_gui', state.id.obj_base[state.type_obj], GROUP_LEADER=state.id.root_base_id
return
end

;**********************************************************************************************************

pro display_object, state

case state.type_obj of

     0 : ;no display available

     1 : ;no display available

     2 : ;no display available

     3 : begin
            obj_extension = state.par.neb.extension
            obj_filename  = state.par.neb.obj_file[state.par.obj_nb]
         end
     4 : begin
            obj_extension = state.par.SNR.extension
            obj_filename  = state.par.SNR.obj_file[state.par.obj_nb]
         end
     5 : begin
            obj_extension = state.par.gal.extension[state.par.obj_nb]
            obj_filename  = state.par.gal.obj_file[state.par.obj_nb]
         end
     6 : begin
            obj_extension = state.par.YSO.extension[state.par.obj_nb]
            obj_filename  = state.par.YSO.obj_file[state.par.obj_nb]
         end
     7 : ;no display available

     8 : begin
            obj_extension = state.par.user_def.extension[state.par.obj_nb]
            obj_filename  = state.par.user_def.obj_file[state.par.obj_nb]
         end


     else :
endcase

array_dim  = state.par.array_dim
xx         = 256
yy         = 256

obj_lec = readfits(obj_filename, h, /noscale, /silent)

resize_object, obj_lec, obj_extension, array_dim , state.par.pix_size, obj

a = bytscl(obj, min=min(obj_lec), max=max(obj_lec))

tvscl, congrid(a, xx, yy, /INTERP)

return
end






pro update_obj_par, state

case state.type_obj of

0 : widget_control, state.id.single_star_field_extension, SET_VALUE=state.par.field_extension
1 : widget_control, state.id.bin_field_extension, SET_VALUE=state.par.field_extension
2 : widget_control, state.id.clust_field_extension, SET_VALUE=state.par.field_extension
3 : begin
      widget_control, state.id.neb_field_extension, SET_VALUE=state.par.field_extension
      display_object, state
   end
4 : begin
      widget_control, state.id.SNR_field_extension, SET_VALUE=state.par.field_extension
      display_object, state
   end
5 : begin
      widget_control, state.id.gal_field_extension, SET_VALUE=state.par.field_extension
      ;update galaxy total field magnitude
      if state.par.field_extension lt state.par.gal.extension[state.par.obj_nb] then begin
          state.par.gal_field_mag = state.par.gal.bright[state.par.obj_nb] $
                       -2.5*alog10(state.par.field_extension^2.)
      endif else begin
          state.par.gal_field_mag = state.par.gal.bright[state.par.obj_nb] $
                       -2.5*alog10(state.par.gal.extension[state.par.obj_nb]^2.)
      endelse
      widget_control, state.id.gal_field_mag, SET_VALUE=state.par.gal_field_mag
      display_object, state
   end
6 : begin
      widget_control, state.id.YSO_field_extension, SET_VALUE=state.par.field_extension

      ;update YSO total field magnitude
      if state.par.field_extension lt state.par.YSO.extension[state.par.obj_nb] then begin
         state.par.YSO_field_mag = state.par.YSO.bright[state.par.obj_nb] $
                            -2.5*alog10(state.par.field_extension^2.)
      endif else begin
         state.par.YSO_field_mag = state.par.YSO.bright[state.par.obj_nb] $
                            -2.5*alog10(state.par.YSO.extension[state.par.obj_nb]^2.)
      endelse
      widget_control, state.id.YSO_field_mag, SET_VALUE=state.par.YSO_field_mag
      display_object, state
   end
7 : widget_control, state.id.star_field_extension, SET_VALUE=state.par.field_extension
8 : begin
      widget_control, state.id.user_def_field_extension, SET_VALUE=state.par.field_extension
      ;update user_def_obj total field magnitude
      if state.par.field_extension lt state.par.user_def.extension[state.par.obj_nb] then begin
         state.par.user_def_field_mag = state.par.user_def.bright[state.par.obj_nb]         $
                       -2.5*alog10(state.par.field_extension^2.)
      endif else begin
         state.par.user_def_field_mag = state.par.user_def.bright[state.par.obj_nb]         $
                       -2.5*alog10(state.par.user_def.extension^2.)
      endelse
      widget_control, state.id.user_def_field_mag, SET_VALUE=state.par.user_def_field_mag
      display_object, state
   end
else : ;DO NOTHING!
endcase
return
end




pro obj_gui_event, event

common error_block, error

widget_control, event.top, GET_UVALUE=state
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
    error = !caos_error.cancel
    widget_control, event.top, /DESTROY
endif
widget_control, event.id, GET_UVALUE=uvalue
case uvalue of
'array_dim' : begin
   state.par.array_dim = event.value

   ;change the field extension value , field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state

   ;check if array_dim > 0
   widget_control, state.id.array_dim, GET_VALUE=dummy
   if (dummy le 0) then begin
   dummy = dialog_message("input array dimensions MUST BE greater than zero",              $
                           DIALOG_PARENT=event.top,                                         $
                           TITLE='input array error',                                       $
                           /ERROR                                                           )
   endif
   obj_gui_set, state
end


'pix_size'  : begin
   state.par.pix_size = event.value
   
   ;change the field extension value, field magnitude value and re-display object
   ;in the current type object gui
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   update_obj_par, state
   
   ;check if pixel size > 0
   widget_control, state.id.pix_size, GET_VALUE=dummy
   if (dummy le 0.) then begin
   dummy = dialog_message("pixel size dimension MUST BE greater than zero",                $
                           DIALOG_PARENT=event.top,                                         $
                           TITLE='pixel size error',                                        $
                           /ERROR                                                           )
   endif
   obj_gui_set, state
   end
   
   
'type_obj'  : begin
   
   ; close opened window
   if state.par.flag ne -1 then wdelete, state.par.flag
   state.par.flag = -1
   
   ;Destroy precedent object panel if exists
   ;print, event.index, state.type_obj, state.type_obj_nb
   if state.type_obj le state.type_obj_nb-1                        $
   then widget_control, state.id.obj_base[state.type_obj], /DESTROY
   
   ;update chosen type object
   state.type_obj = event.index
   state.par.type_obj = event.index
   
   ;Set the default object number -->>> NO!!! If the window has several objects, only
   ;the first will be visualized, and not the one really selected (by previous selection).
   ;state.par.obj_nb = 0
   
   ;set the default field extension value , field magnitude value
   state.par.field_extension  = (state.par.pix_size * state.par.array_dim)
   
   ;Realize chosen object panel
   if event.index le state.type_obj_nb-1   then begin
   
   
   case event.index of
   
         0 : single_star_gui, state
   
         1 : binary_star_gui, state
   
         2 : open_cluster_gui, state
   
         3 : planetary_nebulae_gui, state
   
         4 : SNR_gui, state
   
         5 : spiral_galaxy_gui, state
   
         6 : YSO_gui, state
   
         7 : star_surface_gui, state
   
         8 : user_def_gui, state
   
   
   endcase
   obj_gui_set, state
   
   endif
   
   end

'skymag': if (event.type eq 0) then begin
      widget_control, event.id,  GET_VALUE =dummy
      state.par.skymag = dummy
      widget_control, event.top, SET_UVALUE=state
   endif

'save_obj'        : begin
   state.par.save_obj_fits.enable = event.select
   obj_gui_set, state
   end

'obj_filename'    : begin
   widget_control, state.id.save_obj_filename , GET_VALUE=dummy
   state.par.save_obj_fits.file = dummy[0]
   obj_gui_set, state
   end


'save' : begin
   ; check before saving the parameter file if filename already exists
   check_file = findfile(state.sav_file)
   if check_file[0] ne "" then begin
      answ = dialog_message(['a file '+state.sav_file+' already exists:', $
                              'would you like to overwrite it ?'],         $
                           DIALOG_PARENT=event.top,                      $
                           TITLE='OBJ warning',                          $
                           /QUEST                                        )
      if strlowcase(answ) eq "no" then return
   endif else begin
      answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                           DIALOG_PARENT=event.top,                    $
                           TITLE='OBJ information',                    $
                           /INFO                                       )
   ; inform were the parameters will be saved
   endelse
   
   ; save the parameter data file
   obj_gui_set, state
   par = state.par
   save, par, FILENAME=state.sav_file
   
   ; kill the GUI returning a null error
   error = !caos_error.ok
   widget_control, event.top, /DESTROY
   
   end

; handle event from standard help button
'help': begin
   online_help, book=(obj_info()).help, /FULL_PATH
;   CASE !VERSION.OS_FAMILY OF
;   'Windows'   :     begin
;         spawn, !caos_env.browser+" "+(obj_info()).help,/NOSHELL
;         end
;   else    :  begin
;         spawn, !caos_env.browser+" "+(obj_info()).help+" &"
;         end
;   ENDCASE
   end
'restore': begin
   ; restore the desired parameter file
   par = 0
   title = "parameter file to restore"
   restore, filename_gui(state.def_file,                          $
                        title,                                   $
                        GROUP_LEADER=event.top,                  $
                        FILTER=state.par.module.mod_name+'*sav', $
                        /NOEDIT,                                 $
                        /MUST_EXIST,                             $
                        /ALL_EVENTS                              )

   ; update the current module number
   par.module.n_module = state.par.module.n_module


   ; set the default values for all the widgets
   widget_control,  state.id.pix_size,             SET_VALUE=par.pix_size
   widget_control,  state.id.array_dim,            SET_VALUE=par.array_dim
   widget_control,  state.id.save_obj,             SET_VALUE=par.save_obj_fits.enable
   widget_control,  state.id.save_obj_filename,    SET_VALUE=par.save_obj_fits.file

   case state.type_obj of
   
   0 : begin   ; single_star
   widget_control,  state.id.single_star_mag, SET_VALUE=par.single_star_mag
   widget_control,  state.id.single_star_field_extension, SET_VALUE=par.field_extension
   widget_control,  state.id.single_star_band, SET_COMBOBOX_SELECT=par.band_index
   end

   1 : begin   ; binary_star
   widget_control,  state.id.bin_separation,       SET_VALUE=par.bin_separation
   widget_control,  state.id.bin_pa,               SET_VALUE=par.bin_pa
   widget_control,  state.id.bin_mag,              SET_VALUE=par.bin_mag
   widget_control,  state.id.bin_delt_m,           SET_VALUE=par.bin_delt_m
   widget_control,  state.id.bin_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.bin_band,             SET_COMBOBOX_SELECT=par.band_index
   end

   2 : begin  ; open_cluster
   widget_control,  state.id.clust_nb_star,        SET_VALUE=par.clust_nb_star
   widget_control,  state.id.clust_mag_max,        SET_VALUE=par.clust_mag_max
   widget_control,  state.id.clust_mag_min,        SET_VALUE=par.clust_mag_min
   widget_control,  state.id.clust_seed,           SET_VALUE=par.clust_seed
   widget_control,  state.id.clust_field_extension,SET_VALUE=par.field_extension
   widget_control,  state.id.clust_band,           SET_COMBOBOX_SELECT=par.band_index
   end

   3 : begin   ; planetary_nebulae
   widget_control,  state.id.neb_name_obj,         SET_COMBOBOX_SELECT=par.obj_nb
   widget_control,  state.id.neb_extension,        SET_VALUE=par.neb.extension[par.obj_nb]
   widget_control,  state.id.neb_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.neb_spectra_choice,   SET_VALUE=par.neb_spectra_choice
   widget_control,  state.id.neb_width,            SET_VALUE=par.width
   widget_control,  state.id.neb_lambda,           SET_VALUE=par.lambda
   nb_lines = (size(par.neb.em_lines_text))[1]
   for i=0, nb_lines-1 do begin
      widget_control,  state.id.neb_em_lines[i],  $
           SET_VALUE=par.neb.user_def_spectrum.flux[par.neb_spectra_nb,i]
   endfor
   widget_control,  state.id.neb_flux,   SET_VALUE=par.neb.flux
   widget_control,  state.id.neb_sky_flux,   SET_VALUE=par.sky_flux
   end

   4 : begin ; Super-Nova Remnant
   widget_control,  state.id.SNR_name_obj,         SET_COMBOBOX_SELECT=par.obj_nb
   widget_control,  state.id.SNR_extension,        SET_VALUE=par.SNR.extension[par.obj_nb]
   widget_control,  state.id.SNR_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.SNR_spectra_choice,   SET_VALUE=par.SNR_spectra_choice
   widget_control,  state.id.SNR_width,            SET_VALUE=par.width
   widget_control,  state.id.SNR_lambda,           SET_VALUE=par.lambda
   nb_lines = (size(par.SNR.em_lines_text))[1]
   for i=0, nb_lines-1 do begin
      widget_control,  state.id.SNR_em_lines[i],   $
           SET_VALUE=par.SNR.user_def_spectrum.flux[par.SNR_spectra_nb,i]
   endfor
   widget_control,  state.id.SNR_flux,       SET_VALUE=par.SNR.flux
   widget_control,  state.id.SNR_sky_flux,   SET_VALUE=par.sky_flux
   end
   
   5 : begin   ; spiral_galaxy
   widget_control,  state.id.gal_name_obj,         SET_COMBOBOX_SELECT=par.obj_nb
   widget_control,  state.id.gal_extension,        SET_VALUE=par.gal.extension[par.obj_nb]
   widget_control,  state.id.gal_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.gal_broad_band_choice,SET_VALUE=par.gal.wave_range_choice[par.obj_nb]
   widget_control,  state.id.gal_band,             SET_COMBOBOX_SELECT=par.band_index
   widget_control,  state.id.gal_bright_mag_choice,SET_VALUE=par.gal.bright_mag_choice[par.obj_nb]
   widget_control,  state.id.gal_bright,           SET_VALUE=par.gal.bright[par.obj_nb]
   widget_control,  state.id.gal_field_mag,        SET_VALUE=par.gal_field_mag
   widget_control,  state.id.gal_tot_mag_choice,   SET_VALUE=non(par.gal.bright_mag_choice[par.obj_nb])
   widget_control,  state.id.gal_tot_mag,          SET_VALUE=par.gal.tot_mag[par.obj_nb]
   widget_control,  state.id.gal_lambda,           SET_VALUE=par.lambda
   widget_control,  state.id.gal_width,            SET_VALUE=par.width
   widget_control,  state.id.gal_flux,             SET_VALUE=par.gal.flux[par.obj_nb]
   widget_control,  state.id.gal_sky_flux,         SET_VALUE=par.sky_flux
   end
   
   6 : begin   ; YSO
   widget_control,  state.id.YSO_name_obj,         SET_COMBOBOX_SELECT=par.obj_nb
   widget_control,  state.id.YSO_extension,        SET_VALUE=par.YSO.extension[par.obj_nb]
   widget_control,  state.id.YSO_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.YSO_broad_band_choice,SET_VALUE=par.YSO.wave_range_choice[par.obj_nb]
   widget_control,  state.id.YSO_band,             SET_COMBOBOX_SELECT=par.band_index
   widget_control,  state.id.YSO_bright_mag_choice,SET_VALUE=par.YSO.bright_mag_choice[par.obj_nb]
   widget_control,  state.id.YSO_bright,           SET_VALUE=par.YSO.bright[par.obj_nb]
   widget_control,  state.id.YSO_field_mag,        SET_VALUE=par.YSO_field_mag
   widget_control,  state.id.YSO_tot_mag_choice,   SET_VALUE=non(par.YSO.bright_mag_choice[par.obj_nb])
   widget_control,  state.id.YSO_tot_mag,          SET_VALUE=par.YSO.tot_mag[par.obj_nb]
   widget_control,  state.id.YSO_lambda,           SET_VALUE=par.lambda
   widget_control,  state.id.YSO_width,            SET_VALUE=par.width
   widget_control,  state.id.YSO_flux,             SET_VALUE=par.YSO.flux[par.obj_nb]
   widget_control,  state.id.YSO_sky_flux,         SET_VALUE=par.sky_flux
   widget_control,  state.id.YSO_caption1,  $
          SET_VALUE='image caption : '+par.YSO.caption1[par.obj_nb],/DYNAMIC_RESIZE
   widget_control,  state.id.YSO_caption2,  $
          SET_VALUE='                '+par.YSO.caption2[par.obj_nb], /DYNAMIC_RESIZE
   widget_control,  state.id.YSO_caption3,  $
          SET_VALUE='                '+par.YSO.caption3[par.obj_nb], /DYNAMIC_RESIZE
   widget_control,  state.id.YSO_credit,    $
          SET_VALUE='image credit  : '+par.YSO.credit[par.obj_nb], /DYNAMIC_RESIZE
   end
   
   7 : begin    ; star_surface
   widget_control,  state.id.star_band,            SET_COMBOBOX_SELECT=par.band_index
   widget_control,  state.id.star_mag,             SET_VALUE=par.star_mag
   widget_control,  state.id.star_diam,            SET_VALUE=par.star_diam
   widget_control,  state.id.star_ax_ratio,        SET_VALUE=par.star_ax_ratio
   widget_control,  state.id.spot_diam,            SET_VALUE=par.spot_diam
   widget_control,  state.id.spot_flux,            SET_VALUE=par.spot_flux
   widget_control,  state.id.spot_dist,            SET_VALUE=par.spot_dist
   widget_control,  state.id.spot_pa,              SET_VALUE=par.spot_pa
   widget_control,  state.id.star_field_extension, SET_VALUE=par.star_field_extension
   end

   8 : begin
   ; user-defined object
   widget_control,  state.id.user_def_obj_filename,SET_VALUE=par.user_def.obj_file[par.obj_nb]
   widget_control,  state.id.user_def_extension,        SET_VALUE=par.user_def.extension[par.obj_nb]
   widget_control,  state.id.user_def_field_extension,  SET_VALUE=par.field_extension
   widget_control,  state.id.user_def_broad_band_choice,SET_VALUE=par.user_def.wave_range_choice[par.obj_nb]
   widget_control,  state.id.user_def_band,             SET_COMBOBOX_SELECT=par.band_index
   widget_control,  state.id.user_def_bright_mag_choice,SET_VALUE=par.user_def.bright_mag_choice[par.obj_nb]
   widget_control,  state.id.user_def_bright,           SET_VALUE=par.user_def.bright[par.obj_nb]
   widget_control,  state.id.user_def_field_mag,        SET_VALUE=par.user_def_field_mag
   widget_control,  state.id.user_def_tot_mag_choice, SET_VALUE=non(par.user_def.bright_mag_choice[par.obj_nb])
   widget_control,  state.id.user_def_tot_mag,          SET_VALUE=par.user_def.tot_mag[par.obj_nb]
   widget_control,  state.id.user_def_lambda,           SET_VALUE=par.lambda
   widget_control,  state.id.user_def_width,            SET_VALUE=par.width
   widget_control,  state.id.user_def_flux,             SET_VALUE=par.user_def.flux[par.obj_nb]
   widget_control,  state.id.user_def_sky_flux,         SET_VALUE=par.sky_flux
   end


   else :

endcase

widget_control, state.id.skymag,       SET_VALUE=par.skymag
widget_control, state.id.type_obj, SET_COMBOBOX_SELECT=state.par.type_obj

   state.par = par
   obj_gui_set, state

   end

   'cancel'  : begin
      error = !caos_error.cancel
      widget_control, event.top, /DESTROY
   end

endcase


end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function obj_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
common error_block, error

; retrieve the module information
info = obj_info()

; parameter file checks
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
    restore, def_file
    par.module.n_module = n_module
    if (par.module.mod_name ne info.mod_name) then     $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
    restore, sav_file
    if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file      $
              +' is from another module: please generate a new one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse



id = {                                         $
        root_base_id            : 0L         , $
        par_base                : 0L         , $
        band                    : 0L         , $
        pix_size                : 0L         , $
        array_dim               : 0L         , $
        type_obj                : 0L         , $
        save_obj                : 0L         , $
        save_obj_filename       : 0L         , $
        obj_base                : lonarr(par.type_obj_nb)  , $

        single_star_mag         : 0L         , $
        single_star_band        : 0L         , $
        single_star_field_extension : 0L         , $

        bin_separation          : 0L         , $
        bin_pa                  : 0L         , $
        bin_mag                 : 0L         , $
        bin_delt_m              : 0L         , $
        bin_field_extension     : 0L         , $
        bin_band                : 0L         , $

        clust_nb_star           : 0L         , $
        clust_mag_max           : 0L         , $
        clust_mag_min           : 0L         , $
        clust_seed              : 0L         , $
        clust_field_extension   : 0L         , $
        clust_band              : 0L         , $

        neb_name_obj            : 0L         , $
        neb_image               : 0L         , $
        neb_extension           : 0L         , $
        neb_field_extension     : 0L         , $
        neb_spectra_choice      : 0L         , $
        neb_width               : 0L         , $
        neb_lambda              : 0L         , $
        neb_spectra_number      : 0L         , $
        neb_em_lines            : lonarr(32) , $
        neb_flux                : 0L         , $
        neb_sky_flux            : 0L         , $

        SNR_name_obj            : 0L         , $
        SNR_image               : 0L         , $
        SNR_extension           : 0L         , $
        SNR_field_extension     : 0L         , $
        SNR_spectra_choice      : 0L         , $
        SNR_width               : 0L         , $
        SNR_lambda              : 0L         , $
        SNR_spectra_number      : 0L         , $
        SNR_em_lines            : lonarr(32) , $
        SNR_flux                : 0L         , $
        SNR_sky_flux            : 0L         , $

        gal_name_obj            : 0L         , $
        gal_image               : 0L         , $
        gal_bright              : 0L         , $
        gal_extension           : 0L         , $
        gal_field_extension     : 0L         , $
        gal_field_mag           : 0L         , $
        gal_broad_band_choice   : 0L         , $
        gal_width               : 0L         , $
        gal_lambda              : 0L         , $
        gal_flux                : 0L         , $
        gal_sky_flux            : 0L         , $
        gal_bright_mag_choice   : 0L         , $
        gal_tot_mag_choice      : 0L         , $
        gal_tot_mag             : 0L         , $
        gal_band                : 0L         , $
        gal_bright_mag          : 0L         , $
        gal_narrow_band         : 0L         , $

        YSO_name_obj            : 0L         , $
        YSO_image               : 0L         , $
        YSO_bright              : 0L         , $
        YSO_extension           : 0L         , $
        YSO_field_extension     : 0L         , $
        YSO_caption1            : 0L         , $
        YSO_caption2            : 0L         , $
        YSO_caption3            : 0L         , $
        YSO_credit              : 0L         , $
        YSO_field_mag           : 0L         , $
        YSO_broad_band_choice   : 0L         , $
        YSO_width               : 0L         , $
        YSO_lambda              : 0L         , $
        YSO_flux                : 0L         , $
        YSO_sky_flux            : 0L         , $
        YSO_bright_mag_choice   : 0L         , $
        YSO_tot_mag_choice      : 0L         , $
        YSO_tot_mag             : 0L         , $
        YSO_band                : 0L         , $
        YSO_bright_mag          : 0L         , $
        YSO_narrow_band         : 0L         , $

        star_mag                : 0L         , $
        star_diam               : 0L         , $
        star_ax_ratio           : 0L         , $
        spot_diam               : 0L         , $
        spot_flux               : 0L         , $
        spot_dist               : 0L         , $
        spot_pa                 : 0L         , $
        star_field_extension    : 0L         , $
        star_band               : 0L         , $

        user_def_obj_filename      : 0L         , $
        user_def_image             : 0L         , $
        user_def_bright            : 0L         , $
        user_def_extension         : 0L         , $
        user_def_field_extension   : 0L         , $
        user_def_field_mag         : 0L         , $
        user_def_broad_band_choice : 0L         , $
        user_def_width             : 0L         , $
        user_def_lambda            : 0L         , $
        user_def_flux              : 0L         , $
        user_def_sky_flux          : 0L         , $
        user_def_bright_mag_choice : 0L         , $
        user_def_tot_mag_choice    : 0L         , $
        user_def_tot_mag           : 0L         , $
        user_def_band              : 0L         , $
        user_def_bright_mag        : 0L         , $
        user_def_narrow_band       : 0L         , $

        skymag : 0L                               $

  }




; general state id. struc.
state =                                        $
   {                                           $
   sav_file                : sav_file        , $
   def_file                : def_file        , $
   id                      : id              , $
   par                     : par             , $
   type_obj_tab            : par.type_obj_tab    , $
   type_obj                : par.type_obj_nb     , $
   type_obj_nb             : par.type_obj_nb       $
                                               }
; root base
modal = n_elements(group) ne 0
title = strupcase(info.mod_name)+' parameter setting GUI'
state.id.root_base_id = $
   widget_base(TITLE=title, MODAL=modal, /COL, GROUP_LEADER=group)
widget_control, state.id.root_base_id, SET_UVALUE=state

; parameters base

   state.id.par_base = widget_base(state.id.root_base_id, FRAME=10, ROW=2)
   dummy = widget_label(state.id.par_base, VALUE='parameters', /FRAME)
   par_base_id = widget_base(state.id.par_base, /COL)

; ---------------------- Map parameters widgets --------------------------------

  pix_band_dim_id= widget_base(par_base_id, /COL)

  dummy= widget_label(pix_band_dim_id, VALUE='Map parameters: ', /ALIGN_LEFT)

  pix_size_all= widget_base(pix_band_dim_id, /ROW)
  state.id.pix_size  = cw_field(pix_size_all, $
                       TITLE='  map scale (arcsec/pixel)  [ENTER]   ',$
                       /FLOATING, VALUE=state.par.pix_size, /RETURN_EVENTS, $
                       UVALUE='pix_size', xsize=12)
  array_dim_all= widget_base(pix_band_dim_id, /ROW)
  state.id.array_dim = cw_field(array_dim_all,$
                       TITLE='  map dimensions (pixels)   [ENTER]   ',$
                       /INTEGER, VALUE=state.par.array_dim, /RETURN_EVENTS, $
                       UVALUE='array_dim', xsize=12)




; ---------------------- Object parameters widgets --------------------------------

  obj_type_id= widget_base(par_base_id)
  list_all= widget_base(obj_type_id, /ROW)
  dummy= widget_label(list_all, VALUE='Select an object: ')
  state.id.type_obj= widget_combobox(list_all,      $
                   VALUE=state.type_obj_tab, UVALUE='type_obj')
  ;dummy = widget_label(list_all, VALUE='(Selected object: '                    $
  ;                 +string(state.par.type_obj_tab[state.par.type_obj])+')' )
  back_base_id = widget_base(par_base_id, /COL)
  dummy = widget_label(back_base_id, value=' ')
  dummy = widget_label(back_base_id,/ALIGN_LEFT, $
                  VALUE= "Sky background magnitudes vs. spectral bands")
  dummy = n_phot(0., BAND=dumband)
  band=dumband[3:7]             ; select bands from R to K
  state.id.skymag = widget_table(back_base_id, ROW_LABELS=['mag.'],    $
                    COLUMN_LABELS=band,VALUE=state.par.skymag, $
                    UVALUE='skymag',/EDITABLE, YSIZE=1)
  save_obj_id= widget_base(par_base_id, /COL)
 state.id.save_obj= cw_bgroup(save_obj_id,                               $
                  ['save object map in .fits file  '],       $
                  SET_VALUE=state.par.save_obj_fits.enable,  $
                   UVALUE='save_obj',  /NONEXCLUSIVE)
  state.id.save_obj_filename = cw_filename(save_obj_id,                      $
                       TITLE=' ', VALUE=state.par.save_obj_fits.file,    $
                        UVALUE='obj_filename',  /ALL_EVENTS )




   ; button base for control buttons (standard buttons)
   btn_base_id = widget_base(state.id.root_base_id, FRAME=10, /ROW)
      dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
      cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
      if modal then widget_control, cancel_id, /CANCEL_BUTTON
      dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                            UVALUE="restore")
      save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS", $
                              UVALUE="save")
      if modal then widget_control, save_id, /DEFAULT_BUTTON


; final stuff

widget_control, state.id.root_base_id, SET_UVALUE=state
widget_control, state.id.root_base_id, /REALIZE
widget_control, state.id.type_obj, SET_COMBOBOX_SELECT=state.par.type_obj

xmanager, 'obj_gui', state.id.root_base_id, GROUP_LEADER=group

return, error
end
