; $Id: obj_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;       obj_init
;
; PURPOSE:
;       obj_init executes the initialization for the OBJect
;       (OBJ) module, that is:
;
;         0- check the formal validity of the output structure.
;         1- initialize the output structure(s) out_obj_t.
;
;       (see obj.pro's header --or file airy_help.html-- for details
;       about the module itself).
;
; CATEGORY:
;       module's initialization routine
;
; CALLING SEQUENCE:
;       error = obj_init(out_obj_t, $
;                        par,       $
;                        INIT=init  $
;                       )
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see obj.pro's help for a detailed description.
;
; MODIFICATION HISTORY:
;    program written: may 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : september 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -Added an object type (SN remnants).
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : november 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -procedure model_binary_star-->function model_binary_star
;                     for error management in order to possibly stop the
;                     program in the standard "clean" CAOS way.
;                    -within sub-routine model_binary_star: warning added
;                     when binary sep. and/or binary pos. angle has
;                     been changed in order to match the pixels.
;                    -skymag is now a selectable parameter...
;                     (no more given only by default value in routine n_phot)
;                    -sky magnitude calculation debugged
;                     (was fixed to default values in routine n_phot even when
;                     requested different)
;                   : november 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -added the object type "single star"
;                   : february 2005,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -corrected bug in single star object (x_cent instead of
;                     x_main and y_cent instead of y_main)
;                   : january 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Flux constants are now "float" instead of "int".
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -keywords added to the header (that is now created
;                     whether the map is saved or not), most commons are:
;                     OBJECT, BAND, and PIX_SIZE. Other keywords are
;                     described in the keywordlist_definition.txt file. 
;-                   
;


pro phot_vect, wave_range_choice, band_and_flux, band_ind, mag, own_lambda, own_width, flux, back, $
               n_ph, background, lambda, width, skymag


; begin with R-band
band_ind = band_ind + 3

; retrieve default sky background magnitudes
;;dummy = n_phot(0., BACK_MAG=skymag)

; retrieve lambdas, widths and bands
dummy   = n_phot(0., BAND=band, LAMBDA=lambda, WIDTH=width)
n_bands = n_elements(band)

; nb of photon and background vectors
n_ph     = fltarr(n_bands)
background = fltarr(n_bands)

; wave_range_choice = 0 -> magnitude and pre-defined band
;                   = 1 -> flux and user-defined band
;                          band_and_flux     = 1 -> flux and pre-defined band

if wave_range_choice eq 0 then begin

    n_ph[band_ind]       = (n_phot(mag, BAND=band[band_ind]))[0]
    background[band_ind] = (n_phot(skymag[band_ind-3], band=band[band_ind]))[0]

endif else begin

    if band_and_flux eq 1 then begin

        n_ph[band_ind]       = flux
        background[band_ind] = (n_phot(skymag[band_ind-3], band=band[band_ind]))[0]

    endif else begin

        band[n_bands-1]       = 'own'
        lambda[n_bands-1]     = own_lambda
        width[n_bands-1]      = own_width
        n_ph[n_bands-1]       = flux
        background[n_bands-1] = back

    endelse

endelse

end

;-------------------------------------------------------------------------------------------

function obj_init, out_obj_t, par, INIT=init

error = !caos_error.ok

; STANDARD CHECKS
;================

info = obj_info()                                           ; Retrieve the Input & Output info.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Compute and test the requested number of obj arguments
n_par = 1                       ; the parameter structure is always
                                ; present within the arguments

if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp = n_elements(inp_type)
endif else n_inp = 0

if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out = n_elements(out_type)
endif else n_out = 0

n_par = n_par + n_inp + n_out

; test the number of passed parameters
if n_params() ne n_par then message, 'wrong number of parameters'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'OBJ: par must be a structure'
if n ne 1 then message, 'OBJ: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module obj'


; NO INPUT => no test on the input argument !!
;--------------------------------------------


print, '-------------------------------------------------------------------'
print, 'OBJ:'

; Case of the object type
case par.type_obj of

    0 : begin
                                ; single star
       
        x_cent = par.array_dim/2 + 0.5 ;centered on the pixel
        y_cent = par.array_dim/2 + 0.5
        
        error = model_single_star(x_cent, y_cent, par.single_star_mag, par.array_dim, par.band_tab[par.band_index], $
                                  par.pix_size, map, n_phot)
        
        
        mkhdr, header, map
        fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] single star'
        fxaddpar, header, 'BAND', strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
        fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
        fxaddpar, header, 'MAGNITUD', par.bin_mag, '[OBJ] Magnitude'
        fxaddpar, header, 'X_POS', x_cent, '[OBJ] X-position [pixel]'
        fxaddpar, header, 'Y_POS', y_cent, '[ OBJ] Y-position [pixel]'
        
        if par.save_obj_fits.enable then  writefits, par.save_obj_fits.file, map, header
        
     end
    1 : begin
                                ; binary star
       
       x_cent = par.array_dim/2 + 0.5 ;centered on the pixel
       y_cent = par.array_dim/2 + 0.5
       
       x_main = x_cent - (par.bin_separation/par.pix_size*cos(par.bin_pa*!pi/180))/2
       y_main = y_cent + (par.bin_separation/par.pix_size*sin(par.bin_pa*!pi/180))/2
       
       x_comp = x_main + par.bin_separation/par.pix_size*cos(par.bin_pa*!pi/180)
       y_comp = y_main - par.bin_separation/par.pix_size*sin(par.bin_pa*!pi/180)
       
       error = model_binary_star(x_main, y_main, x_comp, y_comp, par.bin_separation, par.bin_pa,    $
                                 par.bin_mag, par.bin_delt_m, par.array_dim, par.band_tab[par.band_index], $
                                 par.pix_size, map, n_phot)
       
       mkhdr, header, map
       fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] binary star'
       fxaddpar, header, 'BAND',     strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
       fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
       fxaddpar, header, 'SEP', 	  par.bin_separation*1e3, '[OBJ] Binary star separation [arcsec]'
       fxaddpar, header, 'PA', 	  par.bin_pa, 		'[OBJ] Binary star Position Angle [deg.]'
       fxaddpar, header, 'MAGNITUD', 	  par.bin_mag, 		'[OBJ] Main component magnitude'
       fxaddpar, header, 'MAG_DIFF',  par.bin_delt_m,       '[OBJ] Magnitude difference'
       fxaddpar, header, 'X_MAIN',   x_main,               '[OBJ] X-main component position [pixel]'
       fxaddpar, header, 'Y_MAIN',   y_main,               '[OBJ] Y-main component position [pixel]'
       
       if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
       
    end
    

    2 : begin
                                ;Random generated open cluster

       model_random_clust, par.clust_nb_star, par.clust_mag_max, par.clust_mag_min,     $
                           par.clust_seed, par.array_dim, par.band_tab[par.band_index], $
                           par.pix_size, map, n_phot
       
       mkhdr, header, map
       fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] open cluster'
       fxaddpar, header, 'BAND',     strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
       fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
       fxaddpar, header, 'STAR_NB', par.clust_nb_star, 	'[OBJ] Stars number in the generated cluster'
       fxaddpar, header, 'MIN_MAG', par.clust_mag_min, 	'[OBJ] Minimum star magnitude (before apodization)'
       fxaddpar, header, 'MAX_MAG', par.clust_mag_max,     '[OBJ] Maximum star magnitude (before apodization)'
       fxaddpar, header, 'SEED',    par.clust_seed,        '[OBJ] Randomn cluster generation seed'


       if par.save_obj_fits.enable then  writefits, par.save_obj_fits.file, map, header
    end
    
    
    3 : begin
                                ; planetary nebulae
       
       model_extended_obj_flux, par.neb.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                par.neb.extension[par.obj_nb], par.neb.flux, map, n_phot
       mkhdr, header, map
       fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] planetary nebula'
       fxaddpar, header, 'BAND',     strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
       fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
       fxaddpar, header, 'LAMBDA',  par.lambda,                           '[OBJ] Observation filter central wavelength [A]'
       fxaddpar, header, 'WIDTH',   par.width,                            '[OBJ] Observation filter width [A]'
       fxaddpar, header, 'OBJ_FILE', 	 par.neb.obj_file[par.obj_nb], 	       '[OBJ] Object definition filename'
       fxaddpar, header, 'OBJ_FLUX',    par.neb.flux, 	                       '[OBJ] Object flux [ph/s/m2]'
       fxaddpar, header, 'EXTENS',  par.neb.extension[par.obj_nb],        '[OBJ] Total object extension [arcsec]'
                                ;Save object map in .fits
       if par.save_obj_fits.enable then  writefits, par.save_obj_fits.file, map, header
    end

    4 : begin
                                ; SN remnants
       model_extended_obj_flux, par.SNR.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                par.SNR.extension[par.obj_nb], par.SNR.flux, map, n_phot
       mkhdr, header, map
       fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] SN remnant'
       fxaddpar, header, 'BAND', strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
       fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
       fxaddpar, header, 'LAMBDA',  par.lambda, '[OBJ] Observation filter central wavelength [A]'
       fxaddpar, header, 'WIDTH',   par.width,  '[OBJ] Observation filter width [A]'
       fxaddpar, header, 'OBJ_FILE', par.SNR.obj_file[par.obj_nb], '[OBJ] Object definition filename'
       fxaddpar, header, 'OBJ_FLUX', par.SNR.flux,  '[OBJ] Object flux [ph/s/m2]'
       fxaddpar, header, 'EXTENS',  par.SNR.extension[par.obj_nb],  '[OBJ] Total object extension [arcsec]'
                                ;Save object map in .fits
       if par.save_obj_fits.enable then  writefits, par.save_obj_fits.file, map, header
    end
    
    5 : begin
        ; Spiral galaxy
       if par.gal.wave_range_choice[par.obj_nb] eq 0 then begin
          if par.gal.bright_mag_choice[par.obj_nb] eq 0 then begin
             model_extended_obj_mag_tot, par.gal.obj_file[par.obj_nb], par.pix_size, par.array_dim,  $
                                         par.gal.extension[par.obj_nb], par.gal.tot_mag[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] spiral galaxy'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE',  par.gal.obj_file[par.obj_nb],  '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.gal.extension[par.obj_nb], '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'TOT_MAG', par.gal.tot_mag[par.obj_nb], 	            '[OBJ] Total object magnitude'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endif else begin
             model_extended_obj_bright,  par.gal.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                         par.gal.extension[par.obj_nb], par.gal.bright[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] spiral galaxy'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE',  par.gal.obj_file[par.obj_nb],  '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.gal.extension[par.obj_nb], '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'BRIGHT',  par.gal.bright[par.obj_nb], 	            '[OBJ] Average object brightness'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endelse
       endif else begin
          model_extended_obj_flux, par.gal.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                   par.gal.extension[par.obj_nb], par.gal.flux[par.obj_nb],   $
                                   map, n_phot
          mkhdr, header, map
          fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] spiral galaxy'
          fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
          fxaddpar, header, 'OBJ_FILE',  par.gal.obj_file[par.obj_nb],  '[OBJ] Object definition filename'
          fxaddpar, header, 'EXTENS',  par.gal.extension[par.obj_nb], '[OBJ] Total object extension [arcsec]'
          fxaddpar, header, 'LAMBDA',  par.lambda,                       '[OBJ] Observation filter central wavelength [A]'
          fxaddpar, header, 'WIDTH',   par.width,                        '[OBJ] Observation filter width [A]'
          fxaddpar, header, 'OBJ_FLUX',    par.gal.flux[par.obj_nb], 	       '[OBJ] Object flux [ph/s/m2]'
                                ;Save object map in .fits
          if par.save_obj_fits.enable then  writefits, par.save_obj_fits.file, map, header
       endelse
    end

    6 : begin
        ; YSO
       if par.YSO.wave_range_choice[par.obj_nb] eq 0 then begin
          if par.YSO.bright_mag_choice[par.obj_nb] eq 0 then begin
             model_extended_obj_mag_tot, par.YSO.obj_file[par.obj_nb], par.pix_size, par.array_dim,  $
                                         par.YSO.extension[par.obj_nb], par.YSO.tot_mag[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] YSO'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE', par.YSO.obj_file[par.obj_nb], '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.YSO.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'TOT_MAG', par.YSO.tot_mag[par.obj_nb], 	            '[OBJ] Total object magnitude'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endif else begin
             model_extended_obj_bright,  par.YSO.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                         par.YSO.extension[par.obj_nb], par.YSO.bright[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] YSO'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE', par.YSO.obj_file[par.obj_nb], '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.YSO.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'BRIGHT',  par.YSO.bright[par.obj_nb], 	            '[OBJ] Average object brightness'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endelse          
       endif else begin
          model_extended_obj_flux, par.YSO.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                   par.YSO.extension[par.obj_nb], par.YSO.flux[par.obj_nb],   $
                                   map, n_phot
          mkhdr, header, map
          fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] YSO'
          fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
          fxaddpar, header, 'OBJ_FILE', par.YSO.obj_file[par.obj_nb], '[OBJ] Object definition filename'
          fxaddpar, header, 'EXTENS',  par.YSO.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
          fxaddpar, header, 'LAMBDA',  par.lambda,                       '[OBJ] Observation filter central wavelength [A]'
          fxaddpar, header, 'WIDTH',   par.width,                        '[OBJ] Observation filter width [A]'
          fxaddpar, header, 'OBJ_FLUX',    par.YSO.flux[par.obj_nb], 	       '[OBJ] Object flux [ph/s/m2]'
                                ;Save object map in .fits
          if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
       endelse
    end
    
    7 : begin
                                ; Stellar surface
       
       model_stellar_surf, par.star_mag, par.star_diam, par.star_ax_ratio, par.spot_diam, $
                           par.spot_flux, par.spot_dist, par.spot_pa,                     $
                           par.pix_size, par.array_dim, par.band_tab[par.band_index],     $
                           map, n_phot
       mkhdr, header, map
       fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] Stellar surface'
       fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
       fxaddpar, header, 'BAND', strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
       fxaddpar, header, 'OBJ_MAG', par.star_mag,'[OBJ] Star magnitude'
       fxaddpar, header, 'STAR_DI',  par.star_diam,'[OBJ] Star diameter [arcsec]'
       fxaddpar, header, 'AX_RATIO',  par.star_ax_ratio, '[OBJ] Star axial ratio'
       fxaddpar, header, 'SPOT_DI',  par.spot_diam, '[OBJ] Spot diameter [arcsec]'
       fxaddpar, header, 'SPOT_FL',  par.spot_flux, '[OBJ] Spot flux [% of the total flux]'
       fxaddpar, header, 'SPOT_DIS',  par.spot_dist, '[OBJ] Spot location (radius from center) [arcsec]'
       fxaddpar, header, 'SPOT_PA',  par.spot_pa, '[OBJ] Spot position angle [deg.]'
       
                                ;Save object map in .fits
       if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
    end
    
    8 : begin
                                ; User-defined object
       if par.user_def.wave_range_choice[par.obj_nb] eq 0 then begin
          
          if par.user_def.bright_mag_choice[par.obj_nb] eq 0 then begin
             model_extended_obj_mag_tot, par.user_def.obj_file[par.obj_nb], par.pix_size, par.array_dim,       $
                                         par.user_def.extension[par.obj_nb], par.user_def.tot_mag[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] user-defined'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE', par.user_def.obj_file[par.obj_nb], '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.user_def.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'TOT_MAG', par.user_def.tot_mag[par.obj_nb], 	'[OBJ] Total object magnitude'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endif else begin
             model_extended_obj_bright,  par.user_def.obj_file[par.obj_nb], par.pix_size, par.array_dim, $
                                         par.user_def.extension[par.obj_nb], par.user_def.bright[par.obj_nb], $
                                         par.band_tab[par.band_index], map, n_phot
             mkhdr, header, map
             fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] user-defined'
             fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
             fxaddpar, header, 'OBJ_FILE', par.user_def.obj_file[par.obj_nb], '[OBJ] Object definition filename'
             fxaddpar, header, 'EXTENS',  par.user_def.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
             fxaddpar, header, 'BAND',    strcompress(par.band_tab[par.band_index]), '[OBJ] Observation band'
             fxaddpar, header, 'BRIGHT',  par.user_def.bright[par.obj_nb], 	'[OBJ] Average object brightness'
                                ;Save object map in .fits
             if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
          endelse
       endif else begin
          model_extended_obj_flux, par.user_def.obj_file[par.obj_nb], par.pix_size, par.array_dim,    $
                                   par.user_def.extension[par.obj_nb], par.user_def.flux[par.obj_nb], $
                                   map, n_phot
          mkhdr, header, map
          fxaddpar, header, 'OBJECT', par.type_obj, '[OBJ] user-defined'
          fxaddpar, header, 'PIX_SIZE', par.pix_size, '[OBJ] Pixel size [arcsec]'
          fxaddpar, header, 'OBJ_FILE', par.user_def.obj_file[par.obj_nb], '[OBJ] Object definition filename'
          fxaddpar, header, 'EXTENS',  par.user_def.extension[par.obj_nb],          '[OBJ] Total object extension [arcsec]'
          fxaddpar, header, 'LAMBDA',  par.lambda,                       '[OBJ] Observation filter central wavelength [A]'
          fxaddpar, header, 'WIDTH',   par.width,                        '[OBJ] Observation filter width [A]'
          fxaddpar, header, 'OBJ_FLUX',    par.user_def.flux[par.obj_nb], 	'[OBJ] Object flux [ph/s/m2]'
          
                                ;Save object map in .fits
          if par.save_obj_fits.enable then writefits, par.save_obj_fits.file, map, header
       endelse
    end
        
 endcase

; put the photons number and background in the array bands (Johnson or not Johnson...)


case par.type_obj of

    0 : begin
        wave_range_choice = 1
        band_and_flux = 1
        flux = n_phot
        end

    1 : begin
        wave_range_choice = 1
        band_and_flux = 1
        flux = n_phot
        end

    2 : begin
        wave_range_choice = 1
        band_and_flux = 1
        flux = n_phot
        end

    3 : begin
        wave_range_choice = 1
        band_and_flux = 0
        flux = par.neb.flux
        end

    4 : begin
        wave_range_choice = 1
        band_and_flux = 0
        flux = par.SNR.flux
        end

    5 : begin
        wave_range_choice = par.gal.wave_range_choice[par.obj_nb]
        band_and_flux = 0
        if par.gal.bright_mag_choice[par.obj_nb] eq 0 then mag = par.gal.tot_mag[par.obj_nb] $
        else mag = par.gal_field_mag
        flux = par.gal.flux[par.obj_nb]
        end

    6 : begin
        wave_range_choice = par.YSO.wave_range_choice[par.obj_nb]
        band_and_flux = 0
        if par.YSO.bright_mag_choice[par.obj_nb] eq 0 then mag = par.YSO.tot_mag[par.obj_nb] $
        else mag = par.YSO_field_mag
        flux = par.YSO.flux[par.obj_nb]
        end

    7 : begin
        wave_range_choice = 0
        band_and_flux = 0
        mag = par.star_mag
        end

    8 : begin
        wave_range_choice = par.user_def.wave_range_choice
        band_and_flux = 0
        if par.user_def.bright_mag_choice eq 0 then mag = par.user_def.tot_mag[par.obj_nb] $
        else mag = par.user_def_field_mag
        flux = par.user_def.flux[par.obj_nb]
        end

endcase

phot_vect, wave_range_choice, band_and_flux, par.band_index, mag, par.lambda*1.e-10, par.width*1.e-10, flux, $
           par.sky_flux, n_phot, background, lambda, width, par.skymag

; map normalisation to energy unity
map = map / total(map)

; init structure
init =                                              $
   {                                                $
   off_axis  : 0.,                                  $ ; [rd] ;angular pos. of
   pos_ang   : 0.,                                  $ ; [rd] ;source wrt main tel.
   dist_z    : 0.,                                  $ ; [m] dist. main tel.-object
                                                      ;(infinity if astro. object)
   map       : map,                                 $ ; source map
   scale_xy  : par.pix_size/206265.,                $ ; map scale (if any) [rd/px]
   coord     : 0.,                                  $ ; [not used]
   scale_z   : 0.,                                  $ ; [not used]

   n_phot    : n_phot,                              $ ; nbs of photons  [/s/m^2]
   background: background,                          $ ; sky backgrounds [/s/m^2/arcsec^2]

   lambda    : lambda,                              $ ; wavelengths [m]
   width     : width,                               $ ; band-widths [m]
   constant  : 0.                                   $ ; constant (wrt time) source ?

   }

; output structure initialisation
out_obj_t =                       $
   {                              $
   data_type  : info.out_type[0], $
   data_status: !caos_data.valid, $
   off_axis   : init.off_axis,    $
   pos_ang    : init.pos_ang,     $
   dist_z     : init.dist_z,      $
   map        : init.map,         $
   scale_xy   : init.scale_xy,    $
   coord      : init.coord,       $
   scale_z    : init.scale_z,     $
   n_phot     : init.n_phot,      $
   background : init.background,  $
   lambda     : init.lambda,      $
   width      : init.width,       $
   constant   : init.constant,    $
   header     : PTR_NEW('')       $
   }

*out_obj_t.header = header  ; update the pointer

return, error
end
