; $Id: pex.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pex
;
; ROUTINE'S PURPOSE:
;    pex manages the simulation for the PEX module, that is to call 
;    the module's initialization routine pex_init at the first iteration 
;    of the simulation project.
;
; MODULE'S PURPOSE:
;    PEX performs PSF extrapolation from a cube of images (or from a single 
;    image). The extraction can be done either when project is "set up" (when 
;    parameters of the modules are chosen and saved) or when project runs.
;    This operation is divided between PEX_OPEN_GUI and PEX_INIT procedures. 
;    PEX_OPEN_GUI: users can load the image, set up scale, zoom, etc., 
;             then they can select one or more star with a simple 
;             point&click operation, and also select a suitable 
;             'radius of extraction' (R) and the value of the beta parameter.
;             Users have to complete the information about PSF 
;             (pixel-size, band, etc.).
;    PEX_INIT: PSF is extracted by following steps:
;             For each selected star,
;             - centroid of the star is computed and its position is shifted 
;               to the center of the image with a sub-pixel precision;
;             - a domain of size 2R x 2R is extracted from the central region 
;               (this image will be denoted by H);
;             - Moffat extrapolation of H(n) (where n=[n1,n2] is the multi-index 
;               characterizing the pixels of the image) is executed by following 
;               sub-steps:
;             -- a first estimation of background is performed with a bilinear 
;                function: B(n) = a0 + a1*n1 + a2*n2 
;             -- the image H - a1*n1 - a2*n2 is fitted with a circular 
;                symmetric Moffat function:
;                                      b1
;                M0(r) = b0 + -------------------    where r = sqrt(n1^2 + n2^2)
;                             (1 + r^2 / b2^2)^beta
;             -- the estimated values of the parameters ai and bi are used as 
;                initial guesses for a least-squares best-fitting of H(n) with  
;                a rotated elliptical Moffat function superimposed with a 
;                linear varying background:
;	 	                                                   c3
;                M(n) = c0 + c1*n1 + c2*n2 + -----------------------------------
;                                             (1 + (n1'/c4)^2 + (n2'/c5)^2)^beta
;                where n1'=n1*cos(q) + n2*sin(q) and n2'=-n1*sin(q) + n2*cos(q).
;                The value of b2 is the initial guess for both c4 and c5, 
;                while the initial guess of the rotation angle q in zero.
;             -- the extracted image H(n) is merged into the sampled function 
;                M(n) defined on the full image and the linearly background is 
;                subtracted and the result is normalized to unit volume.
;             If several stars are selected in the same image, the median of 
;             all the results of previous procedure is computed and this is 
;             normalized again to unit volume.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = pex(out_img_t, par)
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INCLUDED OUTPUTS:
;    out_img_t: structure of type "img_t" containing the extracted PSF 
;               array, the following fields are contained in
;               this structure:
;out_img_t =                           $
;   {                                  $
;   data_type  : out_type[0],          $
;   data_status: !caos_data.valid,     $
;   image      : image,                $ ; image/psf data cube
;   npixel     : _N_out,               $ ; number of pixels
;   resolution : par.resolution,       $ ; Pixel size [arcsec/px]
;   lambda     : par.lambda,           $ ; Mean detection wavelength [m]
;   width      : par.width,            $ ; Wavelength band width [m]
;   time_integ : int_time,             $ ; time is zero
;   time_delay : 0.,                   $ ; [NOT USED WITHIN AIRY]
;   psf        : par.psf,              $ ; IT'S A PSF!
;   background : background,           $ ; the PSF's background is zero!
;   snr        : fltarr(360),          $ ; [NOT YET EVALUATED]
;   header     : PTR_NEW('')           $ ; header of the output
;   }
;*out_img_t.header = header ; update the pointer
;
; OUTPUT IMAGE = PSF EXTRACTED
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
; PEX_APPEND, PEX_REMOVE, PEX_CIRCLE (called by PEX_GUI, see PEX_GUI_LIB dir)
; PEX_PSF_EXTRAPOLATION (called by PEX_INIT, see PEX_LIB dir)
; MPFIT2DFUN, MPFIT, MPFIT2DPEAK (called by PSF_EXTRAPOLATION - 
;     routines written by Craig B. Markwardt, see each file *.pro 
;     in PEX_LIB directory for details)
; PEX_CIRCULAR_MASK (called by PEX_PSF_EXTRAPOLATION, see PEX_LIB directory)
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  : 
;
; MODULE MODIFICATION HISTORY:
;    module written : Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  
;       : January 2007,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]:	
;        -PEX module is projected to be used in case of Space-Variant PSF 
;         extraction, by splitting the image into severals domains and 
;         extracting the PSF from each domain. 
;         This operation is NOT IMPLEMENTED YET, however data structures 
;         (esp. xsave, ysave and nb_click arrays) include a "domain" 
;         subdivision of the image.
;         (In case of Space-Invariant PSF, the image coincides with the 
;         unique domain).
;         PLEASE DON'T DELETE ANY COMMENTED (three ";") LINE.
;       : October 2008,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;        -Added the possibility to extract the PSF when project runs, LPX_GUI
;         now is a sort of "before-gui". A new procedure called LPX_OPEN_GUI has
;         been created for managing the Graphical User Interface.        
;       : February 2011, 
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]:	
;        -routine debugged. Domains deleted (not really used) 
;       : December 2011,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;        -file "find_dim.pro" has been deleted [not really used]
;        -files "circular_mask.pro", "psf_extrapolation.pro", "append.pro",
;         "remove.pro", and "circle.pro" have been renamed by adding the prefix
;         'pex_' to each filename. Apt changes to the functions/procedures.
;       : February 2012,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;        -New way to call AIRY_HELP. By using the "online_help" 
;         routine, we resolved a known-issue of the Soft.Pack.
;       : May 2012,
;         Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;        -Implemented an automatic search procedure for beta (exponent of the 
;         Moffat function).
;        -Visualization options changed: scale, zoom and color (*NEW*) can be
;         now set up by right-clicking on the draw areas. 
;        -Circular mask is computed more efficiently (pex_circular_mask.pro 
;         improvements).
;       : January 2014,
;         Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;        -added the possibility for choosing whether the
;         PSF is extrapolated with a Moffat function (already existing)
;         of not (only PSF extraction is performed - just added)
;       : May 2016,
;         Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;        -The header of the input/output image is kept
;         up-to-date with extrapolation information
;        -the background is now an array. 
;        -keywords added to the header: SKY_TYPE, SKY_NB, SKY_BGn,
;         SKY_FILE, BIAS_MAP, DARK_MAP, FLAT_MAP, BADP_MAP
;       : May 2017,
;         Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;        -Routine MPFIT2DFUN, MPFIT, MPFIT2DPEAK have been removed 
;         from PEX_LIB and replaced with the whole 
;         package MPFIT by Craig B. Markwardt. 
;         See installation instructions for more details.
;-
;
function pex, inp_img_t, out_img_t, par

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialisation
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = pex_init(inp_img_t, out_img_t, par)
endif

; back to calling program.
return, error
end
