; $Id: pex_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pex_gen_default
;
; PURPOSE:
;    pex_gen_default generates the default parameter structure for the PEX
;    module and save it in the right location.
;    (see pex.pro's header --or file airy_help.html-- for details about the
;    module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    pex_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  : January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:	
;                    -PEX module is projected to be used in case of Space-Variant 
;                     PSF extraction, by splitting the image into several 
;                     domains and extracting PSF from each domain. This 
;                     operation is not yet implemented, however data structures
;                     (esp. xsave, ysave and nb_click arrays) include a "domain"
;                     subdivision of the image. (In case of Space-Invariant PSF 
;                     the image coincides with the unique domain).
;                     PLEASE DON'T DELETE ANY COMMENTED (three ";") LINE.
;                   : February 2011, 
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:	
;                    -routine debugged. Domains deleted (not really used) 
;                   : May 2012, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -added some parameters for the automatic search procedure
;                     of the exponent of the Moffat function (beta) 
;                   : January 2014,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -added the possibility for choosing whether the
;                     PSF is extrapolated with a Moffat function
;                     (already existing) of not (only PSF extraction is
;                     performed - just added)
;                   : December 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -changed visualization options (map^power added; gamma, sqrt, 
;                     and square removed). The routine GMASCL.PRO is NO more NEEDED.
;-
;
pro pex_gen_default

; obtain module info
info = pex_info()

; generate module description structure
module = gen_def_module(info.mod_name, 0)

save_default=make_array(50,360,value=-1.) 
;50 is the max number of selectable stars; if you change this value, please
;check also "pex_append.pro" file within "pex_gui_lib" folder.
click_default=make_array(360,value=0)
int_time= make_array(360,2)
int_time[*,1]=1.

back_val  = make_array(360)

; parameter structure
par =                         $
   {                          $
   pex,	                      $ ; structure named pex
   mode:0,                    $ ; extraction procedure mode: 
                                ; 0=now, 1=when project runs
   name:'',                   $ ; filename (input image)
   resolution:0.005,          $ ; pixel size [arcsec/pixel]
   lambda:2.2e-6,             $ ; filter center wavelength [meters]
   width:0.4e-6,              $ ; filter width [meters]
   psf:1B,                    $ ; psf choice (yes = 1B)
   scale:0,                   $ ; scale (0=linear,1=log,2=power)
   power:1.,                  $ ; default power = 1 (linear scale)
   color: 0,                  $ ; color table (by loadct)                             
   zoom:1.,                   $ ; zoom values: 25%, 50%, 100%, 200%, 
                                ; 400% of original size
   inumber:0,                 $ ; number of image [3rd index] when used 
                                ; for text, add +1
   find_beta: 0B,             $ ; Automatic procedure to search for beta?
   moffat_beta: 1.,           $ ; Extrapolation with a MOFFAT function with a given
                                ; power law index  (the default value correspond
                                ; to a Lorentzian function).
   moffat_beta1:0.2,          $ ; Initial value of the automatic procedure 
   moffat_beta2:10.,          $ ; Initial value of the automatic procedure
   moffat_tol:1e-3,           $ ; Tolerance value of the automatic procedure  
   mag:fltarr(360),           $ ; magnitude of the reference-star (1 per image)
   back_user:0B,              $ ; user-defined background enable?      
   back_variable: 0B,         $ ; Different background selection 
                                ; 0 = Constant value [default]
                                ; 1 = Variable array
   back_val: back_val,        $ ; background value [photons/pixel]
   back_file: '',             $ ; filename of the background array [must be 
                                ; a fits file] 			
   time_integ:int_time,       $ ; integration time (s)
   tel_surf:104.,             $ ; TELESCOPE SURFACE (m^2)
   efficiency: 0.3,           $ ; overall efficiency    
   radius:16,                 $ ; default radius of circle (selection psf)
   extrap:1B,                 $ ; extraction + extrapolation (1) or extraction only (0) ?
   label:'Load an image...',  $ ; first instruction!
   p_xsave:save_default,      $ ; positions of selected star(s) (PSF extraction)
   p_ysave:save_default,      $
   nb_click:click_default,    $ ; number of selected star(s)
   N_out: 256,                $ ; output size "x" (cols)
   firsttime : 'yes',         $ ; first time GUI open flag
   str_def_file: '',          $ ; default save file string 
   str_sav_file: '',          $ ; save file string  
   module: module             $ ; module description structure
   }

; save the default parameters structure in the file def_file
save, par, FILENAME=info.def_file

;back to calling program
end
