; $Id: pex_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pex_gui
;
; PURPOSE:
;    pex_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the PEX module.
;    a parameter file called pex_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    the file is stored in the project directory proj_name located
;    in the working directory.
;    (see pex.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = pex_gui(n_module, $
;                    proj_name )
;
; INPUTS:
;    n_module : number associated to the instance of the PEX module
;               [integer scalar -- n_module > 0].
;    proj_name: name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;   pex_image - with: image_read (original image), N,M,P (dimensions), 
;                     fits header and domain array of structure:
;                     dom = {  domain_info, value:' ',  $
;                              xi:0., xf:0., yi:0., yf:0., $
;                              xs:float(N)/float(state.par.dnx), $
;                              ys:float(N)/float(state.par.dny)}
;   wavelenghts - with: band_tab, lambda_tab, width_tab, flag_band, f_band
;   centroids   - with: xsave, ysave and nb_click (arrays)
;
;
; CALLED NON-IDL FUNCTIONS:
;	APPEND3D, REMOVE3D, CIRCLE      ;see the PEX_GUI_LIB subdirectory
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  : January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:	
;                    -PEX module is projected to be used in case of 
;                     Space-Variant PSF extraction, by splitting the image 
;                     into severals domains and extracting PSF from each 
;                     domain. This operation is not implemented yet, however 
;                     data structures (esp. xsave, ysave and nb_click arrays) 
;                     include a "domain" subdivision of the image.
;                     (In case of Space-Invariant PSF the image coincides with
;                     the unique domain).
;                     PLEASE DON'T DELETE ANY COMMENTED (three ";") LINE.
;                   : October 2008,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Added the possibility to extract the PSF when project 
;                     runs, PEX_GUI now is a sort of "pre-gui". A new procedure 
;                     called PEX_OPEN_GUI has been created for managing the
;                     Graphical User Interface. See PEX.pro for details.
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -routine debugged. Domains deleted (not really used)
;                   : December 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -small changes in the text of the GUI
;                    -file "circle.pro" has been renamed by adding the prefix
;                     'pex_' to the filename. Same change to the procedure
;                     (pex_open_gui.pro, l.204 and pex_circle.pro, l.1)
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2012, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]: 
;                    -implemented an automatic search procedure for beta (Moffat
;                     power law index). 
;                    -Visualization options changed: scale, zoom and color 
;                     (*NEW*) can be set up by right-clicking on the draw areas. 
;                   : January 2014,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -added the possibility for choosing whether the
;                     PSF is extrapolated with a Moffat function (already existing)
;                     of not (only PSF extraction is performed - just added)
;-
;
; ------------------------------------------------------------------
; PEX GUI - generation code
; ------------------------------------------------------------------
function pex_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
COMMON error_block, error

; retrieve the module information
info = pex_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored: display input image with star selected
; otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   firsttime='yes'
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file+$
       ') is from another module: please take the right one'
endif else begin
   firsttime='no'
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file+    $
               ' is from another module: please generate a new one'
endelse

par.firsttime = firsttime
par.str_sav_file = sav_file
par.str_def_file = def_file

;Open_gui? FORM
dummy = [ $
   '0, LABEL, PEX_GUI initial settings, CENTER', $
   '1, BASE,,COLUMN,FRAME', $
     '1, BASE,, COLUMN,', $
       '0, BUTTON, loaded within the GUI (no input for PEX in this case)|'+$
                  'loaded by a RFT module and linked to the PEX''s input,'+$
                  'EXCLUSIVE, '+$
           'LABEL_TOP= PSF extraction performed from an image that will be:, '+$
           'COLUMN,TAG=select, '+$
           'SET_VALUE='+strtrim(par.mode,1), $
       '0, LABEL,N.B. In the first case\, the GUI will appear after, LEFT', $
       '0, LABEL,having closed this window\, in the latter case\,  , LEFT', $    
       '2, LABEL,the GUI will appear when the project will run.    , LEFT', $    
   '1, BASE,,ROW, FRAME, RIGHT', $
   '0, BUTTON, Cancel, QUIT, TAG=cancel', $
   '2, BUTTON, Continue, QUIT, TAG=continue']

a=CW_FORM(dummy, title='PEX_GUI', /COLUMN)
if a.cancel then return, !CAOS_ERROR.cancel
if a.continue then begin 
   par.mode = a.select
   if a.select EQ 0 then begin
      ;YES! Open GUI and select image, stars, etc...
      out=pex_open_gui(par)
      return, out.(0)
   endif else begin
      ;NO! Save parameters and exit!
      ; check before saving the parameter file if filename already exists
      check_file = findfile(sav_file)
      if check_file[0] ne '' then begin
         answ = dialog_message(['file '+sav_file+' already exists.', $
               'do you want to overwrite it ?'], TITLE='PEX warning', /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
        if strlowcase(answ) eq "no" then message, 'NOTHING TO DO!', /info 
      endif else begin
         answ = dialog_message(['file '+sav_file+' will be saved.'], $
                 TITLE='PEX information', /INFO)
         ;inform were the parameters will be saved
      endelse
      par.firsttime='no'
      
      CASE !VERSION.OS_FAMILY OF
         'Windows'   :     begin
                    spawn, 'del '+sav_file, /NOSHELL
                   end
         else    :  begin
                     spawn, 'rm '+sav_file
                   end
      ENDCASE
      
      save, par, FILENAME = sav_file
      return, !caos_error.ok
 endelse
endif

end
