; $Id: pex_init.pro v.7.2 last revision 2017/12/02 Andrea La Camera $ 
;+
; NAME:
;    pex_init
;
; PURPOSE:
;    pex_init executes the initialization for PEX module, that is:
;       0- to check the formal validity of the input/output structure
;       1- to extract PSF (with Moffat extrapolation)
;       2- to initialize the output structure(s) out_img_t
;
;    (see pex.pro's header --or file AIRY_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's initialization routine
;
; CALLING SEQUENCE:
;    error = pex_init(out_img_t, par) 
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;    see pex.pro's header for a detailed description.
;
; CALLED NON-IDL FUNCTIONS:
;    PEX_PSF_EXTRAPOLATION (see PEX_LIB directory)
;    MPFIT2DFUN, MPFIT, MPFIT2DPEAK (called by PSF_EXTRAPOLATION - 
;       routines written by Craig B. Markwardt, see each file *.pro 
;       in PEX_LIB directory for details)
;    PEX_CIRCULAR_MASK (called by PEX_PSF_EXTRAPOLATION, see PEX_LIB directory)
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  : January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;    -PEX module is projected to be used in case of Space-Variant PSF 
;     extraction, by splitting the image into severals domains and extracting 
;     PSF from each domain. This operation is not implemented yet, however 
;     data structures (esp. xsave, ysave and nb_click arrays) include a 
;     "domain" subdivision of the image. (In case of Space-Invariant PSF the 
;     image coincides with the unique domain).
;     PLEASE DON'T DELETE ANY COMMENTED (three ";") LINE.
;                   : February 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -routine debugged. Domains deleted (not really used)
;                   : December 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -file "find_dim.pro" has been deleted [not really used]
;                    -files "circular_mask.pro", "psf_extrapolation.pro", "append.pro",
;                     "remove.pro", and "circle.pro" have been renamed by adding the prefix
;                     'pex_' to each filename. Apt changes to the functions/procedures.
;                   : January 2014,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -added the possibility for choosing whether the
;                     PSF is extrapolated with a Moffat function (already existing)
;                     of not (only PSF extraction is performed - just added)
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -The header of the input/output image is kept
;                     up-to-date with extrapolation information
;                    -The background is now an array (modifications done
;                     also within pex_psf_extrapolation*.pro files) 
;                   : November 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -debugged errors on background and SNR definitions (output structure)
;-
;

function pex_init, inp_img_t, out_img_t, par

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = pex_info()

; get the individual output structure type
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

; STANDARD CHECKS
;
; compute and test the requested number of pex arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n_el) then $
   message, 'PEX error: par must be a structure'
if n_el ne 1 then message, 'PEX error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module PEX'


;check if the input is 4D
if par.mode then begin
   pos=where(inp_img_t.time_integ[*,1] gt 1,num)
   if ((size(inp_img_t.image))[0] eq 4) then begin
     if (num eq 0) then begin 
        inp_img_t.image=reform(inp_img_t.image,(size(inp_img_t.image))[1],$
                                                (size(inp_img_t.image))[2],$
                                                (size(inp_img_t.image))[3]) 
     endif else begin
        dummy=dialog_message('WRONG INPUT TYPE (MULTI-FRAME input is not'+$
                             ' supported in this module)',/ERROR)
        message,'WRONG INPUT TYPE (MULTI-FRAME input is not supported in'+$
                ' this module)'
     endelse
   endif
endif


; PSF EXTRACT

; INIZIALIZATION
; load image
if par.mode EQ 0 then begin
   image_read = readfits(par.name, header)
endif else begin 
   image_read = inp_img_t.image
   header = *inp_img_t.header
endelse

;GUI was open? If not, it will be open now.
if par.mode then begin
;search for a previous background definition through keyword SKY_TYPE 
   a=fxpar(header, 'SKY_TYPE', count=count)
   if (count GT 0) then begin
      par.back_user = 1 
      if a EQ 'FILE    ' then begin
         par.back_variable = 1
         par.back_file = fxpar(header, 'SKY_FILE')
      endif else begin
         par.back_variable = 0
         sky_nb =  fxpar(header, 'SKY_NB')
         for k=0,sky_nb-1 do begin
            par.back_val[k]=fxpar(header, 'SKY_BG'+strtrim(k,1))
         endfor
      endelse
   endif else par.back_user = 0 ;if not already set --> call MMM procedure
   par.time_integ=inp_img_t.time_integ
   par.lambda=inp_img_t.lambda
   par.width=inp_img_t.width
   out = pex_open_gui(par, image=inp_img_t.image)
   error_gui = out.(0)
   restore, par.str_sav_file
   if error_gui NE !caos_error.ok then message,  $
       'ERROR: Nothing to do!'   
endif

_N=(size(image_read))[1]
_M=(size(image_read))[2]
if (size(image_read))[0] EQ 3 then _P=(size(image_read))[3]
; # of images into the cube
if (size(image_read))[0] EQ 2 then _P=1	   	  				; SINGLE IMAGE!

_N_out = par.N_out
_M_out = par.N_out

message, 'Please, wait...', /INFO

;-----------BACKGROUND EVALUATION (used in the extrapolation procedure)------------
back = fltarr(_N,_M,_P)
; AUTO --- mmm
if (par.back_user EQ 0B) then begin
   fxaddpar, header, 'SKY_TYPE', 'VALUE', '[PEX] Sky background (evaluated by SKY)' 
   fxaddpar, header, 'SKY_NB', _P, '[PEX] Sky background number of values' 
   for j=0, _P-1 do begin 
      print, 'SKY background evaluation (image '+strtrim(j,1)+') :'
      dummy = image_read[*,*,j]
      dummy = dummy[where(dummy GT 0.)]
      sky, dummy, sky_mode, sky_sig, /silent ; Sky background evaluation using the Astrolib procedure SKY.
                                ; test the validity of the sky value (non <0)
      if sky_mode lt 0. then begin                                            
         print, 'Sky evaluation error (sky < 0)  ==>  sky = 0' 
         sky_mode = 0.
      endif else begin
         print, 'Sky background value =', sky_mode
         print, 'Standard deviation of sky background =', sky_sig
      endelse
      back[*,*,j] = sky_mode
      fxaddpar, header, 'SKY_BG'+strtrim(j,1), sky_mode, '[PEX] Sky background value' 
   endfor     
endif else begin
; USER DEFINED ---
; -- constant value:       
   if (par.back_variable EQ 0B) then begin
      fxaddpar, header, 'SKY_TYPE', 'USER', '[PEX] Sky background (user-defined)' 
      fxaddpar, header, 'SKY_NB', _P, '[PEX] Sky background number of values' 
      for i=0, _P-1 do begin
         back[*,*,i] = par.back_val[i]
         fxaddpar, header, 'SKY_BG'+strtrim(i,1), par.back_val[i], '[PRE] Sky background value' 
      endfor 
   endif else begin
; -- variable array: 
      fxaddpar, header, 'SKY_TYPE', 'FILE', '[PEX] Sky background (user-defined FITS file)' 
      fits_read, par.back_file, back_map
      if (size(back_map))[1] NE _N OR (size(back_map))[2] NE _M then message, $
         "The loaded BACKGROUND array has not the same dimensions of the input image."
      if (size(back_map))[0] EQ 3 AND (size(back_map))[3] NE _P then message, $
         "Wrong number of images within the cube."
      back=back_map
      fxaddpar, header, 'SKY_FILE', par.back_file, '[PEX] Sky background filename'       
   endelse     
endelse 

;---------------------------------START EXTRACTION-------------------
image=make_array(_N_out, _M_out, _P) ; final image
image1=make_array(_N_out, _M_out) ; final image
for j=0,_P-1 do begin
    IF _P GT 1 then img=image_read[*,*,j] ;select current image (CUBE case)
    IF _P EQ 1 then img=image_read[*,*] ;select current image (SINGLE case)
        ;number of star selected for this domain/image
        nb=par.nb_click[j]
        if nb LE 0 then message, 'ERROR! Image # '+strtrim(j+1,1)+$
                                'has no star(s) selected.'
        ;working images
        img_cube=make_array(_N_out,_M_out,nb, /double)
        for i=0,nb-1 do begin
            ;each star is centered with sub-pixel precision (FFTSHIFT)
            dummy=fftshift(img, _N/2.-par.p_xsave[i,j], $
                                _M/2.-par.p_ysave[i,j])
            ;extraction of the 2r x 2r region
            img_tmp=dummy[_N/2.-par.radius:_N/2.+par.radius-1,$
                        _M/2.-par.radius:_M/2.+par.radius-1]
                        
            ;extraction / extrapolation with psf_extrapolation function
            if (par.extrap) then begin
               if par.find_beta EQ 0 then begin
                  pex_psf_extrapolation,img_tmp,_N_out,_M_out,par.extrap,par,dummy2,f_beta,j, back[*,*,j]
               endif else begin
                  pex_psf_extrapolation_find_beta,img_tmp,_N_out,_M_out,par.extrap,par,dummy2,f_beta,j, back[*,*,j]
               endelse                              
               print, '   Extrapolation r.m.s. error (angle '+$
                      strtrim(j,1)+'): ', abs(f_beta), $
                      ' (beta = '+strtrim(par.moffat_beta,1)+')'
            endif else pex_psf_extrapolation,img_tmp,_N_out,_M_out,par.extrap,par,dummy2,f_beta,j, back[*,*,j]
            
            img_cube[0,0,i] = dummy2
            
                                ;normalization 
            img_cube[0,0,i] = img_cube[*,*,i] / total(img_cube[*,*,i],/double)
         endfor
                                ;median image if necessary
        if nb GT 1 then image1=median(img_cube, dim=3, /EVEN) $
        else image1=reform(img_cube,_N_out,_M_out)
        image1=positivization(image1,/zero)
                                ;normalize output image
        if _P EQ 1 then image=image1 / total(image1,/double) $
        else image[0,0,j]= image1 / total(image1,/double)
endfor; cicle on images (j)
message, '...PSF EXTRACTED!', /INFO
;-------------------------------END EXTRACTION!----------------------

;updating the header 
FXADDPAR, header, 'PSF', 1, '[PEX] PSF array'
if par.extrap then begin
   FXADDPAR, header, 'PSF_EXTR', par.extrap, '[PEX] Extrapolated'
   FXADDPAR, header, 'MOFFAT', par.moffat_beta, '[PEX] Moffat parameter'
endif else begin
   FXADDPAR, header, 'PSF_EXTR', par.extrap, '[PEX] Extracted'
endelse



; INITIALIZE THE OUTPUT STRUCTURE(S)
;even if alreay defined (e.g. in the image), 
;the background of a PSF must be ZERO!
;PSFs do not have background!!!
; same for SNR
background = fltarr(_N_out, _M_out, _P) 
snr = fltarr(360)

;time_integ input (time integ = 0 sec):
int_time=make_array(360,2)
int_time[*,1]=1.

; initializing output structure
out_img_t =                           $
   {                                  $
   data_type  : out_type[0],          $
   data_status: !caos_data.valid,     $
   image      : image,                $ ; image/psf data cube
   npixel     : _N_out,               $ ; number of pixels
   resolution : par.resolution,       $ ; Pixel size [arcsec/px]
   lambda     : par.lambda,           $ ; Mean detection wavelength [m]
   width      : par.width,            $ ; Wavelength band width [m]
   time_integ : int_time,             $ ; time is zero
   time_delay : 0.,                   $ ; [NOT USED WITHIN AIRY]
   psf        : par.psf,              $ ; IT'S A PSF!
   background : background,           $ ; the PSF's background is zero!
   snr        : fltarr(360),          $ ; [NOT YET EVALUATED]
   header     : PTR_NEW('')           $ ; header of the output
   }

*out_img_t.header = header ; update the pointer

; back to calling program
return, error
end
