; $Id: pex_psf_extrapolation.pro, 2017/05/16 andrea la camera$
;+
; NAME:
;   pex_psf_extrapolation
;
; PURPOSE:
;   This routine performs PSF extrapolation as described into PEX.pro file 
;   header. THIS ROUTINE WORKS TOGETHER WITH PEX MODULE!
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   pex_psf_extrapolation, psf,N,M, choice, par, image_out, f_beta,j
;
; INPUTS:
;   psf - PSF
;   N,M - dimensions of full image
;   choice - insert "0" if you want to EXTRACT the PSF 
;            insert "1" for EXTRAPOLATION.
;   par - the whole parameter structure
;   j   - the number of image (cube) now processing 
;
; OPTIONAL INPUTS:
;   none.
;
; KEYWORD PARAMETERS:
;   none.
;
; OUTPUT:
;   psf extrapolated and the value of f_beta (defined by 1-F2/F1, see under)
;
; OPTIONAL OUTPUTS:
;   none.
;
; COMMON BLOCKS:
;   none.
;
; SIDE EFFECTS:
;   none.
;
; RESTRICTIONS:
;   none.
;
; CALLED NON-IDL FUNCTIONS:
;   MPFIT2DFUN, MPFIT, MPFIT2DPEAK (routines written by Craig B. Markwardt, 
;                     see installation instructions for details)
;   PEX_CIRCULAR_MASK
;
; EXAMPLE:
;   none.
;
; WRITTEN BY:
;   Andrea La Camera, January 2007
;   [lacamera@disi.unige.it]
;
; MODIFICATION HISTORY:
;         : February 2011,
;           Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;          -routine debugged. 
;         : December 2011,
;           Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;          -routines renamed from "psf_extrapolation" to "pex_psf_extrapolation"
;           and from "circular_mask.pro" to "pex_circular_mask.pro".
;         : May 2012,
;           Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;          -this is a routine now. I modified input/output for the 
;           automatic search procedure for beta parameter (see 
;           the file pex_psf_extrapolation_find_beta.pro).
;          -Computation within 'Moffat function' has 
;           been improved thanks to Patrizia Boccacci.
;         : January 2014,
;           Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;          -added the possibility for choosing whether the
;           PSF is extrapolated with a Moffat function (already existing)
;           of not (only PSF extraction is performed - just added)
;-
;

;###################################################################
;###################################################################
;###################################################################

function plane_moffat, x,y, par

common mycommon, moffat_beta

N=(size(x))[1]
theta = par[6]
xc=x-(N/2.);-0.5
yc=y-(N/2.);-0.5
wx = abs(par[4]) > 1e-5
wy = abs(par[5]) > 1e-5
u =((xc*cos(theta)-yc*sin(theta))/wx)^2. + $
   ((xc*sin(theta)+yc*cos(theta))/wy)^2.
lor = par[0] + par[1]*x + par[2]*y + par[3] / (u+1.)^moffat_beta
return, lor
end


;###################################################################
;###################################################################
;###################################################################

function moffat, x,y,par,R

common mycommon, moffat_beta

N=(size(x))[1]
lor=make_array(N,N,/double)
theta = par[6]
a0=max([abs(par[4]),abs(par[5])]) > 1e-5
b0=min([abs(par[4]),abs(par[5])]) > 1e-5
b_save=make_array(ceil(N*sqrt(2)/2.))
alpha=a0/b0
;for xx=0,N-1 do begin
;   for yy=0,N-1 do begin
;      xc=xx-(N/2.)
;      yc=yy-(N/2.)
;      rr=sqrt(xc^2.+yc^2.)
;      if rr LT R then begin
;         b = b0
;         b_save[rr]=b
;      endif else begin
;         b = (a0-b0)*(1-exp(-(rr-R)/(alpha*a0)))+b0
;         b_save[rr]=b
;      endelse
;      u = ((xc*cos(theta)-yc*sin(theta))/a0)^2. + $
;          ((xc*sin(theta)+yc*cos(theta))/b) ^2.
;      lor[xx,yy] = par[3] / (u+1.)^moffat_beta
;   endfor
;endfor
;Contribution of P.Boccacci -- start 
xy=dindgen(N)
xc=xy-N/2 ;non erano prefettamente centrati
xx=rebin(reform(xc, [N,1,1]),[N,N,1])
yy=transpose(rebin(reform(xc, [N,1,1]),[N,N,1]))
rr=sqrt(xx^2.+yy^2.)
b_save = (a0-b0)*(1-exp(-(rr-R)/(alpha*a0)))+b0
indexes=where(rr LT R)
b_save[indexes]=b0
u = ((xx*cos(theta)-yy*sin(theta))/a0)^2. + $
    ((xx*sin(theta)+yy*cos(theta))/b_save)^2.
lor = par[0] + par[3] / (u+1.)^moffat_beta
;Contribution of P.Boccacci -- end
return, lor	
end

;###################################################################
;###################################################################
;###################################################################

function plane, x,y, par
N=(size(x))[1]
return, par[0] + par[1]*x + par[2]*y
end

;###################################################################
;###################################################################
;###################################################################

pro pex_psf_extrapolation, psf,N,M, choice, par, image_out, f_beta,j, background

common mycommon, moffat_beta
moffat_beta=par.moffat_beta

;input = psf [small size: default 32x32 pixels]
;3 steps:
; i) best-fit: plane, circular Moffat, elliptical Moffat
; ii) subtraction
; iii) merging
; + positivization

;dimensions and arrays declaration
dim=(size(psf))[1]
x=(dindgen(dim)) # (dblarr(dim)+1)
y=(dblarr(dim)+1) # (dindgen(dim))
xx=(dindgen(N)) # (dblarr(M)+1)
yy=(dblarr(N)+1) # (dindgen(M))

psf_tmp = make_array(N,M,value=0.)

;i) best-fit
;-- plane
param_a=replicate({value:0.D, fixed:0, limited:[0,0], limits:[0.,0.], tied:''}, 3)
param_a[0].value=median(background)
param_a[0].fixed=1
par_a=mpfit2dfun('plane',x,y,psf,sqrt(psf),param_a.value,parinfo=param_a,/quiet)
psf2 = psf - plane(x,y,par_a) + par_a[0]

;-- Moffat circular peak
yfit = MPFIT2DPEAK(psf2, par_b, /circular, /MOFFAT)

;--Moffat elliptical function
param_c = replicate({value:1.D, fixed:0, limited:[0,0], limits:[0.,0.], tied:''}, 8)
param_c[4:5].limited[0]=1 ;p(4) and p(5) must be positive!
param_c[4:5].limits[0]=1e-2
param_c[0].value = median(background)
param_c[0].fixed = 1
param_c[1].value = par_a[1]
param_c[2].value = par_a[2]
param_c[3].value = par_b[1]
param_c[4].value = par_b[2]
param_c[5].value = par_b[3]
param_c[6].value = 0.
param_c[7].value = 1.
par_c=mpfit2dfun('plane_moffat', x,y,psf,sqrt(psf),param_c.value,$
                  parinfo=param_c,/quiet)
par_c[6] = par_c[6] mod (2.*!DPI)



;ii) subtraction
psf_tmp[N/2-dim/2,M/2-dim/2]=psf2 ;plane is already subtracted...

;iii) merging with "morphing moffatian" function
if (choice) then begin
   image=moffat(xx,yy,par_c,dim/2.)
   image_out = (image * pex_circular_mask(N,dim/2,'out')) + $
               (psf_tmp * pex_circular_mask(N,dim/2,'in'))
   image_out =image_out-par_c[0]            
endif else image_out = psf_tmp ;* pex_circular_mask(N,dim/2,'in')

;positivization
image_out=positivization(image_out,/zero)

fact = total(image_out,/DOUBLE)
image_out = image_out / fact

flux = (n_phot(par.mag[j],lambda=par.lambda, width=par.width, $
               surf=par.tel_surf,delta_t=par.time_integ[j,0]))[0]*par.efficiency
F1 = total((psf-background)*pex_circular_mask(dim,dim/2,'in'),/DOUBLE)
F2 = total(image_out*pex_circular_mask(N,dim/2,'in'), /DOUBLE)*flux
f_beta=1.-(F2/F1)
end

