; $Id: pre.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pre
;
; ROUTINE'S PURPOSE:
;    pre manages the simulation for the pre-processing (pre) module,
;    that is to call the module's initialization routine pre_init at the first
;         iteration of the simulation project.
;
; MODULE'S PURPOSE:
;    pre permits to pre-process a cube of images before a subsequent image
;    restoration/deconvolution process. More precisely, the module permits:
;    - data reduction (also called data calibration),
;    - shift-and-add procedure for multi-frame images,
;    - positivization of the calibrated images,
;    - sky background evaluation,
;    - SNR evaluation,
;    - images re-centering.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = pre(inp_img_t, out_img_t, par, INIT=init ) 
;
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_img_t: structure of type img_t.
;    par      : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_img_t: structure of type "img_t" containing the characteristics of 
;               the pre-processed map, the following fields are contained in
;               this structure:
;
;out_img_t = $
;   {        $
;   data_type  : out_type[0]          , $
;   data_status: !caos_data.valid     , $
;   image      : pre_map              , $ ; image map
;   npixel     : inp_img_t.npixel     , $ ; number of pixels
;   resolution : inp_img_t.resolution , $ ; pixel size [arcsec/pix]
;   lambda     : inp_img_t.lambda     , $ ; mean detection wavelength [m]
;   width      : inp_img_t.width      , $ ; wavelength band width [m]
;   time_integ : inp_img_t.time_integ , $ ; Integration time for each image [s]
;   time_delay : inp_img_t.time_delay , $ ; [NOT USED WITHIN AIRY]
;   psf        : inp_img_t.psf        , $ ; image or PSF ?
;   background : back                 , $ ; estimated sky background level of the
;                                         ; pre-processed maps
;   snr        : snr                  , $ ; estimated peak SNR of the
;                                         ; pre-processed maps
;   header     : PTR_NEW('')       $ ; header of the FITS File (modified event.)
;   }
;*out_img_t.header = header ; update the pointer
;
; KEYWORD PARAMETERS:
;    INIT: initialization data structure.
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;     1-In some cases, namely when the object is a double star of faint
;     magnitude the MMM algorithm can return an incorrect value (0) for
;     the background evaluation.
;     For this kind of cases it is recommended to use an alternative method
;     for background evaluation.
;
;     2- The background estimation done by PRE module can be contaminated
;     by the object if the extension
;     of the object is a large fraction of the image frame. The estimated mean value
;     of the sky background and especially its rms value are then biased.
;     A simple solution is to compute directly the mean background using
;     the routine n_phot (found in .../caos/lib/, help at the IDL/CAOS prompt with
;     the command <<doc_library, "n_phot">>). For example, with a 0.01 arcsec pixel
;     size, a telescope collecting surface of 10 m^2, an integration time of 100 s
;     and an overall efficiency of 0.3, the K band mean background (per pixel) is:
;     
;        0.3*(n_phot(0.,BAND="K",DELTA_T=100.,SURF=10.))[1]*0.01^2
;     
;     Then it is necessary to enter by hand the value found within the PRE module GUI
;     (using the option "consider a user-defined background").
;
; CALLED NON-IDL FUNCTIONS:
;    phot_rec.pro : re-center the map wrt the photocenter location
;    sky_back_sub : estimate the sky background level of a map 
;                   using the astrolib routine SKY. 
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                    -call to routine pre_prog eliminated (useless).
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminated for version 4.0
;                     of the whole system CAOS.
;
; MODULE MODIFICATION HISTORY:
;    module written : Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : for version 1.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -help completed.
;                    -call to pre_prog eliminated (useless).
;                   : for version 2.0,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -no more use of the common variable "calibration" and
;                     the tag "calib" (structure "info") for version 4.0 of
;                     the whole Software System CAOS.
;		              : for version 2.0 too,
;   		             Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;	                  -added the possility for the user to subtract sky
;		                background from the images.
;		              : for version 4.0,
;                     Gabriele Desidera' (DISI) [desider@disi.unige.it]:
;                    -use of the variable "time_integ" activated.
;                   : for version 5.0,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -added possibility to simulate multi-frame images.
;                     Multi-frames images cannot be saved (by WFT), but only 
;                     auto-reduced by PRE module (this module),
;                    -Added DATA REDUCTION for raw images (BAD-PIXEL MAP,
;                     FLAT-FIELD MAP, DARK CURRENT MAP and BIAS MAP),
;                    -Added SHIFT-AND-ADD method. The values of the shift can 
;                     be either stored in a text file (x_shift, y_shift in 
;                     every line) or calculated by correlation (with integer or 
;                     sub-pixel accuracy),
;                    -Apodization has been removed (obsolete),
;                    -Changed Re-centering procedure, the new one is based on 
;                     correlation among the images (with integer or sub-pixel
;                     accuracy),
;                    -New GUI's design with "TABs".
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : March 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -routine "find_boundary" renamed to "pre_find_boundary"
;                     for compatibility with other libraries (such as the 
;                     well-known "Coyote" one);
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. The user can now
;                     load the background contained in a FITS file
;                     (that must have the correct dimensions). 
;                    -deleted procedure "sky_back_sub.pro" (useless!)
;                     and added the necessary steps directly in pre_init.
;                    -Moved here the definitions of the 
;                     Read-Out Noise value and the Gain of the CCD, by 
;                     default they are read from the header of the
;                     FITS file (be sure they are defined by "RON" and
;                     "GAIN" keywords)
;                    -keywords added to the header: RON, GAIN,
;                     SKY_TYPE, SKY_NB, SKY_BGn, SKY_FILE, 
;                     BIAS_MAP, DARK_MAP, FLAT_MAP, BADP_MAP 
;                    -bug corrections in the data-reduction procedure
;                   : December 2016-January 2017,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@oca.eu]:
;                    -Re-centering can now be done also for single images.
;                    -Back to correl_optimize from astrolib (instead of
;                     correl_optimize2 from CAOS lib., now eliminated), thanks
;                     to new keyword INIT_FACTOR, here chosen =4 instead of
;                     default 8 (which fails for images as small as 64x64).
;-
;
function pre, inp_img_t, out_img_t, par, INIT=init

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialisation
error = !caos_error.ok

; module's action
if (this_iter eq 0) then $
   error = pre_init(inp_img_t, out_img_t, par, INIT=init)

return, error
end
