; $Id: pre_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pre_gui
;
; PURPOSE:
;    pre_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the pre-processing (pre) module.
;    a parameter file called pre_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    the file is stored in the project directory proj_name located
;    in the working directory.
;    (see pre.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = pre_gui(n_module, $
;                    proj_name )
;
; INPUTS:
;    n_module : number associated to the instance of the pre module
;               [integer scalar -- n_module > 0].
;    proj_name: name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    none
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -GUI completed with a message saying what the module
;                     is doing exactly.
;                   : ???,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -restucturation of the GUI with parameters.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(pre_info()).help stuff added (instead of !caos_env.help).
;                   : october 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;                    -insert possility for the user to subtract sky
;                     background from the image.
;                   : november 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -sky background evaluation AND sky background subtraction
;                     are clearly separated.
;                    -choice on sky background subtraction now a real parameter
;                     (was a priori set to 'no').
;                    -global GUI reordering for better fitting in screens !
;                    -some other minor changes...
;                   : september 2005
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;                    -added help for windows version
;                   : may 2007, 
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -simplest way to call AIRY_HELP 
;                   : December 2009,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Added DATA REDUCTION for raw images (BAD-PIXEL MAP, 
;                     FLAT-FIELD MAP and BIAS MAP)
;                    -Added SHIFT-AND-ADD method. The values of the shift can be
;                     either stored in a text file (x_shift, y_shift in every 
;                     line) or calculated by correlation (with integer or sub-pixel 
;                     accuracy)
;                    -Apodization has been removed (obsolete)
;                    -Changed Re-centering procedure, the new one is based on 
;                     correlation among the images (with integer or sub-pixel 
;                     accuracy)
;                    -New GUI's design with "TABs"
;                   : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -DARK MAP added (data reduction procedure completed)
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                    -routine debugged.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. The user can now
;                     load the background contained in a FITS file
;                     (that must have the correct dimensions). 
;                    -BACK_ENABLE -> BACK_SUB (Better name!)
;                    -new panel with "CCD INFO" --> definition of the
;                     Read-Out Noise value and the Gain of the CCD, by 
;                     default they are read from the header of the
;                     FITS file (be sure they are defined by "RON" and
;                     "GAIN" keywords)
;                    -small changes in the GUI (text corrections)
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting provedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro pre_gui_set, state

if (state.par.buser_enable eq 0B) then begin
     ;widget_control, state.id.back_group, SET_VALUE=0
     widget_control, state.id.val_back, SENSITIVE=0
     widget_control, state.id.reset_val_back, SENSITIVE=0
     widget_control, state.id.val_back,SET_VALUE=['']
     widget_control, state.id.back_arr_sel, SENSITIVE=0
     widget_control, state.id.back_file, SENSITIVE=0 
endif else begin
     widget_control, state.id.back_arr_sel, /SENSITIVE
     if (state.par.back_variable EQ 0B) then begin
         ;widget_control, state.id.back_group, SET_VALUE=1
         widget_control, state.id.val_back, /SENSITIVE
         widget_control, state.id.reset_val_back, /SENSITIVE
         widget_control, state.id.val_back,SET_VALUE=transpose(state.par.back_val)
         widget_control, state.id.back_file, SENSITIVE=0
     endif else begin
         ;widget_control, state.id.back_group, SET_VALUE=0
         widget_control, state.id.val_back, SENSITIVE=0
         widget_control, state.id.reset_val_back, SENSITIVE=0
         widget_control, state.id.val_back,SET_VALUE=['']
         widget_control, state.id.back_file, /SENSITIVE
     endelse    
endelse

if state.par.saa_enable then begin
   widget_control, state.id.saa_method, /sensitive
   if state.par.saa_method EQ 1 then begin
      widget_control, state.id.saa_file, /sensitive
      widget_control, state.id.saa_auto, sensitive=0
   endif
   if state.par.saa_method EQ 2 then begin
      widget_control, state.id.saa_auto, /sensitive
      widget_control, state.id.saa_file, sensitive=0
   endif  
endif else begin
   widget_control, state.id.saa_method, sensitive=0
   widget_control, state.id.saa_file, sensitive=0
   widget_control, state.id.saa_auto, sensitive=0
endelse

if state.par.red_enable then begin
  widget_control, state.id.red_bp, /sensitive
  widget_control, state.id.red_flat, /sensitive
  widget_control, state.id.red_dark, /sensitive
  widget_control, state.id.red_bias, /sensitive
  if state.par.red_bp then widget_control, state.id.red_bp_file, /sensitive $
  else widget_control, state.id.red_bp_file, sensitive=0
  if state.par.red_flat then widget_control, state.id.red_flat_file, /sensitive $
  else widget_control, state.id.red_flat_file, sensitive=0
  if state.par.red_dark then widget_control, state.id.red_dark_file, /sensitive $
  else widget_control, state.id.red_dark_file, sensitive=0  
  if state.par.red_bias then widget_control, state.id.red_bias_file, /sensitive $
  else widget_control, state.id.red_bias_file, sensitive=0  
endif else begin
  widget_control, state.id.red_bp, sensitive=0
  widget_control, state.id.red_bp_file, sensitive=0
  widget_control, state.id.red_flat, sensitive=0
  widget_control, state.id.red_flat_file, sensitive=0
  widget_control, state.id.red_dark, sensitive=0
  widget_control, state.id.red_dark_file, sensitive=0
  widget_control, state.id.red_bias, sensitive=0
  widget_control, state.id.red_bias_file, sensitive=0
endelse

if state.par.pos_enable then begin
   widget_control, state.id.pos_method, /SENSITIVE
endif else begin
   widget_control, state.id.pos_method, SENSITIVE=0
endelse

if state.par.re_center_enable then begin
   widget_control, state.id.re_center_accuracy, /SENSITIVE
endif else begin
   widget_control, state.id.re_center_accuracy, SENSITIVE=0
endelse

if (state.par.header_ron EQ 1B) then begin
   widget_control, state.id.sigma_ron, SENSITIVE=0
   widget_control, state.id.gain_ron, SENSITIVE=0
endif else begin
   widget_control, state.id.sigma_ron, /SENSITIVE
   widget_control, state.id.gain_ron, /SENSITIVE
endelse

end

;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;
pro pre_gui_event, event

; pre_gui error management block
common error_block, error

; read the GUI state structure
widget_control, event.top, GET_UVALUE=state

; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

; handle all the other events
widget_control, event.id, GET_UVALUE=uvalue
case uvalue of

  'line_SNR': begin
      state.par.line_SNR = event.value
   end

  'header_ron': begin
     state.par.header_ron = event.value
     pre_gui_set, state
  end

  'sigma_ron' : begin
     state.par.sigma_ron=event.value
  end
  
  'gain_ron': begin
     state.par.gain_ron=event.value
  end

  'back_sel': begin
    dummy=event.value
    state.par.buser_enable=dummy
    pre_gui_set, state
    end

   'back_arr_sel': begin
      dummy= event.value
      ;widget_control, state.id.back_arr_sel, GET_VALUE=dummy
      if (dummy EQ 0) then begin
          state.par.back_variable = 0B
      endif else begin
          state.par.back_variable=1B
      endelse
      pre_gui_set,state
   end
   'back_val'  : begin
      dummyx = event.x
      dummyy = event.y
      widget_control, state.id.val_back, get_Value=dummy
      state.par.back_val[dummyy]=dummy[dummyx,dummyy]
   end
    
   'reset_val_back' : begin
      state.par.back_val[*]=0.0
      widget_control, state.id.val_back, SET_Value=transpose(state.par.back_val)
   end

   'back_file': begin
   if event.value EQ '' then return
   state.par.back_file=event.value
   end
                       
  'back_sub' : begin
         state.par.back_sub=event.value
         pre_gui_set, state
  end

  'saa_enable': begin
      state.par.saa_enable = event.select
      pre_gui_set, state
   end
   
   'saa_method': begin
   state.par.saa_method=event.value
   pre_gui_set,state
   end
   
   'saa_file': begin
   if event.value EQ '' then return
   state.par.saa_file=event.value
   end
   
   'saa_auto': begin
   state.par.saa_auto=float(2.^event.value)
   end
   
   'red_enable': begin
      state.par.red_enable = event.select
      pre_gui_set,state
   end
   
   'red_bp': begin
      state.par.red_bp = event.select
      pre_gui_set,state
   end
   
   'red_bp_file': begin
   if event.value EQ '' then return
   state.par.red_bp_file=event.value
   end
   
   'red_flat': begin
      state.par.red_flat = event.select
      pre_gui_set,state
   end
   
   'red_flat_file': begin
   if event.value EQ '' then return
   state.par.red_flat_file=event.value
   end
      
   'red_dark': begin
      state.par.red_dark = event.select
      pre_gui_set,state
   end

   'red_bias': begin
      state.par.red_bias = event.select
      pre_gui_set,state
   end
   
   'red_dark_file': begin
   if event.value EQ '' then return
   state.par.red_dark_file=event.value
   end

   'red_bias_file': begin
   if event.value EQ '' then return
   state.par.red_bias_file=event.value
   end
   
   'pos_enable' : begin
                 state.par.pos_enable = event.select
                 pre_gui_set, state
                 end

   'pos_method'   : begin
                 state.par.pos_method = event.value
                 pre_gui_set, state
                 end

   're_center_enable': begin
   state.par.re_center_enable=event.select
   pre_gui_set,state
   end 
   
   're_center_accuracy': begin
   state.par.re_center_accuracy=float(2.^(-event.value))
   end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


   'save': begin

      ; check before saving the parameter file if filename already exists
      check_file = findfile(state.sav_file)
      if check_file[0] ne "" then begin
         answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                'do you want to overwrite it ?'],          $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='pre warning',                        $
                               /QUEST)
         ; return without saving if the user doesn't want to overwrite the
         ; existing parameter file
         if strlowcase(answ) eq "no" then return
      endif else begin
         answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                               DIALOG_PARENT=event.top,                    $
                               TITLE='pre information',                    $
                               /INFO                                       )
         ; inform were the parameters will be saved
      endelse

      ; save the parameter data file
      par = state.par
      save, par, FILENAME = state.sav_file

      ; kill the GUI returning a null error
      error = !caos_error.ok
      widget_control, event.top, /DESTROY
      return

   end

  ; handle event from standard help button
  'help': begin
   online_help, book=(pre_info()).help, /FULL_PATH
;         widget_control, /HOURGLASS
;         
;       CASE !VERSION.OS_FAMILY OF
;         'Windows'   :     begin
;                    spawn, !caos_env.browser+" "+(pre_info()).help,/NOSHELL
;                   end
;         else    :  begin
;                     spawn, !caos_env.browser+" "+(pre_info()).help+" &"
;                   end
;       ENDCASE
   end
   
   'restore': begin

      ; restore the desired parameter file
      par = 0
      title = "parameter file to restore"
      restore, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      ; update the current module number
      par.module.n_module = state.par.module.n_module

      ; set the default values for all the widgets
      widget_control,  state.id.line_SNR,       SET_VALUE=par.line_SNR
      widget_control, state.id.header_ron,    SET_VALUE=par.header_ron
      widget_control, state.id.sigma_ron,      SET_VALUE=par.sigma_ron
      widget_control, state.id.gain_ron,       SET_VALUE=par.gain_ron
      widget_control, state.id.back_group, SET_VALUE=par.buser_enable
      widget_control,  state.id.back_sub,   SET_VALUE=par.back_sub
      widget_control,  state.id.val_back,  SET_VALUE=transpose(par.back_val)
      widget_control, state.id.saa_enable, set_value=par.saa_enable
      widget_control, state.id.saa_method, set_value=par.saa_method
      widget_control, state.id.saa_file, set_value=par.saa_file
      widget_control, state.id.saa_auto, set_value=fix(alog(par.saa_auto)/alog(2))
      widget_control, state.id.red_enable, set_value=par.red_enable
      widget_control, state.id.red_bp, set_value=par.red_bp
      widget_control, state.id.red_bp_file, set_value=par.red_bp_file
      widget_control, state.id.red_flat, set_value=par.red_flat
      widget_control, state.id.red_flat_file, set_value=par.red_flat_file
      widget_control, state.id.red_dark, set_value=par.red_dark
      widget_control, state.id.red_dark_file, set_value=par.red_dark_file
      widget_control, state.id.red_bias, set_value=par.red_bias
      widget_control, state.id.red_bias_file, set_value=par.red_bias_file
      widget_control, state.id.pos_enable, set_value=par.pos_enable
      widget_control, state.id.pos_method, set_value=par.pos_method
      widget_control, state.id.re_center_enable,  set_value=par.re_center_enable
      widget_control, state.id.re_center_accuracy,$
         set_value=fix(-alog(par.re_center_accuracy)/alog(2.))
      

      ; update the state structure
      state.par = par
      pre_gui_set, state

   end

   'cancel': begin
      error = !caos_error.cancel
      widget_control, event.top, /DESTROY
      return
   end
   '': begin
   ;NOTHING TODO
   end
 

endcase

; write the GUI state structure
widget_control, event.top, SET_UVALUE=state

return

end

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
function pre_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
common error_block, error

; retrieve the module information
info = pre_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a new one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = {                             $
     root_base_id    : 0L,        $
     line_SNR        : 0L,        $
     header_ron      : 0L, $
     sigma_ron       : 0L, $
     gain_ron        : 0L, $     
     back_sub        : 0L,        $
     back_group      : 0L,        $
     back_arr_sel    : 0L,        $
     val_back        : 0L,        $
     reset_val_back  : 0L,        $
     back_file       : 0L,        $
     saa_enable      : 0L,        $
     saa_method      : 0L,        $
     saa_file        : 0L,        $
     saa_auto        : 0L,        $
     red_enable : 0L,             $
     red_bp : 0L,                 $
     red_bp_file: 0L,             $
     red_flat : 0L,               $
     red_flat_file: 0L,           $
     red_dark : 0L,               $
     red_dark_file: 0L,           $
     red_bias : 0L,               $
     red_bias_file: 0L,           $
     pos_enable      : 0L,        $
     pos_method      : 0L,        $
     re_center_enable:  0L,  $
     re_center_accuracy:0L   $
     }

; build the state structure were par, id, sav_file and def_file will be stored
; (and passed to pre_gui_event).
state = $
   {    $                ; widget state structure
   sav_file: sav_file, $ ; actual name of the file where save params
   def_file: def_file, $ ; default name of the file where save params
   id      : id,       $ ; widget id structure
   par     : par       $ ; parameter structure
   }

; root base definition
modal = n_elements(group) ne 0
dummy = strupcase(info.mod_name)+' parameters setting GUI'
root_base_id = widget_base(TITLE=dummy, MODAL=modal, /ROW, GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

   ; message widget
   globalbase = widget_base(root_base_id, /COL)

   basetab = widget_tab(globalbase, location=0, multiline=4, uvalue='')
   basetabA=WIDGET_BASE(basetab, /COL, TITLE='Data Reduction ')
   basetabB=WIDGET_BASE(basetab, /COL, TITLE='Shift and Add  ')
   basetabC=WIDGET_BASE(basetab, /COL, TITLE='Positivization ')
   basetabD=WIDGET_BASE(basetab, /COL, TITLE='Sky Background ')
   basetabE=WIDGET_BASE(basetab, /COL, TITLE='SNR Evaluation ')
   basetabF=WIDGET_BASE(basetab, /COL, TITLE='CCD Information')
   basetabG=WIDGET_BASE(basetab, /COL, TITLE='Re-centering   ')

; DATA REDUCTION
   red = widget_base(basetabA, /frame, /COL, /ALIGN_CENTER)
   dummy= widget_label(red, VALUE='Data Reduction:')
   state.id.red_enable=cw_bgroup(red, $
                                 ['Enable Data Reduction for raw images'], $
                                 SET_VALUE=state.par.red_enable, UVALUE='red_enable',    $
                                 /NONEXCLUSIVE )
   
   red_bias = widget_base(red, /COL, /FRAME)
   red_bias1 = widget_base(red_bias, /ROW)
   state.id.red_bias=cw_bgroup(red_bias1, $
                               ['BIAS MAP        '], $
                               SET_VALUE=state.par.red_bias, UVALUE='red_bias',    $
                               /NONEXCLUSIVE )
   state.id.red_bias_file=CW_FILENAME(red_bias1, TITLE="Load a FITS file",$
                                      VALUE=state.par.red_bias_file, UVALUE="red_bias_file",/ALL_EVENTS)
   
   red_flat = widget_base(red, /COL, /FRAME)
   red_flat1 = widget_base(red_flat, /ROW)
   state.id.red_flat=cw_bgroup(red_flat1, $
                               ['FLAT-FIELD MAP  '], $
                               SET_VALUE=state.par.red_flat, UVALUE='red_flat',    $
                               /NONEXCLUSIVE )
   state.id.red_flat_file=CW_FILENAME(red_flat1, TITLE="Load a FITS file",$
                                      VALUE=state.par.red_flat_file, UVALUE="red_flat_file",/ALL_EVENTS)
   dummy=WIDGET_LABEL(red_flat,VALUE="If BIAS MAP is not provided, the FF map must be already ",/ALIGN_LEFT)
   dummy=WIDGET_LABEL(red_flat,VALUE="pre-processed and it will be normalized to unit average.",/ALIGN_LEFT)
   
   red_dark = widget_base(red, /COL, /FRAME)
   red_dark1 = widget_base(red_dark, /ROW)
   state.id.red_dark=cw_bgroup(red_dark1, $
                               ['DARK MAP'], $
                               SET_VALUE=state.par.red_dark, UVALUE='red_dark',    $
                               /NONEXCLUSIVE )
   state.id.red_dark_file=CW_FILENAME(red_dark1, TITLE="Load a FITS file",$
                                      VALUE=state.par.red_dark_file, UVALUE="red_dark_file",/ALL_EVENTS)
   
   red_bp = widget_base(red, /COL, /FRAME)
   red_bp1 = widget_base(red_bp, /ROW)
   state.id.red_bp=cw_bgroup(red_bp1, $
                             ['BAD PIXEL MAP   '], $
                             SET_VALUE=state.par.red_bp, UVALUE='red_bp',    $
                             /NONEXCLUSIVE )
   state.id.red_bp_file=CW_FILENAME(red_bp1, TITLE="Load a FITS file",$
                                    VALUE=state.par.red_bp_file, UVALUE="red_bp_file",/ALL_EVENTS)
   dummy = WIDGET_LABEL(red_bp, VALUE="The map must have 0 in bad positions "+$
                        "and 1 in good pixels.",/ALIGN_LEFT)  
   
;SHIFT AND ADD
   saa = widget_base(basetabB, /frame, /COL)
   dummy= widget_label(saa, VALUE='Shift and Add [MULTIFRAMES ONLY]:')
   state.id.saa_enable=cw_bgroup(saa, $
             ['Enable SHIFT and ADD for multi-frames images'], $
             SET_VALUE=state.par.saa_enable, UVALUE='saa_enable',    $
             /NONEXCLUSIVE )

   state.id.saa_method = cw_bgroup(saa, $
             ['Only ADD (frames are already aligned)', $
              'Shift values are stored into a text file', $
              'Automatic procedure based on correlation'], $
             SET_VALUE=state.par.saa_method, UVALUE='saa_method',    $
             /EXCLUSIVE )
             
    state.id.saa_file=CW_FILENAME(saa, TITLE="Load a text file",$
                        VALUE=state.par.saa_file,$
                        UVALUE="saa_file",/ALL_EVENTS)

    state.id.saa_auto=cw_bgroup(saa,                             $
             ['1 pixel', '1/2 pixel', '1/4 pixel', '1/8 pixel'], $
             SET_VALUE=fix(alog(state.par.saa_auto)/alog(2)),    $
             UVALUE='saa_auto',                                  $
             label_top='Accuracy (sub-pixel accuracy is very slow):',$
             /EXCLUSIVE )


;POSITIVIZATION
   pos = widget_base(basetabC, FRAME=1, /COL)
   dummy = widget_label(pos, VALUE='Positivization:')
   state.id.pos_enable = cw_bgroup(pos, $
       ['Positivization of the non-positive values'],$
       SET_VALUE=state.par.pos_enable,$
       UVALUE='pos_enable', /NONEXCLUSIVE)

   state.id.pos_method = cw_bgroup(pos,$
       COLUMN=2, LABEL_LEFT='Positivization values :',$
       ['Less than 0','Less-equal than 0'], SET_VALUE=state.par.pos_method,$
       UVALUE='pos_method', /EXCLUSIVE)



;SKY BACKGROUND
   background = widget_base(basetabD, FRAME=1, /COL)
   dummy = widget_label(background, VALUE='Sky background evaluation')

   choice = widget_base(background, /COL)

   values = ['Consider the background evaluated by the MMM algorithm', $
             'Consider a user-defined background'                      ]

   state.id.back_group = cw_bgroup (choice,        $
                                    values,        $
                                    ROW=2,         $
                                    SET_VALUE=state.par.buser_enable,   $
                                    UVALUE='back_sel', $
                                    /EXCLUSIVE     )

   bgbase = widget_base(choice,/COL,/FRAME)
   state.id.back_arr_sel = cw_bgroup (bgbase,        $
                                     ['Constant background (single value per image)',$
                                      'Variable background (Fits file shaped '+$
                                      'as input image)'],       $
                                     ROW=2,         $
                                     SET_VALUE=state.par.back_variable,   $
                                     UVALUE='back_arr_sel', $
                                     /EXCLUSIVE     )


   back_table=widget_base(bgbase,xsize=200,ysize=160)
   rlabel=make_array(360,/string,value='Angle #')
   num=indgen(360)
   rlabel=rlabel+strcompress(string(num),/remove_all)

   button_bkg=widget_base(back_table,xoffset=58,yoffset=3,xsize=64,ysize=26)
   state.id.reset_val_back=widget_button(button_bkg,$
                      VALUE="Reset Val", $
                      UVALUE="reset_val_back")

   state.id.val_back = widget_table(back_table,$
                       ALIGNMENT=2,$
                       xoffset=55,yoffset=0,$
                       value=transpose(state.par.back_val),$
                       uvalue="back_val",$
                       editable=1,$
                       COLUMN_LABELS=['Background flux'],$
                       ROW_LABELS=rlabel,$
                       COLUMN_WIDTHS=150,$
                       y_scroll_size=5   )

   state.id.back_file=CW_FILENAME(bgbase, TITLE="Load a FITS file",$
                        VALUE=state.par.back_file,$
                        UVALUE='back_file',/ALL_EVENTS)


backsub = widget_base(basetabD, FRAME=1, /COL)
dummy = widget_label(backsub, VALUE='Sky background subtraction')
state.id.back_sub = cw_bgroup(backsub,         $
          LABEL_LEFT='Would you like to subtract the sky background ?', $
          ['no','yes'], COLUMN=2, SET_VALUE=state.par.back_sub,      $
          UVALUE='back_sub', /EXCLUSIVE )


;SNR 
   SNR   = widget_base(basetabE, FRAME=1, /COL)
   dummy = widget_label(SNR, VALUE='Peak SNR evaluation')
   state.id.line_SNR = cw_field(SNR, $
             TITLE='marginal bands width for evaluation [px]  ',     $
             /FLOATING, VALUE=state.par.line_SNR, UVALUE='line_SNR', $
             XSIZE=11, /ALL_EVENTS )


; CCD INFO  
   base_ccd   = widget_base(basetabF, FRAME=1, /COL)
   dummy = widget_label(base_ccd, VALUE='Read-Out Noise and Gain:')   
   state.id.sigma_ron = cw_field(base_ccd, $
                                 TITLE='Read-Out Noise [e-/px]:            ', $
                                 /ROW, xsize=12, /FLOAT, $
                                 UVALUE='sigma_ron', value=state.par.sigma_ron, /ALL_EVENTS)
   state.id.gain_ron = cw_field(base_ccd, $
                                TITLE='Gain of the detector [e-/ADU]:     ', $
                                /ROW, xsize=12, /FLOAT, $
                                UVALUE='gain_ron', value=state.par.gain_ron, /ALL_EVENTS)
   state.id.header_ron=cw_bgroup (base_ccd,        $
                                  [ 'Enter user-defined values (will be added to header)', $
                                    'Read from FITS header'], $
                                  ROW=2, SET_VALUE=state.par.header_ron,   $
                                  UVALUE='header_ron', /EXCLUSIVE     )

;RE-CENTERING
   re_center   = widget_base(basetabG, FRAME=1, /COL)
   dummy = widget_label(re_center, VALUE='Re-centering parameters')
   state.id.re_center_enable=CW_BGROUP(re_center,  $
      set_value=state.par.re_center_enable, $
      ["Enable re-centering"], /NONEXCLUSIVE, $
       uvalue="re_center_enable")      
   state.id.re_center_accuracy=cw_bgroup(re_center,                    $
             ['integer', '1/2 pixel', '1/4 pixel', '1/8 pixel'],  $
             SET_VALUE=fix(-alog(state.par.re_center_accuracy)/alog(2)), $
             UVALUE='re_center_accuracy', label_top='Accuracy:',  $
             /EXCLUSIVE )

   dummy=widget_label(re_center,VALUE=' ')
   dummy=widget_label(re_center,VALUE='                 WARNING!                ')
   dummy=widget_label(re_center,VALUE='The image re-centering is based on       ')
   dummy=widget_label(re_center,VALUE='correlation among the images of the cube.')
   dummy=widget_label(re_center,VALUE=' ')
   dummy=widget_label(re_center,VALUE='The sub-pixel accuracy may take minutes  ')
   dummy=widget_label(re_center,VALUE='and the subsequent fft-shift procedure,  ')
   dummy=widget_label(re_center,VALUE='in some cases, can introduce artifacts   ')
   dummy=widget_label(re_center,VALUE='in the image.                            ')



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; button base for control buttons (standard buttons)
btn_base_id = widget_base(globalbase, FRAME=10, /ROW)
dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
if modal then widget_control, cancel_id, /CANCEL_BUTTON
dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                      UVALUE="restore"                         )
save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS",  $
                        UVALUE="save"                          )
if modal then widget_control, save_id, /DEFAULT_BUTTON

; save the state structure of the GUI in the top base uvalue
widget_control, root_base_id, SET_UVALUE=state

; draw the GUI
widget_control, root_base_id, /REALIZE

pre_gui_set, state

; launch xmanager
xmanager, 'pre_gui', root_base_id, GROUP_LEADER=group


return, error
end
