; $Id: pre_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+ 
; NAME:
;    pre_init 
; 
; PURPOSE: 
;    pre_init executes the initialization for the pre-processing
;    (pre) module, that is:
;
;       0- check the formal validity of the input/output structure
;       1- initialize the output structure(s) out_img_t
;
;    (see pre.pro's header --or file airy_help.html-- for details
;    about the module itself).
; 
; CATEGORY: 
;    module's initialization routine 
; 
; CALLING SEQUENCE: 
;    error = pre_init(inp_img_t,  $ ; img_t input structure
;                     out_img_t,  $ ; img_t output structure
;                     par,        $ ; parameters structure
;                     INIT=init   $ ; initialization data structure
;                     ) 
; 
; INPUTS/OUTPUTS/KEYWORDS/ETC.: 
;    see pre.pro's help for a detailed description. 
;
; ROUTINE MODIFICATION HISTORY: 
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -a few minor modifications.
;                   : octobre 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -added apodization item.
;                   : april 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -added module number in line display.
;                   : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -debugged computation of sky background.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : march 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -wrong end type debugged.
;                   : october 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;                    -insert possility for the user to subtract sky
;                     background in the image.
;                   : november 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -image positivity now conserved after sky subtraction.
;                    -**Peak** SNR now computed over the four margins of image.
;                   : december 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -use of variable "time_integ" activated.
;                   : may 2007,
;                     Marcel Carbillet (LUAN) [marcel.carbillet@unice.fr]:
;                    -now works also with one-dimensional data (snr calculation
;                     adapted).
;                   : may 2008,
;                     Andrea La Camera and Gabriele Desidera' (DISI)
;                     [lacamera@disi.unige.it], [desidera@disi.unige.it]:
;                    -automatic background estimation now uses the image
;                     where it is greater than zero. In this way, the MMM
;                     procedure should give a more reliable background.
;                    -SNR is now computed within an image-boundaries shaped
;                     ring, made by means of the new-added function
;                     pre_find_boundary.pro (contained in the sub-dir pre_lib)
;                   : December 2009,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Added DATA REDUCTION for raw images (BAD-PIXEL MAP, 
;                     FLAT-FIELD MAP and BIAS MAP)
;                    -Added SHIFT-AND-ADD method. The values of the shift can be
;                     either stored in a text file (x_shift, y_shift in every 
;                     line) or calculated by correlation (with integer or sub-pixel 
;                     accuracy)
;                    -Apodization has been removed (obsolete)
;                    -Changed Re-centering procedure, the new one is based on 
;                     correlation among the images (with integer or sub-pixel 
;                     accuracy)
;                   : February 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -DARK CURRENT MAP added (data reduction procedure completed)
;                   : October 2012,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -routine "positivization.pro" moved to CAOS Library (v5.4)
;                   : March 2013,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -routine "find_boundary" renamed to "pre_find_boundary"
;                     for compatibility with other libraries (such as the 
;                     well-known "Coyote" one);
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. The user can now
;                     load the background contained in a FITS file
;                     (that must have the correct dimensions). 
;                    -BACK_ENABLE -> BACK_SUB (Better name!)
;                    -deleted procedure "sky_back_sub.pro" (useless!)
;                     and added the necessary steps in the code.
;                    -Moved here the definitions of the 
;                     Read-Out Noise value and the Gain of the CCD, by 
;                     default they are read from the header of the
;                     FITS file (be sure they are defined by "RON" and
;                     "GAIN" keywords)
;                    -bug corrections in the data reduction procedure
;                   : December 2016-January 2017,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@oca.eu]:
;                    -Re-centering can now be done also for single images.
;                    -Back to correl_optimize from astrolib (instead of
;                     correl_optimize2 from CAOS lib., now eliminated), thanks
;                     to new keyword INIT_FACTOR, here chosen =4 instead of
;                     default 8 (which fails for images as small as 64x64).
;-
;
function pre_init, inp_img_t, out_img_t, par, INIT=init

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = pre_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

; STANDARD CHECKS
;
; compute and test the requested number of pre arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'pre error: par must be a structure'
if n ne 1 then message, 'pre error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module pre'

; check the input arguments

; test if any optional input exists
if n_inp gt 0 then begin
   inp_opt = info.inp_opt
endif

dummy = test_type(inp_img_t, TYPE=type)
if type eq 0 then begin         ; undefined variable
   inp_img_t = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif
if test_type(inp_img_t, /STRUC, N_EL=n, TYPE=type) then $
   message, 'pre error: wrong definition for the first input.'
if n ne 1 then message, $
   'pre error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_img_t.data_status eq !caos_data.not_valid and not inp_opt[0] then $
      message, 'undefined input is not allowed'

print, '-------------------------------------------------------------------'
print, 'pre module nb '+strcompress(par.module.n_module, /REMOVE_ALL)+' :'

;-------------DATA REDUCTION-------------
reduc_img=inp_img_t.image
N=(size(reduc_img))[1]
M=(size(reduc_img))[2]
P=1
F=1
if (size(reduc_img))[0] EQ 3 then P=(size(reduc_img))[3]
if (size(reduc_img))[0] EQ 4 then F=(size(reduc_img))[4]
;load dark, bias, flat field, and bad pixel

header = *inp_img_t.header

if par.red_bias then begin
   fits_read, par.red_bias_file, bias_map
   if (size(bias_map))[1] NE N OR (size(bias_map))[2] NE M then message, $
      "The loaded BIAS MAP file has not the same dimensions of the input image."
   if (size(bias_map))[0] EQ 3 then begin
      message, "The loaded BIAS MAP file is a cube (3D-array), "+ $
               "it has been reduced to a single image (2D-array).", /INFO
      bias_map=median(bias_map,DIM=3,/EVEN)
      fxaddpar, header, 'BIAS_MAP', par.red_bias_file, '[PRE] Bias map used in calibration'
   endif
endif

if par.red_dark then begin
   fits_read, par.red_dark_file, dark_map
   if (size(dark_map))[1] NE N OR (size(dark_map))[2] NE M then message, $
      "The loaded DARK MAP file has not the same dimensions of the input image."
   if (size(dark_map))[0] EQ 3 then begin
      message, "The loaded DARK FRAME file is a cube (3D-array), "+ $
               "it has been reduced to a single image (2D-array).", /INFO
      dark_map=median(dark_map,DIM=3,/EVEN)
      fxaddpar, header, 'DARK_MAP', par.red_dark_file, '[PRE] Dark map used in calibration'
   endif
endif

if par.red_flat then begin
   fits_read, par.red_flat_file, flat_map
   if (size(flat_map))[1] NE N OR (size(flat_map))[2] NE M then message, $
      "The loaded FLAT-FIELD MAP file has not the same dimensions of the input image."
   if (size(flat_map))[0] EQ 3 then begin
      message, "The loaded FLAT MAP file is a cube (3D-array), "+ $
               "it has been reduced to a single image (2D-array).", /INFO
      if par.red_bias then begin
         for k=0,(size(flat_map))[3]-1 do flat_map[*,*,k]=flat_map[*,*,k]-bias_map
      endif
      flat_map=median(flat_map,DIM=3,/EVEN)
   endif else begin
      if par.red_bias then flat_map=flat_map-bias_map
   endelse
   flat_map = flat_map / mean(flat_map)
   fxaddpar, header, 'FLAT_MAP', par.red_flat_file, '[PRE] Flat-Field map used in calibration'
endif

if par.red_bp then begin
   fits_read, par.red_bp_file, bp_map
   if (size(bp_map))[1] NE N OR (size(bp_map))[2] NE M then message, $
      "The loaded BAD-PIXEL MAP file has not the same dimensions of the input image."
   fxaddpar, header, 'BADP_MAP', par.red_bp_file, '[PRE] Bad-pixels map used in calibration'
endif

;DATA REDUCTIO / CALIBRATION PROCEDURE
for i=0,P-1 do begin
   for j=0,F-1 do begin
      frame=reduc_img[*,*,i,j]
      if par.red_bias then frame=frame-bias_map
      if par.red_dark then frame=frame-dark_map
      if par.red_flat then frame=frame/flat_map
      if par.red_bp   then frame=frame*bp_map
      reduc_img[*,*,i,j]=frame
   endfor
endfor


;-------------MULTI-FRAME-----------------------------------
if (size(reduc_img))[0] eq 4 then begin
   if par.saa_enable EQ 1 then begin
      if par.saa_method EQ 0 then begin     
         reduc_img=total(reduc_img,4,/double)
      endif
      if par.saa_method EQ 1 then begin
         openr, 1, par.saa_file
         nframes=(size(reduc_img))[4]
         nangles=(size(reduc_img))[3]
         b=""
         count=0
         while ~EOF(1) do begin
            readf,1,b
            count=count+1
         endwhile
         point_lun,1,0
         dx=make_array(count)
         dy=make_array(count)
         for line=0, count-1 do begin
            readf,1,b
            dx[line]=(strsplit(b,",",/extract))[0]
            dy[line]=(strsplit(b,",",/extract))[1]  
         endfor
         num=count / nangles
         num2=nframes / num
         for angle=0,nangles-1 do begin
            for frame=0,nframes-1 do begin
               tmp = reduc_img[*,*,angle,frame]
               pos=angle * num + frame/num2
               tmp=shift(tmp,-dx[pos],-dy[pos])
               reduc_img[*,*,angle,frame]=tmp
            endfor
         endfor
         close,1
         reduc_img=total(reduc_img,4,/double)
      endif
      if par.saa_method EQ 2 then begin
         nframes=(size(reduc_img))[4]
         nangles=(size(reduc_img))[3]
         for angle=0,nangles-1 do begin
            for frame=1,nframes-1 do begin
                now=reduc_img[*,*,angle,frame]
                correl_optimize, reduc_img[*,*,angle,0],now,dx,dy,INIT_FACTOR=4
                ;print, angle, frame, dx,dy
                now=fftshift(now, dx, dy)
                while abs(dx-0) GT 1./par.saa_auto AND $
                      abs(dy-0) GT 1./par.saa_auto do begin
                         correl_optimize, reduc_img[*,*,angle,0],now,dx,dy, $
                                             magnification=par.saa_auto,INIT_FACTOR=4
                         now=fftshift(now, dx, dy)
                         ;print, angle, frame, dx, dy
                endwhile
                reduc_img[*,*,angle,frame]=now
            endfor
         endfor  
         reduc_img=total(reduc_img,4,/double)
      endif

;   dummy=dialog_message(['              MULTI-FRAME input type', $
;           '    Automatic reduction to SINGLE-FRAME input type', $
;           'by means of addition of the frames of equal orientation'], $
;           title='PRE Information')
;   print,'MULTI-FRAME input tipe. Automatic reduction to SINGLE-FRAME input type'
   endif else message,                                              $
      'MULTI-FRAME input type. Please, select SHIFT and ADD '+      $
      'in order to reduce it to a single frame for each hour angle.'
endif
;------------END-------------------------------------------------------

; frame number in the data cube
if (size(reduc_img))[0] eq 3 then nb_fram = (size(reduc_img))[3] $
  else nb_fram = 1
pre_map = reduc_img
reduc_img=0.

;positivization
if par.pos_enable eq 1B then begin
   nb_pix = (size(pre_map))[1]
    print, '<<<<< Positivization >>>>>'
    if par.pos_method eq 0 then begin
      for i=0,nb_fram-1 do begin
        tmp=pre_map[*,*,i]
        pre_map[*,*,i]=positivization(tmp)
      endfor
    endif else begin
      for i=0,nb_fram-1 do begin
        tmp=pre_map[*,*,i]
        pre_map[*,*,i]=positivization(tmp,/ZERO)
      endfor
    endelse
endif


; sky background level estimation ---------------------------------------------------
; update the header meanwhile!
back = inp_img_t.background ;is now an array!

print, '<<<<< sky background >>>>>'
; AUTO --- mmm
if (par.buser_enable EQ 0B) then begin
   fxaddpar, header, 'SKY_TYPE', 'VALUE', '[PRE] Sky background (evaluated by MMM)' 
   fxaddpar, header, 'SKY_NB', nb_fram, '[PRE] Sky background number of values' 
   for i=0, nb_fram-1 do begin 
      print, 'SKY background evaluation within frame nb.', strtrim(i,1),' :'
      dummy = pre_map[*,*,i]
      dummy = dummy[where(dummy GT 0.)]
      sky, dummy, sky_mode, sky_sig, /silent ; Sky background evaluation using the Astrolib procedure SKY.
                                ; test the validity of the sky value (non <0)
      if sky_mode lt 0. then begin                                            
         print, 'Sky evaluation error (sky < 0)  ==>  sky = 0' 
         sky_mode = 0.
      endif else begin
         print, 'Sky background value =', sky_mode
         print, 'Standard deviation of sky background =', sky_sig
      endelse
      back[*,*,i] = sky_mode
      fxaddpar, header, 'SKY_BG'+strtrim(i,1), sky_mode, '[PRE] Sky background value' 
   endfor    
endif else begin
; USER DEFINED ---
; -- constant value:       
   if (par.back_variable EQ 0B) then begin
      fxaddpar, header, 'SKY_TYPE', 'USER', '[PRE] Sky background (user-defined)' 
      fxaddpar, header, 'SKY_NB', nb_fram, '[PRE] Sky background number of values' 
      for i=0, nb_fram-1 do begin
         back[*,*,i] = par.back_val[i]
         fxaddpar, header, 'SKY_BG'+strtrim(i,1), par.back_val[i], '[PRE] Sky background value' 
      endfor
   endif else begin
; -- variable array: 
      fxaddpar, header, 'SKY_TYPE', 'FILE', '[PRE] Sky background (user-defined FITS file)' 
      fits_read, par.back_file, back_map
      if (size(back_map))[1] NE N OR (size(back_map))[2] NE M then message, $
         "The loaded BACKGROUND array has not the same dimensions of the input image."
      if (size(back_map))[0] EQ 3 AND (size(back_map))[3] NE P then message, $
         "Wrong number of images within the cube."
      back=back_map
      fxaddpar, header, 'SKY_FILE', par.back_file, '[PRE] Sky background filename'       
   endelse     
endelse

if (par.back_sub EQ 1B) then begin
   for i=0, nb_fram-1 do begin 
      pre_map[*,*,i] = (pre_map[*,*,i]-back[*,*,i])>0.
      back[*,*,i] = 0.
   endfor
endif


; Peak SNR estimation ---------------------------------------------------------------
snr    = fltarr(360)
width= par.line_snr
print, '<<<<< Peak SNR computation >>>>>'

for i=0, nb_fram-1 do begin
    dummy=pre_map[*,*,i]
    snr[i]=(max(dummy)-back[i]) / $
           sqrt((moment(dummy*pre_find_boundary(dummy,width)))[1])
    print, 'Peak SNR within frame nb.', strtrim(i,2),' = ', snr[i]
endfor

; apodization -----------------------------------------------------------------
; construct the apodizing filter for the fftshift

;if par.apo_enable eq 1B then begin 
;
;   print, '<<<<< apodization >>>>>'
;   print, 'Zero padding band inside the frame [pixel]    =', par.marge_apo
;   print, 'Sigma of the gaussian shape transition [pixel]=', par.sigma_apo
;   print, 'construction of the apodizing filter ...'
;
;   nb_pix = (size(pre_map))[1]
;   apo    = fltarr(nb_pix, nb_pix)
;
;   for i=0, nb_pix-1 do begin
;      for j=0, nb_pix-1 do begin
;         apo[i,j] = exp(-((i-nb_pix/2)^2+(j-nb_pix/2)^2)/(2*par.sigma_apo^2))
;      endfor
;   endfor
;   apo    = apo/max(apo)
;   temp   = fltarr(nb_pix,nb_pix)
;   temp[par.marge_apo:nb_pix-par.marge_apo-1,par.marge_apo:nb_pix-par.marge_apo-1] = 1.
;   convolution, temp, apo, apo, nb_pix
;   apo    = apo/max(apo)
;
;   if nb_fram eq 1 then begin
;      pre_map = pre_map*apo 
;   endif else begin
;      for i=0, nb_fram-1 do begin
;         pre_map[*,*,i] = pre_map[*,*,i] * apo
;      endfor
;   endelse
;
;endif 

; map re-centering ------------------------------------------------------------ 
if par.re_center_enable then begin
   print, '<<<<< Re-centering >>>>>'

;;+ MCa - dec.2016
;   if (size(pre_map))[0] EQ 3 then begin
;      N=(size(pre_map))[1]
;      P=(size(pre_map))[3]
;      for j=1,P-1 do begin
;         dummy=pre_map[*,*,j]
;         correl_optimize2, pre_map[*,*,0], dummy, dx, dy
   if (size(pre_map))[0] EQ 3 or (size(pre_map))[0] EQ 2 then begin
      N=(size(pre_map))[1]
      M=(size(pre_map))[2]
      if (size(pre_map))[0] EQ 3 then P=(size(pre_map))[3] else P=1
      ref=fltarr(N,M) & ref[N/2-1:N/2,M/2-1:M/2]=1.
      for j=0,P-1 do begin
          dummy=pre_map[*,*,j]
          correl_optimize, ref, dummy, dx, dy, INIT_FACTOR=4
;;-
          dummy=shift(dummy, dx, dy)
         if par.re_center_accuracy LT 1. then begin
            print, 'Re-centering frame nb.'+strtrim(j,1)+$
                   ' - Please wait, this may take several minutes.'
            while sqrt(dx*dx+dy*dy) GT par.re_center_accuracy do begin
;;+ MCa - dec.2016
;               correl_optimize2, pre_map[*,*,0], dummy, dx, dy, $
               correl_optimize, ref, dummy, dx, dy, INIT_FACTOR=4, $
;;+
                                  magnification=fix(1./par.re_center_accuracy)
               dummy=fftshift(dummy, dx, dy)
            endwhile
            dummy=positivization(dummy)
        endif
         pre_map[*,*,j]=dummy
      endfor
   endif
endif


if (par.header_ron EQ 0B) then begin
                                ; add parameters to the header
   fxaddpar, header, 'RON', par.sigma_ron, '[PRE] Read-Out Noise in [e-/px]'
   fxaddpar, header, 'GAIN', par.gain_ron, '[PRE] Gain of the detector in [e-/ADU]'
   print, '>>> Header modified with the user-defined values'
endif else begin
                                ; read parameters from the header, if
                                ; errors add to header default values 
   print, '>>> Header values: '
   dummy = fxpar(header, 'RON', count=a)
   if (a) then begin
      print, '    RON =', dummy 
   endif else begin
      print, '   RON not found in the header --> default value (0) will be assumed)'
      fxaddpar, header, 'RON', 0., '[PRE] Read-Out Noise in [e-/px]'
   endelse
   dummy = fxpar(header, 'GAIN', count=a)
   if (a) then begin
      print, '    GAIN =', dummy 
   endif else begin
      print, '   GAIN not found in the header --> default value (1) will be assumed)'
      fxaddpar, header, 'GAIN', 1., '[PRE] Gain of the detector in [e-/ADU]'
   endelse
endelse


; INITIALIZE THE OUTPUT STRUCTURE ----------------------------------------
;
out_img_t = $
   {        $
   data_type  : out_type[0]          , $
   data_status: !caos_data.valid     , $
   image      : pre_map              , $ ; image map
   npixel     : inp_img_t.npixel     , $ ; number of pixels
   resolution : inp_img_t.resolution , $ ; pixel size [arcsec/pix]
   lambda     : inp_img_t.lambda     , $ ; mean detection wavelength [m]
   width      : inp_img_t.width      , $ ; wavelength band width [m]
   time_integ : inp_img_t.time_integ , $ ; Integration time for each image [s]
   time_delay : inp_img_t.time_delay , $ ; [NOT USED WITHIN AIRY]
   psf        : inp_img_t.psf        , $ ; image or PSF ?
   background : back                 , $ ; estimated sky background level of the
                                         ; pre-processed maps
   snr        : snr                  , $ ; estimated peak SNR of the
                                         ; pre-processed maps
   header     : PTR_NEW('')       $ ; header of the FITS File (modified event.)
   }

*out_img_t.header = header ; update the pointer

return, error 
end
