; $Id: photcenter.pro,v 6.1 last revision 2013/04/19 Andrea La Camera $
;+
; NAME:
;       photcenter
;
; PURPOSE:
;
;       Compute the photocenter location of a given array.
;       The treshold value is computed in number of background rms, adding 
;       the background level.
;       The value of the background is computed as the rms value in 
;       a border of the array. 
;
; CATEGORY:
;       ...
;
; CALLING SEQUENCE:
;
;       [ph_cent_x, ph_cent_y] = photcenter (in, TRESH=tresh, LINE=line)
;
; INPUTS:
;
;	in		= input array [FLOAT] 
;			   
;
; OPTIONAL INPUTS:
;       none.
;
; KEYWORD PARAMETERS:
;
;	TRESH	        = Treshold value of the array to perform the photocenter computation.
;                         Expressed in rms background. [FLOAT] (default = 5.)
;	LINE		= X lines of the to use to estimate the backgound rms value (default = 2)
;
; OUTPUT:
;
;	[ph_cent_x, ph_cent_y] =  photocenter location [FLOAT] 
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       none.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
; EXAMPLE:
;       ...
;
; MODIFICATION HISTORY:
;       Aug 2000: written (Serge Correia, OAA).
;       sep 2000: cleared confusion between background level 
;                 and rms background. (S.C.)
;-

function photcenter, in,          $
                     TRESH=tresh, $
                     LINE=line

if not(n_elements(tresh))    then tresh    = 5.
if not(n_elements(line))     then line     = 2

nb_pix = (size(in))[1]
dummy  = moment(in[0:nb_pix-1, 0:line-1])           
back   = dummy[0]
back_rms = sqrt(dummy[1])

sum=0.
for i=0, nb_pix-1 do begin
    for j=0, nb_pix-1 do begin
        if in[i,j] gt back+tresh*back_rms  $
          then begin
            sum = sum + in[i,j]
        endif
    endfor
endfor

ph_cent_x=0.
ph_cent_y=0.
for i=0, nb_pix-1 do begin
    for j=0, nb_pix-1 do begin
        if in[i,j] gt tresh*back  $
          then begin
            ph_cent_x = ph_cent_x + i*in[i,j]/sum
            ph_cent_y = ph_cent_y + j*in[i,j]/sum
        endif
    endfor
endfor

return, [ph_cent_x, ph_cent_y] 
end
