; $Id: rti.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;   RTI
;
; ROUTINE'S PURPOSE:
;    RTI manages the simulation for the RoTate Image (RTI) module,
;    that is to call the module's initialization routine rti_init at the first
;         iteration of the simulation project.
;
; MODULE'S PURPOSE:
;   To rotate either images or objects in two different ways: 
;    - rotation in pixel space: by using the IDL routine "ROT";
;    - rotation in frequencies space: by rotating the Fourier transform of the input
;   N.B. The rotation can generate artifacts, in particular in the case of 
;   point objects/stars.
;
; CATEGORY:
;    main module's routine
;
; CALLING SEQUENCE:
;    error = rti(inp_yyy_t, out_yyy_t, par)
; OUTPUT:
;    error: long scalar (error code, see !caos_error var in caos_init.pro).
;
; INPUTS:
;    inp_yyy_t: structure of type "gen_t". 
;    par      : parameters structure.
;
; INCLUDED OUTPUTS:
;    out_yyy_t: structure of type "gen_t" containing the rotated
;               map/image(s). Depending on the type of data, the
;               following fields are contained in this structure:
;'img_t': 
;      out_yyy_t =  {                             $
;                   data_type    : 'img_t'              , $
;                   data_status  : !caos_data.valid     , $
;                   image        : img_rot              , $
;                   psf          : inp_yyy_t.psf        , $
;                   npixel       : N*par.out            , $
;                   resolution   : inp_yyy_t.resolution , $
;                   lambda       : inp_yyy_t.lambda     , $
;                   width        : inp_yyy_t.width      , $
;                   time_integ   : inp_yyy_t.time_integ , $
;                   time_delay   : inp_yyy_t.time_delay , $
;                   background   : background           , $
;                   snr          : inp_yyy_t.snr        , $
;                   header       : PTR_NEW('')            $
;                   }
;      *out_yyy_t.header = header ; updated the pointer
;   'src_t': 
;      out_yyy_t = {                                   $
;                  data_type  : 'src_t',              $
;                  data_status: !caos_data.valid,     $
;                  off_axis   : inp_yyy_t.off_axis,   $
;                  pos_ang    : inp_yyy_t.pos_ang,    $ 
;                  dist_z     : inp_yyy_t.dist_z,     $
;                  map        : img_rot,              $
;                  scale_xy   : inp_yyy_t.scale_xy,   $
;                  coord      : inp_yyy_t.coord,      $
;                  scale_z    : inp_yyy_t.scale_z,    $
;                  n_phot     : inp_yyy_t.n_phot,     $
;                  background : inp_yyy_t.background, $
;                  lambda     : inp_yyy_t.lambda,     $
;                  width      : inp_yyy_t.width,      $
;                  constant   : inp_yyy_t.constant,   $
;                  header     : PTR_NEW('')           $
;                 }
;      *out_yyy_t.header = header ; update the header
;
; KEYWORD PARAMETERS:
;
; COMMON BLOCKS:
;    common caos_block, tot_iter, this_iter
;
;    tot_iter   : total number of iteration during the simulation run.
;    this_iter  : current iteration number.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;
; ROUTINE MODIFICATION HISTORY:
;    program written: March 2007,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]
;    modifications  : 
;
; MODULE MODIFICATION HISTORY:
;    program written: March 2007,
;         Andrea La Camera (DISI) [lacamera@disi.unige.it]
;    modifications  : 
;                   : february 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]
;                    -module debugged (in particular, for tags "angle" and 
;                     "time_integ")
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array.
;                    -keywords added to the header: NB_ANGLES,
;                     ANGLEn, ROT_TYPE
;-
;
function rti, inp_yyy_t, out_yyy_t, par

common caos_block, tot_iter, this_iter

; initialization of the error code: set to "no error" as default
error = !caos_error.ok

; module's actions
if (this_iter eq 0) then begin
   ; initialisation section
   error = rti_init(inp_yyy_t, out_yyy_t, par)
endif 

; back to calling program
return, error
end
