; $Id: rti_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    rti_gui
;
; PURPOSE:
;    rti_gui generates the Graphical User Interface (GUI) for
;    setting the parameters of the rti module.
;    A parameter file called rti_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    The file is stored in the project directory proj_name located
;    in the working directory.
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = rti_gui(n_module, proj_name)
;
; INPUTS:
;    n_module : integer scalar. Number associated to the instance
;               of the sav module. n_module > 0.
;    proj_name: string. Name of the current project.
;
; OUTPUTS:
;    error: long scalar, error code (see !caos_error var in caos_init.pro).
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    ...
;
; ROUTINE MODIFICATION HISTORY:
;    program written:  March 2007,
;					 Andrea La Camera (DISI) [lacamera@disi.unige.it]
;    modifications  : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;
;-
;
pro rti_gui_set, state
  
  if state.par.method EQ 1 then begin
     widget_control, state.id.zp, /sensitive
     widget_control, state.id.test, /sensitive
  endif else begin
     widget_control, state.id.zp, sensitive=0 
     widget_control, state.id.test, sensitive=0 
  endelse
  
  
  widget_control, state.id.method, set_value=state.par.method
  widget_control, state.id.zp, set_value=state.par.zp
  widget_control, state.id.out, set_value=state.par.out-1
  
  widget_control, state.id.val_ang,        SET_VALUE=transpose(state.par.angle)
end

;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;
pro rti_gui_event, event
  
  common error_block, error
  COMMON zero_padding, zp
  
; read the GUI state structure
  widget_control, event.top, GET_UVALUE=state
  
; handle a kill request (considered as a cancel event).
  if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
     error = !caos_error.cancel
     widget_control, event.top, /DESTROY
  endif
  
; handle other events.
; get the user value of the event sender
  widget_control, event.id, GET_UVALUE=uvalue
  
  case uvalue of
     
;handle event from angle field
;'angle':begin
;	dummy=float(strsplit(event.value, ',', /EXTRACT))
;	state.par.angle=dummy
;end
     
     'reset_val_ang' : begin
        state.par.angle[*]=!VALUES.F_NAN
        widget_control, state.id.val_ang, SET_Value=transpose(state.par.angle)
     end
     
     'ang_val'  : begin
        dummyx = event.x
        dummyy = event.y
        widget_control, state.id.val_ang, get_Value=dummy
        state.par.angle[dummyy]=dummy[dummyx,dummyy]
     end
     
     
;handle event from method button
     'method': begin
        state.par.method=event.value
        rti_gui_set, state
     end
     
     
;handle event from accuration button 
     'zp':begin
        dummy=event.value
        state.par.zp=dummy
        if state.par.zp LE 1 then begin
           dummy=dialog_message(['Zero-padding factor MUST be',$
                                 'an integer number greater than 1'],$
                                title='RTI Warning')
        endif
        rti_gui_set, state
     end
     
     
;handle event from test button
     'test' : 	begin
        zp=state.par.zp
        dummy=memory_test_gui(zp,512,GROUP_LEADER=event.top)
        state.par.zp=zp
        rti_gui_set,state
        
     end
     
;handle event from output button
     'output':  begin
        state.par.out=event.value+1
        rti_gui_set, state
     end
     
     
; handle event from standard save button
     'save': begin
                                ; check if "angle" field is empty
        pos=where(state.par.angle NE 1.e37,P)
        if P EQ 0 then begin
           answ =dialog_message(['Please, select at least one angle',$
                                 'before saving the parameter file'], DIALOG_PARENT=event.top, $
                                TITLE='RTI error', /ERROR)
           return
        endif
        
                                ; check before saving the parameter file if filename already exists
        check_file = findfile(state.rti_file)
        if check_file[0] ne "" then begin
           answ = dialog_message(['file '+state.rti_file+' already exists.', $
                                  'do you want to overwrite it ?'],          $
                                 DIALOG_PARENT=event.top,                    $
                                 TITLE='RTI warning',                        $
                                 /QUEST)
                                ; return without saving if the user doesn't want to overwrite the
                                ; existing parameter file
           if strlowcase(answ) eq "no" then return
        endif else begin
           answ = dialog_message(['file '+state.rti_file+' will be saved.'], $
                                 DIALOG_PARENT=event.top,                    $
                                 TITLE='RTI information',                    $
                                 /INFO                                       )
                                ; inform were the parameters will be saved
        endelse
        
                                ; save the parameter data file
        par = state.par
        save, par, FILENAME = state.rti_file
        
                                ; kill the GUI returning a null error
        error = !caos_error.ok
        widget_control, event.top, /DESTROY
        return
     end
     
; standard help button
     'help': begin
        online_help, book=(rti_info()).help, /FULL_PATH
;         widget_control, /HOURGLASS
;       CASE !VERSION.OS_FAMILY OF
;         'Windows'   :     begin
;                    spawn, !caos_env.browser+" "+(rti_info()).help+"#" $
;                      +strupcase(state.par.module.mod_name),/NOSHELL
;                   end
;         else    :  begin
;                     spawn, !caos_env.browser+" "+(rti_info()).help+"\#" $
;                          +strupcase(state.par.module.mod_name)+"&"
;                    end
;
;       ENDCASE
     end


; standard restore button:
     'restore': begin
        
                                ; restore the desired parameter file
        par = 0
        title = "parameter file to restore"
        restore, filename_gui(state.def_file,                          $
                              title,                                   $
                              GROUP_LEADER=event.top,                  $
                              FILTER=state.par.module.mod_name+"*sav", $
                              /NOEDIT,                                 $
                              /MUST_EXIST,                             $
                              /ALL_EVENTS                              )
        
                                ; update the current module number
        par.module.n_module = state.par.module.n_module
        
                                ; set the default values for all the widgets
        
                                ; update the state structure
        state.par = par
        
                                ; reset the setting parameters status
        rti_gui_set, state
        
                                ; write the GUI state structure
        widget_control, event.top, SET_UVALUE=state
        
     end
     
; standard cancel button (exit without saving)
     'cancel': begin
        error = !caos_error.cancel
        widget_control, event.top, /DESTROY
        return
     end
     
  endcase
  
; write the GUI state structure
  WIDGET_CONTROL, event.top, SET_UVALUE=state
  return
end




;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;

function rti_gui, n_module, proj_name, GROUP_LEADER=group
  
; error status from the event handler procedure
  common error_block, error
  
; retrieve the module information
  info = rti_info()
  
; check if a saved parameter file exists. If it exists it is restored,
; otherwise the default parameter file is restored.
  rti_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
  def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
  par=0
  check_file = findfile(rti_file)
  if check_file[0] eq '' then begin
     restore, def_file
     par.module.n_module = n_module
     if par.module.mod_name ne info.mod_name then        $
        message, 'the default parameter file ('+def_file $
                 +') is from another module: please take the right one'
  endif else begin
     restore, rti_file
     if (par.module.mod_name ne info.mod_name) then $
        message, 'the parameter file '+rti_file     $
                 +' is from another module: please generate a new one'
     if (par.module.ver ne info.ver) then       $
        message, 'the parameter file '+rti_file $
                 +' is not compatible: please generate it again'
  endelse
  id = $
     { $                        ; widget id structure
     base            : 0L, $    ; parameters widget base
     base2           : 0L, $
     base3           : 0L, $
     base4           : 0L, $
     angle           : 0L, $
     val_ang       : 0L, $
     reset_val_ang   : 0L, $
     method          : 0L, $ 
     zp 	           : 0L, $
     test            : 0L, $
     out             : 0L  $
     }
  
  state = $
     {    $                     ; widget state structure
     rti_file: rti_file, $      ; actual name of the file where save params
     def_file: def_file, $      ; default name of the file where save params
     id      : id,       $      ; widget id structure
     par     : par       $      ; parameter structure
     }
  
; root base
  modal = n_elements(group) ne 0
  root_base_id = widget_base(                                                 $
                 TITLE=strupcase(info.mod_name)+' parameters setting GUI', $
                 MODAL=modal,                                              $
                 /COL,                                                     $
                 GROUP_LEADER=group                                        )
  
; set the status structure
  widget_control, root_base_id, SET_UVALUE=state
  
                                ; parameters base
  state.id.base = widget_base(root_base_id, /FRAME, /COL)
  dummy = widget_label(state.id.base, VALUE='parameters', /FRAME)
  state.id.base2 = widget_base(state.id.base, /COL)
                                ;state.id.angle=CW_FIELD(state.id.base2,$
                                ;       TITLE='baseline angles (DEG): [ENTER]', $
                                ;	     VALUE='', UVALUE='angle', /RETURN_EVENT,/col)
  
                                ;modifica x + angoli
  
  ang_table=widget_base(state.id.base2,xsize=200,ysize=160)
  rlabel=make_array(360,/string,value='Angle #')
  num=indgen(360)
  rlabel=rlabel+strcompress(string(num),/remove_all)
  
  button_ang=widget_base(ang_table,xoffset=38,yoffset=3,xsize=64,ysize=26)
  state.id.reset_val_ang=widget_button(button_ang,$
                                       VALUE="Reset Val", $
                                       UVALUE="reset_val_ang")
  
  state.id.val_ang = widget_table(ang_table,$
                                  ALIGNMENT=2,$
                                  xoffset=35,yoffset=0,$
                                  value=transpose(state.par.angle),$
                                  uvalue="ang_val",$
                                  editable=1,$
                                  COLUMN_LABELS=['Baseline angles (DEG)'],$
                                  ROW_LABELS=rlabel,$
                                  COLUMN_WIDTHS=150,$
                                  y_scroll_size=5   )	  
                                ;fine modifica x + angoli
  
  dummy=WIDGET_LABEL(state.id.base2, value='positive angles = clockwise rotations (0 is up)')
  state.id.method=cw_bgroup(state.id.base2,$
                            ['space domain rotation','frequency domain rotation'],$
                            row=2, label_left=' ',/exclusive,uvalue='method',$
                            set_value=state.par.method)
  state.id.base3=WIDGET_BASE(state.id.base2,/col)
  state.id.zp=cw_field(state.id.base3,TITLE='zero-padding (before FFT) factor: [ENTER]',$
                       VALUE=state.par.zp, UVALUE='zp', /return_event,/col,/int)
  state.id.base4=WIDGET_BASE(state.id.base3,/row,/align_right)
  dummy=widget_label(state.id.base4,value='   (recommended)    ')
  state.id.test=WIDGET_BUTTON(state.id.base4, VALUE='MEMORY TEST', $
                              uvalue='test',xsize=100)
  state.id.out=cw_bgroup(state.id.base2,$
                         ['original size (cropped)','double size (zero-padded)'],$
                         row=2, label_left='output : ',/exclusive,uvalue='output',$
                         set_value=state.par.out-1)
  
                                ; button base for control buttons (standard buttons)
  btn_base_id = widget_base(root_base_id, /FRAME, /ROW)
  dummy = widget_button(btn_base_id, VALUE="HELP", UVALUE="help")
  cancel_id = widget_button(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
  if modal then widget_control, cancel_id, /CANCEL_BUTTON
  dummy = widget_button(btn_base_id, VALUE="RESTORE PARAMETERS", $
                        UVALUE="restore"                         )
  save_id = widget_button(btn_base_id, VALUE="SAVE PARAMETERS",  $
                          UVALUE="save"                          )
  if modal then widget_control, save_id, /DEFAULT_BUTTON
  
; initialize all the sensitive states
  rti_gui_set, state
  
; save the state structure of the GUI in the top base uvalue
  widget_control, root_base_id, SET_UVALUE=state
  
; draw the GUI
  widget_control, root_base_id, /REALIZE
  
; launch xmanager
  xmanager, 'rti_gui', root_base_id, GROUP_LEADER=group
  
; back to the main calling program
  return, error
end
