; $Id: rti_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+ 
; NAME: 
;    rti_init 
; 
; PURPOSE: 
;    rti_init executes the initialization for the RTI utility module.
; 
; CATEGORY: 
;    initialization program 
; 
; CALLING SEQUENCE: 
;    error = rti_init(inp_yyy_t, out_yyy_t, par)
; 
; INPUTS/OUTPUTS/KEYWORDS/ETC.: 
;    see module help for a detailed description. 
; 
; ROUTINE MODIFICATION HISTORY: 
;    program written: March 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]
;    modifications  : 
;                   : april 2011,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]
;                    -routine debugged (in particular, for tags "angle" and 
;                     "time_integ")
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. 
;                    -keywords added to the header: NB_ANGLES,
;                     ANGLEn, ROT_TYPE
;- 
;
function rti_init, inp_yyy_t, out_yyy_t, par

; CAOS global common block
common caos_block, tot_iter, this_iter

; error code initialization
error = !caos_error.ok

; retrieve the module's informations
info = rti_info()

; STANDARD CHECKS
;
; compute and test the requested number of RTI arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'RTI error: par must be a structure'
if n ne 1 then message, 'RTI error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module RTI'

; check the input arguments
dummy = test_type(inp_yyy_t, TYPE=type)
if type eq 0 then begin         ; undefined variable
    inp_yyy_t = $
       {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
       }
endif
if test_type(inp_yyy_t, /STRUC, N_EL=n, TYPE=type) then $
   message, 'RTI error: wrong definition for the first input.'
if n ne 1 then message, $
   'RTI error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_yyy_t.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_yyy_t.data_type $
              +' ('+inp_type[0]+' expected).'
endif

if inp_yyy_t.data_status eq !caos_data.not_valid and not info.inp_opt[0] then $
      message, 'undefined input is not allowed'
;
if inp_yyy_t.data_type EQ 'img_t' then begin
pos=where(inp_yyy_t.time_integ[*,1] gt 1,num)
if ((size(inp_yyy_t.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_yyy_t.image=reform(inp_yyy_t.image,(size(inp_yyy_t.image))[1],(size(inp_yyy_t.image))[2],(size(inp_yyy_t.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE (MULTI-FRAME input is not supported)',/ERROR)
      message,'WRONG INPUT TYPE (MULTI-FRAME input is not supported)'
   endelse
endif
endif
; END OF STANDARD CHECKS


; DEFINITIONS  
pos=WHERE(FINITE(par.angle) EQ 1,NB_ANGLES)
if NB_ANGLES EQ 0 then message, "RTI error: angle array is empty."+    $
                              "Please, check angles in RTI GUI!"
case inp_yyy_t.data_type of
   'img_t': begin
      img=inp_yyy_t.image   
      bkg = inp_yyy_t.background
      bg_rot=1
   end
   'src_t' :begin
      img=inp_yyy_t.map
      bg_rot=0
; in the case of SRC_T, the  variable "background" is a single value
; and not an array. Thus in the rotation process it is not considered.
   end
   else: message, "Output type must be either 'img_t' or 'src_t'"
endcase

N=(size(img))[1]
M=(size(img))[2]
if (size(img))[0] EQ 2 then P=1 else P=(size(img))[3]


;rotation procedure..........................................
if P GT 1 then begin
   if P NE NB_ANGLES then message, "RTI ERROR: the number of angles is not equal to the number of images in the cube"
;cube images: each frame is rotated by the correspondent angle (j-th image by 
; the j-th element of the angle array)
;also the background array is rotated in the same way
   img_rot = make_array(par.out*N,par.out*M,P,type = size(img, /type))
   if bg_rot then background = make_array(par.out*N,par.out*M,P,type = size(bkg, /type))
   for j=0,P-1 do begin
      if FINITE(par.angle[j]) eq 1 then begin
         if par.method eq 0B then dummy=RTI_ROT_ROTATE(img[*,*,j],par.angle[j])
         if par.method eq 1B then dummy=RTI_FFT_ROTATE(img[*,*,j],par.angle[j],zero_padding=par.zp)
                                ; img is rotated according to the
                                ; selected method, while the
                                ; background is always rotated in
                                ; pixel-space 
         if bg_rot then dummy2=RTI_ROT_ROTATE(bkg[*,*,j], par.angle[j], missing=median(bkg[*,*,j]))
         
         IF par.out EQ 1 THEN begin
            img_rot[0,0,j]=dummy[N/2.:3*N/2.-1,N/2.:3*N/2.-1]
            if bg_rot then background[0,0,j]=dummy2[N/2.:3*N/2.-1,N/2.:3*N/2.-1]
         endif   ELSE begin
            img_rot[0,0,j]=dummy
            if bg_rot then background[0,0,j]=dummy2
         endelse
      endif else begin
         message, "RTI ERROR: one or more angles are NaN. "+$
                  "Please, check again the GUI!"
      endelse
   endfor     
endif else begin
; single images are rotated by NB_ANGLES angles (the output will be a cube)
   img_rot = make_array(par.out*N,par.out*M,NB_ANGLES,type = size(img, /type))
   if bg_rot then background = make_array(par.out*N,par.out*M,NB_ANGLES,type = size(bkg, /type))
   for j=0,NB_ANGLES-1 do begin
      if FINITE(par.angle[j]) eq 1 then begin
         if par.method eq 0B then dummy=RTI_ROT_ROTATE(img,par.angle[j])
         if par.method eq 1B then dummy=RTI_FFT_ROTATE(img,par.angle[j], zero_padding=par.zp)
                                ; img is rotated according to the
                                ; selected method, while the
                                ; background is always rotated in
                                ; pixel-space
         if bg_rot then dummy2=RTI_ROT_ROTATE(bkg, par.angle[j], missing=median(bkg))
         
         IF par.out EQ 1 THEN begin
            img_rot[0,0,j]=dummy[N/2.:3*N/2.-1,N/2.:3*N/2.-1]
            if bg_rot then background[0,0,j]=dummy2[N/2.:3*N/2.-1,N/2.:3*N/2.-1]
         endif ELSE begin
            img_rot[0,0,j]=dummy
            if bg_rot then background[0,0,j]=dummy2
         endelse
      endif else begin
         message, "RTI ERROR: the angle"+$
                  " can't be equal to NaN value! Check angles in RTI GUI"
      endelse
   endfor
endelse

;update the header 
header = *inp_yyy_t.header

if par.method eq 0B then fxaddpar, header, 'ROT_TYPE', 'P', '[RTI] rotation in the pixel space'
if par.method eq 1B then fxaddpar, header, 'ROT_TYPE', 'F', '[RTI] rotation in the Fourier space'
fxaddpar, header, 'NB_ANGLES', NB_ANGLES, '[RTI] number of angles'
for j=0,NB_ANGLES-1 do fxaddpar, header, 'ANGLE'+strtrim(j,1), par.angle[j], '[RTI] Rotation angle for img#'


; INITIALIZE THE OUTPUT STRUCTURES
; set the output structure type
case inp_yyy_t.data_type of
   'img_t': begin
      out_yyy_t =  {                             $
                   data_type    : 'img_t'              , $
                   data_status  : !caos_data.valid     , $
                   image        : img_rot              , $
                   psf          : inp_yyy_t.psf        , $
                   npixel       : N*par.out            , $
                   resolution   : inp_yyy_t.resolution , $
                   lambda       : inp_yyy_t.lambda     , $
                   width        : inp_yyy_t.width      , $
                   time_integ   : inp_yyy_t.time_integ , $
                   time_delay   : inp_yyy_t.time_delay , $
                   background   : background           , $
                   snr          : inp_yyy_t.snr        , $
                   header       : PTR_NEW('')            $
                   }
      *out_yyy_t.header = header ; updated the pointer
   end
   'src_t': begin
      out_yyy_t = {                                   $
                  data_type  : 'src_t',              $
                  data_status: !caos_data.valid,     $
                  off_axis   : inp_yyy_t.off_axis,   $
                  pos_ang    : inp_yyy_t.pos_ang,    $ 
                  dist_z     : inp_yyy_t.dist_z,     $
                  map        : img_rot,              $
                  scale_xy   : inp_yyy_t.scale_xy,   $
                  coord      : inp_yyy_t.coord,      $
                  scale_z    : inp_yyy_t.scale_z,    $
                  n_phot     : inp_yyy_t.n_phot,     $
                  background : inp_yyy_t.background, $
                  lambda     : inp_yyy_t.lambda,     $
                  width      : inp_yyy_t.width,      $
                  constant   : inp_yyy_t.constant,   $
                  header     : PTR_NEW('')           $
                 }
      *out_yyy_t.header = header ; update the header
   end
   else: message, "Output type must be either 'img_t' or 'src_t'"
endcase

end
