function Delta_c, img, PSF, xc, yc, n_alpha, out, $
                  ALPHA1=alpha1, ALPHA2=alpha2, TRESH=tresh
;+
;
; Inputs:
;  img = binary original image
;  PSF = estimated point-spread function
;  [xc,yc] = measured separation between components of the binary
;  n_alpha = number of steps for searching for intensity ratio alpha
;            between the two binary components
;            (with 1/n_alpha<=alpha<=1 or alpha1<=alpha<=alpha2)
;  ALPHA1,ALPHA2 = optional range for alpha search
;  TRESH = optional treshold for img
;
; Output:
;  all inputs
;  out = {alpha_range, alpha0}
;  Delta_c = discrepancy between img and synthesized images
;  (with a minimum for alpha0, and normalized to sum of img)
;
; Usage:
;  (1)
;  > Delta_c = Delta_c(img, PSF, xc, yc, n_alpha, out)
;  > plot, out.alpha_range, Delta_c                         --> see minimum
;  > print, out.alpha0                                      --> print alpha
;  (2)
;  > Delta_c = Delta_c(img, PSF, xc, yc, n_alpha, ALPHA1=alpha1, ALPHA2=alpha2, out)
;  > plot, out.alpha_range, Delta_c                         --> see minimum
;  > print, out.alpha0                                      --> print alpha
;
; Called routines:
;  - fftshift.pro [CAOS Library].
;
; Modification history:
;  - written 13/01-17/03/2016
;  [Marcel Carbillet, marcel.carbillet@unice.fr, Lagrange (UCA, OCA, CNRS)].
;
;-

; arrays linear dimension
dim=(size(img))[1]
if dim ne (size(PSF))[1] then message, "dimension problem between image & PSF"

; threshold
if not(keyword_set(TRESH)) then tresh = 0.

; recenter img wrt its maximun, positivize it, and normalize it to 1
idx=array_indices(img,where(img eq max(img)))
img=fftshift(img,dim/2-idx[0],dim/2-idx[1])
img=img>tresh & img[where(img eq tresh)]=0.
img=img/total(img,/DOUBLE)

; recenter PSF wrt its maximum, positivize it, and normalize it to 1
idx=array_indices(PSF,where(PSF eq max(PSF)))
PSF=fftshift(PSF,dim/2-idx[0],dim/2-idx[1])
PSF=PSF>0
PSF=PSF/total(PSF,/DOUBLE)

; initialize discrepancy between img and synthesized images (Delta_c),
; and ratio between first and second component of binary (alpha_range)
Delta_c=fltarr(n_alpha)
if not(keyword_set(ALPHA1)) and not(keyword_set(ALPHA2)) then $
   alpha_range=(1+findgen(n_alpha))/n_alpha
if keyword_set(ALPHA1) and not(keyword_set(ALPHA2)) then $
   message, "choose ALPHA2 too..."
if keyword_set(ALPHA2) and not(keyword_set(ALPHA1)) then $
   message, "choose ALPHA1 too..."
if keyword_set(ALPHA1) and keyword_set(ALPHA2) then begin
   if alpha2 gt alpha1 then                                         $
   alpha_range=alpha1+findgen(n_alpha)*(alpha2-alpha1)/(n_alpha-1L) $
   else message, "alpha2 must be greater than alpha1..."
endif

; compute Delta_c in function of alpha
for i=0,n_alpha-1 do begin

      PSF2 = fftshift(PSF,xc,yc)
      PSF2 = PSF2>0
      PSF2 = PSF2/total(PSF2,/DOUBLE)

      img_synth = PSF+alpha_range[i]*PSF2
      img_synth = img_synth/total(img_synth,/DOUBLE)

      Delta_c[i] = sqrt(total((img_synth-img)^2,/DOUBLE))

endfor

; prepare output out
Delta_C = Delta_c/total(img,/DOUBLE)                 ;  normalize Delta_C to sum of img
alpha0 = alpha_range(where(Delta_c eq min(Delta_c))) ; find minimum
out = {alpha_range:alpha_range, alpha0:alpha0}

; return Delta_c
return, Delta_c
end