function barycenter, image, DELTA_I=Delta_I
;+
;
; Input:
;  image : image for which calculate the barycenter
;  Delta_I : precision over intensity (if already known)
; 
; Returns:
;  out.X & out.Y = coordinates of barycenter
;  out.delta_X & out.delta_Y = precisions on [X,Y]
;
; Usage (example):
;  message, /INFO, barycenter(image)
;
; Modification history:
;  - routine originally created in 2011--2012
;  - modified on 13-27/01/2016:
;  calculate Delta_I, calculate precision, /DOUBLE, help, output, positivization, etc.
;  [Marcel Carbillet, marcel.carbillet@unice.fr, Lagrange (UCA, OCA, CNRS)]
;
;-

; inputs dimensions, vectors xx & yy (in units of pixel)
nx = (size(image))[1]
xx = findgen(nx)
ny = (size(image))[2]
yy = findgen(ny)
message, /INFO, '---'
message, /INFO, 'image peack value = ' + string(max(image))
message, /INFO, 'image minimum = ' + string(min(image))
message, /INFO, 'image mean = ' + string(mean(image))
message, /INFO, '---'

; input precision
; if Delta_I unknown: Delta_I=background rms,

; if Delta_I=0: calculate precision of array type
; (if "float" or "double", 0.5 if "integer" or "long").
if not(keyword_set(DELTA_I)) then begin
   ; 1st background rms evaluation (raw algo.)
   back_dim  = nx/4
   stat 	  =(moment(image[0:back_dim-1, 0:ny-1]) + moment(image[nx-back_dim:nx-1, 0:ny-1]) $
              + moment(image[0:nx-1, 0:back_dim-1]) + moment(image[0:nx-1, ny-back_dim:ny-1]))/4.
   back 	  = stat[0]
   back_rms  = sqrt(stat[1])
   message, /INFO, 'raw background rms (background) = '+string(back_rms)+' ('+string(back)+')'
   ; 2nd background rms evaluation (MMM algo.)
   sky, image, back_MMM, back_rms_MMM, /MEANBACK, /SILENT
   message, /INFO, 'MMM background rms (background) = '+string(back_rms_MMM)+' ('+string(back_MMM)+')'
   ; 3rd background rms evaluation (MEANCLIP algo.)
   sky, image, back_MCP, back_rms_MCP, /SILENT
   message, /INFO, 'MEANCLIP background rms (background) = ', back_rms_MCP+' ('+string(back_MCP)+')'
   ; worst background rms value => Delta_I
   Delta_I = max([back_rms, back_rms_MMM, back_rms_MCP])
   ; in case Delta_I is (still) null...
   if Delta_I eq 0. then begin
      if size(image,/TYPE) eq 4 then begin        ; "float" type arrays
         Delta_I=(machar()).eps
      endif else if size(image,/TYPE) eq 5 then begin
         Delta_I=(machar(/DOUBLE)).eps            ; "double" type arrays
      endif else Delta_I=.5                       ; "integer" or "long" type arrays
   endif
   message, /INFO, '=> intensity precision Delta_I = '+string(Delta_I)
   message, /INFO, '---'
endif

; barycenter computation (+precision)
img = image-min(image) ; positivization

I_tot  = total(img,/DOUBLE)
x_tot  = total(xx,/DOUBLE)
y_tot  = total(yy,/DOUBLE)
xI_tot = nx*total(xx##img,/DOUBLE) ; in order to avoid a double loop and a 2D array for xx
yI_tot = ny*total(img##yy,/DOUBLE) ; id. for yy

X = xI_tot/I_tot
Y = yI_tot/I_tot

delta_X = X*delta_I*( x_tot/xI_tot + (nx+ny)/I_tot )
delta_Y = Y*delta_I*( y_tot/yI_tot + (nx+ny)/I_tot )

; return barycenter & its precision
message, /INFO, 'X_barycenter = '+string(X)+'+/-'+string(delta_X)
message, /INFO, 'Y_barycenter = '+string(Y)+'+/-'+string(delta_Y)
message, /INFO, '---'
out = {X:X, Y:Y, delta_X:delta_X, delta_Y:delta_Y}
return, out
end