function detect_binary, img, d, out, DELTA_I=Delta_I, TRESH=tresh
;+
;
; Description:
;  Detect the positions of the two binary components in the array "img", given a treshold
;  for barycenters and maxima computations TRESH (if any), the original background rms
;  "DELTA_I" (if any), and the diameter "d" of the supposed components [px].
;  Error calculus is performed from an a priori of .5 pixel error for the positions and a
;  background rms evaluation for the intensities (if 0. then from precision of array type).
;  Result is put into structure out, where:
;     out = {rhox, Delta_rhox, rhoy, Delta_rhoy, I0, I1, alpha, Delta_alpha,
;            rho, Delta_rho, PA, Delta_PA, Dm, Delta_Dm},
;  and: rhox,rhoy=separation angle [px], alpha=intensity ratio (<1).
;       rho=separation angle [px], PA=position angle [deg.], Dm=diff. of mag. (>0).
;
; Example of usage:
;  > print, detect_binary(rec_sup, 3, out, TRESH=500000., DELTA_I=1000.)
;  > help, out
;  with "rec_sup" coming from a super-res. reconstruction, "Tresh" the threshold used for
;  super-res., and "Delta_I" the backgound computed from the original data.
;
; Called routines:
;   - barycenter.pro [associated routine].
;
; Modification history:
;  From older AIRY routine detect_binary_star.pro (Serge Correia, OAA-INAF (Arcetri), 1998-2000).
;  Modifications: - 19/01-16/03/2016: lots of modifications,
;                   ~specialized for outputs of super-res.
;                 [Marcel Carbillet, marcel.carbillet@unice.fr, Lagrange (UCA, OCA, CNRS)].
;
;-

; array dimension
dim = (size(img))[1]

; background evaluation & Delta_I stuff
back_dim  = 5L
stat 	 =(moment(img[0:back_dim-1, 0:dim-1]) + moment(img[dim-back_dim:dim-1, 0:dim-1]) $
         + moment(img[0:dim-1, 0:back_dim-1]) + moment(img[0:dim-1, dim-back_dim:dim-1]))/4.
back 	 = stat[0]
back_rms = sqrt(stat[1])
if not(keyword_set(TRESH)) then tresh = 3*back_rms+back

message, 'Background value =' + string(back), /INFO
message, 'Background rms   =' + string(back_rms), /INFO
message, 'Treshold value   =' + string(tresh), /INFO

if not(keyword_set(DELTA_I)) then begin
   if back_rms ne 0. then begin
      Delta_I = back_rms
   endif else begin
      if size(img,/TYPE) eq 4 then begin          ; "float" type arrays
         Delta_I=(machar()).eps
      endif else if size(img,/TYPE) eq 5 then begin
         Delta_I=(machar(/DOUBLE)).eps            ; "double" type arrays
      endif else Delta_I=0.                       ; "integer" or "long"
   endelse
endif

; initializations
mask 	  = img
mask_2    = fltarr(dim, dim)
mask_out  = fltarr(dim, dim)
res       = replicate({x:0., Delta_x:0., y:0., Delta_Y:0., value:0.}, 2)
out       = {rho_x:0., Delta_rhox:0., rho_y:0., Delta_rhoy:0., I0:0., I1:0., alpha:0., Delta_alpha:0., $
             rho:0., Delta_rho:0., PA:0., Delta_PA:0., Dm:0., Delta_Dm:0.}
flag      = 1B ; flag=0B if binary not detected (companion or both components)

; main component intensity and position
max = max(mask, pos_max)

; loop on components
for n = 0, 1 do begin

   message, 'Component detected, maximum =' + string(max), /INFO

   if max ge tresh then begin 

      mask_2 = img
      
	  res[n].value = max

	  x_max = pos_max mod dim
	  y_max = pos_max/dim

      out_bar = barycenter(img[x_max-d/2:x_max+d/2,y_max-d/2:y_max+d/2]>tresh, DELTA_I=Delta_I)
	  res[n].x = out_bar.X+x_max
	  res[n].y = out_bar.Y+y_max
      res[n].Delta_x = out_bar.Delta_X
      res[n].Delta_y = out_bar.Delta_Y

	  for i = 0, dim-1 do begin
	     for j=0, dim-1 do begin
		    if ((i-x_max)^2 + (j-y_max)^2) lt (d^2/4) then mask[i,j] = 0.
			if (((i-x_max)^2 + (j-y_max)^2) ge (d^2/4)) or 		$
			    (mask_out[i,j] ne 0.) then mask_2[i,j] = 0.			
		 endfor
	  endfor
	
	  mask_out = mask_out + mask_2
	  max = max(mask, pos_max)

   endif else begin 
      if n eq 0 then message, 'Main component not detected', /INFO
      if n eq 1 then message, 'Second component not detected', /INFO
      flag=0B
      return, flag
   endelse 

endfor

; detection => compute rho, PA, Dm, and corresponding errors
if flag then begin

   out.rho_x       = res[1].x-res[0].x
   out.Delta_rhox  = (res[0].Delta_x+res[1].Delta_x)/2.
   out.rho_y       = res[1].y-res[0].y
   out.Delta_rhoy  = (res[0].Delta_y+res[1].Delta_y)/2.
   out.I0          = res[0].value
   out.I1          = res[1].value
   out.alpha       = out.I1/out.I0                          ; intensity ratio (<=1)
   out.Delta_alpha = Delta_I*out.alpha*(1./out.I0+1./out.I1)

   out.rho         = sqrt(out.rho_x^2+out.rho_y^2)          ; separation [px]
   out.Delta_rho   = out.rho*(out.Delta_rhox*out.rho_x+out.Delta_rhoy*out.rho_y)
   out.PA          = atan(out.rho_y, out.rho_x)*!RADEG      ; position angle [deg.]
   out.Delta_PA    = (out.rho_y*out.Delta_rhox+out.rho_x*out.Delta_rhoy)/out.rho^2*!RADEG
   out.Dm          = 2.5*alog10(1./out.alpha)               ; magnitude difference (>0)
   out.Delta_Dm    = 2.5/alog(10)*out.Delta_alpha/out.alpha ; rmq: 1/alog(10)=alog10(exp(1))

endif

; return flag (1B=detection)
return, flag
end