function reg_f, fn, par
;+
; reg_f - compute the value of the regularizer
;   This function compute the value of the chosen regularizer R(x) at the
;   current iterate
;
; SYNOPSIS
;   f0 = reg_f(fn, par)
;
; INPUT
;   fn (double array)  - the reconstructed image at k-th iteration
;   par (structure)    - parameters' structure from the GUI
;
; OUTPUT
;   out - the value of R(x) in fn
;
;-

dim = size(fn,/dimensions)
c_entropy = 1. ; to be decided

case par.reg of ;choice of the type of regularization

   1: begin ; Tikhonov - zeroth order
      out = total(abs(fn)^2.,/double)*0.5
      end
   2: begin ; Tikhonov - second order (Laplacian)
      D=make_array(3,3,/double)
      D[0,1]=0.25
      D[1,0]=0.25
      D[1,2]=0.25
      D[2,1]=0.25
      dummy = fn - convol(fn,D)
      out = total(abs(dummy)^2.,/double)*0.5
      end
   3: begin ; Entropy
      f_ref=make_array(dim[0],dim[1],value=1.)
      f_ref=f_ref/total(f_ref,/double)*c_entropy
      out = total(fn*alog(fn/f_ref)+f_ref-fn, /double)
      end
   4: begin ; MRF
      psi_tot=0.
      for mm=0,7 do begin
        case mm of
        0 : begin ;dx
            dm=shift(fn,-1,0)
            dm[dim[0]-1,*]=dm[dim[0]-2,*]
            psi=2.*(sqrt(((dm-fn)^2.)+par.delta^2)-par.delta)
            end
        1 : begin ;sx
            dm=shift(fn,+1,0)
            dm[0,*]=dm[1,*]
            psi=2.*(sqrt(((dm-fn)^2.)+par.delta^2)-par.delta)
            end
        2 : begin ;up
            dm=shift(fn,0,-1)
            dm[*,dim[1]-1]=dm[*,dim[1]-2]
            psi=2.*(sqrt(((dm-fn)^2.)+par.delta^2)-par.delta)
            end
        3 : begin ;dwn
            dm=shift(fn,0,+1)
            dm[*,0]=dm[*,1]
            psi=2.*(sqrt(((dm-fn)^2.)+par.delta^2)-par.delta)
            end
        4 : begin ;up-dx
            dm=shift(fn,-1,-1)
            dm[dim[0]-1,*]=dm[dim[0]-2,*]
            dm[*,dim[1]-1]=dm[*,dim[1]-2]
            psi=2.*(sqrt((((dm-fn)/sqrt(2.))^2)+par.delta^2)-par.delta)
            end
        5 : begin ;dwn-dx
            dm=shift(fn,+1,-1)
            dm[0,*]=dm[1,*]
            dm[*,dim[1]-1]=dm[*,dim[1]-2]
            psi=2.*(sqrt((((dm-fn)/sqrt(2.))^2)+par.delta^2)-par.delta)
            end
        6 : begin ;up-sx
            dm=shift(fn,-1,+1)
            dm[dim[0]-1,*]=dm[dim[0]-2,*]
            dm[*,0]=dm[*,1]
            psi=2.*(sqrt((((dm-fn)/sqrt(2.))^2.)+par.delta^2)-par.delta)
            end
        7 : begin ;dwn-sx
            dm=shift(fn,+1,+1)
            dm[0,*]=dm[1,*]
            dm[*,0]=dm[*,1]
            psi=2.*(sqrt((((dm-fn)/sqrt(2.))^2.)+par.delta^2)-par.delta)
            end
        endcase
      psi_tot=psi_tot+psi
      endfor
      out = total(psi_tot,/double)*0.25
      end
   5: begin ; HDR
      out = 0.5*par.delta^2*total(fn^2. /(fn^2. + par.delta^2), /double)
      end
   6: begin ; MISTRAL
      dummy1=(shift(fn,-1,0)-fn)^2+(shift(fn,0,-1)-fn)^2  ;fn=shift(fn,0,0)
      out = total(sqrt(dummy1)-par.delta*alog(1.+1./par.delta*sqrt(dummy1)),/double)
      end
   7: begin ; Hypersurface (aka TV Smooth)
      dx1 = [fn[1:dim[0]-1,*]-fn[0:dim[0]-2,*] , fn[0,*]-fn[dim[0]-1,*]]
      dx2 = [[fn[*,1:dim[1]-1]-fn[*,0:dim[1]-2]], [fn[*,0]-fn[*,dim[1]-1]]]
      out = total(sqrt(dx1^2 + dx2^2 + par.delta^2),/double)
      end
   8: begin ; Tikhonov - first order
      dx1 = [fn[1:dim[0]-1,*]-fn[0:dim[0]-2,*] , fn[0,*]-fn[dim[0]-1,*]]
      dx2 = [[fn[*,1:dim[1]-1]-fn[*,0:dim[1]-2]], [fn[*,0]-fn[*,dim[1]-1]]]
      out = 0.5*total(dx1^2 + dx2^2,/double)
      end
   9: begin ; L1 regularization
      out = total(abs(fn),/double)
      end
endcase
return, out
end

; =====================
; End of reg_f.pro file
; =====================
