function reg_grad_dec, fn, par, eps = eps

; reg_grad_dec - compute the splitting of the gradient of the regularizer
;   This function splits the gradient of a chosen regularizer R(x) into the difference
;   of two non-negative parts [1]
;                \nabla R(x) = V_R(x) - U_R(x),    U_R(x)>=0, V_R(x)>=0
;
;   [1] H. Lantri, M. Roche and C. Aime, "Penalized maximum likelihood image
;       restoration with positivity constraints: multiplicative algorithms",
;       Inverse Probl. 18, 1397 - 1419 (2002)
;
; SYNOPSIS
;   grad_dec = reg_grad_dec(fn, par, [eps=eps])
;
; INPUT
;   fn (double array)  - the reconstructed image at k-th iteration
;   par (structure)    - parameters' structure from the GUI
;
; OUTPUT
;   grad_dec (struct) - gradient decomposition
;                       grad_dec.U_R: U_R part
;                       grad_dec.V_R: V_R part
;
; OPTIONAL
;   eps (double)   - threshold for Entropy regularization
;
; ------------------------------------------------------------------------------

dim = size(fn,/dimensions)
apo = 1. ; to be decided
c_entropy = 1. 

if (keyword_set(eps) - 1) then eps = 1.e-10     ; eps threshold in Entropy regularization

case par.reg of                 ;choice of the type of regularization
     1: begin ; Tikhonov - zeroth order
         U = 0.
         V = fn
        end
     2: begin ; ; Tikhonov - second order (Laplacian)
         D = make_array(3,3,/double)
         D[0,1] = 0.25
         D[1,0] = 0.25
         D[1,2] = 0.25
         D[2,1] = 0.25
         V = convol(fn,D)
         U = 2.0*V
         V = convol(V,D)
         V = V+fn
         U = U*apo
         V = V*apo
        end
     3: begin ; Entropy
         V = 2.0*alog(dim[0])
         U = alog(c_entropy) - alog(fn+eps)
        end
     4: begin ; MRF
         U=0
         V=0
         for mm = 0,7 do begin
             case mm of
                  0: begin ;dx
                      dm = shift(fn,-1,0)
                      dm[dim[0]-1,*] = dm[dim[0]-2,*]
                      vm = 1./sqrt(((dm-fn)^2.)+par.delta^2)
                     end
                  1: begin ;sx
                      dm = shift(fn,+1,0)
                      dm[0,*] = dm[1,*]
                      vm = 1./sqrt(((dm-fn)^2.)+par.delta^2)
                     end
                  2: begin ;up
                      dm = shift(fn,0,-1)
                      dm[*,dim[1]-1] = dm[*,dim[1]-2]
                      vm = 1./sqrt(((dm-fn)^2.)+par.delta^2)
                     end
                  3: begin ;dwn
                      dm = shift(fn,0,+1)
                      dm[*,0] = dm[*,1]
                      vm = 1./sqrt(((dm-fn)^2.)+par.delta^2)
                     end
                  4: begin ;up-dx
                      dm = shift(fn,-1,-1)
                      dm[dim[0]-1,*] = dm[dim[0]-2,*]
                      dm[*,dim[1]-1] = dm[*,dim[1]-2]
                      vm = (1./sqrt((((dm-fn)/(sqrt(2.)))^2.)+par.delta^2))*$
                      (1./(sqrt(2.)))
                     end
                  5: begin ;dwn-dx
                      dm = shift(fn,+1,-1)
                      dm[0,*] = dm[1,*]
                      dm[*,dim[1]-1] = dm[*,dim[1]-2]
                      vm = (1./sqrt((((dm-fn)/(sqrt(2.)))^2.)+par.delta^2))*$
                      (1./(sqrt(2.)))
                     end
                  6: begin ;up-sx
                      dm = shift(fn,-1,+1)
                      dm[dim[0]-1,*] = dm[dim[0]-2,*]
                      dm[*,0] = dm[*,1]
                      vm = (1./sqrt((((dm-fn)/(sqrt(2.)))^2.)+par.delta^2))*$
                      (1./(sqrt(2.)))
                     end
                  7: begin ;dwn-sx
                      dm = shift(fn,+1,+1)
                      dm[0,*] = dm[1,*]
                      dm[*,0] = dm[*,1]
                      vm = (1./sqrt((((dm-fn)/(sqrt(2.)))^2.)+par.delta^2))*$
                       (1./(sqrt(2.)))
                     end
             endcase
             V = V+vm
             U = U+(vm*dm)
         endfor
         V = fn*V
         U = 0.5*U*apo
         V = 0.5*V*apo
        end
     5: begin ; HDR
         U=0
         V = par.delta^4*fn/((par.delta^2 + fn^2)^2)
        end
     6: begin ; MISTRAL
         dummy1 = (shift(fn,-1,0)-fn)^2+(shift(fn,0,-1)-fn)^2
         dummy2 = (fn-shift(fn,1,0))^2+(shift(fn,1,-1)-shift(fn,1,0))^2
         dummy3 = (shift(fn,-1,1)-shift(fn,0,1))^2+(fn-shift(fn,0,1))^2
         den1 = par.delta + sqrt(dummy1)
         den2 = par.delta + sqrt(dummy2)
         den3 = par.delta + sqrt(dummy3)
         U = (shift(fn,-1,0)+shift(fn,0,-1))/den1 + shift(fn,1,0)/den2 + shift(fn,0,1)/den3
         V = fn * (2./den1 + 1./den2 + 1./den3)
        end
     7: begin ; Hypersurface (aka TV Smooth)
         dx1 = [fn[1:dim[0]-1,*]-fn[0:dim[0]-2,*] , fn[0,*]-fn[dim[0]-1,*]]
         dx2 = [[fn[*,1:dim[1]-1]-fn[*,0:dim[1]-2]], [fn[*,0]-fn[*,dim[1]-1]]]
         densq = sqrt(dx1^2 + dx2^2 + par.delta^2)
         dx1 = dx1/densq
         dx2 = dx2/densq
         g = [-dx1[0,*]+dx1[dim[0]-1,*],-dx1[1:dim[0]-1,*]+dx1[0:dim[0]-2,*]] + [[-dx2[*,0]+dx2[*,dim[1]-1]],[-dx2[*,1:dim[1]-1]+dx2[*,0:dim[1]-2]]]
         V = fn*(2./densq + 1./[densq[dim[0]-1,*], densq[0:dim[0]-2,*]] + 1./[[densq[*,dim[1]-1]] , [densq[*,0:dim[1]-2]]])
         U = V - g
        end
     8: begin ; Tikhonov - first order
         V = 4*fn
         U = dblarr(dim)
         U[1:dim[0]-2,1:dim[1]-2] = fn[2:dim[0]-1,1:dim[1]-2] + fn[1:dim[0]-2,2:dim[1]-1] + fn[0:dim[0]-3,1:dim[1]-2] + fn[1:dim[0]-2,0:dim[1]-3]
         U[1:dim[0]-2,0]          = fn[2:dim[0]-1,0] + fn[1:dim[0]-2,1] + fn[0:dim[0]-1,0] + fn[1:dim[0]-2,dim[1]-1]
         U[1:dim[0]-2,dim[1]-1]   = fn[2:dim[0]-1,dim[1]-1] + fn[1:dim[0]-2,0] + fn[0:dim[0]-3,dim[1]-1] + fn[1:dim[0]-2,dim[1]-2]
         U[0,1:dim[1]-2]          = fn[1,1:dim[1]-2] + fn[0,2:dim[1]-1] + fn[dim[0]-1,1:dim[1]-2] + fn[0,0:dim[1]-3]
         U[dim[0]-1,1:dim[1]-2]   = fn[0,1:dim[1]-2] + fn[dim[0]-1,2:dim[1]-1] + fn[dim[0]-2,1:dim[1]-2] + fn[dim[0]-1,0:dim[1]-3]
         U[0,0]                   = fn[1,0] + fn[0,1] + fn[dim[0]-1,0] + fn[0,dim[1]-1]
         U[0,dim[1]-1]            = fn[1,dim[1]-1] + fn[0,0] + fn[dim[0]-1,dim[1]-1] + fn[0,dim[1]-2]
         U[dim[0]-1,0]            = fn[0,0] + fn[dim[0]-1,1] + fn[dim[0]-2,0] + fn[dim[0]-1,dim[1]-1]
         U[dim[0]-1,dim[1]-1]     = fn[0,dim[1]-1] + fn[dim[0]-1,0] + fn[dim[0]-2,dim[1]-1] + fn[dim[0]-1,dim[1]-2]
        end
     9: begin ; L1 regularization
         V = 1.
         U = 0.
        end
endcase
return, {U_R:U, V_R:V}

end
