; $Id: deconvolve_sgp.pro, v 7.2 last revision 2017/07/22 Andrea La Camera $
;+
; PURPOSE:
;    DECONVOLVE_SGP computes the generic iteration of the 
;    Scaled Gradient Projection (SGP) algorithm. See [1] for reference.
;    [1]Bonettini, S., Zanella, R. & Zanni, L. A scaled gradient projection 
;     method for constrained image deblurring. Inverse Problems 25, 015002 (2009). 
;
; INPUT: 
;   x - initial point
;   init - the structure that contains initialization of the algorithm
;   par - the structure that contains parameters (from GUI)
;
; OUTPUT:
;   out - the iterate after the SGP steps
;
; COMMON BLOCK(s):
;   caos_block --> tot_iter (total number of iterations)
;              --> this_iter (current iteration)
;
; Written: 2013/04/09,
; Andrea La Camera (DIBRIS, USG, Italy) [andrea.lacamera@unige.it]
;    modifications  : December 2013,
;                    -Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]
;                     1D case added (debugged).
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -regularizations added; general debugging
;                   : July 2017,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -if a stopping rule is satisfied, the variable "last_iter" is 
;                     used for storing current iteration, next "this_iter" is set to the 
;                     total iteration number, and then "tot_iter" is updated with 
;                     the current iteration number (i.e the very last iteration).  
;-

pro deconvolve_sgp, x, init, par, out
  
  common caos_block, tot_iter, this_iter
  
  common dec_iter, last_iter
  
  init.Valpha[0:init.Malpha-2] = init.Valpha[1:init.Malpha-1]
  if init.M gt 1 then init.Fold[0:init.M-2] = init.Fold[1:init.M-1]
  init.Fold[init.M-1] = init.fv
                                ; compute step along the negative scaled gradient
  y = x - init.alpha*init.XX*init.g
  
                                ; projection 
    if (par.flux_con) then begin
       y[init.regionR_index] = project_fletcher_nonneg(reform(init.ATM[init.regionR_index]), init.c*init.dim[2], reform(y[init.regionR_index])*init.D[init.regionR_index], init.D[init.regionR_index])
    endif else begin
       temp =  where(y lt 0, n_nonpos)
       if (n_nonpos gt 0) then y[temp] = 0.
    endelse
  y_nonpos = where(y lt 0, n_nonpos)
  if (n_nonpos gt 0) then y[y_nonpos] = 0
                                ; in the case of super-resolution
                                ; another projection is needed
  if (par.init_mask) then y = init.image_mask * y


                                ; compute descent direction
  d = y - x
  gd = total(d*init.g,/double)
                                ; Armijo linesearch
  lam = 1.
  fcontinue = 1B
  d_tf = afunction(d,init.TF) ; exploiting linearity
  fr = max(init.Fold)
  while fcontinue do begin
     xplus = x + lam*d
     x_tf_try = init.x_tf + lam*d_tf
     temp = init.gn/(x_tf_try + init.bg)
     if (par.reg GT 0) then begin
        f_R = reg_f(xplus, par)
     endif else begin
        f_R = 0.
     endelse
     
     if (init.P*init.obj_size[1] EQ 1) then begin
        f_0 = total(init.gn*alog(temp),/double) + total(x_tf_try,/double) - total(init.flux,/double)
     endif  else begin
        f_0=total(init.gn[init.obj_index_multi]*alog(temp[init.obj_index_multi]),/double)+total(x_tf_try[init.obj_index_multi],/double)-total(init.flux,/double)
     endelse
     init.fv=f_0+par.beta*f_R
     if ( (init.fv le fr + init.beta*lam*gd) OR (lam lt 1.e-12) ) then begin
        x = xplus          ;clear xplus
        sk = lam*d
        init.x_tf = x_tf_try
        gtemp = afunction(temp,init.CTF)
        if init.P gt 1 then begin
           gtemp = total(gtemp,3,/double)
        endif
        if par.reg gt 0 then begin
           grad_dec = reg_grad_dec(x, par)
        endif else begin
           grad_dec = {U_R:0., V_R:0.}
        endelse
        gtemp = init.ATM - gtemp + par.beta*(grad_dec.V_R - grad_dec.U_R)
        yk = gtemp - init.g
        init.g = gtemp
        fcontinue = 0B
     endif else begin
        lam = lam * init.theta
     endelse
  endwhile
  
  if (init.fv ge fr) AND (par.verb) then print, 'WARNING: fv >= fr'
  
                                ; compute new steplength and scaling matrices
                                ; scaling matrix - diffuse part
  temp = init.ATM + par.beta*grad_dec.V_R
  Weight_R = dblarr(init.work_size)
  Weight_R[init.regionR_index] = 1./temp[init.regionR_index]
  init.XX = Weight_R*x
  XX_toolow = where(init.XX lt init.X_low_bound, nlow)
  if (nlow gt 0) then init.XX[XX_toolow] = init.X_low_bound
  XX_toohigh = where(init.XX gt init.X_upp_bound, nhigh)
  if (nhigh gt 0) then init.XX[XX_toohigh] = init.X_upp_bound
  if (par.bound) then begin
     temp = dblarr(init.work_size)
     temp[init.regionR_index] = init.XX[init.regionR_index]
     init.XX = temp
    endif
  init.D=dblarr(init.work_size)
  init.D[init.regionR_index] = 1./init.XX[init.regionR_index]
                                ; steplenght
  sk2 = sk*init.D
  yk2 = yk*init.XX
  bk = total(sk2*yk)
  ck = total(yk2*sk)
  if (bk le 0) then begin
     alpha1 = min([10.*init.alpha,init.alpha_max])
  endif else begin
     alpha1BB = total(sk2*sk2)/bk
     alpha1 = min([init.alpha_max, max([init.alpha_min, alpha1BB])])
  endelse
  if (ck le 0) then begin
     alpha2 = min([10.*init.alpha,init.alpha_max])
  endif else begin
     alpha2BB = ck/total(yk2*yk2)
     alpha2 = min([init.alpha_max, max([init.alpha_min, alpha2BB])])
  endelse
  init.Valpha[init.Malpha-1] = alpha2
  if (this_iter le 20) then begin
     init.alpha = min(init.Valpha)
  endif else begin
     if (alpha2/alpha1 lt init.tau) then begin
        init.alpha = min(init.Valpha)
        init.tau = init.tau*0.9
     endif else begin
        init.alpha = alpha1
        init.tau = init.tau*1.1
     endelse
  endelse
  init.alpha = double(init.alpha)  
  init.discrepancy[this_iter] = init.Discr_coeff * init.fv
  init.obj_fun[this_iter] = init.fv * init.scaling
   
                                ; error
    if init.errflag then begin
        e = x[init.obj_index] - init.obj[*]
        init.rho[this_iter] = sqrt(total(e*e)/init.obj_sum)
    endif
                                ; stop criteria 
  case par.stopping_rule of
                                ; STOPPING RULE #1 (TOT_ITER REACHED!)
      0: begin                  ;really un-useful ;)
         if this_iter GE tot_iter then begin
            last_iter = this_iter
            if (par.verb) then print, '[#1]  --> STOP AT ITER = ' , this_iter
         endif
      end
                                ; STOPPING RULE #2 (NORM OF THE OBJECT)
     1: begin  
        normstep = total(sk*sk,/double) / total(x*x,/double)
        if (normstep lt par.stop_tol) then begin
           if (par.verb) then print, '[#2]  --> STOP AT ITER = ' , this_iter
           if (par.verb) then print, ' || f(k) - f(k-1) ||^2 / || f(k) ||^2 = '+strtrim(normstep,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
        endif
     end
                                ;  STOPPING RULE #3  (TOTAL FUNCTIONAL IS APPROX CONSTANT)
     2: begin
        reldecrease = abs(init.fv - init.Fold[init.M-1])/abs(init.fv)
        if (reldecrease lt par.stop_tol) then begin
           if (par.verb) then print, '[#3]  --> STOP AT ITER = ' , this_iter
           if (par.verb) then print, ' | J(k) - J(k-1) | / | J(k) | = '+strtrim(reldecrease,1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
        endif
     end
                                ;  STOPPING RULE #4  (DISCREPANCY CROSSES THE GIVEN TOLERANCE ~~
                                ;  should be around 1)
     3: begin
        if (this_iter GT 2) AND (init.discrepancy[this_iter] lt par.stop_tol) then begin
           if (par.verb) then print, '[#4]  --> STOP AT ITER = ' , this_iter
           if (par.verb) then print, ' D(k) = '+strtrim(init.discrepancy[this_iter],1), ' ;   tol = '+strtrim(par.stop_tol,1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
        endif
     end
                                ;  STOPPING RULE #5  (MINIMUM OF THE RESTORATION ERROR)
     4: begin
        if (this_iter GT 2) AND (init.rho[this_iter] GT init.rho[this_iter-1]) then begin
           if (par.verb) then print, '[#5]  --> STOP AT ITER = ' , this_iter
           if (par.verb) then print, ' RHO (k) = '+strtrim(init.rho[this_iter],1)
               last_iter = LONG(this_iter)
               this_iter=LONG(tot_iter)
               tot_iter=LONG(last_iter)
        endif
      end
     else: message, 'Unknown stopping criterion:', stopcrit
  endcase
    
  if par.verb gt 0 then begin
     print, 'Iteration:', this_iter,  '  Fobj:', init.fv, '  Alpha:', init.alpha, '  Lambda:', lam
  endif



                                ; save & exit 
  out = x
end
